@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -114,14 +114,6 @@ WBContextModel::WBContextModel(WBContextUI *wbui)
   _sidebar_dockpoint = NULL;
   _template_panel = NULL;
 
-  // Default table templates list
-  grt::DictRef options(_wbui->get_wb()->get_root()->options()->options());
-  if (!options.has_key("TableTemplates"))
-  {
-    grt::ListRef<db_Table> templates = grt::ListRef<db_Table>::cast_from(_wbui->get_wb()->get_grt_manager()->get_grt()->unserialize(make_path(_wbui->get_wb()->get_datadir(), "data/table_templates.xml")));
-    options.set("TableTemplates", templates);
-  }
-
   scoped_connect(_wbui->get_wb()->get_root()->options()->signal_dict_changed(),boost::bind(&WBContextModel::option_changed, this, _1, _2, _3));
 
   setup_secondary_sidebar();
@@ -166,6 +158,32 @@ void WBContextModel::setup_secondary_sidebar()
   _secondary_sidebar->add_page(_template_panel, _("Templates"));
 }
 
+void WBContextModel::notify_catalog_tree_view(const CatalogNodeNotificationType &notify_type, grt::ValueRef value, const std::string &diagram_id)
+{
+  std::map<std::string, ModelDiagramForm*>::iterator it;
+  if (diagram_id.empty())
+  {
+    for (it = _model_forms.begin(); it != _model_forms.end(); ++it)
+      it->second->notify_catalog_tree(notify_type, value);
+  }
+  else
+  {
+    it = _model_forms.find(diagram_id);
+    if (it != _model_forms.end())
+    {
+      it->second->notify_catalog_tree(notify_type, value);
+    }
+  }
+}
+
+void WBContextModel::refill_catalog_tree()
+{
+  std::map<std::string, ModelDiagramForm*>::iterator it;
+  for (it = _model_forms.begin(); it != _model_forms.end(); ++it)
+    it->second->refill_catalog_tree();
+
+}
+
 
 mforms::TreeNodeView *WBContextModel::create_user_type_list()
 {
@@ -191,18 +209,6 @@ mforms::TreeNodeView* WBContextModel::create_history_tree()
 
 //--------------------------------------------------------------------------------------------------
 
-mforms::TreeNodeView* WBContextModel::create_catalog_tree()
-{
-  CatalogTreeBE *model = _wbui->get_wb()->get_component<WBComponentPhysical>()->get_catalog_tree_model();
-  CatalogTreeView *tree = new CatalogTreeView(model);
-
-  // No tree loading/refresh for now. This will happen when the diagram is ready
-  // and we know which object is already placed.
-  return tree;
-}
-
-//--------------------------------------------------------------------------------------------------
-
 grt::GRT *WBContextModel::get_grt()
 {
   return _wbui->get_wb()->get_grt();
@@ -621,6 +627,7 @@ void WBContextModel::activate_canvas_object(const model_ObjectRef &object, ssize
 void WBContextModel::register_diagram_form(ModelDiagramForm *view)
 {
   _model_forms[view->get_model_diagram().id()]= view;
+  view->refill_catalog_tree();
 }
 
 
@@ -1054,7 +1061,7 @@ int WBContextModel::get_object_list_popup_items(bec::UIForm *form,
   item.type= MenuAction;
   items.push_back(item);
 
-  if (nodes.empty() || nodes[0][0] != 0)
+  if ((nodes.empty() || nodes[0][0] != 0) && objects.count() > 0)
   {
     item.caption= label.empty() ? _("Remove Figure") : strfmt(_("Remove Figure %s"), label.c_str());
     item.name= "builtin:removeFigure";
@@ -1409,13 +1416,14 @@ void WBContextModel::add_new_diagram(const model_ModelRef &model)
   
  // model_DiagramRef view(model_DiagramRef::cast_from(_wbui->get_wb()->execute_in_grt_thread("Create new diagram",
    //                                                                       boost::bind(&model_Model::addNewDiagram, &model.content(), true))));
+  _wbui->get_wb()->lock_gui(true);
   model_DiagramRef view = model->addNewDiagram(true);
   if (view.is_valid())
   {
     model->currentDiagram(view);
     view->get_data()->realize();
   }
-  
+  _wbui->get_wb()->lock_gui(false);
   _wbui->get_wb()->show_status_text(_("Diagram added."));
 }
 
@@ -1447,7 +1455,23 @@ bool WBContextModel::delete_object(model_ObjectRef object)
   FOREACH_COMPONENT(_wbui->get_wb()->_components, iter)
   {
     if ((*iter)->handles_figure(object))
-      return (*iter)->delete_model_object(object, false);
+    {
+      grt::ValueRef value;
+      grt::ObjectRef obj;
+      if (object.is_instance(model_Figure::static_class_name()))
+      {
+        obj = (*iter)->get_object_for_figure(model_FigureRef::cast_from(object));
+        value = (*iter)->get_object_for_figure(model_FigureRef::cast_from(object));
+
+      }
+
+      if ((*iter)->delete_model_object(object, false))
+      {
+        notify_catalog_tree_view(NodeDelete, value);
+        return true;
+      }
+      return false;
+    }
   }
   return false;
 }
@@ -1460,7 +1484,18 @@ bool WBContextModel::remove_figure(model_ObjectRef object)
   FOREACH_COMPONENT(_wbui->get_wb()->_components, iter)
   {
     if ((*iter)->handles_figure(object))
-      return (*iter)->delete_model_object(object, true);
+    {
+      grt::ValueRef value;
+      if (object.is_instance(model_Figure::static_class_name()))
+        value = (*iter)->get_object_for_figure(model_FigureRef::cast_from(object));
+
+      if ((*iter)->delete_model_object(object, true))
+      {
+        notify_catalog_tree_view(NodeUnmark, value, view->id());
+        return true;
+      }
+      return false;
+    }
   }
   return false;
 }
@@ -1540,9 +1575,9 @@ mforms::View *WBContextModel::shared_secondary_sidebar()
 
 //--------------------------------------------------------------------------------------------------
 
-static struct RegisterNotifDocs
+static struct RegisterNotifDocs_wb_context_model
 {
-  RegisterNotifDocs()
+  RegisterNotifDocs_wb_context_model()
   {
     base::NotificationCenter::get()->register_notification("GRNModelOpened",
                                                            "modeling",
@@ -1562,5 +1597,5 @@ static struct RegisterNotifDocs
                                                            "ui.ModelPanel instance",
                                                            "");
   }
-} initdocs;
+} initdocs_wb_context_model;
 
