@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "workbench/wb_context.h"
 #include "wb_diagram_options.h"
 
@@ -232,7 +230,7 @@ void DiagramOptionsBE::set_xpages(int c)
   int xc, yc;
   get_min_size_in_pages(xc, yc);
   
-  if (c > 0 && c != get_xpages() && c >= xc)
+  if (c > 0 && c != get_xpages() && c >= xc && _sizer)
   {
     _sizer->_width= _sizer->_paper_size.width * c;
     _sizer->set_needs_render();
@@ -247,7 +245,7 @@ void DiagramOptionsBE::set_ypages(int c)
   int xc, yc;
   get_min_size_in_pages(xc, yc);
 
-  if (c > 0 && c != get_ypages() && c >= yc)
+  if (c > 0 && c != get_ypages() && c >= yc && _sizer)
   {
     _sizer->_height= _sizer->_paper_size.height * c;
     _sizer->set_needs_render();
