@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,9 +17,6 @@
  * 02110-1301  USA
  */
 
-
-#include "stdafx.h"
-
 #include "wb_overview_diagram.h"
 #include "wb_overview_physical.h"
 #include "wb_overview_physical_schema.h"
@@ -66,7 +63,7 @@ PhysicalSchemataNode::PhysicalSchemataNode(workbench_physical_ModelRef amodel)
   type= OverviewBE::ODivision;
   object= amodel->catalog();
   model= amodel;
-  label= _("Physical Schemata");
+  label= _("Physical Schemas");
   small_icon= 0;
   large_icon= 0;
   expanded= true;
@@ -92,7 +89,7 @@ OverviewBE::Node * PhysicalSchemataNode::create_child_node(db_SchemaRef schema)
 
 bool PhysicalSchemataNode::add_object(WBContext *wb)
 {
-  wb->get_component<WBComponentPhysical>()->add_new_db_schema(model);
+  wb->get_grt_manager()->open_object_editor(wb->get_component<WBComponentPhysical>()->add_new_db_schema(model));
   return true;
 }
 
@@ -325,7 +322,7 @@ bool SQLScriptsNode::add_new(WBContext *wb)
 {
   WBComponentPhysical *compo= wb->get_component<WBComponentPhysical>();
   
-  compo->add_new_stored_script(_model, "");
+  wb->get_grt_manager()->open_object_editor(compo->add_new_stored_script(_model, ""));
   return true;
 }
 
@@ -401,7 +398,7 @@ bool NotesNode::add_new(WBContext *wb)
 {
   WBComponentPhysical *compo= wb->get_component<WBComponentPhysical>();
   
-  compo->add_new_stored_note(_model, "");
+  wb->get_grt_manager()->open_object_editor(compo->add_new_stored_note(_model, ""));
   return true;
 }
 
@@ -473,31 +470,85 @@ public:
   }
 };
 
-
-
+//----------------- PhysicalOverviewBE -------------------------------------------------------------
 
 PhysicalOverviewBE::PhysicalOverviewBE(WBContext *wb)
 : OverviewBE(wb), _menu(0), _toolbar(0)
 {
   _schemata_node_index= 1;
   _menu = 0;
+  NotificationCenter::get()->add_observer(this, "GNColorsChanged");
 }
 
+//--------------------------------------------------------------------------------------------------
 
 PhysicalOverviewBE::~PhysicalOverviewBE()
 {
+  NotificationCenter::get()->remove_observer(this);
+  delete _toolbar;
   delete _menu;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+void PhysicalOverviewBE::handle_notification(const std::string &name, void *sender, base::NotificationInfo &info)
+{
+  if (name == "GNColorsChanged")
+  {
+    // Single colors or the entire color scheme changed.
+    update_toolbar_icons();
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
 mforms::ToolBar *PhysicalOverviewBE::get_toolbar()
 {
   if (!_toolbar)
   {
     _toolbar = _wb->get_ui()->get_command_ui()->create_toolbar("data/model_toolbar.xml");
+    update_toolbar_icons();
   }
   return _toolbar;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+// Implemented in wb_sql_editor_form_ui.cpp.
+extern std::string find_icon_name(std::string icon_name, bool use_win8);
+
+void PhysicalOverviewBE::update_toolbar_icons()
+{
+  bool use_win8;
+
+  switch (base::Color::get_active_scheme())
+  {
+  case base::ColorSchemeStandardWin8:
+  case base::ColorSchemeStandardWin8Alternate:
+    use_win8 = true;
+    break;
+
+  default:
+    use_win8 = false;
+  }
+
+  mforms::ToolBarItem *item = _toolbar->find_item("wb.toggleSidebar");
+  if (item != NULL)
+  {
+    item->set_icon(find_icon_name(item->get_icon(), use_win8));
+    item->set_alt_icon(find_icon_name(item->get_alt_icon(), use_win8));
+  }
+
+  item = _toolbar->find_item("wb.toggleSecondarySidebar");
+  if (item != NULL)
+  {
+    item->set_icon(find_icon_name(item->get_icon(), use_win8));
+    item->set_alt_icon(find_icon_name(item->get_alt_icon(), use_win8));
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
 OverviewBE::ContainerNode * PhysicalOverviewBE::create_root_node(workbench_physical_ModelRef model, PhysicalOverviewBE *owner)
 {
   return new PhysicalRootNode(model, owner);
@@ -508,9 +559,9 @@ bool PhysicalOverviewBE::can_close()
   return _wb->can_close_document();
 }
 
-bool PhysicalOverviewBE::close()
+void PhysicalOverviewBE::close()
 {
-  return _wb->close_document();
+  _wb->close_document();
 }
 
 std::string PhysicalOverviewBE::identifier() const
@@ -553,6 +604,12 @@ model_ModelRef PhysicalOverviewBE::get_model()
 }
 
 
+int PhysicalOverviewBE::get_default_tab_page_index()
+{
+  return (int)_model->catalog()->schemata().get_index(_model->catalog()->defaultSchema());
+}
+
+
 static bool has_selection(PhysicalOverviewBE *overview)
 {
   grt::ListRef<GrtObject> selection(overview->get_selection());
@@ -775,7 +832,7 @@ bool PhysicalOverviewBE::get_file_data_for_node(const bec::NodeId &node, char *&
   {
     std::string text= _wb->get_attached_file_contents(note->filename());
     
-    data= (char*)g_memdup(text.data(), text.size());
+    data = (char*)g_memdup(text.data(), (int)text.size());
     length= text.size();
 
     return true;
