@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "wb_overview_physical_schema.h"
 
 #include "workbench/wb_context.h"
@@ -46,7 +44,7 @@ using namespace base;
 //-----------------------------------------------------------------------------
 
 
-SchemaObjectNode::SchemaObjectNode(const db_DatabaseObjectRef &dbobject) : OverviewBE::Node() 
+SchemaObjectNode::SchemaObjectNode(const db_DatabaseObjectRef &dbobject) : OverviewBE::ObjectNode() 
 {
   object= dbobject; 
   label= object->name();
@@ -264,7 +262,7 @@ public:
 
   virtual int count_detail_fields() 
   {
-    return _fields.size();
+    return (int)_fields.size();
   }
 
   virtual std::string get_detail_name(int field) 
@@ -452,7 +450,8 @@ void PhysicalSchemaNode::focus(OverviewBE *sender)
 {
   db_SchemaRef schema(db_SchemaRef::cast_from(object));
 
-  db_CatalogRef::cast_from(schema->owner())->defaultSchema(schema);
+  if (schema->owner().is_valid())
+    db_CatalogRef::cast_from(schema->owner())->defaultSchema(schema);
 }
 
 
