@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "db_sql_editor_log.h"
 #include "sqlide/recordset_data_storage.h"
 
@@ -27,11 +25,15 @@
 #include "base/wb_memory.h"
 #include "base/file_functions.h"
 #include "base/file_utilities.h"
+#include "base/log.h"
 
 #include "mforms/utilities.h"
 #include "wb_sql_editor_form.h"
+#include "wb_sql_editor_panel.h"
 #include <boost/foreach.hpp>
 
+DEFAULT_LOG_DOMAIN("SqlEditorLog");
+
 using namespace bec;
 using namespace grt;
 using namespace base;
@@ -46,10 +48,9 @@ DbSqlEditorLog::DbSqlEditorLog(SqlEditorForm *owner, GRTManager *grtm, int max_e
   : VarGridModel(grtm), _owner(owner), _max_entry_count(max_entry_count)
 {
   reset();
-
-  std::string log_dir = grtm->get_user_datadir() + "/log/";
+  std::string log_dir = base::join_path(grtm->get_user_datadir().c_str(), "log", "");
   create_directory(log_dir, 0700);
-  _log_file_name = log_dir + sanitize_file_name("sql_actions_" + owner->get_session_name() + ".log");
+  _log_file_name = base::join_path(log_dir.c_str(), sanitize_file_name("sql_actions_" + owner->get_session_name() + ".log").c_str(), "");
 
   _context_menu.add_item("Copy Row", "copy_row");
   _context_menu.add_item("Copy Action", "copy_action");
@@ -137,16 +138,16 @@ void DbSqlEditorLog::handle_context_menu(const std::string &action)
   else if (action == "append_selected_items")
   {
     sql = get_selection_text(false, true, false, false);
-    Sql_editor::Ref editor(_owner->active_sql_editor());
+    SqlEditorPanel *editor(_owner->active_sql_editor_panel());
     if (editor)
-      editor->append_text(sql);
+      editor->editor_be()->append_text(sql);
   }
   else if (action == "replace_sql_script")
   {
     sql = get_selection_text(false, true, false, false);
-    Sql_editor::Ref editor(_owner->active_sql_editor());
+    SqlEditorPanel *editor(_owner->active_sql_editor_panel());
     if (editor)
-      editor->sql(sql.c_str());
+      editor->editor_be()->sql(sql.c_str());
   }
   else if (action == "clear")
   {
@@ -236,7 +237,7 @@ public:
 
 //--------------------------------------------------------------------------------------------------
 
-IconId DbSqlEditorLog::get_field_icon(const NodeId &node, int column, IconSize size)
+IconId DbSqlEditorLog::get_field_icon(const NodeId &node, ColumnId column, IconSize size)
 {
   IconId icon= 0;
 
@@ -273,7 +274,7 @@ static std::string sanitize_text(const std::string &text)
 
 //--------------------------------------------------------------------------------------------------
 
-bool DbSqlEditorLog::get_field(const bec::NodeId &node, int column, std::string &value)
+bool DbSqlEditorLog::get_field(const bec::NodeId &node, ColumnId column, std::string &value)
 {
   if (VarGridModel::get_field(node, column, value))
   {
@@ -288,7 +289,7 @@ bool DbSqlEditorLog::get_field(const bec::NodeId &node, int column, std::string
 
 //--------------------------------------------------------------------------------------------------
 
-bool DbSqlEditorLog::get_field_description(const bec::NodeId &node, int column, std::string &value)
+bool DbSqlEditorLog::get_field_description(const bec::NodeId &node, ColumnId column, std::string &value)
 {
   return VarGridModel::get_field(node, column, value);
 }
@@ -301,10 +302,16 @@ RowId DbSqlEditorLog::add_message(int msg_type, const std::string &context, cons
     return -1;
 
   std::string time = current_time();
+  if (!_log_file_name.empty())
   {
     base::FILE_scope_ptr fp = base_fopen(_log_file_name.c_str(), "a");
     fprintf(fp, "[%u, %s] %s: %s\n",  _next_id, time.c_str(), context.c_str(), msg.c_str());
   }
+  else
+  {
+    log_error("DbSqlEditorLog::add_message called with no log file name set\n");
+    return -1;
+  }
 
   {
     base::RecMutexLock data_mutex(_data_mutex);
