@@ -28,6 +28,8 @@
 #include <boost/weak_ptr.hpp>
 #include "base/trackable.h"
 #include "base/notifications.h"
+#include "mforms/utilities.h"
+
 class SqlEditorPanel;
 class SqlEditorForm;
 
@@ -43,6 +45,7 @@ namespace wb
     std::list<boost::weak_ptr<SqlEditorForm> > _open_editors;
     app_ToolbarRef _toolbar;
 
+    mforms::TimeoutHandle _auto_save_handle;
     ssize_t _auto_save_interval;
     bool _auto_save_active;
     bool _option_change_signal_connected;
@@ -67,7 +70,6 @@ namespace wb
     bool activate_live_object(GrtObjectRef object);
 
     void open_document(const std::string &path);
-    void run_file(const std::string &path);
     
     bool request_quit();
 
@@ -83,6 +85,7 @@ namespace wb
   private:
     void call_in_editor(void (SqlEditorForm::*method)());    
     void call_in_editor_str(void (SqlEditorForm::*method)(const std::string &arg), const std::string &arg);
+    void call_in_editor_str2(void (SqlEditorForm::*method)(const std::string &arg1, bool arg2, bool arg3), const std::string &arg1, bool arg2, bool arg3);
     void call_in_editor_bool(void (SqlEditorForm::*method)(bool arg), bool arg);
 
     void call_in_editor_panel(void (SqlEditorPanel::*method)());
