@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -115,7 +115,8 @@ namespace wb
 
       ForeignKeyColumn,
       IndexColumn,
-      Any
+      Any,
+      None
     };
 
     // This will be used on different object type validations
@@ -158,7 +159,7 @@ namespace wb
       ObjectType _type;
     public:
       ColumnData(ObjectType type = TableColumn) : 
-          LSTData(), is_pk(false), is_fk(false), is_id(false) { _type = type; }
+          LSTData(), is_pk(false), is_fk(false), is_id(false), is_idx(false) { _type = type; }
       // NOTE than name will contain the column name duplicating this info as it
       //      is already on the TreeNode. Duplicating it is better than having
       //      full HTML for all the columns on all the tables in the db.
@@ -299,9 +300,13 @@ namespace wb
       virtual std::string get_object_name() { return _("Schema"); }
     };
 
-    typedef boost::function<void (const std::string& schema_name, std::list<std::string>* tables, std::list<std::string>* views, std::list<std::string>* procedures, std::list<std::string>* functions, bool just_append)> NewSchemaContentArrivedSlot;
-    typedef boost::function<void (const std::string& schema_name, const std::string& object_name, ObjectType obj_type, ObjectType child_type, const std::map<std::string, LSTData*> &children)> NewObjectDetailsArrivedSlot;
-    typedef boost::function<bool (mforms::TreeNodeRef, std::list<std::string>&, ObjectType, bool sorted, bool just_append)>NodeChildrenUpdaterSlot;
+    typedef boost::function<void(const std::string& schema_name, base::StringListPtr tables,
+      base::StringListPtr views, base::StringListPtr procedures, base::StringListPtr functions,
+      bool just_append)> NewSchemaContentArrivedSlot;
+    typedef boost::function<void (const std::string& schema_name, const std::string& object_name,
+      ObjectType obj_type, ObjectType child_type, const std::map<std::string, LSTData*> &children)> NewObjectDetailsArrivedSlot;
+    typedef boost::function<bool(mforms::TreeNodeRef, base::StringListPtr, ObjectType, bool sorted,
+      bool just_append)>NodeChildrenUpdaterSlot;
 
     struct FetchDelegate
     {
@@ -333,9 +338,12 @@ namespace wb
 
     bool _case_sensitive_identifiers;
 
-    void schema_contents_arrived(const std::string &schema_name, std::list<std::string> *tables, std::list<std::string> *views, std::list<std::string> *procedures, std::list<std::string> *functions, bool just_append);
+    void schema_contents_arrived(const std::string &schema_name, base::StringListPtr tables,
+      base::StringListPtr views, base::StringListPtr procedures, base::StringListPtr functions,
+      bool just_append);
     void load_table_details(mforms::TreeNodeRef& node, int fetch_mask);
-    void fetch_table_details(ObjectType object_type, const std::string schema_name, const std::string object_name, int fetch_mask);
+    void fetch_table_details(ObjectType object_type, const std::string schema_name,
+      const std::string object_name, int fetch_mask);
     void load_routine_details(mforms::TreeNodeRef& node);
     void load_schema_content(mforms::TreeNodeRef& schema_node);
     void reload_object_data(mforms::TreeNodeRef& node);
@@ -343,11 +351,14 @@ namespace wb
 
     bool identifiers_equal(const std::string &a, const std::string &b);
 
+    std::vector<std::string> overlay_icons_for_tree_node(mforms::TreeNodeRef node);
+
     // Filtering functions
     std::string get_filter_wildcard(const std::string& filter, FilterType type = LocalLike);
     void clean_filter();
     void filter_children_collection(mforms::TreeNodeRef& source, mforms::TreeNodeRef& target);
-    bool filter_children(ObjectType type, mforms::TreeNodeRef& source, mforms::TreeNodeRef& target, GPatternSpec* pattern = NULL);
+    bool filter_children(ObjectType type, mforms::TreeNodeRef& source, mforms::TreeNodeRef& target,
+      GPatternSpec* pattern = NULL);
     bool is_object_type(ObjectTypeValidation validation, ObjectType type);
   public:
     LiveSchemaTree(grt::GRT* grtm);
@@ -390,13 +401,16 @@ namespace wb
 
     void set_base(LiveSchemaTree *base) { _base = base; }
 
-    bool update_node_children(mforms::TreeNodeRef parent, std::list<std::string>& children, ObjectType type, bool sorted = false, bool just_append = false);
-    void update_change_data(mforms::TreeNodeRef parent, std::list<std::string>& children, ObjectType type, std::vector<mforms::TreeNodeRef>& to_remove);
+    bool update_node_children(mforms::TreeNodeRef parent, base::StringListPtr children,
+      ObjectType type, bool sorted = false, bool just_append = false);
+    void update_change_data(mforms::TreeNodeRef parent, base::StringListPtr children,
+      ObjectType type, std::vector<mforms::TreeNodeRef>& to_remove);
     mforms::TreeNodeRef insert_node(mforms::TreeNodeRef parent, const std::string& name, ObjectType type);
-    void setup_node(mforms::TreeNodeRef node, ObjectType type, mforms::TreeNodeData* data = NULL, bool ignore_null_data = false);
+    void setup_node(mforms::TreeNodeRef node, ObjectType type, mforms::TreeNodeData* data = NULL,
+      bool ignore_null_data = false);
     void update_node_icon(mforms::TreeNodeRef node);
 
-    void update_schemata(std::list<std::string> &schema_list);
+    void update_schemata(base::StringListPtr schema_list);
     mforms::TreeNodeRef binary_search_node(const mforms::TreeNodeRef& parent, int first, int last, const std::string &name, ObjectType type, int &position);
     mforms::TreeNodeRef get_child_node(const mforms::TreeNodeRef& parent, const std::string& name, ObjectType type = Any, bool binary_search = true);
     bool find_child_position(const mforms::TreeNodeRef& parent, const std::string& name, ObjectType type, int &position);
@@ -420,7 +434,7 @@ namespace wb
     GPatternSpec* _object_pattern;
     LSTData *notify_on_reload_data;
 
-    static const char* _schema_tokens[15];
+    static const char* _schema_tokens[16];
     
     std::map<ObjectType, std::string> _icon_paths;
     
