@@ -56,7 +56,7 @@ void SqlEditorForm::auto_save()
   {
     try
     {
-      save_workspace(sanitize_file_name(_connection->name()), true);
+      save_workspace(sanitize_file_name(_connection.is_valid() ? _connection->name() : "unconnected"), true);
     }
     catch (std::exception &exc)
     {
@@ -186,7 +186,7 @@ std::string SqlEditorForm::find_workspace_state(const std::string &workspace_nam
         const char *end = strrchr(name, '.');
         int new_index = 0;
         if (end)
-          new_index = atoi(std::string(name + workspace_name.size()+1, end-name-(workspace_name.size()+1)).c_str());
+          new_index = base::atoi<int>(std::string(name + workspace_name.size()+1, end-name-(workspace_name.size()+1)), 0);
 
         if (g_str_has_suffix(name, ".autosave"))
         {
@@ -312,12 +312,12 @@ bool SqlEditorForm::load_workspace(const std::string &workspace_name)
       try { base::remove(info_file); }
       catch (std::exception &e)
       {
-        log_error("Could not delete autosave file %s\n", info_file.c_str());
+        log_error("Could not delete autosave file %s\n%s\n", info_file.c_str(), e.what());
       }
       try { base::remove(text_file); }
       catch (std::exception &e)
       {
-        log_error("Could not delete autosave file %s\n", text_file.c_str());
+        log_error("Could not delete autosave file %s\n%s\n", text_file.c_str(), e.what());
       }
     }
     // remove the pre-created editor
@@ -380,7 +380,7 @@ bool SqlEditorForm::load_workspace(const std::string &workspace_name)
       try { base::remove(file.first); }
       catch (std::exception &e)
       {
-        log_error("Could not delete autosave file %s\n", file.first.c_str());
+        log_error("Could not delete autosave file %s\n%s\n", file.first.c_str(), e.what());
       }
     }
     // remove the pre-created editor
@@ -551,13 +551,38 @@ void SqlEditorForm::sql_editor_panel_switched()
 
 void SqlEditorForm::sql_editor_panel_closed(mforms::AppView *view)
 {
-  if (_tabdock->view_count() == 0 && !_closing)
-    new_sql_scratch_area();
+  if (!_closing)
+  {
+    if (_tabdock->view_count() == 0)
+        new_sql_scratch_area();
+    else if(dynamic_cast<SqlEditorPanel*>(view))
+    {
+      // We check if the tab was closed is SqlEditorPanel if so,
+      // we need to find if there's at least one query tab left if not,
+      // we create new one
+      bool panel_found = false;
+      for (int i = 0; i < _tabdock->view_count(); ++i)
+      {
+        if (sql_editor_panel(i))
+        {
+          panel_found = true;
+          break;
+        }
+      }
+      if (!panel_found)
+        new_sql_scratch_area();
+
+    }
+  }
 }
 
 void SqlEditorForm::save_workspace_order(const std::string &prefix)
 {
   std::ofstream order_file;
+  if (prefix.empty())
+  {
+    log_error("save with empty path\n");
+  }
 
   order_file.open(bec::make_path(prefix, "tab_order").c_str(), std::ofstream::out);
 
@@ -625,8 +650,11 @@ void SqlEditorForm::sql_editor_reordered(SqlEditorPanel *panel, int to)
 
   grtobj()->queryEditors()->reorder(from_index, to_index);
 
-  /// Rename autosave files to keep the order
-  save_workspace_order(_autosave_path);
+  if (!_autosave_path.empty())
+  {
+    /// Rename autosave files to keep the order
+    save_workspace_order(_autosave_path);
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -675,24 +703,27 @@ SqlEditorPanel *SqlEditorForm::new_sql_scratch_area(bool start_collapsed)
 
 //--------------------------------------------------------------------------------------------------
 
-void SqlEditorForm::open_file(const std::string &path, bool in_new_tab)
+void SqlEditorForm::open_file(const std::string &path, bool in_new_tab, bool askForFile)
 {
   std::string file_path = path;
 
   _grtm->replace_status_text(base::strfmt(_("Opening %s..."), path.c_str()));
 
-  if (file_path.empty())
+  if (askForFile)
   {
-    mforms::FileChooser opendlg(mforms::OpenFile);
-    opendlg.set_title(_("Open SQL Script"));
-    opendlg.set_extensions("SQL Files (*.sql)|*.sql|Query Browser Files (*.qbquery)|*.qbquery", "sql");
-    if (opendlg.run_modal())
-      file_path = opendlg.get_path();
-  }
-  if (file_path.empty())
-  {
-    _grtm->replace_status_text(_("Cancelled open file"));
-    return;
+    if (file_path.empty())
+    {
+      mforms::FileChooser opendlg(mforms::OpenFile);
+      opendlg.set_title(_("Open SQL Script"));
+      opendlg.set_extensions("SQL Files (*.sql)|*.sql|Query Browser Files (*.qbquery)|*.qbquery", "sql");
+      if (opendlg.run_modal())
+        file_path = opendlg.get_path();
+    }
+    if (file_path.empty())
+    {
+      _grtm->replace_status_text(_("Cancelled open file"));
+      return;
+    }
   }
 
   SqlEditorPanel *panel = NULL;
@@ -717,7 +748,16 @@ void SqlEditorForm::open_file(const std::string &path, bool in_new_tab)
 
   try
   {
-    panel->load_from(file_path);
+    if (askForFile && panel->load_from(file_path) == SqlEditorPanel::RunInstead)
+    {
+      if (in_new_tab)
+        remove_sql_editor(panel);
+      grt::BaseListRef args(_grtm->get_grt());
+      args.ginsert(grtobj());
+      args.ginsert(grt::StringRef(file_path));
+      _grtm->get_grt()->call_module_function("SQLIDEUtils", "runSQLScriptFile", args);
+      return;
+    }
   }
   catch (std::exception &exc)
   {
