@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,6 +33,7 @@
 #include "sqlide/db_sql_editor_log.h"
 #include "sqlide/db_sql_editor_history_be.h"
 #include "sqlide/wb_context_sqlide.h"
+#include "sqlide/wb_live_schema_tree.h"
 
 #include "cppdbc.h"
 
@@ -114,6 +115,7 @@ public:
   typedef boost::weak_ptr<SqlEditorForm> Ptr;
   static SqlEditorForm::Ref create(wb::WBContextSQLIDE *wbsql, const db_mgmt_ConnectionRef &conn);
   static void report_connection_failure(const std::string &error, const db_mgmt_ConnectionRef &target);
+  static void report_connection_failure(const grt::server_denied &info, const db_mgmt_ConnectionRef &target);
 
   void set_tab_dock(mforms::DockingPoint *dp);
 
@@ -121,7 +123,7 @@ public:
   boost::function<void (int)> set_busy_tab;
 
 protected:
-  SqlEditorForm(wb::WBContextSQLIDE *wbsql, const db_mgmt_ConnectionRef &conn);
+  SqlEditorForm(wb::WBContextSQLIDE *wbsql);
 
   void update_menu_and_toolbar();
   void update_toolbar_icons();
@@ -168,6 +170,10 @@ private:
 
   mforms::DockingPoint *_tabdock;
 
+  // Set when we triggered a refresh asynchronously.
+  boost::signals2::connection _overviewRefreshPending;
+  boost::signals2::connection _editorRefreshPending;
+
   bool _autosave_disabled;
   bool _loading_workspace;
   bool _cancel_connect;
@@ -254,6 +260,8 @@ public:
   bool collect_field_info() const;
   void toggle_collect_ps_statement_events();
   bool collect_ps_statement_events() const;
+
+  void set_connection(db_mgmt_ConnectionRef conn);
   
   void run_editor_contents(bool current_statement_only);
 
@@ -285,9 +293,9 @@ private:
   void create_connection(sql::Dbc_connection_handler::Ref &dbc_conn, db_mgmt_ConnectionRef db_mgmt_conn, boost::shared_ptr<sql::TunnelConnection> tunnel, sql::Authentication::Ref auth, bool autocommit_mode, bool user_connection);
   void init_connection(sql::Connection* dbc_conn_ref, const db_mgmt_ConnectionRef& connectionProperties, sql::Dbc_connection_handler::Ref& dbc_conn, bool user_connection);
   void close_connection(sql::Dbc_connection_handler::Ref &dbc_conn);
-  base::RecMutexLock ensure_valid_dbc_connection(sql::Dbc_connection_handler::Ref &dbc_conn, base::RecMutex &dbc_conn_mutex);
-  base::RecMutexLock ensure_valid_usr_connection();
-  base::RecMutexLock ensure_valid_aux_connection();
+  base::RecMutexLock ensure_valid_dbc_connection(sql::Dbc_connection_handler::Ref &dbc_conn, base::RecMutex &dbc_conn_mutex, bool throw_on_block = false);
+  base::RecMutexLock ensure_valid_usr_connection(bool throw_on_block = false);
+  base::RecMutexLock ensure_valid_aux_connection(bool throw_on_block = false);
 
 public:
   base::RecMutexLock ensure_valid_aux_connection(sql::Dbc_connection_handler::Ref &conn);
@@ -379,15 +387,16 @@ private:
   int sql_script_apply_error(long long, const std::string&, const std::string&, std::string&);
   int sql_script_apply_progress(float);
   int sql_script_stats(long, long);
-  
+
+  void abort_apply_object_alter_script();
 public:
-  void apply_object_alter_script(std::string &alter_script, bec::DBObjectEditorBE* obj_editor, RowId log_id);
+  void apply_object_alter_script(const std::string &alter_script, bec::DBObjectEditorBE* obj_editor, RowId log_id);
   bool run_live_object_alteration_wizard(const std::string &alter_script, bec::DBObjectEditorBE* obj_editor, RowId log_id, const std::string &log_context);
 
 private:
   void apply_changes_to_recordset(Recordset::Ptr rs_ptr);
   bool run_data_changes_commit_wizard(Recordset::Ptr rs_ptr, bool skip_commit);
-  void apply_data_changes_commit(std::string &sql_script_text, Recordset::Ptr rs_ptr, bool skip_commit);
+  void apply_data_changes_commit(const std::string &sql_script_text, Recordset::Ptr rs_ptr, bool skip_commit);
   void update_editor_title_schema(const std::string& schema);
 
 public:  
@@ -399,16 +408,14 @@ public:
   SqlEditorPanel *new_sql_script_file();
   SqlEditorPanel *new_sql_scratch_area(bool start_collapsed = false);
   void new_scratch_area() { new_sql_scratch_area(false); }
-  void open_file(const std::string &path, bool in_new_tab);
-  void open_file(const std::string &path = "") { open_file(path, true); }
+  void open_file(const std::string &path, bool in_new_tab, bool askForFile = true);
+  void open_file(const std::string &path = "") { open_file(path, true, !path.empty()); }
 
 public:
   void active_schema(const std::string &value);
   std::string active_schema() const;
-  void schema_meta_data_refreshed(const std::string &schema_name,
-                                  const std::vector<std::pair<std::string,bool> >& tables, 
-                                  const std::vector<std::pair<std::string,bool> >& procedures, 
-                                  bool just_append);
+  void schema_meta_data_refreshed(const std::string &schema_name, base::StringListPtr tables,
+    base::StringListPtr views, base::StringListPtr procedures, base::StringListPtr functions);
 private:
   void cache_active_schema_name();
 
