@@ -165,7 +165,9 @@ void SqlEditorResult::copy_all_column_names()
   if (rset)
   {
     std::string text;
-    for (Recordset::Column_names::const_iterator col = rset->column_names()->begin(); col != rset->column_names()->end(); ++col)
+    size_t visibleColumnCount = rset->get_column_count();
+    Recordset::Column_names::const_iterator end = rset->column_names()->end();
+    for (Recordset::Column_names::const_iterator col = rset->column_names()->begin(); col != end && visibleColumnCount > 0; ++col, --visibleColumnCount)
       text.append(", ").append(*col);
     if (!text.empty())
       text = text.substr(2);
@@ -753,7 +755,7 @@ void SqlEditorResult::create_spatial_view_panel_if_needed()
       if (!spatial::Projection::get_instance().check_libproj_availability())
       {
         mforms::Utilities::show_message_and_remember("Unable to initialize Spatial Viewer",
-                                                     "Spatial support requires the PROJ.4 library (libproj). If you already have it installed, please set the PROJSO environment variable to its location before starting Worbench.",
+                                                     "Spatial support requires the PROJ.4 library (libproj). If you already have it installed, please set the PROJSO environment variable to its location before starting Workbench.",
                                                      "Ok", "", "",
                                                      "SqlEditorResult.libprojcheck", "");
         return;
