@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,7 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "tut_stdafx.h"
 #include <grtpp.h>
 #include "grt_test_utility.h"
 #include "wb_helpers.h"
@@ -54,7 +53,7 @@ TEST_FUNCTION(1)
 
 TEST_FUNCTION(2)
 {
-  std::vector<int> columns;
+  std::vector<ssize_t> columns;
 
   columns.push_back(wb::OverviewBE::Label);
   columns.push_back(wb::OverviewBE::NodeType);
