@@ -1052,6 +1052,7 @@ void WBContextUI::refresh_home_connections(bool clear_state)
   std::vector<db_mgmt_ConnectionRef> invalid_connections;
   std::map<std::string, std::string> invalid_connection_ids;
 
+  std::vector<db_mgmt_ConnectionRef> oldAuthList;
   for (grt::ListRef<db_mgmt_Connection>::const_iterator end = connections.end(),
       inst = connections.begin(); inst != end; ++inst)
   {
@@ -1064,6 +1065,13 @@ void WBContextUI::refresh_home_connections(bool clear_state)
     else
     {
       grt::DictRef dict((*inst)->parameterValues());
+      if (dict.has_key("useLegacyAuth"))
+      {
+        if ((size_t)dict.get_int("useLegacyAuth", 0) == 0)
+          (*inst)->parameterValues().remove("useLegacyAuth"); // If it's not used (old val), we silently remove it.
+        else
+          oldAuthList.push_back(*inst);
+      }
 
       // Any fabric managed connection referencing a 
       if (dict.has_key("fabric_managed") && invalid_connection_ids.count(dict.get_string("fabric_managed")))
@@ -1094,6 +1102,7 @@ void WBContextUI::refresh_home_connections(bool clear_state)
     }
   }
 
+  _home_screen->oldAuthConnections(oldAuthList);
 
   // Deletes the fabric managed connections
   for (std::vector<db_mgmt_ConnectionRef>::const_iterator iterator = invalid_connections.begin();
