@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,6 +18,7 @@
  */
 
 #include <algorithm>
+#include <fcntl.h>
 
 #include <zip.h>
 #include "wb_model_file.h"
@@ -36,6 +37,7 @@
 #include "base/util_functions.h"
 
 #include "mforms/utilities.h"
+#include "mdc_image.h"
 
 #include "grt/common.h"
 #include "grt/grt_manager.h"
@@ -505,8 +507,16 @@ std::list<std::string> ModelFile::unpack_zip(const std::string &zipfile, const s
     throw grt::os_error(strfmt(_("Cannot create temporary directory for open document: %s"), destdir.c_str()), errno);
 
   int err;
-  zip *z= zip_open(zipfile.c_str(), 0, &err);
-  if (!z)
+#ifdef ZIP_DISABLE_DEPRECATED
+  // Would be good if we could test for zip_fdopen, but there's no way in the preprocessor.
+  // And there's no version macro either for libzip.
+  // Define ZIP_DISABLE_DEPRECATED to use the newer APIs.
+  int fd = base_open(zipfile, O_RDONLY, S_IREAD); // Error checking is done already before.
+  zip *z = zip_fdopen(fd, 0, &err);
+#else
+  zip *z = zip_open(zipfile.c_str(), 0, &err); // Older versions of libzip.
+#endif
+  if (z == NULL)
   {
     if (err == ZIP_ER_NOZIP)
       throw std::runtime_error("The file is not a Workbench document.");
@@ -515,7 +525,11 @@ std::list<std::string> ModelFile::unpack_zip(const std::string &zipfile, const s
     else if (err == ZIP_ER_NOENT)
       throw grt::os_error("File not found.");
 
-    int len= zip_error_to_str(NULL, 0, 0, err);
+  #ifdef ZIP_DISABLE_DEPRECATED
+    // the new API doesn't offer a way to extract an error without a pertaining zip*, which is NULL in this case
+    std::string msg = "error opening zip archive";
+  #else
+	int len= zip_error_to_str(NULL, 0, 0, err);
     std::string msg;
     if (len > 0)
     {
@@ -526,12 +540,13 @@ std::list<std::string> ModelFile::unpack_zip(const std::string &zipfile, const s
     }
     else
       msg= "error opening zip archive";
+  #endif
 
     zip_close(z);
     throw std::runtime_error(strfmt(_("Cannot open document file: %s"), msg.c_str()));
   }
 
-#ifdef zip_int64_t
+#ifdef ZIP_DISABLE_DEPRECATED
   zip_int64_t count = zip_get_num_entries(z, 0);
 #else
   int count = zip_get_num_files(z);
@@ -665,7 +680,7 @@ static void zip_dir_contents(zip *z, const std::string &destdir, const std::stri
         if (!add_directories)
         {
           zip_source *src= zip_source_file(z, tmp.c_str(), 0, 0);
-#ifdef zip_file_add
+#ifdef _WIN32
           if (!src || zip_file_add(z, tmp.c_str(), src, ZIP_FL_OVERWRITE | ZIP_FL_ENC_UTF_8) < 0)
           {
             zip_source_free(src);
@@ -691,7 +706,6 @@ static void zip_dir_contents(zip *z, const std::string &destdir, const std::stri
 
 void ModelFile::pack_zip(const std::string &zipfile, const std::string &destdir, const std::string &comment)
 {
-  int err= 0;
   std::string curdir;
 
   {
@@ -708,7 +722,18 @@ void ModelFile::pack_zip(const std::string &zipfile, const std::string &destdir,
 
   // zip_open will open an existing file even if ZIP_CREATE is specified, so
   // we have to 1st delete the file...
-  zip *z= zip_open(zipfile.c_str(), ZIP_CREATE, &err);
+  int err = 0;
+  /* XXX: doesn't work yet as zip_close creates a temporary file without considering the name encoding
+   *      and opening with zip_fdopen doesn't set the file name member of the zip struct
+   *      which then crashes when an attempt is made to derived a temp name from that.
+#ifdef ZIP_DISABLE_DEPRECATED
+  int fd = base_open(zipfile, O_CREAT, S_IWRITE);
+  zip *z = zip_fdopen(fd, 0, &err);
+#else
+  zip *z = zip_open(zipfile.c_str(), 0, &err); // Older versions of libzip.
+#endif
+#*/
+  zip *z = zip_open(zipfile.c_str(), ZIP_CREATE, &err);
   if (!z)
   {
     if (err == ZIP_ER_MEMORY)
@@ -1078,24 +1103,14 @@ bool ModelFile::undelete_file(const std::string &path)
   return true;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 cairo_surface_t *ModelFile::get_image(const std::string &path)
 {
-  cairo_surface_t *image;
-  cairo_status_t st;
-
-  image= cairo_image_surface_create_from_png(get_path_for(path).c_str());
-  if (!image || (st= cairo_surface_status(image)) != CAIRO_STATUS_SUCCESS)
-  {
-    if (image)
-      cairo_surface_destroy(image);
-    return 0;
-  }
-
-  return image;
+  return mdc::surface_from_png_image(get_path_for(path));
 }
 
-
+//--------------------------------------------------------------------------------------------------
 
 void ModelFile::check_and_fix_data_file_bug()
 {
