@@ -28,7 +28,9 @@
 #include "grts/structs.workbench.h"
 #include "base/trackable.h"
 
+#ifndef _WIN32
 #include <cairo.h>
+#endif
 
 #define MAIN_DOCUMENT_NAME "document.mwb.xml"
 #define MAIN_DOCUMENT_AUTOSAVE_NAME "document-autosave.mwb.xml"
@@ -103,7 +105,7 @@ namespace wb {
 
   private:
     base::LockFile *_temp_dir_lock;
-    GStaticRecMutex _mutex;
+    base::RecMutex _mutex;
     std::string _temp_dir; //< temporary files directory
     std::string _content_dir; //< path for directory where document contents are stored in disk
     std::list<std::string> _delete_queue; //< files marked for deletion
