@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -44,10 +44,10 @@ public:
 
   void set_context(WBContext *wb);
   std::string getSystemInfo(bool indent);
+  std::map<std::string, std::string> getSystemInfoMap();
 
   DEFINE_INIT_MODULE(WBModule_VERSION, "Oracle Corporation", grt::ModuleImplBase,
     DECLARE_MODULE_FUNCTION(WorkbenchImpl::getPluginInfo),
-    DECLARE_MODULE_FUNCTION(WorkbenchImpl::refreshHomeConnections),
 
 
     // Non-plugin functions
@@ -146,6 +146,8 @@ public:
     DECLARE_MODULE_FUNCTION(WorkbenchImpl::showQueryConnectDialog),
     DECLARE_MODULE_FUNCTION(WorkbenchImpl::saveConnections),
     DECLARE_MODULE_FUNCTION(WorkbenchImpl::saveInstances),
+    DECLARE_MODULE_FUNCTION(WorkbenchImpl::refreshHomeConnections),
+
     DECLARE_MODULE_FUNCTION(WorkbenchImpl::showGRTShell),
     DECLARE_MODULE_FUNCTION(WorkbenchImpl::newGRTFile),
     DECLARE_MODULE_FUNCTION(WorkbenchImpl::openGRTFile),
@@ -160,7 +162,10 @@ public:
 
     DECLARE_MODULE_FUNCTION(WorkbenchImpl::createConnectionsFromLocalServers),
     DECLARE_MODULE_FUNCTION(WorkbenchImpl::createInstancesFromLocalServers),
-    DECLARE_MODULE_FUNCTION(WorkbenchImpl::initializeOtherRDBMS)
+    DECLARE_MODULE_FUNCTION(WorkbenchImpl::create_connection),
+    DECLARE_MODULE_FUNCTION(WorkbenchImpl::initializeOtherRDBMS),
+    DECLARE_MODULE_FUNCTION(WorkbenchImpl::deleteConnection),
+    DECLARE_MODULE_FUNCTION(WorkbenchImpl::deleteConnectionGroup)
     );
 
 protected:
@@ -327,7 +332,7 @@ private:
 #endif
 
   db_mgmt_ConnectionRef create_connection(const std::string& host, const std::string& user,
-    const std::string socket_or_pipe_name, bool can_use_networking, bool can_use_socket_or_pipe,
+    const std::string socket_or_pipe_name, int can_use_networking, int can_use_socket_or_pipe,
     int port, const std::string& name);
   grt::DictListRef getLocalServerList();
   int createConnectionsFromLocalServers();
@@ -336,6 +341,8 @@ private:
   std::string getVideoAdapter();
   std::string getFullVideoAdapterInfo(bool indent);
   int initializeOtherRDBMS();
+  int deleteConnection(const db_mgmt_ConnectionRef &connection);
+  int deleteConnectionGroup(const std::string& group);
 };
 
 };
