@@ -1,10 +1,9 @@
 
 #define APP_MAJOR_NUMBER 6
 #define APP_MINOR_NUMBER 3
-#define APP_RELEASE_NUMBER 4
-#define APP_REVISION_NUMBER 0
-#define APP_BUILD_NUMBER 828
-#define APP_RELEASE_TYPE ""
+#define APP_RELEASE_NUMBER 6
+#define APP_BUILD_NUMBER 511
+#define APP_RELEASE_TYPE "CE"
 
 #define APP_LICENSE_TYPE "GPL"
 #define APP_EDITION_NAME "Community"
