@@ -1162,7 +1162,7 @@ void CatalogHelper::apply_defaults(db_mysql_CatalogRef cat, std::string default_
     if(strlen(schema->defaultCharacterSetName().c_str()) == 0)
       schema->defaultCharacterSetName(cat->defaultCharacterSetName());
     if(strlen(schema->defaultCollationName().c_str()) == 0)
-      schema->defaultCollationName(get_cs_def_collation(schema->defaultCharacterSetName().c_str()));
+      schema->defaultCollationName(defaultCollationForCharset(schema->defaultCharacterSetName().c_str()));
 
     for(size_t j= 0, tables_count= schema->tables().count(); j < tables_count; j++)
     {
@@ -1179,7 +1179,7 @@ void CatalogHelper::apply_defaults(db_mysql_CatalogRef cat, std::string default_
       else
       {
         if(strlen(table->defaultCollationName().c_str()) == 0)
-          table->defaultCollationName(get_cs_def_collation(table->defaultCharacterSetName()));
+          table->defaultCollationName(defaultCollationForCharset(table->defaultCharacterSetName()));
       }
       
       if(strlen(table->tableEngine().c_str()) == 0)
@@ -1346,13 +1346,13 @@ static bool parse_type(const std::string &type,
   if (simpleType->characterMaximumLength() != bec::EMPTY_TYPE_MAXIMUM_LENGTH
     || simpleType->characterOctetLength() != bec::EMPTY_TYPE_OCTET_LENGTH)
   {
-    if (walker.token_type() != INTEGER)
+    if (walker.token_type() != INT_NUMBER)
       return false;
     length = base::atoi<int>(walker.token_text().c_str());
     return true;
   }
 
-  if (!walker.is(INTEGER))
+  if (!walker.is(INT_NUMBER))
   {
     // ENUM or SET.
     do 
@@ -1511,5 +1511,5 @@ std::string bec::TableHelper::generate_comment_text(const std::string& comment_t
   else if (!comment_text.empty())
     return "'"+base::escape_sql_string(comment_text)+"'";
 
-  return "''";
+  return "";
 }
