@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,7 @@
  * 02110-1301  USA
  */
 
-#ifndef _EDITOR_ROUTINE_H_
-#define _EDITOR_ROUTINE_H_
+#pragma once
 
 #include "grtdb/editor_dbobject.h"
 
@@ -26,33 +25,12 @@ namespace bec {
 
 class WBPUBLICBACKEND_PUBLIC_FUNC RoutineEditorBE : public DBObjectEditorBE
 {
-protected:
-  db_RoutineRef _routine;
-  bool _has_syntax_error;
-
 public:
-  RoutineEditorBE(GRTManager *grtm, const db_RoutineRef &routine, const db_mgmt_RdbmsRef &rdbms); 
+  RoutineEditorBE(GRTManager *grtm, const db_RoutineRef &routine); 
 
   virtual std::string get_title();
-
-  virtual db_DatabaseObjectRef get_dbobject() { return get_routine(); }
-  virtual db_RoutineRef get_routine() { return _routine; }
-
+  db_RoutineRef get_routine() { return db_RoutineRef::cast_from(get_dbobject()); }
   virtual std::string get_sql();
-  virtual void set_sql(const std::string &sql, bool sync);
-  
-  std::string get_sql_definition_header();
-  virtual std::string get_sql_template(const std::string &template_name, int &cursor_pos);
-  
-  std::string get_formatted_sql_for_editing(int &cursor_pos);
-  
-  grt::ValueRef parse_sql(grt::GRT*, grt::StringRef sql);
-  bool has_syntax_error() { return _has_syntax_error; }
-
-protected:
-  virtual std::string get_object_type();
 };
 
 } // namespace bec 
-
-#endif /* _EDITOR_ROUTINE_H_ */
