@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,32 +16,27 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-#ifndef _EDITOR_SCHEMA_H_
-#define _EDITOR_SCHEMA_H_
+
+#pragma once
 
 #include "grtdb/editor_dbobject.h"
-#include "wbpublic_public_interface.h"
 
 namespace bec {
 
   class WBPUBLICBACKEND_PUBLIC_FUNC SchemaEditorBE : public DBObjectEditorBE
   {
-  protected:
-    db_SchemaRef _schema;
   public:
-    virtual db_DatabaseObjectRef get_dbobject() { return get_schema(); }
-    virtual db_SchemaRef& get_schema() { return _schema; }
+    SchemaEditorBE(GRTManager *grtm, const db_SchemaRef &schema);
+
+    virtual std::string get_title();
+
+    virtual db_SchemaRef get_schema() = 0;
 
     virtual void set_name(const std::string &name);
 
     // table options
     virtual void set_schema_option_by_name(const std::string& name, const std::string& value);
     virtual std::string get_schema_option_by_name(const std::string& name);
-
-    SchemaEditorBE(GRTManager *grtm, const db_SchemaRef &schema, const db_mgmt_RdbmsRef &rdbms);
   };
 
 };
-
-
-#endif /* _EDITOR_SCHEMA_H_ */
