@@ -1,4 +1,4 @@
-/* 
+/*
  * Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -355,6 +355,67 @@ private:
 
 //--------------------------------------------------------------------------------
 
+class JsonDataViewer : public BinaryDataViewer
+{
+public:
+  JsonDataViewer(BinaryDataEditor *owner, JsonParser::JsonValue &value, const std::string &encoding)
+    : BinaryDataViewer(owner), _encoding(encoding)
+  {
+    set_spacing(8);
+    _jsonView.setJson(value);
+    add(&_jsonView, true, true);
+    scoped_connect(_jsonView.editorDataChanged(), boost::bind(&JsonDataViewer::edited, this, _1));
+  }
+
+  virtual void data_changed()
+  {
+    if (!_owner->data())
+    {
+      _jsonView.clear();
+      return;
+    }
+    GError *error = NULL;
+    gsize bread = 0, bwritten = 0;
+    char *converted = g_convert(_owner->data(), static_cast<gssize>(_owner->length()), "UTF-8", _encoding.c_str(), &bread, &bwritten, &error);
+    if (!converted || _owner->length() != bread)
+    {
+      _jsonView.clear();
+      return;
+    }
+    std::string dataToTest = converted;
+    size_t pos = dataToTest.find_first_not_of(SPACES);
+    if (pos != std::string::npos && dataToTest.at(pos) != '{' &&  dataToTest.at(pos) != '[')
+    {
+      _jsonView.clear();
+      return;
+    }
+
+    JsonParser::JsonValue value;
+    try
+    {
+      JsonParser::JsonReader::read(converted, value);
+      if (_jsonView.text() != converted)
+        _jsonView.setJson(value);
+    }
+    catch (JsonParser::ParserException &)
+    {
+      _jsonView.setText(converted);
+    }
+  }
+
+private:
+
+  void edited(const std::string &text)
+  {
+     _owner->assign_data(text.data(), text.length());
+  }
+
+  mforms::JsonTabView _jsonView;
+  std::string _encoding;
+};
+
+//--------------------------------------------------------------------------------
+
 class GeomDataViewer : public BinaryDataViewer
 {
 public:
@@ -445,9 +506,19 @@ BinaryDataEditor::BinaryDataEditor(bec::GRTManager *grtm, const char *data, size
   add_viewer(new TextDataViewer(this, "LATIN1", read_only), "Text");
   if (ImageDataViewer::can_display(data, length))
     add_viewer(new ImageDataViewer(this, read_only), "Image");
-    
+
+  int activeTab = 0;
   if (tab.is_valid())
-    _tab_view.set_active_tab((int)*tab);
+    activeTab = (int)*tab;
+  if (tab.is_valid() && *tab >= _tab_view.page_count())
+  {
+    grt::DictRef dict(grt::DictRef::cast_from(_grtm->get_app_option("")));
+    if (dict.is_valid())
+      dict.gset("BlobViewer:DefaultTab", 0);
+    activeTab = 0;
+  }
+  _tab_view.set_active_tab(activeTab);
+
   tab_changed();
 }
 
@@ -457,6 +528,7 @@ BinaryDataEditor::BinaryDataEditor(bec::GRTManager *grtm, const char *data, size
   set_name("blob_editor");
   _data = 0;
   _length = 0;
+  _updating = false;
 
   grt::IntegerRef tab = grt::IntegerRef::cast_from(_grtm->get_app_option("BlobViewer:DefaultTab"));
 
@@ -473,9 +545,20 @@ BinaryDataEditor::BinaryDataEditor(bec::GRTManager *grtm, const char *data, size
     add_viewer(new ImageDataViewer(this, read_only), "Image");
 
   assign_data(data, length);
+  add_json_viewer(read_only, text_encoding, "JSON");
 
+  int activeTab = 0;
   if (tab.is_valid())
-    _tab_view.set_active_tab((int)*tab);  
+    activeTab = (int)*tab;
+  if (tab.is_valid() && *tab >= _tab_view.page_count())
+  {
+    grt::DictRef dict(grt::DictRef::cast_from(_grtm->get_app_option("")));
+    if (dict.is_valid())
+      dict.gset("BlobViewer:DefaultTab", 0);
+    activeTab = 0;
+  }
+
+  _tab_view.set_active_tab(activeTab);  
   tab_changed();
 }
 
@@ -517,7 +600,7 @@ void BinaryDataEditor::setup()
   scoped_connect(_import.signal_clicked(),boost::bind(&BinaryDataEditor::import_value, this));
   scoped_connect(_export.signal_clicked(),boost::bind(&BinaryDataEditor::export_value, this));
 
-  set_size(640, 500);
+  set_size(800, 500); // Golden ratio.
   center();
 }
 
@@ -528,6 +611,9 @@ void BinaryDataEditor::notify_edit()
 
 void BinaryDataEditor::assign_data(const char *data, size_t length, bool steal_pointer)
 {
+  if (_updating)
+    return;
+
   if (data != _data)
   {
     g_free(_data);
@@ -537,7 +623,7 @@ void BinaryDataEditor::assign_data(const char *data, size_t length, bool steal_p
       _data = (char*)g_memdup(data, (guint)length);
 
     for (size_t i = 0; i < _viewers.size(); i++)
-      _viewers[i].second = true;
+      _pendingUpdates.insert(_viewers[i]);
   }
   _length = length;
 
@@ -548,17 +634,25 @@ void BinaryDataEditor::tab_changed()
 {
   int i = _tab_view.get_active_tab();
   if (i < 0)
-    i= 0;
+    i = 0;
 
   grt::DictRef dict(grt::DictRef::cast_from(_grtm->get_app_option("")));
   if (dict.is_valid())
     dict.gset("BlobViewer:DefaultTab", i);
-
+  if (i >= _tab_view.page_count())
+  {
+    grt::DictRef dict(grt::DictRef::cast_from(_grtm->get_app_option("")));
+    if (dict.is_valid())
+      dict.gset("BlobViewer:DefaultTab", 0);
+    i = 0;
+  }
   try
   {
-    if (_viewers[i].second && _data)
-      _viewers[i].first->data_changed();
-    _viewers[i].second = false;
+    _updating = true;
+    if (_pendingUpdates.count(_viewers[i]) > 0 && _data != NULL)
+      _viewers[i]->data_changed();
+    _pendingUpdates.erase(_viewers[i]);
+    _updating = false;
   }
   catch (std::exception &exc)
   {
@@ -568,14 +662,50 @@ void BinaryDataEditor::tab_changed()
 
 void BinaryDataEditor::add_viewer(BinaryDataViewer *viewer, const std::string &title)
 {
-  _viewers.push_back(std::make_pair(viewer, true));
+  _viewers.push_back(viewer);
+  _pendingUpdates.insert(viewer);
   
   _tab_view.add_page(mforms::manage(viewer), title);
 }
 
+void BinaryDataEditor::add_json_viewer(bool read_only, const std::string& text_encoding, const std::string& title)
+{
+  if (!data())
+    return;
+  GError *error = NULL;
+  gsize bread = 0, bwritten = 0;
+  char *converted = g_convert(data(), static_cast<gssize>(length()), "UTF-8", text_encoding.c_str(), &bread, &bwritten, &error);
+  if (!converted || length() != bread)
+  {
+    //convert problem
+    return;
+  }
+  std::string dataToTest = converted;
+  size_t pos = dataToTest.find_first_not_of(SPACES);
+  if (pos != std::string::npos && dataToTest.at(pos) != '{' && dataToTest.at(pos) != '[')
+    return;
+
+  bool isJson = true;
+  JsonParser::JsonValue value;
+  try
+  {
+    JsonParser::JsonReader::read(converted, value);
+  }
+  catch (JsonParser::ParserException &)
+  {
+    isJson = false;
+  }
+  if (isJson)
+  {
+    add_viewer(new JsonDataViewer(this, value, text_encoding), title.c_str());
+    _type = "JSON";
+  }
+}
+
 void BinaryDataEditor::save()
 {
   signal_saved();
+  close();
 }
 
 void BinaryDataEditor::import_value()
