@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,8 +19,6 @@
 #ifndef _DB_CONN_BE_H_
 #define _DB_CONN_BE_H_
 
-#include "wb_config.h"
-
 #include "grts/structs.db.mgmt.h"
 #include "cppdbc.h"
 #include <vector>
@@ -43,7 +41,9 @@ enum ControlType {
   ctNumericUpDown,
   ctButton,
   ctDirSelector,
-  ctFileSelector
+  ctFileSelector,
+  ctEnumSelector,
+  ctText
 };
 
 
@@ -59,11 +59,15 @@ public:
     ptBoolean,
     ptTristate,
     ptDir,
-    ptFile
+    ptFile,
+    ptEnum,
+    ptIntEnum,
+    ptText,
+    ptButton
   };
 
 private:
-  static ParamType decode_param_type(std::string type_name);
+  static ParamType decode_param_type(std::string type_name, std::string real_type);
 
   db_mgmt_DriverParameterRef _inner;
   ParamType _type;
@@ -73,6 +77,7 @@ private:
   DbDriverParam(
     const db_mgmt_DriverParameterRef &driver_param,
     const db_mgmt_ConnectionRef &stored_conn);
+  DbDriverParam(const db_mgmt_DriverParameterRef &driver_param, const grt::ValueRef &value);
 
   ControlType get_control_type() const;
 
@@ -86,6 +91,7 @@ public:
   const grt::ValueRef & get_value() const { return _value; }
   const grt::StringRef get_value_repr() const { return _value.repr(); }
   void set_value(const grt::ValueRef &value);
+  std::vector<std::pair<std::string, std::string> > get_enum_options();
 };
 
 
@@ -102,6 +108,8 @@ private:
   DbDriverParams(const DbDriverParams&) {}
   void free_dyn_mem();
 
+  bool parameter_not_valid(const db_mgmt_DriverRef& driver, const std::string& param);
+
 public:
   DbDriverParams() {}
   ~DbDriverParams() { free_dyn_mem(); }
@@ -111,7 +119,7 @@ public:
     const db_mgmt_ConnectionRef &stored_conn,
     const boost::function<void (bool)> &suspend_layout,
     const boost::function<void ()> &begin_layout,
-    const boost::function<void (DbDriverParam*, ControlType, const MySQL::Geometry::ControlBounds&,
+    const boost::function<void (DbDriverParam*, ControlType, const base::ControlBounds&,
       const std::string &)> &create_control,
     const boost::function<void ()> &end_layout,
     bool skip_schema=false,
@@ -129,55 +137,46 @@ class WBPUBLICBACKEND_PUBLIC_FUNC DbConnection
 {
 private:
   db_mgmt_ManagementRef _mgmt;
-  db_mgmt_RdbmsRef _rdbms;
-  int _active_db_driver_index;
   DbDriverParams _db_driver_param_handles;
+  db_mgmt_DriverRef _active_driver;
   db_mgmt_ConnectionRef _connection;
   bool _skip_schema;
   
   boost::function<void ()> _begin_layout;
   boost::function<void ()> _end_layout;
   boost::function<void (bool)> _suspend_layout;
-  boost::function<void (DbDriverParam*, ControlType, const MySQL::Geometry::ControlBounds&,
+  boost::function<void (DbDriverParam*, ControlType, const base::ControlBounds&,
     const std::string &)> _create_control;
 
-  db_mgmt_DriverRef get_active_driver() { return _active_db_driver_index >= 0 ? _rdbms->drivers().get(_active_db_driver_index) : db_mgmt_DriverRef(); }
-  
   void init_dbc_connection(sql::Connection* dbc_conn, const db_mgmt_ConnectionRef& connectionProperties);
 
 public:
-  DbConnection() : _active_db_driver_index(-1), _skip_schema(false) {}
+  DbConnection(const db_mgmt_ManagementRef &mgmt, const db_mgmt_DriverRef &driver, bool skip_schema);
+  
   ~DbConnection();
   grt::GRT * get_grt() const { return _mgmt.get_grt(); }
   
-  void init(const db_mgmt_ManagementRef &mgmt, bool skip_schema);
   void set_control_callbacks(
     const boost::function<void (bool)> &suspend_layout,
     const boost::function<void ()> &begin_layout,
-    const boost::function<void (DbDriverParam*, ControlType, const MySQL::Geometry::ControlBounds&,
+    const boost::function<void (DbDriverParam*, ControlType, const base::ControlBounds&,
       const std::string &)> &create_control,
     const boost::function<void ()> &end_layout);
 
   DbDriverParams *get_db_driver_param_handles() { return &_db_driver_param_handles; }
   
-  void set_connection(const db_mgmt_ConnectionRef &connection);
+  void update();
+  void set_connection_and_update(const db_mgmt_ConnectionRef &connection);
   void set_connection_keeping_parameters(const db_mgmt_ConnectionRef &connection);
   db_mgmt_ConnectionRef get_connection();
   
   void save_changes();
   
   sql::ConnectionWrapper get_dbc_connection();
-  db_mgmt_RdbmsRef get_rdbms() const { return _rdbms; }
-  grt::ListRef<db_mgmt_Rdbms> get_rdbms_list();
-  db_mgmt_ManagementRef get_db_mgmt() const { return _mgmt; }
+  db_mgmt_ManagementRef get_db_mgmt() { return _mgmt; }
+  db_mgmt_DriverRef driver() { return _active_driver; }
 
-  void set_active_rdbms(int index);
-  grt::ListRef<db_mgmt_Driver> get_driver_list();
-
-  void set_active_driver(int driver_index);
-
-  int get_rdbms_default_driver_index() const;
-  int get_default_rdbms_index() const;
+  void set_driver_and_update(db_mgmt_DriverRef);
 
   bool test_connection();
   std::string validate_driver_params() const;
