@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,7 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-#include "stdafx.h"
 
 #include "wizard_object_filter_page.h"
 /**
@@ -45,15 +44,16 @@ WizardObjectFilterPage::~WizardObjectFilterPage()
 
 
 DBObjectFilterFrame *WizardObjectFilterPage::add_filter(const std::string &class_name,
-                                       const std::string &caption_fmt,
-                                       bec::GrtStringListModel *model,
-                                       bec::GrtStringListModel *excl_model)
+                                                        const std::string &caption_fmt,
+                                                        bec::GrtStringListModel *model,
+                                                        bec::GrtStringListModel *excl_model,
+                                                        bool *enabled_flag)
 {
   DBObjectFilterFrame *filter;
     
   filter= new DBObjectFilterFrame(_form->grtm());
   filter->set_object_class(class_name, caption_fmt);
-  filter->set_models(model, excl_model);
+  filter->set_models(model, excl_model, enabled_flag);
   _box.add(mforms::manage(filter), false, true);
   _filters.push_back(filter);
 //  filter->set_active(true);
