@@ -1,4 +1,21 @@
-#include "stdafx.h"
+/*
+* Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
+*
+* This program is free software; you can redistribute it and/or
+* modify it under the terms of the GNU General Public License as
+* published by the Free Software Foundation; version 2 of the
+* License.
+*
+* This program is distributed in the hope that it will be useful,
+* but WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+* GNU General Public License for more details.
+*
+* You should have received a copy of the GNU General Public License
+* along with this program; if not, write to the Free Software
+* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+* 02110-1301  USA
+*/
 
 #include <grts/structs.workbench.physical.h>
 
@@ -32,8 +49,8 @@ void workbench_physical_RoutineGroupFigure::routineGroup(const db_RoutineGroupRe
   if (_routineGroup.is_valid() && value.is_valid())
     throw std::runtime_error("Cannot change routineGroup field of figure after its set");
 
-  if (_is_global && _routineGroup.is_valid()) _routineGroup.valueptr()->unmark_global();
-  if (_is_global && value.is_valid()) value.valueptr()->mark_global();
+  if (_is_global && _routineGroup.is_valid()) _routineGroup.unmark_global();
+  if (_is_global && value.is_valid()) value.mark_global();
 
   grt::ValueRef ovalue(_routineGroup);
   get_data()->set_routine_group(value);
