@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -92,6 +92,7 @@ Recordset::Recordset(GRTManager *grtm)
   _id = g_atomic_int_get(&next_id);
   g_atomic_int_inc(&next_id);
 
+  task->desc("Recordset task");
   task->send_task_res_msg(false);
   apply_changes_cb= boost::bind(&Recordset::apply_changes_, this);
   register_default_actions();
@@ -157,7 +158,7 @@ bool Recordset::reset(Recordset_data_storage::Ptr data_storage_ptr, bool rethrow
       _column_names.push_back("id");
       _column_types.push_back(int());
       _real_column_types.push_back(int());
-      _column_quoting.push_back(false);
+      _column_flags.push_back(0);
 
       {
         sqlite::query q(*data_swap_db, "select coalesce(max(id)+1, 0) from `data`");
@@ -184,7 +185,7 @@ bool Recordset::reset(Recordset_data_storage::Ptr data_storage_ptr, bool rethrow
   }
 
   data_edited();
-  refresh_ui();
+  //refresh_ui(); This is the wrong place for a GUI refresh. Reset is called from many places, including the c-tor.
 
   return res;
 }
@@ -268,6 +269,8 @@ void Recordset::rollback()
 {
   if (!reset(false))
     task->send_msg(grt::ErrorMsg, _("Rollback failed"), _("Rollback recordset changes"));
+  else
+    refresh_ui();
 }
 
 
@@ -362,6 +365,9 @@ Recordset::Cell Recordset::cell(RowId row, ColumnId column)
     for (ColumnId col= 0; _column_count > col; ++col, ++new_cell)
       *(new_cell)= sqlite::null_t();
     _data[_data.size() - _column_count + _rowid_column] = (int)rowid;
+
+    if (rows_changed)
+      rows_changed();
   }
 
   return VarGridModel::cell(row, column);
@@ -1082,7 +1088,7 @@ void Recordset::paste_rows_from_clipboard(ssize_t dest_row)
         mforms::Utilities::show_error("Cannot Paste Row",
                                       strfmt("Number of fields in pasted data doesn't match the columns in the table (%zi vs %zi).\n"
                                              "Data must be in the same format used by the Copy Row Content command.",
-                                             (long)parts.size(), (long)get_column_count()),
+                                             parts.size(), get_column_count()),
                                       "OK");
 
         if (rows_changed && row != rows.begin())
@@ -1732,14 +1738,17 @@ public:
 
 void Recordset::set_field_value(RowId row, ColumnId column, BinaryDataEditor *data_editor)
 {
-  set_field_raw_data(row, column, data_editor->data(), data_editor->length());
+  if (!data_editor)
+    return;
+  set_field_raw_data(row, column, data_editor->data(), data_editor->length(), data_editor->isJson());
 }
 
 
-void Recordset::set_field_raw_data(RowId row, ColumnId column, const char *data, size_t data_length)
+void Recordset::set_field_raw_data(RowId row, ColumnId column, const char *data, size_t data_length, bool isJson /*= false*/)
 {
   DataValueConv data_value_conv(data, data_length);
-  sqlite::variant_t value= boost::apply_visitor(data_value_conv, _real_column_types[column]);
+  sqlite::variant_t valueString = std::string("");
+  sqlite::variant_t value = boost::apply_visitor(data_value_conv, isJson ? valueString : _real_column_types[column]);
   if (sqlide::is_var_unknown(value))
     throw std::logic_error("Can't save value of this data type.");
   bec::NodeId node(row);
