@@ -27,7 +27,7 @@
 #include <boost/foreach.hpp>
 #include <algorithm>
 #include <sstream>
-
+#include "mforms/jsonview.h"
 
 using namespace bec;
 using namespace grt;
@@ -68,6 +68,34 @@ std::string PrimaryKeyPredicate::operator()(std::vector<boost::shared_ptr<sqlite
 
 //------------------------------------------------------------------------------
 
+class JsonTypeFinder : public boost::static_visitor<bool>
+{
+public:
+  result_type operator()(const sqlite::unknown_t&, const std::string &text) const
+  {
+    bool ret = false;
+    try
+    {
+      JsonParser::JsonValue value;
+      JsonParser::JsonReader::read(text, value);
+      ret = true;
+    }
+    catch (JsonParser::ParserException &)
+    {
+      ret = false;
+    }
+    return ret; 
+  }
+
+  template<typename T, typename V>
+  result_type operator()(const T &t, const V &v) const
+  {
+    return false;
+  }
+};
+
+
+//------------------------------------------------------------------------------
 
 std::string Recordset_sql_storage::statements_as_sql_script(const Sql_script::Statements &statements)
 {
@@ -120,7 +148,7 @@ void Recordset_sql_storage::do_unserialize(Recordset *recordset, sqlite::connect
   Recordset::Column_names &column_names= get_column_names(recordset);
   Recordset::Column_types &column_types= get_column_types(recordset);
   Recordset::Column_types &real_column_types= get_real_column_types(recordset);
-  Recordset::Column_quoting &column_quoting= get_column_quoting(recordset);
+  Recordset::Column_flags &column_flags= get_column_flags(recordset);
 
   _pkey_columns.clear();
   _fields_order.clear(); //! make auto var & bind like var_list
@@ -141,8 +169,8 @@ void Recordset_sql_storage::do_unserialize(Recordset *recordset, sqlite::connect
     real_column_types.reserve(column_names.size());
     std::fill_n(std::back_inserter(real_column_types), column_names.size(), std::string());
 
-    column_quoting.reserve(column_names.size());
-    std::fill_n(std::back_inserter(column_quoting), column_names.size(), true);
+    column_flags.reserve(column_names.size());
+    std::fill_n(std::back_inserter(column_flags), column_names.size(), Recordset::NeedsQuoteFlag);
 
     if (!column_names.empty())
     {
@@ -354,7 +382,7 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
   const Recordset::Column_names &column_names= get_column_names(recordset);
   const Recordset::Column_types &column_types= get_column_types(recordset);
   const Recordset::Column_types &real_column_types= get_real_column_types(recordset);
-  const Recordset::Column_quoting &column_quoting = get_column_quoting(recordset);
+  const Recordset::Column_flags &column_flags = get_column_flags(recordset);
   const size_t partition_count= recordset->data_swap_db_partition_count();
   const sqlide::VarToStr var_to_str;
 
@@ -370,6 +398,8 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
 
   std::string full_table_name= this->full_table_name();
 
+  JsonTypeFinder jsonTypeFinder;
+  bool unknownAsStringOrginal = qv.store_unknown_as_string;
   if (0 == editable_col_count)
     return;
 
@@ -465,9 +495,12 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
                 boost::shared_ptr<sqlite::result> &data_row_rs= data_row_results[partition];
 
                 v = data_row_rs->get_variant((int)partition_column);
-
+                if (!qv.store_unknown_as_string && boost::apply_visitor(jsonTypeFinder, column_types[column], v))
+                  qv.store_unknown_as_string = true;
                 values+= strfmt("%s, ",
                     boost::apply_visitor(qv, column_types[column], v).c_str());
+                if (unknownAsStringOrginal != qv.store_unknown_as_string)
+                  qv.store_unknown_as_string = unknownAsStringOrginal;
                 if (blob_columns[column])
                   sql_bindings.push_back(v);
               }
@@ -508,9 +541,14 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
                 boost::shared_ptr<sqlite::result> &data_row_rs= data_row_results[partition];
 
                 v = data_row_rs->get_variant((int)partition_column);
+                
+                if (!qv.store_unknown_as_string && boost::apply_visitor(jsonTypeFinder, column_types[column], v))
+                  qv.store_unknown_as_string = true;
                 values+= strfmt("`%s`=%s, ",
                   column_names[column].c_str(),
                   boost::apply_visitor(qv, column_types[column], v).c_str());
+                if (unknownAsStringOrginal != qv.store_unknown_as_string)
+                  qv.store_unknown_as_string = unknownAsStringOrginal;
                 if (blob_columns[column] && _binding_blobs)
                   sql_bindings.push_back(v);
               }
@@ -556,7 +594,7 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
             ColumnId partition_column= col - col_begin;
             v = data_rs->get_variant((int)partition_column);
             values+= strfmt("%s, ",
-                  column_quoting[partition_column] || sqlide::is_var_null(v)?
+                  (column_flags[partition_column] & Recordset::NeedsQuoteFlag) || sqlide::is_var_null(v)?
                   boost::apply_visitor(qv, column_types[partition_column], v).c_str():
                   boost::apply_visitor(var_to_str, v).c_str());
 
@@ -589,7 +627,7 @@ void Recordset_sql_storage::generate_inserts(const Recordset *recordset, sqlite:
   const Recordset::Column_names &column_names= get_column_names(recordset);
   const Recordset::Column_types &column_types= get_column_types(recordset);
   const Recordset::Column_types &real_column_types= get_real_column_types(recordset);
-  const Recordset::Column_quoting &column_quoted= get_column_quoting(recordset);
+  const Recordset::Column_flags &column_flags= get_column_flags(recordset);
   const size_t partition_count= recordset->data_swap_db_partition_count();
 
   //RowId min_new_rowid= recordset->min_new_rowid();
@@ -632,7 +670,14 @@ void Recordset_sql_storage::generate_inserts(const Recordset *recordset, sqlite:
           sqlide::VarToStr var_to_str;
 
           std::string value;
-          if (column_quoted[col] || sqlide::is_var_null(v))
+          if (sqlide::is_var_null(v))
+          {
+            if ((column_flags[col] & Recordset::NotNullFlag) != 0)
+              value = "DEFAULT";
+            else
+              value = boost::apply_visitor(qv, column_types[col], v);
+          }
+          else if ((column_flags[col] & Recordset::NeedsQuoteFlag))
             value = boost::apply_visitor(qv, column_types[col], v);
           else
           {
