@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -172,6 +172,11 @@ static std::string escape_sql_string_(const std::string &s)
   return base::escape_sql_string(s, false);
 }
 
+static std::string escape_json_string_(const std::string &s)
+{
+  return base::escape_json_string(s);
+}
+
 void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::connection *data_swap_db)
 {
   const TemplateInfo &info(template_info(_data_format));
@@ -203,7 +208,7 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
       if (!pre_tpl)
         g_warning("Failed to open template file: `%s`", pre_tpl_path.c_str()); 
       else
-        pre_tpl->ReloadIfChanged();
+        pre_tpl->ReloadAllIfChanged();
     }      
     if (g_file_test((name+".post.tpl").c_str(), G_FILE_TEST_EXISTS))
     {
@@ -212,7 +217,7 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
       if (!post_tpl)
         g_warning("Failed to open template file: `%s`", post_tpl_path.c_str());
       else
-        post_tpl->ReloadIfChanged();
+        post_tpl->ReloadAllIfChanged();
     }
   }    
   
@@ -221,7 +226,7 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
       throw std::runtime_error(strfmt("Failed to open output file: `%s`", _file_path.c_str()));
   }
 
-  tpl->ReloadIfChanged();
+  tpl->ReloadAllIfChanged();
 
   std::auto_ptr<TemplateDictionary> dict(new TemplateDictionary("/"));
   BOOST_FOREACH (const Parameters::value_type &param, _parameters)
@@ -230,14 +235,17 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
   const Recordset::Column_names *column_names= recordset->column_names();
   const Recordset::Column_types &column_types= get_column_types(recordset);
   const Recordset::Column_types &real_column_types= get_real_column_types(recordset);
-  const Recordset::Column_quoting &column_quoting= get_column_quoting(recordset);
+  const Recordset::Column_flags &column_flags= get_column_flags(recordset);
   
   ColumnId visible_col_count= recordset->get_column_count();
   sqlide::QuoteVar qv;
   {
     if (info.quote != "")
       qv.quote = info.quote;
-    qv.escape_string= std::ptr_fun(escape_sql_string_);
+    if (_data_format == "JSON")
+      qv.escape_string= std::ptr_fun(escape_json_string_);
+    else
+      qv.escape_string= std::ptr_fun(escape_sql_string_);
     // swap db (sqlite) stores unknown values as quoted strings
     qv.store_unknown_as_string= true;
     qv.allow_func_escaping= false;
@@ -348,7 +356,7 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
               if (is_null)
                 field_value = null_syntax;
               else if (strings_are_pre_quoted)
-                field_value= column_quoting[col] || sqlide::is_var_null(v)?
+                field_value= (column_flags[col] & Recordset::NeedsQuoteFlag) || sqlide::is_var_null(v)?
                      boost::apply_visitor(qv, column_types[col], v):
                      boost::apply_visitor(var_to_str, v);
               else
@@ -416,7 +424,7 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
               sqlide::VarToStr var_to_str;
               
               if (strings_are_pre_quoted)
-                field_value= column_quoting[col] || sqlide::is_var_null(v)?
+                field_value= (column_flags[col] & Recordset::NeedsQuoteFlag) || sqlide::is_var_null(v)?
                     boost::apply_visitor(qv, column_types[col], v):
                     boost::apply_visitor(var_to_str, v);
               else
