@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2012, 2013 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,21 +17,15 @@
  * 02110-1301  USA
  */
 
-#include "tut_stdafx.h"
 #include "connection_helpers.h"
 #include "test_fixture.h"
 #include "base/file_utilities.h"
 #include "sqlide/wb_sql_editor_form.h"
 #include "sqlide/autocomplete_object_name_cache.h"
 
-
 using namespace grt;
 using namespace wb;
 
-// Helper to wait for cache operations to finish.
-static base::Mutex cache_mutex;
-static base::Cond cache_condition;
-
 BEGIN_TEST_DATA_CLASS(autocompletion_cache_test)
 public:
   GRT _grt;
@@ -40,12 +34,21 @@ public:
   AutoCompleteCache *_cache;
 
 TEST_DATA_CONSTRUCTOR(autocompletion_cache_test)
-  : _cache(0), _conn(new sql::Dbc_connection_handler())
+  : _conn(new sql::Dbc_connection_handler()), _cache(0)
 {
+  // We want to print this out only once, not for every test, so we put it here
+  // as this is the first test that runs usually.
+#ifdef _WIN32
+  TCHAR path[MAX_PATH];
+  GetCurrentDirectory(MAX_PATH, path);
+  printf("\nTests running in: %s\n\n", base::wstring_to_string(path).c_str());
+#endif
+
   _grt.scan_metaclasses_in("../../res/grt/");
   _grt.end_loading_metaclasses();
 
-  // setup the sakila db in server
+  // Because tests are executed in alphabetic order this is the first one.
+  // Hence we set up the sakila db in server here.
   setup_sakila_db();
 }
 
@@ -67,16 +70,6 @@ END_TEST_DATA_CLASS;
 
 TEST_MODULE(autocompletion_cache_test, "autocompletion object name cache");
 
-static void cache_callback(bool working)
-{
-  if (!working)
-  {
-    cache_mutex.lock();
-    cache_condition.signal();
-    cache_mutex.unlock();
-  }
-}
-
 TEST_FUNCTION(2)
 {
   db_mgmt_ConnectionRef connectionProperties(&_grt);
@@ -88,12 +81,24 @@ TEST_FUNCTION(2)
   _conn->ref = dm->getConnection(connectionProperties);
 }
 
-
 TEST_FUNCTION(3)
 {
   base::remove("testconn.cache");
   _cache = new AutoCompleteCache("testconn", boost::bind(&Test_object_base<autocompletion_cache_test>::get_connection, this, _1),
-    ".", cache_callback);
+    ".", NULL);
+
+  // Right after creation the schema list is empty. Retrieval has just been set up.
+  std::vector<std::string> list = _cache->get_matching_schema_names("sakila");
+  ensure("Schema list is not empty", list.empty());
+
+  // After creation a first data retrieval starts automatically in the background.
+  // Wait a moment to have that finished.
+  g_usleep(2000000);
+
+  // Retrieve db objects in the sakila schema. So they are available when we ask for them
+  // in the following tests.
+  _cache->refresh_schema_cache_if_needed("sakila");
+  g_usleep(2000000);
 }
 
 static void ensure_list_equals(const char *what, const std::vector<std::string> &list, const char **comp)
@@ -111,6 +116,7 @@ static void ensure_list_equals(const char *what, const std::vector<std::string>
   }
   catch (...)
   {
+    // TODO: this should be part of the TUT message, otherwise we might not see it.
     g_message("Result list:");
     for (i = list.begin(); i != list.end(); ++i)
       g_message("  %s", i->c_str());
@@ -124,21 +130,10 @@ static void ensure_list_equals(const char *what, const std::vector<std::string>
 
 TEST_FUNCTION(10)
 {
-  std::vector<std::string> list = _cache->get_matching_schema_names("sakila");
-  // 1st time fetch the list is empty because the data is not cached and the worker will start
-  // fetching
-  ensure("uncached schema list (empty)", list.empty());
-
-  _cache->refresh_schema_list_cache_if_needed(); // TODO: is this really needed? get_matching_schema_names calls it already.
-
-  // The loops are not necessary, but there can be spurious condition signals,
-  // as the glib docs say.
-  while (!_cache->is_schema_list_fetch_done())
-    cache_condition.wait(cache_mutex);
-
-  list = _cache->get_matching_schema_names("");
+  std::vector<std::string> list = _cache->get_matching_schema_names("");
   int found = 0;
-  // this time the schema list should contain sakila and mysql
+
+  // This time the schema list should contain sakila and mysql.
   for (std::vector<std::string>::const_iterator i = list.begin(); i != list.end(); ++i)
   {
     if (*i == "sakila" || *i == "mysql")
@@ -154,7 +149,7 @@ TEST_FUNCTION(10)
     if (*i == "sakila" || *i == "mysql")
       found++;
   }
-  ensure_equals("known schema name matches (sakila*)", found, 1);
+  ensure_equals("known schema name matches (sakila)", found, 1);
 }
 
 TEST_FUNCTION(12)
@@ -168,16 +163,21 @@ TEST_FUNCTION(12)
     "actor_info",
     NULL
   };
+
+  _cache->refresh_schema_cache_if_needed("sakila");
+
+  // Wait for the refresh to settle. The functions to check if a fetch is done
+  // are really unreliable and should be revised.
+  g_usleep(1000000);
+
   std::vector<std::string> list = _cache->get_matching_table_names("sakila", "ac");
-
-  while (!_cache->is_schema_tables_fetch_done("sakila"))
-    cache_condition.wait(cache_mutex);
-
-  // get the list now
-  list = _cache->get_matching_table_names("sakila", "ac");
+  std::vector<std::string> list2 = _cache->get_matching_view_names("sakila", "ac");
+  std::copy(list2.begin(), list2.end(), std::back_inserter(list));
   ensure_list_equals("tables sakila.ac*", list, sakila_ac);
 
   list = _cache->get_matching_table_names("sakila", "actor_");
+  list2 = _cache->get_matching_view_names("sakila", "ac");
+  std::copy(list2.begin(), list2.end(), std::back_inserter(list));
   ensure_list_equals("tables sakila.actor_*", list, sakila_actor_);
 }
 
@@ -188,13 +188,8 @@ TEST_FUNCTION(14)
     "inventory_in_stock",
     NULL
   };
+
   std::vector<std::string> list = _cache->get_matching_function_names("sakila", "inv");
-
-  while (!_cache->is_schema_routines_fetch_done("sakila"))
-    cache_condition.wait(cache_mutex);
-
-  // get the list now
-  list = _cache->get_matching_function_names("sakila", "inv");
   ensure_list_equals("functions sakila.inv*", list, sakila_inv);
 }
 
@@ -205,13 +200,8 @@ TEST_FUNCTION(16)
     "film_not_in_stock",
     NULL
   };
+
   std::vector<std::string> list = _cache->get_matching_procedure_names("sakila", "fi");
-
-  while (!_cache->is_schema_routines_fetch_done("sakila"))
-    cache_condition.wait(cache_mutex);
-
-  // get the list now
-  list = _cache->get_matching_procedure_names("sakila", "fi");
   ensure_list_equals("procedures sakila.fi*", list, sakila_fi);
 }
 
@@ -223,23 +213,21 @@ TEST_FUNCTION(18)
     "actor_id",
     NULL
   };
+
   std::vector<std::string> list = _cache->get_matching_column_names("sakila", "actor", "a");
-
-  while (!_cache->is_schema_table_columns_fetch_done("sakila", "actor"))
-    cache_condition.wait(cache_mutex);
-
-  // get the list now
-  list = _cache->get_matching_column_names("sakila", "actor", "a");
   ensure_list_equals("columns sakila.actor.a*", list, sakila_a);
 }
 
-/// Everything again reusing cache
+// Everything again reusing cache
 TEST_FUNCTION(19)
 {
   _cache->shutdown();
   delete _cache;
   _cache = new AutoCompleteCache("testconn", boost::bind(&Test_object_base<autocompletion_cache_test>::get_connection, this, _1),
-    ".", cache_callback);
+    ".", NULL);
+
+  _cache->refresh_schema_cache_if_needed("sakila");
+  g_usleep(2000000);
 }
 
 
@@ -249,7 +237,8 @@ TEST_FUNCTION(20)
 
   list = _cache->get_matching_schema_names("");
   int found = 0;
-  // this time the schema list should contain sakila and mysql
+  
+  // This time the schema list should contain sakila and mysql.
   for (std::vector<std::string>::const_iterator i = list.begin(); i != list.end(); ++i)
   {
     if (*i == "sakila" || *i == "mysql")
@@ -279,10 +268,15 @@ TEST_FUNCTION(22)
     "actor_info",
     NULL
   };
+
   std::vector<std::string> list = _cache->get_matching_table_names("sakila", "ac");
+  std::vector<std::string> list2 = _cache->get_matching_view_names("sakila", "ac");
+  std::copy(list2.begin(), list2.end(), std::back_inserter(list));
   ensure_list_equals("tables sakila.ac*", list, sakila_ac);
 
   list = _cache->get_matching_table_names("sakila", "actor_");
+  list2 = _cache->get_matching_view_names("sakila", "actor_");
+  std::copy(list2.begin(), list2.end(), std::back_inserter(list));
   ensure_list_equals("tables sakila.actor_*", list, sakila_actor_);
 }
 
@@ -293,9 +287,8 @@ TEST_FUNCTION(24)
     "inventory_in_stock",
     NULL
   };
-  std::vector<std::string> list;
-  // get the list now
-  list = _cache->get_matching_function_names("sakila", "inv");
+
+  std::vector<std::string> list = _cache->get_matching_function_names("sakila", "inv");
   ensure_list_equals("functions sakila.inv*", list, sakila_inv);
 }
 
@@ -306,25 +299,20 @@ TEST_FUNCTION(26)
     "film_not_in_stock",
     NULL
   };
-  std::vector<std::string> list;
 
-  // get the list now
-  list = _cache->get_matching_procedure_names("sakila", "fi");
+  std::vector<std::string> list = _cache->get_matching_procedure_names("sakila", "fi");
   ensure_list_equals("procedures sakila.fi*", list, sakila_fi);
 }
 
 
 TEST_FUNCTION(28)
 {
-  // Just like test 18. See if we still get the same result after all the objects have been loaded.
   static const char *sakila_a[] = {
     "actor_id",
     NULL
   };
-  std::vector<std::string> list;
 
-  // get the list now
-  list = _cache->get_matching_column_names("sakila", "actor", "a");
+  std::vector<std::string> list = _cache->get_matching_column_names("sakila", "actor", "a");
   ensure_list_equals("columns sakila.actor.a*", list, sakila_a);
 }
 
