@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "base_bridge.h"
 #include "grt/grt_manager.h"
 
@@ -36,11 +34,6 @@ bool BridgeBase::is_main_thread()
 
 void BridgeBase::run_later(const boost::function<void ()> &slot)
 {
-    bec::GRTManager::get_instance_for(get_object()->get_grt())->run_once_when_idle(slot);
+    bec::GRTManager::get_instance_for(get_object()->get_grt())->run_once_when_idle(this, slot);
 }
 
-
-void BridgeBase::run_later(grt::GRT *grt, const boost::function<void ()> &slot)
-{
-    bec::GRTManager::get_instance_for(grt)->run_once_when_idle(slot);
-}
