@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,50 +17,11 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "figure_common.h"
 #include "base/string_utilities.h"
 
 using namespace wbfig;
-using namespace MySQL::Drawing;
-using namespace MySQL::Geometry;
-
-mdc::FontSpec parse_font_spec(const std::string &font)
-{
-  std::vector<std::string> parts= base::split(font, " ");
-  mdc::FontSpec fs;
-  int size= 12;
-
-  if (!parts.empty() && sscanf(parts.back().c_str(), "%i", &size) == 1)
-  {
-    fs.size= (float) size;
-    parts.pop_back();
-  }
-
-  for (int i= 0; i < 2 && !parts.empty(); i++)
-  {
-    if (g_strcasecmp(parts.back().c_str(), "bold")==0)
-    {
-      fs.toggle_bold(true);
-      parts.pop_back();
-    }
-
-    if (g_strcasecmp(parts.back().c_str(), "italic")==0)
-    {
-      fs.toggle_italic(true);
-      parts.pop_back();
-    }
-  }
-
-  if (!parts.empty())
-  {
-    fs.family= parts[0];
-    for (unsigned int i= 1; i < parts.size(); i++)
-      fs.family+= " " + parts[i];
-  }
-  return fs;
-}
+using namespace base;
 
 Titlebar::Titlebar(mdc::Layer *layer, FigureEventHub *hub, BaseFigure *owner, bool expander)
   : mdc::Box(layer, mdc::Box::Horizontal), _hub(hub), _owner(owner), _icon_text(layer)
@@ -241,7 +202,7 @@ void Titlebar::set_auto_sizing(bool flag)
 FigureItem::FigureItem(mdc::Layer *layer, FigureEventHub *hub, BaseFigure *owner)
 : mdc::IconTextFigure(layer), _hub(hub), _owner(owner), _dirty(true) 
 {
-  set_font(mdc::FontSpec("Helvetica", mdc::SNormal, mdc::WNormal, 11));
+  set_font(get_view()->get_default_font());
 }
 
 
@@ -350,8 +311,10 @@ BaseFigure::BaseFigure(mdc::Layer *layer, FigureEventHub *hub, const model_Objec
 : mdc::Box(layer, mdc::Box::Vertical), _hub(hub)
 {
   _represented_object= &object.content();
-  
+
+#ifndef __APPLE__ // disabling caching allows for hidpi correctness
   set_cache_toplevel_contents(true);
+#endif
   set_draggable(true);
   _manual_resizing= false;
   _resizing= false;
@@ -458,7 +421,7 @@ void BaseFigure::invalidate_min_sizes()
 bool BaseFigure::on_drag_handle(mdc::ItemHandle *handle, const Point &pos, bool dragging)
 {
   if (!_manual_resizing)
-    set_allow_manual_resizing(true);
+    set_allow_manual_resizing(true); // TODO: what about undo record?
 
   if (dragging)
   {
@@ -471,8 +434,8 @@ bool BaseFigure::on_drag_handle(mdc::ItemHandle *handle, const Point &pos, bool
 
   if (!dragging)
   {
-    _resizing= false;
-    _signal_resize(_initial_bounds);
+    _resizing = false;
+    _signal_interactive_resize(_initial_bounds);
   }
   if (flag)
     set_fixed_size(get_size());
@@ -484,6 +447,8 @@ bool BaseFigure::on_drag_handle(mdc::ItemHandle *handle, const Point &pos, bool
 void BaseFigure::set_content_font(const mdc::FontSpec &font)
 {
   _content_font= font;
+  if (font.family == "Arial")
+    _content_font=font;
 }
 
 
@@ -642,6 +607,7 @@ void BaseFigure::end_sync(mdc::Box &box, ItemList &list, ItemList::iterator iter
 CaptionFigure::CaptionFigure(mdc::Layer *layer, FigureEventHub *hub, model_Object *owner)
 : mdc::TextFigure(layer), _hub(hub), _owner_object(owner)
 {
+  set_draw_outline(true);
 }
 
 
@@ -754,12 +720,12 @@ void ShrinkableBox::resize_to(const Size &size)
       number_of_items_that_fit= (int) floor((size.height - 2 * _ypadding + _spacing) / (min_item_size.height + _spacing));
     }
         
-    _hidden_item_count= _children.size() - number_of_items_that_fit;
+    _hidden_item_count = (int)_children.size() - number_of_items_that_fit;
     if (_hidden_item_count > 0)
     {  
       // add space needed by "X items more..." label
       number_of_items_that_fit= (int) floor((size.height - EXTRA_LABEL_SIZE + _spacing) / (min_item_size.height + _spacing));
-      _hidden_item_count= _children.size() - number_of_items_that_fit;      
+      _hidden_item_count = (int)_children.size() - number_of_items_that_fit;
     }
     
     if (number_of_items_that_fit > 0)
