@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "grts/structs.workbench.model.h"
 
 #include "model_diagram_impl.h"
@@ -34,8 +32,7 @@
 #include "base/string_utilities.h"
 
 using namespace grt;
-using namespace MySQL::Geometry;
-using namespace MySQL::Drawing;
+using namespace base;
 
 // this subclass allows repainting to happen in 3 steps, layer only, connection only and figure only
 // to follow a repaint ordering that's independent from logical ordering
@@ -430,7 +427,7 @@ void model_Diagram::ImplData::member_changed(const std::string &name, const grt:
     if (name == "x" || name == "y")
     {
       if (_canvas_view)
-        _canvas_view->set_offset(MySQL::Geometry::Point(_self->_x, _self->_y));
+        _canvas_view->set_offset(base::Point(_self->_x, _self->_y));
     }
     else if (name == "pageSettings")
     {
