@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "model_figure_impl.h"
 #include "model_diagram_impl.h"
 #include "model_layer_impl.h"
@@ -348,13 +346,13 @@ void model_Figure::ImplData::finish_realize()
     
     font= model->get_string_option(strfmt("%s:TitleFont", self()->class_name().c_str()), "");
     if (!font.empty())
-      figure->set_title_font(parse_font_spec(font));
+      figure->set_title_font(mdc::FontSpec::from_string(font));
 
     font= model->get_string_option(strfmt("%s:ItemsFont", self()->class_name().c_str()), "");
     if (font.empty())
       font= model->get_string_option(strfmt("%s:TextFont", self()->class_name().c_str()), "");
     if (!font.empty())
-      figure->set_content_font(parse_font_spec(font));
+      figure->set_content_font(mdc::FontSpec::from_string(font));
   }
 
   view->stack_figure(self(), get_canvas_item());
