@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,19 +17,10 @@
  * 02110-1301  USA
  */
 
-#ifndef _MODEL_FIGURE_IMPL_H_
-#define _MODEL_FIGURE_IMPL_H_
+#pragma once
 
-#include <mdc.h>
-#include <grtpp.h>
-
-#ifdef _WIN32
-#include <hash_map>
-#define hash_map stdext::hash_map
-#else
-#include <ext/hash_map>
-#define hash_map __gnu_cxx::hash_map
-#endif
+#include "mdc.h"
+#include "grtpp.h"
 
 #include "grts/structs.model.h"
 
@@ -41,21 +32,6 @@
 
 #include "badge_figure.h"
 
-#ifdef __GNUC__
-namespace __gnu_cxx
-{
-  using std::size_t;
-
-  template<>
-    struct hash<grt::internal::Object*>
-    {
-      size_t
-      operator()(grt::internal::Object* __p) const
-      { return (intptr_t)__p; }
-    };
-};
-#endif
-
 namespace bec {
   class UndoAction;
 };
@@ -109,6 +85,3 @@ public:
 private:
   model_Figure *self() const { return (model_Figure*)_self; }
 };
-
-
-#endif // _MODEl_FIGURE_IMPL_H_
