@@ -6,12 +6,13 @@
         Partitioning
         RunStyles
         ContractionState
-
-    To do:
+        CharClassify
         Decoration
         DecorationList
+        CellBuffer
+
+    To do:
         PerLine *
-        CellBuffer *
         Range
         StyledText
         CaseFolder ...
@@ -31,11 +32,13 @@
         WordList
 */
 
-#include <stdio.h>
+#include <cstdio>
+#include <cstdarg>
 
 #include "Platform.h"
 
-#include <gtest/gtest.h>
+#define CATCH_CONFIG_MAIN  // This tells Catch to provide a main() - only do this in one cpp file
+#include "catch.hpp"
 
 // Needed for PLATFORM_ASSERT in code being tested
 
@@ -44,7 +47,11 @@ void Platform::Assert(const char *c, const char *file, int line) {
 	abort();
 }
 
-int main(int argc, char **argv) {
-	testing::InitGoogleTest(&argc, argv);
-	return RUN_ALL_TESTS();
+void Platform::DebugPrintf(const char *format, ...) {
+	char buffer[2000];
+	va_list pArguments;
+	va_start(pArguments, format);
+	vsprintf(buffer, format, pArguments);
+	va_end(pArguments);
+	fprintf(stderr, "%s", buffer);
 }
