@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,7 +17,7 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
+#include "grt_python_debugger.h"
 #include "grt_shell_window.h"
 #include "base/file_functions.h"
 #include "base/string_utilities.h"
@@ -31,14 +31,17 @@ DEFAULT_LOG_DOMAIN("grtshell")
 #include "grt/common.h"
 #include "grt_code_editor.h"
 #include "grt_plugin_wizard.h"
-#include "grt_python_debugger.h"
+
 
 #include "workbench/wb_context.h"
 #include "mforms/app.h"
 #include "mforms/imagebox.h"
 
+#include <glib/gstdio.h>
+
 using namespace base;
 using namespace bec;
+using namespace mforms;
 
 
 #define EDITOR_TAB_OFFSET 2
@@ -73,10 +76,58 @@ _lower_tab(mforms::TabViewDocument),
   set_title(("Workbench Scripting Shell"));
   set_name("shell_window");
   set_content(&_content);
+  set_menubar(&_menu);
   scoped_connect(signal_closed(),boost::bind(&GRTShellWindow::shell_closed, this));
-  
+  set_on_close(boost::bind(&GRTShellWindow::can_close, this));
+ 
   _content.add(&_toolbar, false, true);
 
+  // setup the menubar
+  {
+    mforms::MenuItem *menu = mforms::manage(new mforms::MenuItem("File"));
+    mforms::MenuItem *item;
+    _menu.add_submenu(menu);
+    item = menu->add_item_with_title("New...", boost::bind(&GRTShellWindow::add_new_script, this));
+    item->set_shortcut("Modifier+N");
+    menu->add_item_with_title("New Script", boost::bind(&GRTShellWindow::add_editor, this, true, "python"));
+    item = menu->add_item_with_title("Open...", boost::bind(&GRTShellWindow::open_script_file, this));
+    item->set_shortcut("Modifier+O");
+    menu->add_separator();
+    item = menu->add_item_with_title("Save", boost::bind(&GRTShellWindow::save_file, this, false));
+    item->set_shortcut("Modifier+S");
+    item = menu->add_item_with_title("Save As...", boost::bind(&GRTShellWindow::save_file, this, true));
+    item->set_shortcut("Modifier+Shift+S");
+    menu->add_separator();
+    item = menu->add_item_with_title("Close Script", boost::bind(&GRTShellWindow::close_tab, this));
+    item->set_shortcut("Modifier+W");
+    item = menu->add_item_with_title("Close Window", boost::bind(&GRTShellWindow::close, this));
+#ifdef _WIN32
+    item->set_shortcut("Control+F4");
+#else
+    item->set_shortcut("Modifier+Shift+W");
+#endif
+    menu = mforms::manage(new mforms::MenuItem("Edit"));
+    _menu.add_submenu(menu);
+
+    item = menu->add_item_with_title("Cut", boost::bind(&GRTShellWindow::cut, this));
+    item->set_shortcut("Modifier+X");
+    item = menu->add_item_with_title("Copy", boost::bind(&GRTShellWindow::copy, this));
+    item->set_shortcut("Modifier+C");
+    item = menu->add_item_with_title("Paste", boost::bind(&GRTShellWindow::paste, this));
+    item->set_shortcut("Modifier+V");
+    item = menu->add_item_with_title("Select All", boost::bind(&GRTShellWindow::select_all, this));
+    item->set_shortcut("Modifier+A");
+    menu->add_separator();
+    item = menu->add_item_with_title("Find...", boost::bind(&GRTShellWindow::show_find_panel, this));
+    item->set_shortcut("Modifier+F");
+
+    menu = mforms::manage(new mforms::MenuItem("Script"));
+    _menu.add_submenu(menu);
+    
+    item = menu->add_item_with_title("Run", boost::bind(&GRTShellWindow::execute_file, this));
+    item->set_shortcut("Modifier+R");
+  }
+
 #ifdef _WIN32
   _content.add(&_padding_box, true, true);
   _padding_box.add(&_hsplitter, true, true);
@@ -126,7 +177,7 @@ _lower_tab(mforms::TabViewDocument),
   }
   //  else
   {
-    _run_button = add_tool_button("tiny_execute.png", boost::bind(&GRTShellWindow::execute_file, this),
+    _run_button = add_tool_button("qe_sql-editor-tb-icon_execute.png", boost::bind(&GRTShellWindow::execute_file, this),
       _("Execute script"));
     _continue_button = add_tool_button("debug_continue.png", boost::bind(&GRTShellWindow::debug_continue, this), 
       _("Continue execution until next breakpoint"));
@@ -150,7 +201,7 @@ _lower_tab(mforms::TabViewDocument),
     _pause_button->set_enabled(false);
   }  
 
-#ifndef _WIN32
+#if !defined(_WIN32) && !defined(__APPLE__)
   // TODO: remove as soon as all platforms support closable tabs.
   _close_script_tab_button = add_tool_button("Discard.png", boost::bind(&GRTShellWindow::close_tab, this),
     _("Close this script tab"), false);
@@ -221,7 +272,9 @@ _lower_tab(mforms::TabViewDocument),
 
   _global_box2.add(&_global_entry, false, true);
   _global_entry.set_read_only(true);
+#if defined(_WIN32) | defined(__APPLE__)
   _global_entry.set_back_color("#FFFFFF");
+#endif
   
   _global_box2.add(&_global_list, true, true);
   _global_list.add_column(mforms::IconStringColumnType, "Name", 100);
@@ -314,8 +367,10 @@ _lower_tab(mforms::TabViewDocument),
   _shell_box.add(&_shell_text, true, true);
   _shell_text.set_monospaced(true);
   _shell_text.set_read_only(true);
-  _shell_text.set_front_color("#E0E0E0");
+#if defined(_WIN32) | defined(__APPLE__)
+  _shell_text.set_front_color("#FFFFFF");
   _shell_text.set_back_color("#000000");
+#endif
   _shell_text.set_padding(2);
   _shell_box.add(&_shell_hbox, false, true);
   _shell_hbox.add(&_shell_prompt, false, true);
@@ -354,7 +409,6 @@ _lower_tab(mforms::TabViewDocument),
   _main_tab.add_page(&_snippet_splitter, "Snippets");
 
   scoped_connect(_main_tab.signal_tab_closing(),boost::bind(&GRTShellWindow::on_tab_closing, this, _1));
-  scoped_connect(_main_tab.signal_tab_closed(),boost::bind(&GRTShellWindow::on_tab_closed, this, _1));
   scoped_connect(_main_tab.signal_tab_changed(),boost::bind(&GRTShellWindow::on_tab_changed, this));
 
   //
@@ -369,8 +423,9 @@ _lower_tab(mforms::TabViewDocument),
   {
     _debugger->init_pdb();
   }
-  catch (const std::exception)
+  catch (const std::exception &exc)
   {
+    log_error("Could not initialize the debugger: %s\n", exc.what());
     add_output("Could not initialize the debugger\n");
     delete _debugger;
     _debugger= 0;
@@ -389,6 +444,15 @@ _lower_tab(mforms::TabViewDocument),
   side_tab_changed();  
 }
 
+bool GRTShellWindow::can_close()
+{
+  // Because there's no other way to know if the window is about to close, we'll use this event to stop debugger.
+  if (_stop_button->is_enabled() && _debugger)
+    _debugger->stop();
+
+  // GRTShellWindow is about to close so we ask each editor if it's fine to quit.
+  return request_quit();
+}
 
 void GRTShellWindow::set_splitter_positions()
 {
@@ -441,6 +505,8 @@ void GRTShellWindow::shell_action(mforms::TextEntryAction action)
     }      
     case mforms::EntryCKeyDown:
       break;
+    case mforms::EntryEscape:
+      break;
   }
 }
 
@@ -481,16 +547,8 @@ void GRTShellWindow::refresh_all()
 
   refresh_notifs_list();
 
-  if (grtm()->get_grt()->shell_type() == "lua")
-  {
-    _script_extension = ".lua";
-    _comment_prefix = "-- ";
-  }
-  else if (grtm()->get_grt()->shell_type() == "python")
-  {
-    _script_extension = ".py";
-    _comment_prefix = "# ";
-  }
+  _script_extension = ".py";
+  _comment_prefix = "# ";  
   
   refresh_snippets();
 }
@@ -660,7 +718,8 @@ void GRTShellWindow::side_tab_changed()
     _("File Browser"),
     _("Globals Tree"),
     _("Classes List"),
-    _("Modules List")
+    _("Modules List"),
+    _("Notifications")
   };
 
   _side_header_panel.set_title(side_bar_titles[_side_tab.get_active_tab()]);
@@ -950,12 +1009,6 @@ void GRTShellWindow::scriptize_snippet()
     std::string snippet = node->get_tag();
     std::string language = "python";
     
-    // kind of a hack.. pick language depending on comment type used
-    if (g_str_has_prefix(snippet.c_str(), "#"))
-      language= "python";
-    else if (g_str_has_prefix(snippet.c_str(), "--"))
-      language= "lua";
-    
     GRTCodeEditor *editor= add_editor(true, language);
     editor->set_text(snippet);
   }
@@ -984,12 +1037,6 @@ void GRTShellWindow::run_snippet()
     try
     {
       std::string language = "python";
-      
-      // kind of a hack.. pick language depending on comment type used
-      if (g_str_has_prefix(script.c_str(), "#"))
-        language= "python";
-      else if (g_str_has_prefix(script.c_str(), "--"))
-        language= "lua";
 
       bool ret = execute_script(script, language);
       grtm()->get_grt()->pop_message_handler();
@@ -1150,20 +1197,21 @@ void GRTShellWindow::open_file_in_editor(const std::string &path, bool is_script
   if (g_str_has_suffix(path.c_str(), ".py"))
     language = "python"; // Python script
   else
-    if (g_str_has_suffix(path.c_str(), ".lua"))
-      language = "lua"; // Lua script
-    else
-      if (g_str_has_suffix(path.c_str(), ".sql") || g_str_has_suffix(path.c_str(), ".qbquery"))
-        language = "sql";
+    if (g_str_has_suffix(path.c_str(), ".sql") || g_str_has_suffix(path.c_str(), ".qbquery"))
+      language = "sql";
 
   // Show warning messages if applicable...
   if (language == "")
   {
     std::string text= base::strfmt(_("The file %s has an unsupported extension for this script editor."), path.c_str());
-    Utilities::show_message_and_remember(_("Unsupported File Format"), text, _("OK"), "", "",  "ShellWindowUnknownLanguageFile", "" );
+    if (Utilities::show_message_and_remember(_("Unsupported File Format"), text, _("OK"), _("Cancel"), "",  "ShellWindowUnknownLanguageFile", "") == mforms::ResultCancel)
+      return;
   }
   else if (language == "sql")
-    Utilities::show_message_and_remember(_("Unsupported Execution"), _("This script editor is meant for developing Workbench plugins and scripts. SQL scripts should be opened and executed in the SQL Editor."), _("OK"), "", "",  "ShellWindowSqlLanguageFile", "" );
+  {
+    if (Utilities::show_message_and_remember(_("Unsupported Execution"), _("This script editor is meant for developing Workbench plugins and scripts. SQL scripts should be opened and executed in the SQL Editor."), _("OK"), _("Cancel"), "",  "ShellWindowSqlLanguageFile", "") == mforms::ResultCancel)
+      return;
+  }
 
   GRTCodeEditor *editor= add_editor(is_script, language);
   if (!editor->load(path))
@@ -1185,7 +1233,7 @@ GRTCodeEditor *GRTShellWindow::show_file_at_line(const std::string &path, int li
     add_output(base::strfmt("Cannot open file %s", path.c_str()));
   else
   {
-    int start, length;
+    ssize_t start, length;
     editor->get_editor()->get_range_of_line(line, start, length);
     editor->get_editor()->set_selection(start, 0);
   }
@@ -1238,10 +1286,14 @@ void GRTShellWindow::set_editor_title(GRTCodeEditor *editor, const std::string &
  */
 bool GRTShellWindow::request_quit()
 {
-  for (std::vector<GRTCodeEditor*>::iterator editor = _editors.begin(); editor != _editors.end(); editor++)
+  std::vector<GRTCodeEditor*>::reverse_iterator editor;
+  while ((editor = _editors.rbegin()) != _editors.rend())
+  {
     if (!(*editor)->can_close())
       return false;
-
+    else
+      close_editor(*editor);
+  }
   return true;
 }
 
@@ -1255,7 +1307,7 @@ void GRTShellWindow::add_files_from_dir(mforms::TreeNodeRef parent, const std::s
     
   while (const gchar *name= g_dir_read_name(dir))
   {
-    if (g_str_has_suffix(name, ".py") || g_str_has_suffix(name, ".lua"))
+    if (g_str_has_suffix(name, ".py"))
     {
       mforms::TreeNodeRef node = parent->add_child();
       node->set_string(0, name);
@@ -1437,15 +1489,12 @@ bool GRTShellWindow::on_tab_closing(int index)
     return false;
 
   GRTCodeEditor* editor = _editors[index - EDITOR_TAB_OFFSET];
-  return editor->can_close();
-}
-
-//--------------------------------------------------------------------------------------------------
-
-void GRTShellWindow::on_tab_closed(int index)
-{
-  GRTCodeEditor* editor = _editors[index - EDITOR_TAB_OFFSET];
-  close_editor(editor);
+  if (editor->can_close())
+  {
+    close_editor(editor);
+    return true;
+  }
+  return false;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1455,7 +1504,7 @@ void GRTShellWindow::on_tab_changed()
   GRTCodeEditor *editor = get_active_editor();
   if (editor)
   {
-    bool exec_enabled = (editor->get_language() == "python" || editor->get_language() == "lua");
+    bool exec_enabled = (editor->get_language() == "python");
 
     _save_button->set_enabled(true);
     _save_as_button->set_enabled(true);
@@ -1510,7 +1559,7 @@ GRTCodeEditor* GRTShellWindow::get_editor_for(const std::string& path, bool sele
     if (g_utf8_collate(path1, path2) == 0)
     {
       if (select_tab)
-        _main_tab.set_active_tab(editor - _editors.begin() + EDITOR_TAB_OFFSET);
+        _main_tab.set_active_tab((int)(editor - _editors.begin() + EDITOR_TAB_OFFSET));
 
       g_free(path2);
       return *editor;
@@ -1988,16 +2037,15 @@ static void globals_rescan_list(mforms::TreeNodeRef &node,
                                 const std::string &path,
                                 const grt::BaseListRef &value)
 {
-  int c= value.count();
   char buffer[30];
   
   node->remove_children();
-  for (int i= 0; i < c; i++)
+  for (size_t i = 0; i < value.count(); i++)
   {
     grt::ValueRef v= value.get(i);
     std::string label;
     
-    sprintf(buffer, "%i", i);
+    sprintf(buffer, "%lu", (long unsigned)i);
     
     if (v.is_valid() && !grt::is_simple_type(v.type()))
     {
@@ -2209,7 +2257,7 @@ void GRTShellWindow::refresh_global_list()
   _global_list.clear();
   if (_inspector)
   {
-    for (int c = _inspector->count(), i = 0; i < c; i++)
+    for (size_t c = _inspector->count(), i = 0; i < c; i++)
     {
       mforms::TreeNodeRef node = _global_list.add_node();
       std::string value;
@@ -2250,3 +2298,41 @@ void GRTShellWindow::refresh_notifs_list()
   }
 }
 
+
+void GRTShellWindow::cut()
+{
+  GRTCodeEditor *editor = get_active_editor();
+  if (editor)
+    editor->get_editor()->cut();
+  else if (_shell_entry.has_focus())
+    _shell_entry.cut();
+}
+
+void GRTShellWindow::copy()
+{
+  GRTCodeEditor *editor = get_active_editor();
+  if (editor)
+    editor->get_editor()->copy();
+  else if (_shell_entry.has_focus())
+    _shell_entry.copy();
+}
+
+void GRTShellWindow::paste()
+{
+  GRTCodeEditor *editor = get_active_editor();
+  if (editor)
+    editor->get_editor()->paste();
+  else if (_shell_entry.has_focus())
+    _shell_entry.paste();
+}
+
+void GRTShellWindow::select_all()
+{
+  GRTCodeEditor *editor = get_active_editor();
+  if (editor)
+    editor->get_editor()->select_all();
+  else if (_shell_entry.has_focus())
+    _shell_entry.select(base::Range(0, (size_t)-1));
+}
+
+
