@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -283,12 +283,12 @@ HostAndRemoteTypePage::HostAndRemoteTypePage(WizardForm* host)
   _management_type_panel.add(&_management_type_box);
 
   _win_remote_admin.set_text(_("Native Windows remote management (only available on Windows)"));
-  scoped_connect(_win_remote_admin.signal_toggled(),boost::bind(&HostAndRemoteTypePage::toggle_remote_admin, this));
+  scoped_connect(_win_remote_admin.signal_clicked(),boost::bind(&HostAndRemoteTypePage::toggle_remote_admin, this));
 #ifndef _WIN32
   _win_remote_admin.set_enabled(false);
 #endif
   _ssh_remote_admin.set_text(_("SSH login based management"));
-  scoped_connect(_ssh_remote_admin.signal_toggled(),boost::bind(&HostAndRemoteTypePage::toggle_remote_admin, this));
+  scoped_connect(_ssh_remote_admin.signal_clicked(),boost::bind(&HostAndRemoteTypePage::toggle_remote_admin, this));
 
   _management_type_box.add(&_win_remote_admin, false, true);
   _management_type_box.add(&_ssh_remote_admin, false, true);
@@ -633,7 +633,7 @@ SSHConfigurationPage::SSHConfigurationPage(WizardForm* host)
 #else
     "~";
 #endif
-  _file_selector->initialize(homedir + "/.ssh/id_rsa", mforms::OpenFile, "", "...", true,
+  _file_selector->initialize(homedir + "/.ssh/id_rsa", mforms::OpenFile, "", true,
     boost::bind(&WizardPage::validate, this));
   use_ssh_key_changed();
   
@@ -776,7 +776,7 @@ WindowsManagementPage::WindowsManagementPage(WizardForm* host, wb::WBContext* co
 
   // Setup for configuration file browsing.
   _file_selector= mforms::manage(new FsObjectSelector(&_browse_button, &_config_path));
-  _file_selector->initialize("", mforms::OpenFile, "", "...", true, boost::bind(&WizardPage::validate, this));
+  _file_selector->initialize("", mforms::OpenFile, "", true, boost::bind(&WizardPage::validate, this));
 
   add(&_layout_table, false, true);
 }
@@ -1278,7 +1278,7 @@ PathsPage::PathsPage(WizardForm* host, wb::WBContext* context)
   
   // Setup for local config file browsing. This will be adjusted if we are at a remote location.
   _file_selector= mforms::manage(new FsObjectSelector(&_browse_button, &_config_path));
-  _file_selector->initialize("", mforms::OpenFile, "", "...", true, boost::bind(&WizardPage::validate, this));
+  _file_selector->initialize("", mforms::OpenFile, "", true, boost::bind(&WizardPage::validate, this));
 
   _test_config_path_button.set_text(_("Check Path"));
   scoped_connect(_test_config_path_button.signal_clicked(),boost::bind(&PathsPage::test_path, this));
