@@ -270,8 +270,6 @@ private:
   TextEntry _start_command;
   Label _stop_label;
   TextEntry _stop_command;
-  Label _status_label;
-  TextEntry _status_command;
   
   CheckBox _use_sudo;
 };
