@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,7 +16,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-#include "stdafx.h"
+
 #include "plugin_manager_window.h"
 #include "grt/icon_manager.h"
 #include "base/file_utilities.h"
@@ -176,7 +176,8 @@ void PluginManagerWindow::refresh_plugin_list()
         if (icon_path.empty() || !g_file_test(icon_path.c_str(), G_FILE_TEST_IS_REGULAR))
           icon_path = bec::IconManager::get_instance()->get_icon_path("MySQLPlugin-48.png");
         
-        node->set_icon_path(0, icon_path);
+          //disabled for now b/c TreeNodeView in windows doesn't scale down images to fit the row
+          //node->set_icon_path(0, icon_path);
         node->set_string(1, (*p)->caption());
         node->set_tag((*p)->name());
       }
