@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -229,20 +229,26 @@ public:
 
     mforms::TextEntry *entry = _owner->new_entry_option(option, false);
     entry->set_tooltip(tooltip);
+    entry->set_size(80, -1);      //  Set a default size. If the size is never set, the text entry will not show if the _help_column == true
     
 #ifdef _WIN32
     TableItemFlags descriptionFlags = mforms::HFillFlag;
     bool right_aligned = false;
 #else
-    TableItemFlags descriptionFlags = mforms::HFillFlag;
+    TableItemFlags descriptionFlags = mforms::HFillFlag|mforms::HExpandFlag;
     bool right_aligned = true;
 #endif
 
     mforms::Label* label = new_label(caption, right_aligned);
     _table.add(label, 0, 1, _rows-1, _rows, descriptionFlags);
 //    label->set_size(180, -1);
-    _table.add(entry, 1, 2, _rows-1, _rows, mforms::HFillFlag);
-
+    _table.add(entry, 1, 2, _rows-1, _rows, _help_column ? mforms::HFillFlag : mforms::HFillFlag|mforms::HExpandFlag);
+    if (_help_column)
+    {
+      label = new_label(tooltip);
+      label->set_style(mforms::SmallHelpTextStyle);
+      _table.add(label, 2, 3, _rows-1, _rows, mforms::HFillFlag|mforms::HExpandFlag);
+    }
     return entry;
   }
 
@@ -375,11 +381,7 @@ PreferencesForm::PreferencesForm(wb::WBContextUI *wbui, const workbench_physical
     // Fonts only for now in Mac/Linux
     add_page(NULL, _("Fonts"), create_fonts_and_colors_page());
 #endif
-    add_page(NULL, _("Advanced"), create_advanced_settings_page());
-#ifdef _WIN32
-    // right now, there's only a single option specific to windows
     add_page(NULL, _("Others"), create_others_page());
-#endif
   }
 
 
@@ -398,7 +400,7 @@ PreferencesForm::PreferencesForm(wb::WBContextUI *wbui, const workbench_physical
 
   _switcher.select_node(_switcher.node_at_row(0));
 
-  set_size(750, 600);
+  set_size(800, 600);
   center();
   
   show_values();
@@ -544,7 +546,7 @@ void PreferencesForm::update_path_option(const std::string &option_name, mforms:
 
 void PreferencesForm::update_entry_option_numeric(const std::string &option_name, mforms::TextEntry *entry, int minrange, int maxrange)
 {
-  long value= atoi(entry->get_string_value().c_str());
+  long value= base::atoi<long>(entry->get_string_value(), 0l);
   if (value < minrange)
     value= minrange;
   else if (value > maxrange)
@@ -560,7 +562,7 @@ void PreferencesForm::show_checkbox_option(const std::string &option_name, mform
 
   _wbui->get_wb_options_value(_model.is_valid() ? _model.id() : "", option_name, value);
 
-  checkbox->set_active(atoi(value.c_str()) != 0);
+  checkbox->set_active(base::atoi<int>(value, 0) != 0);
 }
 
 
@@ -794,7 +796,7 @@ mforms::View *PreferencesForm::create_sqlide_page()
                                                   _("A snapshot of all open scripts is saved when the SQL Editor is closed. Next time it is opened to the same connection that state is restored. Unsaved files will remain unsaved, but their contents will be preserved."));
 
       {
-        static const char *auto_save_intervals= "disable:0,10 seconds:10,15 seconds:15,30 seconds:30,1 minute:60,5 minutes:300,10 minutes:600,20 minutes:1200";
+        static const char *auto_save_intervals= "disable:0,5 seconds:5,10 seconds:10,15 seconds:15,30 seconds:30,1 minute:60,5 minutes:300,10 minutes:600,20 minutes:1200";
         mforms::Selector *sel = new_selector_option("workbench:AutoSaveSQLEditorInterval", auto_save_intervals, true);
 
         table->add_option(sel, _("Auto-save scripts interval:"),
@@ -865,6 +867,11 @@ mforms::View *PreferencesForm::create_sqlide_page()
                              _("DBMS connection read time out (in seconds):"),
                              _("Max time the a query can take to return data from the DBMS"));
     entry->set_size(80, -1);
+
+    entry = otable->add_entry_option("DbSqlEditor:ConnectionTimeOut",
+      _("DBMS connection time out (in seconds):"),
+      _("Maximum time to wait before a connection attempt is aborted."));
+    entry->set_size(80, -1);
     box->add(otable, false, true);
   }
 
@@ -882,7 +889,7 @@ mforms::View *PreferencesForm::create_sqlide_page()
 
     {
       otable->add_checkbox_option("DbSqlEditor:SafeUpdates",
-                                  _("\"Safe Updates\".\nForbid UPDATEs and DELETEs with no key in WHERE clause or no LIMIT clause.\nRequires a reconnection."),
+                                  _("\"Safe Updates\". Forbid UPDATEs and DELETEs with no key in WHERE clause or no LIMIT clause. Requires a reconnection."),
                                   _("Enables the SQL_SAFE_UPDATES option for the session.\n"
                                     "If enabled, MySQL aborts UPDATE or DELETE statements\n"
                                     "that do not use a key in the WHERE clause or a LIMIT clause.\n"
@@ -950,6 +957,17 @@ mforms::View *PreferencesForm::create_general_editor_page()
       tbox->add(entry, false, false);
     }
   }
+
+  {
+    OptionTable *table;
+
+    table = mforms::manage(new OptionTable(this, _("Indentation"), true));
+    box->add(table, false, true);
+    table->add_checkbox_option("Editor:TabIndentSpaces", _("Tab key inserts spaces instead of tabs"), "Check if you want the tab key to indent using\nthe configured amount of spaces.");
+
+    table->add_entry_option("Editor:IndentWidth", "Indent width:", "How many spaces to insert when indenting with the tab key.");
+    table->add_entry_option("Editor:TabWidth", "Tab width:", "How many spaces wide are tab characters.");
+  }
   return box;
 }
 
@@ -1004,7 +1022,7 @@ mforms::View *PreferencesForm::create_editor_page()
         // Set initial enabled state of sub settings depending on whether code completion is enabled.
         std::string value;
         _wbui->get_wb_options_value(_model.is_valid() ? _model.id() : "", "DbSqlEditor:CodeCompletionEnabled", value);
-        subsettings_box->set_enabled(atoi(value.c_str()) != 0);
+        subsettings_box->set_enabled(base::atoi<int>(value, 0) != 0);
 
         vbox->add(subsettings_box, false);
       }
@@ -1025,17 +1043,26 @@ mforms::View *PreferencesForm::create_editor_page()
     }
 
     {
-      mforms::Box *tbox= mforms::manage(new mforms::Box(true));
-      tbox->set_spacing(4);
-      vbox->add(tbox, false);
+      mforms::Table *table = mforms::manage(new mforms::Table());
+      table->set_row_count(2);
+      table->set_column_count(2);
+      table->set_column_spacing(4);
+      table->set_row_spacing(4);
+      vbox->add(table, false);
 
-      tbox->add(new_label(_("Max syntax error count:"), true), false, false);
-      mforms::TextEntry *entry= new_entry_option("SqlEditor::SyntaxCheck::MaxErrCount", false);
+      table->add(new_label(_("Max syntax error count:"), true), 0, 1, 0, 1, mforms::HFillFlag);
+      mforms::TextEntry *entry = new_entry_option("SqlEditor::SyntaxCheck::MaxErrCount", false);
       entry->set_size(50, -1);
       entry->set_tooltip(_("Maximum number of errors for syntax checking.\n"
                            "Syntax errors aren't highlighted beyond this threshold.\n"
                            "Set to 0 to show all errors."));
-      tbox->add(entry, false, false);
+      table->add(entry, 1, 2, 0, 1, mforms::HFillFlag);
+
+      table->add(new_label(_("Max number of result sets:"), true), 0, 1, 1, 2, mforms::HFillFlag);
+      entry = new_entry_option("DbSqlEditor::MaxResultsets", false);
+      entry->set_size(50, -1);
+      entry->set_tooltip(_("Maximum number of result sets that can be opened for a single editor."));
+      table->add(entry, 1, 2, 1, 2, mforms::HFillFlag);
     }
   }
 
@@ -1320,15 +1347,67 @@ mforms::View *PreferencesForm::create_model_page()
 
 mforms::View *PreferencesForm::create_others_page()
 {
-  OptionTable *table = mforms::manage(new OptionTable(this, _("Others"), true));
+
+  Box* content = manage(new Box(false));
+  content->set_spacing(8);
+
+  OptionTable *timeouts_table;
+
+  timeouts_table = mforms::manage(new OptionTable(this, _("Timeouts"), true));
+  content->add(timeouts_table, false, true);
   {
+
+    // SSH keepalive
+    {
+      mforms::TextEntry *entry = new_numeric_entry_option("sshkeepalive", 0, 500);
+      entry->set_max_length(5);
+      entry->set_size(50, -1);
+
+      entry->set_tooltip(_(
+        "The interval in seconds without sending any data over the connection, a \"keepalive\" packet will be sent.\nThis option will apply to both SSH tunnel connections and remote management via SSH."));
+
+      timeouts_table->add_option(entry, _("SSH KeepAlive:"),
+        _("SSH keep-alive interval in seconds.\nUse 0 to disable."));
+    }
+
+    // SSH timeout
+    {
+      mforms::TextEntry *entry = new_numeric_entry_option("sshtimeout", 0, 500);
+      entry->set_max_length(5);
+      entry->set_size(50, -1);
+      entry->set_tooltip(_(
+        "Determines how long the process waits for a result."));
+
+      timeouts_table->add_option(entry, _("SSH Timeout:"),
+        _("SSH timeout in seconds.\nUsed only in Online Backup/Restore"));
+    }
+
+    // Fabric timeout
+    {
+      // Using arbitrary max value of 1 Hour for connection timeout.
+      mforms::TextEntry *entry = new_numeric_entry_option("Fabric:ConnectionTimeOut", 0, 3600);
+      entry->set_max_length(5);
+      entry->set_size(50, -1);
+      entry->set_tooltip(_(
+        "The interval in seconds without sending any data over the connection, a \"keepalive\" packet will be sent.\nThis option will apply to both SSH tunnel connections and remote management via SSH."));
+
+      timeouts_table->add_option(entry, _("Fabric Connection Timeout:"),
+        _("Maximum time to wait before a connection\nattempt is aborted."));
+    }
+
+  }
+
 #ifdef _WIN32
+  OptionTable *table = mforms::manage(new OptionTable(this, _("Others"), true));
+  content->add(table, false, true);
+  {
     table->add_checkbox_option("DisableSingleInstance", _("Allow more than one instance of MySQL Workbench to run"), 
       _("By default only one instance of MySQL Workbench can run at the same time.\nThis is more resource friendly "
         "and necessary as multiple instances share the same files (settings etc.). Change at your own risk."));
-#endif
   }
-  return table;
+#endif
+
+  return content;
 }
 
 
@@ -1681,8 +1760,8 @@ mforms::View *PreferencesForm::create_diagram_page()
     check->set_text(_("Draw Line Crossings (slow in large diagrams)"));
     vbox->add(check, false);
 
-    check= new_checkbox_option("workbench.physical.Connection:HideCaptions");
-    check->set_text(_("Hide Captions"));
+    check= new_checkbox_option("workbench.physical.Connection:ShowCaptions");
+    check->set_text(_("Show Captions"));
     vbox->add(check, false);
 
     check= new_checkbox_option("workbench.physical.Connection:CenterCaptions");
@@ -1873,29 +1952,6 @@ mforms::View *PreferencesForm::create_fonts_and_colors_page()
   return content;
 }
 
-mforms::View *PreferencesForm::create_advanced_settings_page()
-{
-  Box* content = manage(new Box(false));
-  content->set_spacing(8);
-
-  OptionTable *table;
-
-  table = mforms::manage(new OptionTable(this, _("SSH"), true));
-  content->add(table, false, true);
-  {
-      mforms::TextEntry *entry= new_numeric_entry_option("sshkeepalive", 0, 500);
-      entry->set_max_length(5);
-      entry->set_size(50, -1);
-      entry->set_tooltip(_(
-      "The interval in seconds without sending any data over the connection, a \"keepalive\" packet will be sent.\nThis option will apply to both SSH tunnel connections and remote management via SSH."));
-
-      table->add_option(entry, _("SSH KeepAlive:"),
-                        _("SSH keep-alive interval in seconds.\nUse 0 to disable."));
-  }
-
-  return content;
-}
-
 
 static std::string separate_camel_word(const std::string &word)
 {
@@ -1982,9 +2038,9 @@ void PreferencesForm::toggle_use_global()
 
 //--------------------------------------------------------------------------------------------------
 
-static struct RegisterPrefsNotifDocs
+static struct RegisterNotifDocs_preferences_form
 {
-  RegisterPrefsNotifDocs()
+  RegisterNotifDocs_preferences_form()
   {
     base::NotificationCenter::get()->register_notification("GRNPreferencesDidCreate",
                                                            "preferences",
@@ -2014,6 +2070,6 @@ static struct RegisterPrefsNotifDocs
                                                            "model-options - the model specific options dictionary being changed\n"
                                                            "model-id - the object id of the model for which the options are being changed\n");
   }
-} initdocs;
+} initdocs_preferences_form;
 
 
