@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "base/log.h"
 
 #include "server_instance_editor.h"
@@ -120,8 +118,10 @@ ServerInstanceEditor::ServerInstanceEditor(bec::GRTManager *grtm, const db_mgmt_
 , _password_box(true)
 , _sys_box(false)
 , _details_panel(mforms::TitledBoxPanel)
+, _custom_sudo_box(true)
 , _connect_panel(new grtui::DbConnectPanel(grtui::DbConnectPanelHideConnectionName))
 , _bottom_hbox(true)
+, _remote_admin_box(false)
 {
   set_name("instance_editor");
   _mgmt= mgmt;
@@ -130,7 +130,7 @@ ServerInstanceEditor::ServerInstanceEditor(bec::GRTManager *grtm, const db_mgmt_
 
   {
     // do a quick validation, removing any instance objects that don't have a matching connection
-    for (int i = _instances.count()-1; i >= 0; --i)
+    for (ssize_t i = _instances.count() - 1; i >= 0; --i)
     {
       if (!_instances[i]->connection().is_valid())
       {
@@ -167,6 +167,7 @@ ServerInstanceEditor::ServerInstanceEditor(bec::GRTManager *grtm, const db_mgmt_
   _content_box.add(&_tabview, true, true);
   _top_hbox.add(&_content_box, true, true);
 
+  _connect_panel->set_driver_changed_cb(boost::bind(&ServerInstanceEditor::driver_changed_cb, this, _1));
   scoped_connect(_tabview.signal_tab_changed(), boost::bind(&ServerInstanceEditor::tab_changed, this));
   scoped_connect(_stored_connection_list.signal_changed(),boost::bind(&ServerInstanceEditor::show_connection, this));
   
@@ -180,24 +181,23 @@ ServerInstanceEditor::ServerInstanceEditor(bec::GRTManager *grtm, const db_mgmt_
 
   // Remote management
   {
-    Box* remote_admin_box = manage(new Box(false));                 // For the content.
-    remote_admin_box->set_padding(MF_PANEL_PADDING);
-    remote_admin_box->set_spacing(4);
+    _remote_admin_box.set_padding(MF_PANEL_PADDING);
+    _remote_admin_box.set_spacing(4);
     mforms::Table* remote_param_table = NewTable(6, 2);
     
     _no_remote_admin.set_text(_("Do not use remote management"));
-    scoped_connect(_no_remote_admin.signal_toggled(),boost::bind(&ServerInstanceEditor::toggle_administration, this));
+    scoped_connect(_no_remote_admin.signal_clicked(),boost::bind(&ServerInstanceEditor::toggle_administration, this));
     _win_remote_admin.set_text(_("Native Windows remote management (only available on Windows)"));
-    scoped_connect(_win_remote_admin.signal_toggled(),boost::bind(&ServerInstanceEditor::toggle_administration, this));
+    scoped_connect(_win_remote_admin.signal_clicked(),boost::bind(&ServerInstanceEditor::toggle_administration, this));
 #ifndef _WIN32
     _win_remote_admin.set_enabled(false);
 #endif
     _ssh_remote_admin.set_text(_("SSH login based management"));
-    scoped_connect(_ssh_remote_admin.signal_toggled(),boost::bind(&ServerInstanceEditor::toggle_administration, this));
+    scoped_connect(_ssh_remote_admin.signal_clicked(),boost::bind(&ServerInstanceEditor::toggle_administration, this));
 
-    remote_admin_box->add(&_no_remote_admin, false, true);
-    remote_admin_box->add(&_win_remote_admin, false, true);
-    remote_admin_box->add(&_ssh_remote_admin, false, true);
+    _remote_admin_box.add(&_no_remote_admin, false, true);
+    _remote_admin_box.add(&_win_remote_admin, false, true);
+    _remote_admin_box.add(&_ssh_remote_admin, false, true);
 
     _remote_param_box.set_spacing(12);
     _remote_param_box.add(manage(remote_param_table), true, true);
@@ -269,9 +269,7 @@ ServerInstanceEditor::ServerInstanceEditor(bec::GRTManager *grtm, const db_mgmt_
     scoped_connect(b->signal_clicked(),boost::bind(&ServerInstanceEditor::browse_file, this));
     remote_param_table->add(box, 1, 2, 4, 5, HFillFlag);
 
-    remote_admin_box->add(&_remote_param_box, true, true);
-
-    _tabview.add_page(remote_admin_box, _("Remote Management"));
+    _remote_admin_box.add(&_remote_param_box, true, true);
   }
   
   {
@@ -288,8 +286,6 @@ ServerInstanceEditor::ServerInstanceEditor(bec::GRTManager *grtm, const db_mgmt_
   }
   
   
-  _tabview.add_page(&_sys_box, _("System Profile"));
-
   // Sys
   {
     Label *label = manage(new Label(_("Information about the server and MySQL configuration, such as path to the configuration file,\n"
@@ -349,7 +345,7 @@ ServerInstanceEditor::ServerInstanceEditor(bec::GRTManager *grtm, const db_mgmt_
     
     _sys_box.add(&_details_panel, false, true);
     
-    table = NewTable(7, 2);
+    table = NewTable(3, 2);
     _details_panel.set_title(_("MySQL Management"));
 #if defined(_WIN32) || defined(__APPLE__)
     _details_panel.add(manage(table));
@@ -357,40 +353,38 @@ ServerInstanceEditor::ServerInstanceEditor(bec::GRTManager *grtm, const db_mgmt_
     // XXX tmp workaround for crash caused by changed destruction routine
     _details_panel.add(table);
 #endif
-
-    table->add(manage(RLabel(_("Check MySQL Status:"))), 0, 1, 0, 1, HFillFlag);
-    table->add(&_status_cmd, 1, 2, 0, 1, HFillFlag | HExpandFlag);
-    scoped_connect(_status_cmd.signal_changed(),boost::bind(&ServerInstanceEditor::entry_changed, this,
-                                                            &_status_cmd));
-
-
-    table->add(&_sudo_status_check, 1, 2, 1, 2, HFillFlag | HExpandFlag);
-#ifndef _WIN32
-    _sudo_status_check.set_text(_("Elevate privileges to execute status check command"));
-#else
-    _sudo_status_check.set_text(_("Acquire administrator rights to execute status check command"));
-#endif
-    scoped_connect(_sudo_status_check.signal_clicked(),boost::bind(&ServerInstanceEditor::check_changed, this,
-                                                                   &_sudo_status_check));
-
     
-    table->add(manage(RLabel(_("Start MySQL:"))), 0, 1, 2, 3, HFillFlag);
-    table->add(&_start_cmd, 1, 2, 2, 3, HFillFlag | HExpandFlag);
+    table->add(manage(RLabel(_("Start MySQL:"))), 0, 1, 0, 1, HFillFlag);
+    table->add(&_start_cmd, 1, 2, 0, 1, HFillFlag | HExpandFlag);
     scoped_connect(_start_cmd.signal_changed(),boost::bind(&ServerInstanceEditor::entry_changed, this,  
                                                   &_start_cmd));
-    table->add(manage(RLabel(_("Stop MySQL:"))), 0, 1, 3, 4, HFillFlag);
-    table->add(&_stop_cmd, 1, 2, 3, 4, HFillFlag | HExpandFlag);
+    table->add(manage(RLabel(_("Stop MySQL:"))), 0, 1, 1, 2, HFillFlag);
+    table->add(&_stop_cmd, 1, 2, 1, 2, HFillFlag | HExpandFlag);
     scoped_connect(_stop_cmd.signal_changed(),boost::bind(&ServerInstanceEditor::entry_changed, this,  
                                                   &_stop_cmd));
 
-    table->add(&_sudo_check, 1, 2, 4, 5, HFillFlag | HExpandFlag);
+    table->add(&_sudo_check, 1, 2, 2, 3, HFillFlag | HExpandFlag);
 #ifndef _WIN32
     _sudo_check.set_text(_("Elevate privileges to execute start/stop commands\nand write configuration data"));
 #else
     _sudo_check.set_text(_("Acquire administrator rights to execute start/stop commands\nand write configuration data"));
 #endif
-    scoped_connect(_sudo_check.signal_clicked(),boost::bind(&ServerInstanceEditor::check_changed, this, 
-                                                  &_sudo_check));
+
+    _sudo_description.set_text(_("When sudo is used in Linux like systems, a certain set of parameters are passed to it.\n"
+                                 "In certain environments, it may be necessary to override these parameters.\n"
+                                 "Look at the Workbench log file to determine the parameters being currently used. Leave it blank if unsure."));
+    _sudo_description.set_style(SmallHelpTextStyle);
+    _sys_box.add(&_sudo_description, false, true);
+    _custom_sudo_box.add(manage(RLabel(_("Override sudo command line:"))), false, true);
+    _custom_sudo_box.add(&_sudo_prefix, true, true);
+    
+    scoped_connect(_sudo_prefix.signal_changed(),boost::bind(&ServerInstanceEditor::entry_changed, this,
+                                                          &_sudo_prefix));
+    scoped_connect(_sudo_check.signal_clicked(),boost::bind(&ServerInstanceEditor::check_changed, this,
+                                                      &_sudo_check));
+    
+    _sys_box.add(&_custom_sudo_box, false, true);
+    
   }
   
 
@@ -413,11 +407,11 @@ ServerInstanceEditor::ServerInstanceEditor(bec::GRTManager *grtm, const db_mgmt_
   _bottom_hbox.add(&_move_up_button, false, true);
   _bottom_hbox.add(&_move_down_button, false, true);
   
-  _bottom_hbox.add_end(&_ok_button, false, true);
+  _bottom_hbox.add_end(&_close_button, false, true);
   _bottom_hbox.add_end(&_test_button, false, true);
   //  _bottom_hbox.add_end(&_cancel_button, false, true);
   
-  _ok_button.set_text(_("Close"));
+  _close_button.set_text(_("Close"));
 
   _test_button.set_text(_("Test Connection"));
 //  _test_button.set_enabled(_grtm->get_grt()->get_module("WbAdmin")!=0);
@@ -425,7 +419,7 @@ ServerInstanceEditor::ServerInstanceEditor(bec::GRTManager *grtm, const db_mgmt_
   
   _add_inst_button.enable_internal_padding(true);
   _del_inst_button.enable_internal_padding(true);
-  _ok_button.enable_internal_padding(true);
+  _close_button.enable_internal_padding(true);
   _test_button.enable_internal_padding(true);
 
   _stored_connection_list.set_size(180, -1);
@@ -435,7 +429,7 @@ ServerInstanceEditor::ServerInstanceEditor(bec::GRTManager *grtm, const db_mgmt_
   _stored_connection_list.add_column(::mforms::StringColumnType, _("MySQL Connections"), 150, false);
   _stored_connection_list.end_columns();
 
-  set_size(900, 750);
+  set_size(900, 600);
   center();
 
   _top_vbox.resume_layout();
@@ -550,18 +544,22 @@ db_mgmt_ServerInstanceRef ServerInstanceEditor::run(db_mgmt_ConnectionRef select
 
   int i = -1;
   if (select_connection.is_valid())
-    i = _connections.get_index(select_connection);
+    i = (int)_connections.get_index(select_connection);
 
-  if (i < 0 || i >= _stored_connection_list.count())
+  if (i >= _stored_connection_list.count())
       i = 0;
-  _stored_connection_list.select_node(_stored_connection_list.node_at_row(i));
-  show_connection();
+
+  if (i != -1)
+  {
+    _stored_connection_list.select_node(_stored_connection_list.node_at_row((int)i));
+    show_connection();
+  }
 
   if (show_admin)
     _tabview.set_active_tab(2);
 
   _top_vbox.resume_layout();
-  run_modal(&_ok_button, NULL);
+  run_modal(NULL, &_close_button);
 
   _grtm->get_grt()->call_module_function("Workbench", "saveConnections", grt::BaseListRef());
   _grtm->get_grt()->call_module_function("Workbench", "saveInstances", grt::BaseListRef());
@@ -624,19 +622,11 @@ void ServerInstanceEditor::system_type_changed()
   
   if (instance.is_valid())
   {
-    std::string system= _os_type.get_string_value();
+    std::string system = _os_type.get_string_value();
     if (!system.empty())
     {
       instance->serverInfo().gset("sys.system", system);
-
-      const bool is_not_windows = system.find("indows") == std::string::npos;
-      _sudo_status_check.set_enabled(is_not_windows);
-
-      if (!is_not_windows)
-        _sudo_status_check.set_active(false);
-
       refresh_profile_list();
-    
       profile_changed();
     }
   }
@@ -776,7 +766,16 @@ void ServerInstanceEditor::toggle_administration()
       std::string value = instance->connection()->parameterValues().get_string("sshHost");
       if (value.empty() || win_administration)
         value = instance->connection()->parameterValues().get_string("hostName");
-      _remote_host.set_value(value);
+      std::size_t char_pos = value.rfind(":");
+      if (std::string::npos != char_pos)
+      {
+        _remote_host.set_value(value.substr(0, char_pos));
+        char_pos++;
+        if (char_pos <= value.size())
+          _ssh_port.set_value(value.substr(char_pos, std::string::npos));
+      }
+      else
+        _remote_host.set_value(value);
     }
     if (ssh_administration)
     {
@@ -818,10 +817,25 @@ void ServerInstanceEditor::tab_changed()
     {
       grt::BaseListRef args(_grtm->get_grt());
       args.ginsert(connection);
-      if (is_local_connection(connection))
-        instance = db_mgmt_ServerInstanceRef::cast_from(_grtm->get_grt()->call_module_function("WbAdmin", "autoDetectLocalInstance", args));
-      else
-        instance = db_mgmt_ServerInstanceRef::cast_from(_grtm->get_grt()->call_module_function("WbAdmin", "autoDetectRemoteInstance", args));
+      try
+      {
+        if (is_local_connection(connection))
+          instance = db_mgmt_ServerInstanceRef::cast_from(_grtm->get_grt()->call_module_function("WbAdmin", "autoDetectLocalInstance", args));
+        else
+          instance = db_mgmt_ServerInstanceRef::cast_from(_grtm->get_grt()->call_module_function("WbAdmin", "autoDetectRemoteInstance", args));
+      }
+      catch (grt::module_error &exc)
+      {
+        log_error("Error calling autoDetectRemoteInstance: %s, %s\n", exc.what(), exc.inner.c_str());
+        mforms::Utilities::show_error("Auto Detect Server Configuration",
+                                      exc.inner, "OK");
+      }
+      catch (std::exception &exc)
+      {
+        log_error("Error calling autoDetectRemoteInstance: %s\n", exc.what());
+        mforms::Utilities::show_error("Auto Detect Server Configuration",
+                                      exc.what(), "OK");
+      }
     }
   }
 
@@ -829,6 +843,29 @@ void ServerInstanceEditor::tab_changed()
     show_instance_info(instance->connection(), instance);
 }
 //--------------------------------------------------------------------------------------------------
+void ServerInstanceEditor::driver_changed_cb(const db_mgmt_DriverRef &driver)
+{
+  db_mgmt_ConnectionRef connection(selected_connection());
+  bool is_managed = connection.is_valid() ? connection->parameterValues().has_key("fabric_managed") : false;
+
+  if (driver->name() == "MySQLFabric" || is_managed)
+  {
+    if (_tabview.get_page_index(&_remote_admin_box) != -1)
+      _tabview.remove_page(&_remote_admin_box);
+
+    if (_tabview.get_page_index(&_sys_box) != -1)
+      _tabview.remove_page(&_sys_box);
+  }
+  else
+  {
+    if (_tabview.get_page_index(&_remote_admin_box) == -1)
+      _tabview.add_page(&_remote_admin_box, _("Remote Management"));
+
+    if (_tabview.get_page_index(&_sys_box) == -1)
+      _tabview.add_page(&_sys_box, _("System Profile"));
+  }
+
+}
 
 void ServerInstanceEditor::add_instance()
 {
@@ -860,6 +897,11 @@ void ServerInstanceEditor::add_instance()
     connection->owner(_mgmt);
     connection->name(name);
     connection->driver(_connect_panel->selected_rdbms()->defaultDriver());
+    if (find_named_object_in_list(connection->driver()->parameters(), "useSSL").is_valid())
+    {
+      // prefer SSL if possible by default
+      connection->parameterValues().set("useSSL", grt::IntegerRef(1));
+    }
     _connections.insert(connection);
     _connect_panel->set_connection(connection);
   }
@@ -881,12 +923,10 @@ void ServerInstanceEditor::delete_instance()
     if (row >= 0 && row < (int)_connections.count())
     {
       db_mgmt_ConnectionRef conn(_connections[row]);
-      for (int i = _instances.count()-1; i >= 0; --i)
+      for (ssize_t i = _instances.count() - 1; i >= 0; --i)
       {
         if (_instances[i]->connection() == conn)
-        {
           _instances.remove(i);
-        }
       }
 
       _connections.remove(row);
@@ -921,6 +961,19 @@ void ServerInstanceEditor::duplicate_instance()
   grt::merge_contents(copy_conn->parameterValues(), orig_conn->parameterValues(), true);
   copy_conn->hostIdentifier(orig_conn->hostIdentifier());
 
+  // Deletes the fabric parameters if a fabric managed connection
+  // is being duplicated
+  if (copy_conn->parameterValues().has_key("fabric_managed"))
+  {
+    std::vector<std::string> params = copy_conn->parameterValues().keys();
+    for (size_t index = 0; index < params.size(); index++)
+    {
+      std::string parameter = params[index];
+      if (base::starts_with(parameter, "fabric_"))
+        copy_conn->parameterValues().remove(parameter);
+    }
+  }
+
   if (orig_inst.is_valid())
   {
     copy_inst->owner(orig_inst->owner());
@@ -1012,7 +1065,7 @@ grt::DictRef ServerInstanceEditor::get_preset(const std::string& system, const s
 
 void ServerInstanceEditor::entry_changed(mforms::TextEntry *sender)
 {
-  const std::string value= sender->get_string_value();
+  const std::string value = base::trim(sender->get_string_value());
   db_mgmt_ConnectionRef connection(selected_connection());
   db_mgmt_ServerInstanceRef instance(selected_instance());
 
@@ -1086,9 +1139,9 @@ void ServerInstanceEditor::entry_changed(mforms::TextEntry *sender)
       _sys_profile_type.set_selected(_sys_profile_type.get_item_count()-1);
       instance->serverInfo().gset("sys.preset", "");
     }
-    else if (&_status_cmd == sender)
+    else if (&_sudo_prefix == sender)
     {
-      instance->serverInfo().gset("sys.mysqld.status", value);
+      instance->serverInfo().gset("sys.mysqld.sudo_override", value);
       _sys_profile_type.set_selected(_sys_profile_type.get_item_count()-1);
       instance->serverInfo().gset("sys.preset", "");
     }
@@ -1108,9 +1161,14 @@ void ServerInstanceEditor::check_changed(mforms::CheckBox *sender)
     if (&_ssh_usekey == sender)
       instance->loginInfo().gset("ssh.useKey", value ? 1 : 0);
     else if (&_sudo_check == sender)
+    {
+      if (_os_type.get_string_value() != "Windows")
+      {
+        _sudo_description.show(value);
+        _custom_sudo_box.show(value);
+      }
       info.gset("sys.usesudo", value ? 1 : 0);
-    else if (&_sudo_status_check == sender)
-      info.gset("sys.usesudostatus", value ? 1 : 0);
+    }
   }
 }
 
@@ -1226,27 +1284,36 @@ void ServerInstanceEditor::show_connection()
 
   _connect_panel->set_active_stored_conn(connection);
 
-  if (connection.is_valid())
-  {
-    _content_box.set_enabled(true);
-    _move_up_button.set_enabled(true);
-    _move_down_button.set_enabled(true);
-    _del_inst_button.set_enabled(true);
-    _dup_inst_button.set_enabled(true);
+  bool valid_connection = connection.is_valid();
+  bool is_managed = valid_connection ? connection->parameterValues().has_key("fabric_managed") : false;
 
-    _name_entry.set_value(connection->name());
+  // On Fabric Connections and Fabric Managed Connections the Remote Management 
+  // and System Profile tabs are hidden
+  if (valid_connection && (connection->driver()->name() == "MySQLFabric" || is_managed))
+  {
+    if (_tabview.get_page_index(&_remote_admin_box) != -1)
+      _tabview.remove_page(&_remote_admin_box);
+
+    if (_tabview.get_page_index(&_sys_box) != -1)
+      _tabview.remove_page(&_sys_box);
   }
   else
   {
-    _content_box.set_enabled(false);
-    _move_up_button.set_enabled(false);
-    _move_down_button.set_enabled(false);
-    _del_inst_button.set_enabled(false);
-    _dup_inst_button.set_enabled(false);
+    if (_tabview.get_page_index(&_remote_admin_box) == -1)
+      _tabview.add_page(&_remote_admin_box, _("Remote Management"));
 
-    _name_entry.set_value("");
+    if (_tabview.get_page_index(&_sys_box) == -1)
+      _tabview.add_page(&_sys_box, _("System Profile"));
   }
 
+  _content_box.set_enabled(valid_connection);
+  _move_up_button.set_enabled(valid_connection && !is_managed);
+  _move_down_button.set_enabled(valid_connection && !is_managed);
+  _del_inst_button.set_enabled(valid_connection && !is_managed);
+  _dup_inst_button.set_enabled(valid_connection);
+
+  _name_entry.set_value(valid_connection ? connection->name() : "");
+
   show_instance_info(connection, instance);
 }
 
@@ -1346,6 +1413,10 @@ void ServerInstanceEditor::show_instance_info(db_mgmt_ConnectionRef connection,
     _sys_win_service_name_label->show(false);
     _sys_win_hint_label.show(false);
   }
+  
+  
+  
+
 
   // If the MySQL connection is to a local server then remote administration makes no sense.
   // Disable it in this case.
@@ -1366,17 +1437,28 @@ void ServerInstanceEditor::show_instance_info(db_mgmt_ConnectionRef connection,
   }
   toggle_administration();
 
+  // Disables the config path controls if locked: means it was found automatically
+  // i.e. on the linux command call for ths server or on the windows service parameter
+  bool enable_config_path = serverInfo.get_int("sys.config.path.lock", 0) != 1;
   _sys_config_path.set_value(serverInfo.get_string("sys.config.path"));
+  _sys_config_path.set_enabled(enable_config_path);
+  _sys_config_path_browse.set_enabled(enable_config_path);
+  
   _sys_myini_section.set_value(serverInfo.get_string("sys.config.section"));
   if (system == "Windows")
     _sys_win_service_name.set_value(serverInfo.get_string("sys.mysqld.service_name"));
 
   _start_cmd.set_value(serverInfo.get_string("sys.mysqld.start"));
   _stop_cmd.set_value(serverInfo.get_string("sys.mysqld.stop"));
-  _status_cmd.set_value(serverInfo.get_string("sys.mysqld.status"));
+  _sudo_prefix.set_value(serverInfo.get_string("sys.mysqld.sudo_override"));
 
   _sudo_check.set_active(serverInfo.get_int("sys.usesudo", 1) != 0);
-  _sudo_status_check.set_active(serverInfo.get_int("sys.usesudostatus", 0) != 0);
+
+
+  bool show_custom_sudo = system != "Windows" && _sudo_check.get_active();
+
+  _sudo_description.show(show_custom_sudo);
+  _custom_sudo_box.show(show_custom_sudo);
 
   if (serverInfo.has_key("sys.sudo"))
   {
