@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2013 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2014 Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,7 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-#include "stdafx.h"
 
 #include "snippet_list.h"
 #include "snippet_popover.h"
@@ -71,7 +70,7 @@ static int image_height(cairo_surface_t* image)
 
 //----------------- Snippet ------------------------------------------------------------------------
 
-class Snippet
+class Snippet: public mforms::Accessible
 {
 private:
   cairo_surface_t* _icon;
@@ -216,11 +215,14 @@ public:
       cairo_stroke(cr);
     }
 
-    cairo_set_source_surface(cr, _icon, bounds.left() + SNIPPET_PADDING, bounds.top() + (int) ceil((bounds.height() - icon_height) / 2));
-    if (_enabled)
-      cairo_paint(cr);
-    else
-      cairo_paint_with_alpha(cr, 0.25);
+    if (_icon)
+    {
+      cairo_set_source_surface(cr, _icon, bounds.left() + SNIPPET_PADDING, bounds.top() + (int) ceil((bounds.height() - icon_height) / 2));
+      if (_enabled)
+        cairo_paint(cr);
+      else
+        cairo_paint_with_alpha(cr, 0.25);
+    }
 
     int text_offset= (int) (bounds.height() - _text_height) / 2;
     cairo_select_font_face(cr, SNIPPET_NORMAL_FONT, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD);
@@ -324,6 +326,11 @@ public:
 
   //------------------------------------------------------------------------------------------------
 
+  // ------ Accesibility Methods -----
+  virtual std::string get_acc_name() { return _title; }
+  virtual mforms::Accessible::Role get_acc_role() { return mforms::Accessible::ListItem;}
+  virtual base::Rect get_acc_bounds() { return _bounds;}
+
   // End of internal class Snippets.
 };
 
@@ -334,7 +341,7 @@ int BaseSnippetList::find_selected_index()
 {
   // Rather unlikely the selected link is not in the vector (since we got it from there) but...
   std::vector<Snippet*>::iterator location = std::find(_snippets.begin(), _snippets.end(), _selected_snippet);
-  return (location == _snippets.end()) ? -1 : location - _snippets.begin();
+  return (location == _snippets.end()) ? -1 : (int)(location - _snippets.begin());
 }
 
 //------------------------------------------------------------------------------------------------
@@ -386,14 +393,16 @@ void BaseSnippetList::clear()
 void BaseSnippetList::refresh_snippets()
 {
   clear();
-  for (int i = 0; i < _model->count(); i++)
+  for (size_t i = 0; i < _model->count(); i++)
   {
     std::string caption;
     _model->get_field(bec::NodeId(i), 0, caption);
     std::string description;
-    _model->get_field(bec::NodeId(i), 1, description);
+    bool skip_image = false;
+    if (!_model->get_field(bec::NodeId(i), 1, description))
+      skip_image = true;
 
-    Snippet* snippet = new Snippet(_image, caption, description, true);
+    Snippet* snippet = new Snippet(skip_image ? NULL : _image, caption, description, true);
     _snippets.push_back(snippet);
   }
   set_layout_dirty(true);
@@ -434,7 +443,7 @@ void BaseSnippetList::layout()
     _layout_width = _left_spacing + _right_spacing;
 
     if (_snippets.size() > 0)
-      _layout_height += _snippets.size() * SNIPPET_HEIGHT + (_snippets.size() - 1) * SNIPPET_SPACING;
+      _layout_height += (int)_snippets.size() * SNIPPET_HEIGHT + ((int)_snippets.size() - 1) * SNIPPET_SPACING;
 
     if (_image != NULL)
     {
@@ -467,19 +476,27 @@ Snippet* BaseSnippetList::selected()
 
 //------------------------------------------------------------------------------------------------
 
-void BaseSnippetList::mouse_leave()
+bool BaseSnippetList::mouse_leave()
 {
+  if (DrawBox::mouse_leave())
+    return true;
+
   if (_hot_snippet != NULL)
   {
     _hot_snippet = NULL;
     set_needs_repaint();
+    return true;
   }
+  return false;
 }
 
 //------------------------------------------------------------------------------------------------
 
-void BaseSnippetList::mouse_move(int x, int y)
+bool BaseSnippetList::mouse_move(mforms::MouseButton button, int x, int y)
 {
+  if (DrawBox::mouse_move(button, x, y))
+    return true;
+
   if (_single_click)
   {
     Snippet* snippet = snippet_from_point(x, y);
@@ -487,50 +504,67 @@ void BaseSnippetList::mouse_move(int x, int y)
     {
       _hot_snippet = snippet;
       set_needs_repaint();
+      return true;
     }
   }
+  return false;
 }
 
 //------------------------------------------------------------------------------------------------
 
-void BaseSnippetList::mouse_down(int button, int x, int y)
+bool BaseSnippetList::mouse_down(mforms::MouseButton button, int x, int y)
 {
+  if (DrawBox::mouse_down(button, x, y))
+    return true;
+
   if (button == MouseButtonLeft || button == MouseButtonRight)
   {
     Snippet* snippet= snippet_from_point(x, y);
     set_selected(snippet);
+    return true;
   }
+  return false;
 }
 
 //------------------------------------------------------------------------------------------------
 
-void BaseSnippetList::mouse_double_click(int button, int x, int y)
+bool BaseSnippetList::mouse_double_click(mforms::MouseButton button, int x, int y)
 {
-  switch (button)
+  if (DrawBox::mouse_double_click(button, x, y))
+    return true;
+
+  bool result = false;
+  if (button == MouseButtonRight)
   {
-    case MouseButtonRight:
+    // If this double click results from a quick !right + right click then handle it as single right click.
+    if (_last_mouse_button != MouseButtonRight)
     {
-      // If this double click results from a quick !right + right click then handle it as single right click.
-      if (_last_mouse_button != MouseButtonRight)
-        _context_menu->popup_at(this, x + 3, y + 3);
+      _context_menu->popup_at(this, x + 3, y + 3);
+      result = true;
     }
   }
   _last_mouse_button = MouseButtonNone;
+
+  return result;
 }
 
 //------------------------------------------------------------------------------------------------
 
-void BaseSnippetList::mouse_click(int button, int x, int y)
+bool BaseSnippetList::mouse_click(mforms::MouseButton button, int x, int y)
 {
+  if (DrawBox::mouse_click(button, x, y))
+    return true;
+
   // Keep the last pressed button. A quick series of two different button clicks might be interpreted
   // as a double click of the second button. So we need to handle that.
-  _last_mouse_button = (MouseButton)button;
-  switch (button)
+  _last_mouse_button = button;
+  if (button== MouseButtonRight)
   {
-    case MouseButtonRight:
       _context_menu->popup_at(this, x + 3, y + 3);
-      break;
+      return true;
   }
+
+  return false;
 }
 
 //------------------------------------------------------------------------------------------------
@@ -598,3 +632,27 @@ base::Rect BaseSnippetList::snippet_bounds(Snippet *snippet)
 {
   return snippet->bounds();
 }
+
+//------------------------------------------------------------------------------------------------
+int BaseSnippetList::get_acc_child_count()
+{ 
+  return (int)_snippets.size();
+}
+
+//------------------------------------------------------------------------------------------------
+Accessible* BaseSnippetList::get_acc_child(int index)
+{ 
+  mforms::Accessible* accessible = NULL;
+
+  if ((size_t)index < _snippets.size())
+    accessible = _snippets[index];
+
+  return accessible;
+}
+
+
+//------------------------------------------------------------------------------------------------
+mforms::Accessible* BaseSnippetList::hit_test(int x, int y)
+{ 
+  return snippet_from_point(x, y);
+}
