@@ -46,7 +46,7 @@ void FormViewBase::add_plugin_tab(PluginEditorBase *plugin)
   {
     ActiveLabel* label = Gtk::manage(new ActiveLabel(plugin->get_title(), sigc::hide_return(sigc::bind(sigc::mem_fun(this, &FormViewBase::close_plugin_tab), plugin))));
 
-    _editor_note->append_page(*plugin, *label);
+    int page_num = _editor_note->append_page(*plugin, *label);
 
     plugin->signal_title_changed().connect(sigc::mem_fun(label, &ActiveLabel::set_text));
 
@@ -57,6 +57,9 @@ void FormViewBase::add_plugin_tab(PluginEditorBase *plugin)
     }
 
     plugin_tab_added(plugin);
+
+    Glib::signal_idle().connect_once(sigc::bind(sigc::mem_fun(_editor_note, &Gtk::Notebook::set_current_page), page_num));
+
   }
   else
     g_warning("active form doesn't support editor tabs");
@@ -68,9 +71,6 @@ void FormViewBase::remove_plugin_tab(PluginEditorBase *plugin)
 {
   if (_editor_note)
   {
-    ActiveLabel* const al = dynamic_cast<ActiveLabel*>(_editor_note->get_tab_label(*plugin));
-    if (al)
-      al->set_menu(NULL);
     _editor_note->remove_page(*plugin);
 
     if (_editor_note->get_n_pages() == 0)
@@ -143,7 +143,10 @@ void FormViewBase::restore_sidebar_layout()
     int w = _grtm->get_app_option_int(_panel_savename+":SidebarWidth", 200);
     _sidebar1_pane->set_position(w);
     if (_grtm->get_app_option_int(_panel_savename+":SidebarHidden", 0))
+    {
+      _toolbar->set_item_checked("wb.toggleSidebar", false);
       _sidebar1_pane->get_child1()->hide();
+    }
     else
       _toolbar->set_item_checked("wb.toggleSidebar", true);
 
@@ -155,7 +158,10 @@ void FormViewBase::restore_sidebar_layout()
     int w = _grtm->get_app_option_int(_panel_savename+":SecondarySidebarWidth", 200);
     _sidebar2_pane->set_position(_sidebar2_pane->get_width()-w);
     if (_grtm->get_app_option_int(_panel_savename+":SecondarySidebarHidden", 0))
+    {
+      _toolbar->set_item_checked("wb.toggleSecondarySidebar", false);
       _sidebar2_pane->get_child2()->hide();
+    }
     else
       _toolbar->set_item_checked("wb.toggleSecondarySidebar", true);     
 
