@@ -1,18 +1,36 @@
+/*
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
+
+#include "mforms/../gtk/lf_view.h"
 #include <gtk/gtkhpaned.h>
 
 #include "model_diagram_panel.h"
 #include "image_cache.h"
 #include "grtdb/db_object_helpers.h"
+#include "model/wb_context_model.h"
 #include "workbench/wb_context_ui.h"
+#include "workbench/wb_context.h"
 #include "mforms/../gtk/lf_toolbar.h"
 #include "mforms/../gtk/lf_menubar.h"
 
-#include "user_types_box.h"
-#include "catalog_tree.h"
-#include "layer_tree.h"
 #include "properties_tree.h"
 #include "navigator_box.h"
-#include "history_tree.h"
 #include "documentation_box.h"
 #include "base/string_utilities.h"
 
@@ -163,6 +181,16 @@ class CanvasViewer : public mdc::GtkCanvas
     return true;
   }
 
+  virtual void on_zoom_in_event()
+  {
+    _be->zoom_in();
+  }
+
+  virtual void on_zoom_out_event()
+  {
+    _be->zoom_out();
+  }
+
 public:
   CanvasViewer(wb::ModelDiagramForm *be, bool use_gl)
     : mdc::GtkCanvas(use_gl ? mdc::GtkCanvas::OpenGLCanvasType : mdc::GtkCanvas::BufferedXlibCanvasType), _be(be)
@@ -224,28 +252,36 @@ ModelDiagramPanel *ModelDiagramPanel::create(wb::WBContextUI *wb)
   return panel;
 }
 
-
-static void fix_size(Gtk::Paned *paned)
+void ModelDiagramPanel::on_activate()
 {
-  int pos = paned->get_position();
-  paned->set_position(paned->get_parent()->get_width() - paned->get_position());
+  mforms::View *sidebar = _wb->get_wb()->get_model_context()->shared_secondary_sidebar();
+  Gtk::Widget *w = mforms::widget_for_view(sidebar);
+  Gtk::Frame *secondary_sidebar;
+  _xml->get_widget("sidebar_frame", secondary_sidebar);
+  if (w->get_parent())
+    gtk_reparent_realized(w,secondary_sidebar);
+  else
+    secondary_sidebar->add(*w);
+  w->show();
+
 }
 
-
 ModelDiagramPanel::ModelDiagramPanel(GtkHPaned *paned, Glib::RefPtr<Gtk::Builder> builder)
-  : Gtk::HPaned(paned)
+  : Gtk::HPaned(paned), FormViewBase("ModelDiagram")
   , _top_box(false, 0)
   , _wb(0)
   , _be(0)
   , _canvas(0)
   , _cursor(0)
   , _inline_editor(this)
+  , _xml(builder)
 {
 }
 
 void ModelDiagramPanel::post_construct(wb::WBContextUI *wb, Glib::RefPtr<Gtk::Builder> xml)
 {
   _wb = wb;
+  _grtm = wb->get_wb()->get_grt_manager();
   _diagram_hbox= 0;
   xml->get_widget("diagram_hbox", _diagram_hbox);
   //_top_box = new Gtk::VBox(false, 0);
@@ -254,11 +290,8 @@ void ModelDiagramPanel::post_construct(wb::WBContextUI *wb, Glib::RefPtr<Gtk::Bu
   _top_box.pack_end(*this, true, true);
   show();
 
-  if (_wb->get_wb()->get_wb_options().get_int("Sidebar:RightAligned", 0))
-  {
-    swap_panned_children(this, true);
-    signal_realize().connect(sigc::bind(sigc::ptr_fun(fix_size), this));
-  }
+  xml->get_widget("diagram_pane", _sidebar1_pane);
+  xml->get_widget("diagram_pane2", _sidebar2_pane);
 
   //xml->get_widget("tools_toolbar", _tools_toolbar);
   xml->get_widget("diagram_box", _vbox);
@@ -268,79 +301,93 @@ void ModelDiagramPanel::post_construct(wb::WBContextUI *wb, Glib::RefPtr<Gtk::Bu
   
   xml->get_widget("model_sidebar", _sidebar);
 
-  Gtk::Notebook *note;
-  Gtk::Label *label;
-
-  // setup sidebar stuff for model
-  xml->get_widget("side_model_note0", note);
-  _navigator_box= Gtk::manage(new NavigatorBox());
-  label = Gtk::manage(new Gtk::Label(_("<small>Navigator</small>")));
-  note->append_page(*_navigator_box, *label);
-  label->set_use_markup(true);
-
-
-  xml->get_widget("side_model_note1", note);
-  _catalog_tree= Gtk::manage(new CatalogTree(wb));
-  label = Gtk::manage(new Gtk::Label(_("<small>Catalog</small>")));
-  note->append_page(*_catalog_tree, *label);
-  label->set_use_markup(true);
-
-  _layer_tree= Gtk::manage(new LayerTree(wb));
-  label = Gtk::manage(new Gtk::Label(_("<small>Layers</small>")));
-  note->append_page(*_layer_tree, *label);
-  label->set_use_markup(true);
-
-  _usertypes_box= Gtk::manage(new UserTypesBox(wb));
-  label = Gtk::manage(new Gtk::Label(_("<small>User Types</small>")));
-  note->append_page(*_usertypes_box, *label);
-  label->set_use_markup(true);
-
-  xml->get_widget("side_model_note2", note);
-  _documentation_box= Gtk::manage(new DocumentationBox(wb));
-  label = Gtk::manage(new Gtk::Label(_("<small>Description</small>")));
-  note->append_page(*_documentation_box, *label);
-  label->set_use_markup(true);
-
-  _properties_tree= Gtk::manage(new PropertiesTree(wb));
-  label = Gtk::manage(new Gtk::Label(_("<small>Properties</small>")));
-  note->append_page(*_properties_tree, *label);
-  label->set_use_markup(true);
-
-  _history_tree= Gtk::manage(new HistoryTree(wb));
-  label = Gtk::manage(new Gtk::Label(_("<small>History</small>")));
-  note->append_page(*_history_tree, *label);
-  label->set_use_markup(true);
 }
 
 
 
 void ModelDiagramPanel::view_realized()
-{    
+{
   _canvas->get_canvas()->set_user_data(dynamic_cast<FormViewBase*>(this));
 
   // changing zoom or scrolling should cancel any editing in place
   _be->scoped_connect(_canvas->get_canvas()->signal_viewport_changed(),sigc::mem_fun(_be, &wb::ModelDiagramForm::stop_editing));
 }
 
-
 void ModelDiagramPanel::init(const std::string &view_id)
 {
   _be= _wb->get_wb()->get_model_context()->get_diagram_form_for_diagram_id(view_id);
   _be->set_frontend_data(dynamic_cast<FormViewBase*>(this));
   _canvas= Gtk::manage(new CanvasViewer(_be, false));//_wb->get_wb()->using_opengl()));
 
+  _toolbar = _be->get_toolbar();
+
+  {
+    Gtk::Notebook *note;
+    Gtk::Label *label;
+    Gtk::Paned *pan;
+
+    _xml->get_widget("model_sidebar", pan);
+    PanedConstrainer::make_constrainer(pan, 64, 90);
+
+    _xml->get_widget("side_model_pane2", pan);
+    PanedConstrainer::make_constrainer(pan, 64, 90);
+
+    _xml->get_widget("diagram_pane", pan);
+    PanedConstrainer::make_constrainer(pan, 130, 0);
+
+    // setup sidebar stuff for model
+    _xml->get_widget("side_model_note0", note);
+    _navigator_box= Gtk::manage(new NavigatorBox());
+    label = Gtk::manage(new Gtk::Label(_("<small>Navigator</small>")));
+    note->append_page(*_navigator_box, *label);
+    label->set_use_markup(true);
+
+
+    _xml->get_widget("side_model_note1", note);
+
+    _catalog_tree = _be->get_catalog_tree();
+    label = Gtk::manage(new Gtk::Label(_("<small>Catalog</small>")));
+    note->append_page(*mforms::widget_for_view(_catalog_tree), *label);
+    label->set_use_markup(true);
+
+    label = Gtk::manage(new Gtk::Label(_("<small>Layers</small>")));
+    note->append_page(*mforms::widget_for_view(_be->get_layer_tree()), *label);
+    label->set_use_markup(true);
+
+    _usertypes_list= _wb->get_wb()->get_model_context()->create_user_type_list();
+    label = Gtk::manage(new Gtk::Label(_("<small>User Types</small>")));
+    note->append_page(*mforms::widget_for_view(_usertypes_list), *label);
+    label->set_use_markup(true);
+
+    _xml->get_widget("side_model_note2", note);
+    _documentation_box= Gtk::manage(new DocumentationBox(_wb));
+    label = Gtk::manage(new Gtk::Label(_("<small>Description</small>")));
+    note->append_page(*_documentation_box, *label);
+    label->set_use_markup(true);
+
+    _properties_tree= Gtk::manage(new PropertiesTree(_wb));
+    label = Gtk::manage(new Gtk::Label(_("<small>Properties</small>")));
+    note->append_page(*_properties_tree, *label);
+    label->set_use_markup(true);
+
+    _history_list = _wb->get_wb()->get_model_context()->create_history_tree();
+    label = Gtk::manage(new Gtk::Label(_("<small>History</small>")));
+    note->append_page(*mforms::widget_for_view(_history_list), *label);
+    label->set_use_markup(true);
+  }
+
   {
     mforms::MenuBar *menubar = _be->get_menubar();
     if (menubar)
     {
-      Gtk::Widget *w = mforms::gtk::widget_for_menubar(menubar);
+      Gtk::Widget *w = mforms::widget_for_menubar(menubar);
       _top_box.pack_start(*w, false, true);
       w->show();
     }
     mforms::ToolBar *toolbar = _be->get_toolbar();
     if (toolbar)
     {
-      Gtk::Widget *w = mforms::gtk::widget_for_toolbar(toolbar);
+      Gtk::Widget *w = mforms::widget_for_toolbar(toolbar);
       _top_box.pack_start(*w, false, true);
       w->show();
       _top_box.pack_start(*(w=Gtk::manage(new Gtk::HSeparator)), false, true);
@@ -351,7 +398,7 @@ void ModelDiagramPanel::init(const std::string &view_id)
     log_debug3("ModelDiagramPanel::init got tools toolbar %p\n", tools_toolbar);
     if (tools_toolbar)
     {
-      _tools_toolbar = dynamic_cast<Gtk::VBox*>(mforms::gtk::widget_for_toolbar(tools_toolbar));
+      _tools_toolbar = dynamic_cast<Gtk::VBox*>(mforms::widget_for_toolbar(tools_toolbar));
       log_debug3("ModelDiagramPanel::init cast _tools_toolbar = %p\n", _tools_toolbar);
       if (_tools_toolbar)
       {
@@ -364,7 +411,7 @@ void ModelDiagramPanel::init(const std::string &view_id)
     mforms::ToolBar *options_toolbar = _be->get_options_toolbar();
     if (toolbar)
     {
-      Gtk::Widget *w = mforms::gtk::widget_for_toolbar(options_toolbar);
+      Gtk::Widget *w = mforms::widget_for_toolbar(options_toolbar);
       _vbox->pack_start(*w, false, true);
       w->show();
     }
@@ -382,45 +429,56 @@ void ModelDiagramPanel::init(const std::string &view_id)
     targets.push_back(Gtk::TargetEntry(*iter, Gtk::TARGET_SAME_APP));
   _canvas->drag_dest_set(targets);
 
+  _canvas->signal_drag_motion().connect(sigc::mem_fun(this, &ModelDiagramPanel::drag_motion));
   _canvas->signal_drag_drop().connect(sigc::mem_fun(this, &ModelDiagramPanel::drag_drop));
   _canvas->signal_drag_data_received().connect(sigc::mem_fun(this, &ModelDiagramPanel::drag_data_received));
 
-  _canvas->signal_realize().connect(sigc::mem_fun(this, &ModelDiagramPanel::view_realized));
 
-  _layer_tree->set_model(_be);
-  
+  _canvas->signal_realize().connect_notify(sigc::mem_fun(this, &ModelDiagramPanel::view_realized));
+
   show_all();
 
   get_panel()->signal_show().connect_notify((sigc::bind(sigc::mem_fun(_be, &wb::ModelDiagramForm::set_closed), false)));
 
-  _catalog_tree->refresh();
-  _layer_tree->refresh();
-}
+//  _catalog_tree->refresh();
 
+  // restore widths of sidebars when shown
+  _sig_restore_sidebar = Glib::signal_idle().connect(sigc::bind_return(sigc::mem_fun(this, &FormViewBase::restore_sidebar_layout), false));
+  
+  //  Set the sidebar pane sizes
+  Gtk::VPaned *pane = NULL;
+  _xml->get_widget("model_sidebar", pane);
+  pane->set_position(_wb->get_wb()->get_grt_manager()->get_app_option_int("Sidebar:VBox1:Position", pane->get_position()));
+
+  _xml->get_widget("side_model_pane2", pane);
+  pane->set_position(_wb->get_wb()->get_grt_manager()->get_app_option_int("Sidebar:VBox2:Position", pane->get_position()));
+    
+}
+bool ModelDiagramPanel::drag_motion(const Glib::RefPtr<Gdk::DragContext> &context, int x, int y, guint time)
+{
+  context->drag_status(context->get_suggested_action(), time);
+
+  this->drag_highlight();
+
+  return true;
+}
 
 ModelDiagramPanel::~ModelDiagramPanel()
 {
+  on_close();
+  _sig_restore_sidebar.disconnect();
+  delete _catalog_tree;
+  delete _usertypes_list;
+  delete _history_list;
   delete _cursor;
 }
 
 
-void ModelDiagramPanel::toggle_sidebar()
-{
-  if (_sidebar->is_visible())
-    _sidebar->hide();
-  else
-    _sidebar->show();
-}
-
 bool ModelDiagramPanel::drag_drop(const Glib::RefPtr<Gdk::DragContext> &context, int x, int y, guint time)
 {
-  //std::vector<Gtk::TargetEntry> targets(context->get_targets());
-  
-  //if (targets
-  {
-    Glib::ustring type= "x-mysql-wb/db.DatabaseObject";
-    drag_get_data(context, type, time);
-  }
+  std::vector<std::string> targets(context->get_targets());
+  if (!targets.empty())
+    drag_get_data(context, targets[0], time);
   
   return true;
 }
@@ -430,10 +488,22 @@ void ModelDiagramPanel::drag_data_received(const Glib::RefPtr<Gdk::DragContext>
                                         int x, int y, const Gtk::SelectionData &selection_data,
                                         guint, guint time)
 {
-  std::string type= selection_data.get_data_type();
-  _be->perform_drop(x, y, type, selection_data.get_data_as_string());
 
-  context->drag_finish(false, false, time);
+  mforms::gtk::DataWrapper *dwrapper = (mforms::gtk::DataWrapper*)selection_data.get_data();
+
+  if (!dwrapper)
+    return;
+
+  std::string tmpstr = std::vector<std::string>(context->get_targets())[0];
+
+  std::string type= selection_data.get_data_type();
+  if ( type == WB_DBOBJECT_DRAG_TYPE)
+  {
+    _be->perform_drop(x, y, type, *reinterpret_cast<std::list<GrtObjectRef> *>(dwrapper->GetData()));
+    context->drag_finish(true, false, time);
+  }
+  else
+    context->drag_finish(false, false, time);
 }
 
 
@@ -530,7 +600,7 @@ void ModelDiagramPanel::update_tool_cursor()
   if (_canvas->is_realized())
   {
     std::string cursor= _be->get_cursor();
-    std::string path= bec::IconManager::get_instance()->get_icon_path(cursor+".cur");
+    std::string path= bec::IconManager::get_instance()->get_icon_path(cursor+".png");
 
     _be->get_options_toolbar(); // refresh options toolbar
 
@@ -550,14 +620,15 @@ void ModelDiagramPanel::update_tool_cursor()
 }
 
 
-void ModelDiagramPanel::set_title(const std::string &title)
-{
-  _title_changed.emit(title);
-}
-
-
 bool ModelDiagramPanel::on_close()
 {
+  Gtk::VPaned *pane = NULL;
+  _xml->get_widget("model_sidebar", pane);
+  _wb->get_wb()->get_grt_manager()->set_app_option("Sidebar:VBox1:Position", grt::IntegerRef(pane->get_position()));
+  
+  _xml->get_widget("side_model_pane2", pane);
+  _wb->get_wb()->get_grt_manager()->set_app_option("Sidebar:VBox2:Position", grt::IntegerRef(pane->get_position()));
+
   _be->set_closed(true);
   get_parent()->hide(); // hide the container
 
@@ -565,12 +636,7 @@ bool ModelDiagramPanel::on_close()
 }
 
 void ModelDiagramPanel::refresh_catalog(bool hard)
-{
-  if (hard)
-    _catalog_tree->refresh();
-  else
-    _catalog_tree->queue_draw();
-}
+{}
 
 
 void ModelDiagramPanel::setup_navigator()
