@@ -78,10 +78,10 @@ void ModelPanel::post_construct(wb::WBContextUI *wb, wb::OverviewBE *overview, G
   {
     mforms::MenuBar *menubar = overview->get_menubar();
     if (menubar)
-      pack_start(*mforms::gtk::widget_for_menubar(menubar), false, true);
+      pack_start(*mforms::widget_for_menubar(menubar), false, true);
     mforms::ToolBar *toolbar = overview->get_toolbar();
     if (toolbar)
-      pack_start(*mforms::gtk::widget_for_toolbar(toolbar), false, true);
+      pack_start(*mforms::widget_for_toolbar(toolbar), false, true);
   }
   show_all();
 
@@ -126,8 +126,9 @@ void ModelPanel::post_construct(wb::WBContextUI *wb, wb::OverviewBE *overview, G
   label->set_use_markup(true);
 
 
+  _sig_restore_layout.disconnect();
   // restore widths of sidebars when shown
-  Glib::signal_idle().connect(sigc::bind_return(sigc::mem_fun(this, &ModelPanel::restore_sidebar_layout), false));
+  _sig_restore_layout = Glib::signal_idle().connect(sigc::bind_return(sigc::mem_fun(this, &ModelPanel::restore_sidebar_layout), false));
 }
 
 
@@ -140,6 +141,8 @@ void ModelPanel::restore_sidebar_layout()
 
 ModelPanel::~ModelPanel()
 {
+  _sig_restore_layout.disconnect();
+  _sig_resize_overview.disconnect();
   delete _history_tree;
   delete _usertypes_box;
 }
@@ -159,7 +162,8 @@ void ModelPanel::resize_overview()
   {
     // this hack is needed to force the iconview to properly resize to accomodate its new
     // container size
-    Glib::signal_idle().connect(sigc::mem_fun(this, &ModelPanel::do_resize_overview));
+    _sig_resize_overview.disconnect();
+    _sig_resize_overview = Glib::signal_idle().connect(sigc::mem_fun(this, &ModelPanel::do_resize_overview));
     _pending_rebuild_overview = true;
   }
 }
