@@ -1,3 +1,22 @@
+/*
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
+
 #include "workbench/wb_overview.h"
 #include "overview_panel.h"
 #include "treemodel_wrapper.h"
@@ -5,6 +24,7 @@
 
 #include "grtdb/db_object_helpers.h"
 #include "workbench/wb_model_file.h"
+#include "workbench/wb_context.h"
 
 #include <glib/gstdio.h>
 #include <gtkmm.h>
@@ -13,8 +33,6 @@
 
 using base::strfmt;
 
-static const char* dbgdomain = "OverviewPanel";
-
 class OverviewDivision : public Gtk::VBox
 {
   bool header_button_release(GdkEventButton *e)
@@ -192,13 +210,12 @@ public:
 
   void add_edit_buttons(const std::string &obj_type)//const sgic::slot<void> &add_slot, const sgic::slot<void> &del_slot)
   {
-    Gtk::Button *btn;
       
-    btn= add_action_button(_header_box, "collapsing_panel_header_tab_add2.png", 
+    add_action_button(_header_box, "collapsing_panel_header_tab_add2.png", 
                            strfmt(_("Add a new %s"), obj_type.c_str()),
                            sigc::mem_fun(this, &OverviewDivision::add_clicked));
     //      btn->set_tooltip_text(_("Add new item"));
-    btn= add_action_button(_header_box, "collapsing_panel_header_tab_del2.png",
+    add_action_button(_header_box, "collapsing_panel_header_tab_del2.png",
                            strfmt(_("Delete the selected %s"), obj_type.c_str()),
                            sigc::mem_fun(this, &OverviewDivision::delete_clicked));
     //      btn->set_tooltip_text(_("Delete selected item"));
@@ -348,7 +365,7 @@ void OverviewItemContainer::activate_item(const Gtk::TreeModel::Path &path)
 void OverviewItemContainer::on_selection_changed(const std::vector<bec::NodeId>& sel)
 {
   _overview->begin_selection_marking();
-  int node_type;
+  ssize_t node_type;
   
   for ( int i = sel.size() - 1; i >= 0; --i )
   {
@@ -358,8 +375,6 @@ void OverviewItemContainer::on_selection_changed(const std::vector<bec::NodeId>&
       _overview->select_node(sel[i]);
   }
   _overview->end_selection_marking();
-
-  grt::ListRef<GrtObject> sellist = _overview->get_selection();
 }
 
 //------------------------------------------------------------------------------ 
@@ -417,7 +432,7 @@ void OverviewItemContainer::drag_data_get(const Glib::RefPtr<Gdk::DragContext> &
         }
       }
     }
-    else if (data.get_target() == "x-mysql-wb/db.DatabaseObject")
+    else if (data.get_target() == WB_DBOBJECT_DRAG_TYPE)
     {
       grt::ListRef<GrtObject> sel = _overview->get_selection();
       std::list<db_DatabaseObjectRef> objects;
@@ -528,7 +543,7 @@ OverviewItemContainer::OverviewItemContainer(wb::OverviewBE *overview, const bec
   if (get_tree_view())
   {
     // create columns for the tree view
-    tv_model->model().append_string_column(wb::OverviewBE::Label, _("Name"), EDITABLE, WITH_ICON);
+    tv_model->model().append_string_column(wb::OverviewBE::Label, _("Name"), EDITABLE_WO_FIRST, WITH_ICON);
     
     int columnCount = _overview->get_details_field_count(node);
     for (int counter = 0; counter < columnCount; counter++)
@@ -702,7 +717,7 @@ public:
     std::string label;
     _overview->get_field(_node, wb::OverviewBE::Label, label);
 
-    _label.set_markup(strfmt("<b>%s</b> <small>(%i items)</small>", label.c_str(), _overview->count_children(_node)-1));
+    _label.set_markup(strfmt("<b>%s</b> <small>(%zi items)</small>", label.c_str(), _overview->count_children(_node)-1));
   }
   
   virtual void refresh()
@@ -810,18 +825,7 @@ public:
 private:
   void page_switched(GtkNotebookPage *page, guint num)
   {
-    if ((_overview->identifier() == "overview.physical.live") && !_is_focus_node_enabled)
-    {
-      if ((_current_page_index != -1) && ((int)num != _current_page_index))
-      {
-        // this is causing random crashes on page switch --alfredo
-        //!set_current_page(_current_page_index);
-
-        Glib::signal_idle().connect(sigc::bind_return(sigc::bind(sigc::mem_fun(this, &OverviewGroupContainer::set_current_page), _current_page_index), false));
-      }
-      return;
-    }
-    if ((int)num < _overview->count_children(_node))
+    if ((size_t)num < _overview->count_children(_node))
       _overview->focus_node(_overview->get_child(_node, num));
     _current_page_index= num;
   }
@@ -939,7 +943,7 @@ public:
 
   void refresh_info(const bec::NodeId &node)
   {    
-    if (get_n_pages() > node.back())
+    if (get_n_pages() > (ssize_t)node.back())
       ((OverviewGroup*)get_nth_page(node.back()))->update_label();
   }
 };
@@ -948,11 +952,10 @@ public:
 //----------------------------------------------------------------------------------------------------
 //----------------------------------------------------------------------------------------------------
 
-OverviewPanel::OverviewPanel(wb::WBContextUI *wb, wb::OverviewBE *overview, bool live_overview)
+OverviewPanel::OverviewPanel(wb::WBContextUI *wb, wb::OverviewBE *overview)
   : _container(Gtk::manage(new Gtk::VBox(false, 0)))
   , _wb(wb)
   , _overview_be(overview)
-  , _live_overview(live_overview)
 {
   add(*_container);
   _container->get_parent()->set_name("overview_viewport");
@@ -972,17 +975,24 @@ OverviewPanel::OverviewPanel(wb::WBContextUI *wb, wb::OverviewBE *overview, bool
 
 //----------------------------------------------------------------------------------------------------
 
+static void delete_container_contents(Gtk::Container *container)
+{
+  std::vector<Gtk::Widget*> children = container->get_children();
+  for (std::vector<Gtk::Widget*>::const_iterator i = children.begin(); i != children.end(); ++i)
+  {
+    container->remove(**i);
+    delete *i;
+  }
+}
+
 void OverviewPanel::reset()
 {
   _freeze= true;
 
-  _container->foreach(sigc::mem_fun(_container, &Gtk::Container::remove));
-  
   _group_containers_by_id.clear();
-  for (std::map<std::string, OverviewItemContainer*>::iterator iter = _item_containers_by_id.begin();
-        iter != _item_containers_by_id.end(); ++iter)
-    iter->second->unset_models();
   _item_containers_by_id.clear();
+
+  delete_container_contents(_container);
   
   Gtk::DrawingArea *filler= Gtk::manage(new Gtk::DrawingArea());
   filler->set_name("overview_filler");
@@ -993,6 +1003,12 @@ void OverviewPanel::reset()
 
 
 //----------------------------------------------------------------------------------------------------
+void OverviewPanel::update_for_resize()
+{
+  // XXX gtk bug with icon reflow, this should be optimized to only refresh all icon views
+  rebuild_all();
+}
+
 void OverviewPanel::rebuild_all()
 {
   if (_rebuilding)
@@ -1001,17 +1017,17 @@ void OverviewPanel::rebuild_all()
   _freeze= false;
   _rebuilding= true;
 
-  _container->foreach(sigc::mem_fun(_container, &Gtk::Container::remove));
+  delete_container_contents(_container);
   
   _group_containers_by_id.clear();
   _item_containers_by_id.clear();
 
   bec::NodeId root= _overview_be->get_root();
   
-  for (int i= 0; i < _overview_be->count_children(root); i++)
+  for (size_t i= 0; i < _overview_be->count_children(root); i++)
   {
     bec::NodeId node= _overview_be->get_child(root, i);
-    int type;
+    ssize_t type;
 
     _overview_be->get_field(node, wb::OverviewBE::NodeType, type);
 
@@ -1027,9 +1043,8 @@ void OverviewPanel::rebuild_all()
   filler->show();
   filler->modify_bg_pixmap(Gtk::STATE_NORMAL, bec::IconManager::get_instance()->get_icon_path("background.png"));
   
-  if (_overview_be->identifier() != "overview.physical.live")
   {
-    int group_index= 0;
+    size_t group_index= 0;
     if (_groups->current_page_index() >= 0)
       group_index= _groups->current_page_index();
     if (group_index < _overview_be->count_children(_groups->node()))
@@ -1040,6 +1055,18 @@ void OverviewPanel::rebuild_all()
 }
 
 
+void OverviewPanel::item_list_selection_changed(const std::vector<bec::NodeId>& nodes, MultiView *mview)
+{
+  if (nodes.empty())
+    return;
+  for (std::map<std::string, OverviewItemContainer*>::iterator iter = _item_containers_by_id.begin();
+       iter != _item_containers_by_id.end(); ++iter)
+  {
+    if (iter->second && iter->second != mview)
+      iter->second->select_node(bec::NodeId());
+  }
+}
+
 //------------------------------------------------------------------------------
 // Build division e.g. EER Diagrams, Physical Schemata, 
 // Schema Privileges in the model overview. @pnode points to a subtree
@@ -1051,22 +1078,15 @@ void OverviewPanel::build_division(Gtk::VBox *container, const bec::NodeId &pnod
   std::string text;
   _overview_be->get_field(pnode, wb::OverviewBE::Label, text);
 
-  bool inside_physical_section= false;
-  if ( (text == "Physical Schemata") || (text == "Overview") )
-    inside_physical_section = true;
-  
   // Check if division is expanded
-  int default_expanded= 1;
+  ssize_t default_expanded= 1;
   _overview_be->get_field(pnode, wb::OverviewBE::Expanded, default_expanded);
 
-  int child_type;
+  ssize_t child_type;
   _overview_be->get_field(pnode, wb::OverviewBE::ChildNodeType, child_type);
 
-  // if _live_overview requested, construct the contents directly (used in live overview)
   OverviewDivision *division = 0;
-  division= Gtk::manage(new OverviewDivision(_overview_be, pnode, text, true, _live_overview));
-  if (inside_physical_section && ! _live_overview)
-    division->add_edit_buttons(_("Schema"));
+  division= Gtk::manage(new OverviewDivision(_overview_be, pnode, text, true, false));
 
   container->pack_start(*division, false, true);
 
@@ -1077,11 +1097,14 @@ void OverviewPanel::build_division(Gtk::VBox *container, const bec::NodeId &pnod
 
     division->pack_start(*_groups, false, false);
 
+    division->add_edit_buttons(_("Schema"));
+
     _group_containers_by_id[pnode.repr()]= _groups;
 
     // assumes child nodes are sections
     // Build groups, like 'mydb' tab and its content for a databases
-    for (int i= 0; i < _overview_be->count_children(pnode); i++)
+    _overview_be->refresh_node(pnode, true);
+    for (size_t i= 0; i < _overview_be->count_children(pnode); i++)
     {
       bec::NodeId node= _overview_be->get_child(pnode, i);
 
@@ -1094,6 +1117,7 @@ void OverviewPanel::build_division(Gtk::VBox *container, const bec::NodeId &pnod
   {
     OverviewItemContainer *item_list= Gtk::manage(new OverviewItemContainer(_overview_be, pnode, true, true));
 
+    item_list->signal_selection_changed().connect(sigc::bind(sigc::mem_fun(this, &OverviewPanel::item_list_selection_changed), item_list));
     item_list->signal_popup_menu().connect(sigc::bind(sigc::mem_fun(this, &OverviewPanel::item_popup_menu),
                                                       item_list));
 
@@ -1112,11 +1136,12 @@ void OverviewPanel::build_division(Gtk::VBox *container, const bec::NodeId &pnod
   }
   else if (child_type == wb::OverviewBE::OSection)
   {
-    for (int i= 0; i < _overview_be->count_children(pnode); i++)
+    for (size_t i= 0; i < _overview_be->count_children(pnode); i++)
     {
       bec::NodeId node= _overview_be->get_child(pnode, i);
       OverviewSection *section= Gtk::manage(new OverviewSection(_overview_be, node, true, true));
 
+      section->signal_selection_changed().connect(sigc::bind(sigc::mem_fun(this, &OverviewPanel::item_list_selection_changed), section));
       _item_containers_by_id[node.repr()]= section;
 
       division->pack_start(*section, false, false);
@@ -1160,17 +1185,17 @@ void OverviewPanel::build_group(OverviewDivision *division, OverviewGroupContain
 
 void OverviewPanel::build_group_contents(OverviewDivision *division, Gtk::VBox *page, const bec::NodeId &pnode)
 {
-  int type;
+  ssize_t type;
   _overview_be->get_field(pnode, wb::OverviewBE::ChildNodeType, type);
     if (type != wb::OverviewBE::OSection)
       throw std::logic_error("unexpected child node type");
   
   std::string child_name;
-  for (int i= 0; i < _overview_be->count_children(pnode); i++)
+  for (size_t i= 0; i < _overview_be->count_children(pnode); i++)
   {
     bec::NodeId node= _overview_be->get_child(pnode, i);
     OverviewSection *section= Gtk::manage(new OverviewSection(_overview_be, node, true, true));
-    
+    section->signal_selection_changed().connect(sigc::bind(sigc::mem_fun(this, &OverviewPanel::item_list_selection_changed), section));
     section->signal_popup_menu().connect(sigc::bind(sigc::mem_fun(this, &OverviewPanel::item_popup_menu),
                                                     section));
     _item_containers_by_id[node.repr()]= section;
@@ -1209,14 +1234,14 @@ void OverviewPanel::pre_refresh_groups()
 void OverviewPanel::refresh_active_group_node_children()
 {
   bec::NodeId group_node_id= _groups->node();
-  int current_page_index= _groups->current_page_index();
-  if (-1 == current_page_index || _groups->get_n_pages() < _overview_be->count_children(group_node_id))
+  ssize_t current_page_index= _groups->current_page_index();
+  if (-1 == current_page_index || _groups->get_n_pages() < (ssize_t)_overview_be->count_children(group_node_id))
   {
     rebuild_all();
     select_default_group_page();
     current_page_index= _groups->current_page_index();
   }
-  if ((current_page_index != -1) && ((int)current_page_index < _overview_be->count_children(group_node_id)))
+  if ((current_page_index != -1) && (current_page_index < (ssize_t)_overview_be->count_children(group_node_id)))
   {
     bec::NodeId schema_node_id= _overview_be->get_child(group_node_id, current_page_index);
 
@@ -1237,7 +1262,7 @@ void OverviewPanel::refresh_active_group_node_children()
       
       if (uid == schema_uid)
       {
-        group->foreach(sigc::mem_fun(group, &Gtk::Container::remove));
+        delete_container_contents(group);
 
         build_group_contents(division, group, schema_node_id);
         
@@ -1279,6 +1304,23 @@ void OverviewPanel::update_group_note(OverviewGroupContainer *group_container, c
       _groups->enable_set_focus_node(false);
       group->hide();
       group->invalidate();
+
+      std::vector<Gtk::Widget*> children(group->get_children());
+      std::map<std::string, OverviewItemContainer*>::iterator it;
+      for (std::vector<Gtk::Widget*>::iterator iter = children.begin();
+              iter != children.end(); ++iter)
+      {
+        OverviewItemContainer* items = dynamic_cast<OverviewItemContainer*>(*iter);
+        if (items)
+        {
+          it = _item_containers_by_id.find(items->get_base_node().repr());
+          if (it != _item_containers_by_id.end())
+          {
+            _item_containers_by_id.erase(it);
+          }
+        }
+      }
+
       group_container->remove_page(*group);
       _groups->enable_set_focus_node(was_focus_node_enabled);
     }
@@ -1337,7 +1379,7 @@ void OverviewPanel::select_default_group_page()
 //------------------------------------------------------------------------------
 void OverviewPanel::refresh_children(const bec::NodeId& node)
 {
-  int type;
+  ssize_t type;
 
   if (!node.is_valid())
   {
@@ -1358,7 +1400,7 @@ void OverviewPanel::refresh_children(const bec::NodeId& node)
     if (_groups->enable_set_focus_node())
     {
       if ((_groups->current_page_index() >= 0) &&
-        (_groups->current_page_index() < _overview_be->count_children(_groups->node())))
+        (_groups->current_page_index() < (ssize_t)_overview_be->count_children(_groups->node())))
       {
         _overview_be->focus_node(_overview_be->get_child(_groups->node(), _groups->current_page_index()));
       }
@@ -1379,7 +1421,7 @@ void OverviewPanel::refresh_node(const bec::NodeId &node)
 {
   if (_freeze) return;
 
-  int type;
+  ssize_t type;
   bec::NodeId parent(_overview_be->get_parent(node));
   _overview_be->get_field(node, wb::OverviewBE::NodeType, type);
 
@@ -1406,7 +1448,7 @@ void OverviewPanel::select_node(const bec::NodeId& node)
 {
   if (_freeze) return;
 
-  int type;
+  ssize_t type;
   bec::NodeId parent(_overview_be->get_parent(node));
   _overview_be->get_field(node, wb::OverviewBE::NodeType, type);
 
