@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -34,6 +34,7 @@
 #include <fcntl.h>
 #include <sys/file.h>
 #endif
+#include <algorithm>
 
 namespace base {
   
@@ -152,10 +153,10 @@ namespace base {
     }
 
     char buffer[32];
-    sprintf(buffer, "%i", GetCurrentProcessId());
+    sprintf_s(buffer, "%i", GetCurrentProcessId());
     DWORD bytes_written;
 
-    if (!WriteFile(handle, buffer, strlen(buffer), &bytes_written, NULL) || bytes_written != strlen(buffer))
+    if (!WriteFile(handle, buffer, (DWORD)strlen(buffer), &bytes_written, NULL) || bytes_written != strlen(buffer))
     {
       CloseHandle(handle);
       DeleteFileW(wpath.c_str());
@@ -172,60 +173,53 @@ namespace base {
   
   LockFile::Status LockFile::check(const std::string &path)
   {
-    std::wstring wpath(string_to_wstring(path));
-    // open the file and see if it's locked
-    HANDLE h = CreateFileW(wpath.c_str(), 
-      GENERIC_WRITE, 
-      FILE_SHARE_WRITE|FILE_SHARE_READ,
-      NULL,
-      OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
-
-    if (h == INVALID_HANDLE_VALUE)
-    {
-      switch (GetLastError())
-      {
-      case ERROR_SHARING_VIOLATION:
-        // if file cannot be opened for writing, it is locked...
-        // so open it for reading to check the owner process id written in it
-        h = CreateFileW(wpath.c_str(), 
-              GENERIC_READ, 
-              FILE_SHARE_WRITE|FILE_SHARE_READ, 
-              NULL,
-              OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
-        if (h != INVALID_HANDLE_VALUE)
-        {
-          char buffer[32];
-          DWORD bytes_read;
-          if (ReadFile(h, buffer, sizeof(buffer), &bytes_read, NULL))
-          {
-            CloseHandle(h);
-            buffer[bytes_read]= 0;
-            if (atoi(buffer) == GetCurrentProcessId())
-              return LockedSelf;
-            return LockedOther;
-          }
-          CloseHandle(h);
-          return LockedOther;
-        }
-        // if the file is locked for read, assume its locked by some unrelated process
-        // since this class never locks it for read
-        return LockedOther;
-        //throw std::runtime_error(strfmt("Could not read process id from lock file (%i)", GetLastError());
-        break;
-      case ERROR_FILE_NOT_FOUND:
-        return NotLocked;
-      case ERROR_PATH_NOT_FOUND:
-        throw std::invalid_argument("Invalid path");
-      default:
-        throw std::runtime_error(strfmt("Could not open lock file (%i)", GetLastError()));
-      }
-    }
-    else
+    // Can we open the file in exclusive mode?
+    std::wstring wpath = string_to_wstring(path);
+    HANDLE h = CreateFile(wpath.c_str(), GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);
+    if (h != INVALID_HANDLE_VALUE)
     {
       CloseHandle(h);
-      // if the file could be opened with DENY_WRITE, it means no-one is locking it
       return NotLocked;
     }
+
+    switch (GetLastError())
+    {
+    case ERROR_SHARING_VIOLATION:
+      // If file cannot be opened for writing it's open somewhere else.
+      // Try to open it and read the first bytes to see if that corresponds to our process id.
+      h = CreateFileW(wpath.c_str(), 
+            GENERIC_READ, 
+            FILE_SHARE_WRITE | FILE_SHARE_READ, 
+            NULL,
+            OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
+      if (h != INVALID_HANDLE_VALUE)
+      {
+        char buffer[32];
+        DWORD bytes_read;
+        if (ReadFile(h, buffer, sizeof(buffer), &bytes_read, NULL))
+        {
+          CloseHandle(h);
+          buffer[bytes_read] = 0;
+          if (base::atoi<int>(buffer, -1) != GetCurrentProcessId())
+            return LockedOther; // TODO: this unreliable. Any non-lock file would qualify here (which is wrong).
+          return LockedSelf;
+        }
+        CloseHandle(h);
+        return LockedOther;
+      }
+
+      // If the file cannot be read assume its locked by another process.
+      return LockedOther;
+
+    case ERROR_FILE_NOT_FOUND:
+      return NotLocked;
+
+    case ERROR_PATH_NOT_FOUND:
+      throw std::invalid_argument("Invalid path");
+
+    default:
+      throw std::runtime_error(strfmt("Unknown error while checking file %s for locks (%i)", path.c_str(), GetLastError()));
+    }
   }
 #else
   LockFile::LockFile(const std::string &apath) throw (std::invalid_argument, std::runtime_error, file_locked_error)
@@ -286,7 +280,7 @@ namespace base {
       if (c < 0)
         return LockedOther;
       pid[c] = 0;
-      if (atoi(pid) != getpid())
+      if (base::atoi<int>(pid, -1) != getpid())
         return LockedOther;
       return LockedSelf;
     }
@@ -303,11 +297,16 @@ namespace base {
   bool create_directory(const std::string &path, int mode, bool with_parents)
   {
 #ifdef _WIN32
+    SetLastError(0);
     if (!CreateDirectoryW(path_from_utf8(path).c_str(), NULL))
     {
+      DWORD error = GetLastError();
+      if (error == ERROR_ALREADY_EXISTS)
+        return false;
+
       if (with_parents)
       {
-        if (GetLastError() == ERROR_PATH_NOT_FOUND)
+        if (error == ERROR_PATH_NOT_FOUND)
         {
           // create parents too
           std::string tmp = path;
@@ -331,11 +330,8 @@ namespace base {
           return true;
         }
       }
-      else
-        if (GetLastError() == ERROR_ALREADY_EXISTS)
-          return false;
-      throw file_error(strfmt("Could not create directory %s",
-                              path.c_str()), GetLastError());
+
+      throw file_error(strfmt("Could not create directory %s", path.c_str()), GetLastError());
     }
 #else
     if (with_parents)
@@ -397,6 +393,12 @@ namespace base {
     return true;
   }
   
+  //--------------------------------------------------------------------------------------------------
+  
+  /**
+   * Deletes file or folder.
+   * Returns false if the object doesn't exist and throws an exception on error.
+   */
   bool remove(const std::string &path)
   {
 #ifdef _WIN32
@@ -430,6 +432,26 @@ namespace base {
     return true;
   }
   
+  //--------------------------------------------------------------------------------------------------
+
+  /**
+   * Tries to delete a file or folder.
+   * No exception is thrown if that fails. Returns true on success, otherwise false.
+   */
+  bool tryRemove(const std::string &path)
+  {
+#ifdef _WIN32
+    if (is_directory(path))
+      return RemoveDirectory(path_from_utf8(path).c_str()) == TRUE;
+    else
+      return DeleteFile(path_from_utf8(path).c_str()) == TRUE;
+#else
+    return ::g_remove(path_from_utf8(path).c_str()) == 0;
+#endif
+  }
+
+  //--------------------------------------------------------------------------------------------------
+
   bool file_exists(const std::string &path)
   {
     char *f = g_filename_from_utf8(path.c_str(), -1, NULL, NULL, NULL);
@@ -442,6 +464,8 @@ namespace base {
     return false;
   }
 
+  //--------------------------------------------------------------------------------------------------
+
   bool is_directory(const std::string &path)
   {
     char *f = g_filename_from_utf8(path.c_str(), -1, NULL, NULL, NULL);
@@ -453,17 +477,9 @@ namespace base {
     g_free(f);
     return false;
   }
-  /*
-  std::string make_path(const std::string &path, const std::string &filename)
-  {
-    if (path.empty())
-      return filename;
-    
-    if (path[path.size()-1] == '/' || path[path.size()-1] == '\\')
-      return path+filename;
-    return path+G_DIR_SEPARATOR+filename;
-  }*/
   
+  //--------------------------------------------------------------------------------------------------
+
   std::string extension(const std::string &path)
   {
     std::string::size_type p = path.rfind('.');
@@ -503,86 +519,14 @@ namespace base {
     return path;
   }
 
-
-  //---
-  /** @brief Wrapper around fopen that expects a filename in UTF-8 encoding
-   @param filename name of file to open
-   @param mode second argument of fopen
-   @return If successful, base_fopen returns opened FILE*.
-   *//////////////////////////////////////////////////////////////////////////////
-  FILE *fopen(const std::string &filename, const char *mode)
-  {
-#ifdef _WIN32
-    
-    // Convert filename from UTF-8 to UTF-16.
-    int required;
-    WCHAR* converted;
-    WCHAR* converted_mode;
-    WCHAR* in;
-    char* out;
-    FILE* result;
-    
-    required= MultiByteToWideChar(CP_UTF8, 0, filename.c_str(), -1, NULL, 0);
-    if (required == 0)
-      return NULL;
-    
-    // Required contains the length for the result string including the terminating 0.
-    converted= new WCHAR[required];
-    memset(converted, 0, sizeof(WCHAR) * required);
-    MultiByteToWideChar(CP_UTF8, 0, filename.c_str(), -1, converted, required);
-    
-    converted_mode= new WCHAR[strlen(mode) + 1];
-    memset(converted_mode, 0, sizeof(WCHAR) * (strlen(mode) + 1));
-    in= converted_mode;
-    out= (char*) mode;
-    while (*out)
-      *in++ = (WCHAR) (*out++);
-    
-    result= _wfopen(converted, converted_mode);
-    delete[] converted;
-    delete[] converted_mode;
-    
-    return result;
-    
-#else
-    
-    FILE *file;
-    char * local_filename;
-    
-    if (! (local_filename= g_filename_from_utf8(filename.c_str(),-1,NULL,NULL,NULL)))
-      return NULL;
-    
-    file= ::fopen(local_filename, mode);
-    
-    g_free(local_filename);
-    
-    return file;
-#endif
-  }
-  
-
-  
-FileHandle::FileHandle(const char *filename, const char *mode, bool throw_on_fail) 
+FileHandle::FileHandle(const char *filename, const char *mode, bool throw_on_fail)
 : _file(NULL) 
 { 
-    _file= base::fopen(filename, mode);
+    _file = base_fopen(filename, mode);
     if (!_file && throw_on_fail)
       throw file_error(std::string("Failed to open file \"").append(filename).append("\""), errno);
 }
 
-//   FILE * FileHandle::open_file(const char *filename, const char *mode, bool throw_on_fail)
-//   {
-//     FILE *file= base::fopen(filename, mode);
-//     if (!file && throw_on_fail)
-//       throw file_error(std::string("Failed to open file \"").append(filename), errno);
-//     return file;
-//   }
-//  
-//   void FileHandle::close_file(FILE *f)
-//   {
-//     ::fclose(f);
-//   }
- 
   FileHandle & FileHandle::operator =(FileHandle &fh)
   {
     dispose();
@@ -603,12 +547,11 @@ FileHandle::FileHandle(const char *filename, const char *mode, bool throw_on_fai
       _file= NULL;
     }
   }
-};
 
 /**
  * Returns the last modification time of the given file.
  */
-bool base::file_mtime(const std::string &path, time_t &mtime)
+bool file_mtime(const std::string &path, time_t &mtime)
 {
 #ifdef _WIN32
   struct _stat stbuf;
@@ -628,3 +571,28 @@ bool base::file_mtime(const std::string &path, time_t &mtime)
   return false;
 }
 
+std::string join_path(const char *prefix, ...)
+{
+  std::string path = prefix;
+  char wrong_path_separator = G_DIR_SEPARATOR == '/' ? '\\' : '/';
+  std::replace(path.begin(), path.end(), wrong_path_separator, G_DIR_SEPARATOR);
+  std::string arg = const_cast<char*>(prefix);
+  va_list ap;
+  va_start(ap, prefix);
+  while (!arg.empty())
+  {
+    arg = va_arg(ap, char*);
+    if (!arg.empty())
+    {
+      if (path[path.size() - 1] == G_DIR_SEPARATOR)
+        path += arg;
+      else
+        path += G_DIR_SEPARATOR + arg;
+    }
+  }
+  va_end(ap);
+
+  return path;
+}
+
+};
