@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,17 +17,16 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "base/string_utilities.h"
 #include "base/log.h"
 #include "mforms/mforms.h"
 
-DEFAULT_LOG_DOMAIN(DOMAIN_MFORMS_BE);
+//DEFAULT_LOG_DOMAIN(DOMAIN_MFORMS_BE);
 
 using namespace mforms;
 
 //--------------------------------------------------------------------------------------------------
+
 static int _serial = 0;
 
 #ifdef _WIN32
@@ -35,13 +34,13 @@ static int _serial = 0;
 AppView::AppView(bool horiz, const std::string &context_name, bool is_main)
   : Box(horiz), _context_name(context_name), _menubar(0), _toolbar(0), _is_main(is_main)
 {
-  _app_view_impl= &ControlFactory::get_instance()->_app_view_impl;
+  _app_view_impl = &ControlFactory::get_instance()->_app_view_impl;
   if (_app_view_impl && _app_view_impl->create)
     _app_view_impl->create(this, horiz);
   
   _identifier = base::strfmt("avid%i", ++_serial);
-
-  _dpoint = 0;
+  _dpoint = NULL;
+  set_name(context_name);
 }
 
 #else
@@ -49,6 +48,7 @@ AppView::AppView(bool horiz, const std::string &context_name, bool is_main)
 AppView::AppView(bool horiz, const std::string &context_name, bool is_main)
 : Box(horiz), _context_name(context_name), _menubar(0), _toolbar(0), _is_main(is_main)
 {
+  set_name(context_name);
 #ifdef __APPLE__
   // default, empty toolbar for mac, to show the 3px bar under the top tabs
   // TODO: move this to the platform layer. It doesn't belong here.
@@ -56,7 +56,6 @@ AppView::AppView(bool horiz, const std::string &context_name, bool is_main)
   set_back_color("#e8e8e8");
 #endif
   _identifier = base::strfmt("avid%i", ++_serial);
-
   _dpoint = 0;
 }
 
@@ -85,11 +84,17 @@ void AppView::set_menubar(mforms::MenuBar *menu)
 {
   if (_menubar != menu)
   {
-    if (_menubar)
+    if (_menubar != NULL)
       _menubar->release();
     _menubar = menu;
-    if (menu)
-      _menubar->retain();
+
+    if (menu != NULL)
+    {
+      if (!menu->release_on_add())
+        _menubar->retain();
+      else
+        _menubar->set_release_on_add(false);
+    }
   }
 }
 
@@ -102,8 +107,14 @@ void AppView::set_toolbar(mforms::ToolBar *toolbar)
     if (_toolbar)
       _toolbar->release();
     _toolbar = toolbar;
-    if (toolbar)
-      _toolbar->retain();
+
+    if (toolbar != NULL)
+    {
+      if (!_toolbar->release_on_add())
+        _toolbar->retain();
+      else
+        _toolbar->set_release_on_add(false);
+    }
   }
 }
 
@@ -111,11 +122,9 @@ void AppView::set_toolbar(mforms::ToolBar *toolbar)
 
 void AppView::set_title(const std::string &title)
 { 
-  _title= title;
+  _title = title;
   if (_dpoint)
     _dpoint->set_view_title(this, title);
-  else
-    log_error("Calling AppView::set_title() on an undocked view\n");
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -125,9 +134,21 @@ std::string AppView::get_title()
   return _title;
 }
 
+//--------------------------------------------------------------------------------------------------
+
 bool AppView::on_close()
 {
   if (_on_close_slot) 
     return _on_close_slot(); 
   return true;
-}
\ No newline at end of file
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void AppView::close()
+{
+  if (_dpoint)
+    _dpoint->undock_view(this);
+}
+
+//--------------------------------------------------------------------------------------------------
