@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "mforms/mforms.h"
 
 using namespace mforms;
@@ -31,7 +29,7 @@ FindPanel::FindPanel(CodeEditor *editor)
 }
 
 
-int FindPanel::perform_action(FindPanelAction action)
+size_t FindPanel::perform_action(FindPanelAction action)
 {
   return _find_impl->perform_action(this, action);
 }
