@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,18 +17,19 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "mforms/mforms.h"
 
 using namespace mforms;
 
+static Form *current_active_form = NULL;
+
 //--------------------------------------------------------------------------------------------------
 
 Form::Form(Form *owner, FormFlag flag)
 {
   _form_impl= &ControlFactory::get_instance()->_form_impl;
-  
+ 
+  _menu = NULL; 
   _content = NULL;
   _fixed_size = false;
   _release_on_close = false;
@@ -41,6 +42,7 @@ Form::Form(Form *owner, FormFlag flag)
 Form::Form()
 {
   _form_impl = &ControlFactory::get_instance()->_form_impl;
+  _menu = NULL; 
   _content = NULL;
   _fixed_size = false;
   _release_on_close = false;
@@ -61,12 +63,34 @@ Form *Form::main_form()
 
 Form::~Form()
 {
-  if (_content && !_content->release_on_add())
+  if (_menu)
+    _menu->release();
+  if (current_active_form == this)
+    current_active_form = NULL;
+  if (_content != NULL)
     _content->release();
 }
 
 //--------------------------------------------------------------------------------------------------
 
+void Form::set_menubar(MenuBar *menu)
+{
+  if (!_content || !dynamic_cast<Box*>(_content))
+    throw std::logic_error("set_menubar() must be called on a window with a Box as it's toplevel content");
+
+  if (menu != _menu)
+  {
+    if (_menu)
+      _menu->release();
+    _menu = menu;
+    _menu->retain();
+
+    _form_impl->set_menubar(this, menu);
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void Form::set_title(const std::string &title)
 {
   if (_form_impl)
@@ -153,6 +177,7 @@ void Form::flush_events()
 
 void Form::activated()
 {
+  current_active_form = this;
   _active = true;
   _activated_signal();
 }
@@ -173,3 +198,17 @@ bool Form::is_active()
 }
 
 //--------------------------------------------------------------------------------------------------
+
+bool Form::can_close()
+{
+  if (_can_close_slot)
+    return _can_close_slot();
+  return true;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+Form* Form::active_form()
+{
+  return current_active_form;
+}
