@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -56,7 +56,8 @@ class UtilitiesImpl
   static std::string get_clipboard_text();
   static void open_url(const std::string &url);
   static std::string get_special_folder(mforms::FolderType type);
-  static void add_timeout(float interval, const boost::function<bool ()> &slot);
+  static TimeoutHandle  add_timeout(float interval, const boost::function<bool ()> &slot);
+  static void cancel_timeout(TimeoutHandle h);
 
   static void store_password(const std::string &service, const std::string &account, const std::string &password);
   static bool find_password(const std::string &service, const std::string &account, std::string &password);
@@ -65,6 +66,11 @@ class UtilitiesImpl
   static bool move_to_trash(const std::string &path);
   static void reveal_file(const std::string &path);
 
+  static void set_thread_name(const std::string &name);
+  static void beep();
+
+  static double get_text_width(const std::string &text, const std::string &font_desc);
+  
 public:
   static void init();
    
