@@ -814,7 +814,7 @@ bool GridModel::get_iter_vfunc(const Path& path, iterator& iter) const
       it->user_data3 = (void*)-1;
       const GridRow* const row = &(_root[gid]);
       if (path_size > 1)
-        it->user_data2 = (void*)(path[1] < (long)row->rows.size() ? path[1]: -1);
+        it->user_data2 = reinterpret_cast<void*>(path[1] < (long)row->rows.size() ? path[1]: -1);
       else
         it->user_data2 = (void*)-1;
       ret = true;
@@ -894,7 +894,7 @@ bool GridModel::iter_nth_child_vfunc(const iterator& parent, int n, iterator& it
       {
         it->stamp      = _stamp;
         it->user_data  = pit->user_data;
-        it->user_data2 = (void*)n;
+        it->user_data2 = reinterpret_cast<void*>(n);
         it->user_data3 = (void*)-1;
         ret = true;
       }
@@ -916,7 +916,7 @@ bool GridModel::iter_nth_root_child_vfunc(int n, iterator& iter) const
     if (n >= 0 && n < (long)_root.size())
     {
       it->stamp      = _stamp;
-      it->user_data  = (void*)n;
+      it->user_data  = reinterpret_cast<void*>(n);
       it->user_data2 = (void*)-1;
       it->user_data3 = (void*)-1;
       ret = true;
@@ -1184,7 +1184,7 @@ int GridModel::get_children_count(const mforms::GridPath& path)
   else if (path_size == 1)
   {
     const int gid = path[0];
-    if (gid >= 0 && gid < _root.size())
+    if (gid >= 0 && (unsigned int)gid < _root.size())
       ret = _root[gid].rows.size();
   }
 
@@ -1271,13 +1271,8 @@ mforms::GridPath GridModel::insert_row(const mforms::GridPath& path)
   const int       gid = path.size() >= 1 ? path[0] : -1;
   const int       rid = path.size() > 1  ? path[1] : -1;
   const int root_size = _root.size();
-  bool      do_append = false;
 
-  if (gid < 0 || gid >= root_size)
-  {
-    do_append = true;
-  }
-  else
+  if (gid >= 0 && gid < root_size)
   {
     if (rid >= 0)
     {
@@ -1285,12 +1280,7 @@ mforms::GridPath GridModel::insert_row(const mforms::GridPath& path)
       GridRow* row = &(_root[gid]);
       if (row)
       {
-        const int rows_size = row->rows.size();
-        if (rows_size <= rid && rid)
-        {
-          do_append = true;
-        }
-        else
+        if ((int)row->rows.size() > rid)
         {
           typedef std::deque<Cells>  RowCells;
           RowCells::iterator it = row->rows.begin() + rid;
@@ -1305,8 +1295,6 @@ mforms::GridPath GridModel::insert_row(const mforms::GridPath& path)
           row_has_child_toggled(ppath, piter);
         }
       }
-      else
-        do_append = true;
     }
     else
     {
@@ -1460,8 +1448,8 @@ bool GridModel::fill(const int gid, const int rid, Gtk::TreeIter* iter, Gtk::Tre
 {
   GtkTreeIter* it = iter->gobj();
   it->stamp = _stamp;
-  it->user_data = (void*)gid;
-  it->user_data2 = (void*)rid;
+  it->user_data = reinterpret_cast<void*>(gid);
+  it->user_data2 = reinterpret_cast<void*>(rid);
   it->user_data3 = (void*)-1;
 
   if (path && gid >= 0)
