@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -47,9 +47,9 @@
 #include "../lf_code_editor.h"
 #include "../lf_menubar.h"
 #include "../lf_toolbar.h"
+#include "../lf_canvas.h"
 
 extern void lf_findpanel_init();
-extern void Grid_init();
 namespace mforms
 {
 namespace gtk
@@ -92,10 +92,10 @@ void ::mforms::gtk::init(bool force_sys_colors_)
   CodeEditorImpl::init();
   lf_menubar_init();
   mforms::gtk::lf_toolbar_init();
-  Grid_init();
   mforms::gtk::HyperText_init();
   mforms::gtk::Popover_init();
   lf_findpanel_init();
+  CanvasImpl::init();
 //  GRTTreeViewImpl::init(); initialized in main program
 }
 
