@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -101,14 +101,14 @@ struct ToolBarImpl : public mforms::gtk::ViewImpl
 };
 
 //------------------------------------------------------------------------------
-Gtk::Widget *mforms::gtk::widget_for_toolbar(mforms::ToolBar *toolbar)
+Gtk::Widget *mforms::widget_for_toolbar(mforms::ToolBar *toolbar)
 {
   return toolbar->get_data< ::ToolBarImpl>()->get_outer();
 }
 
 
 //------------------------------------------------------------------------------
-Gtk::Widget *mforms::gtk::widget_for_toolbar_item_named(mforms::ToolBar *toolbar, const std::string& name)
+Gtk::Widget *mforms::widget_for_toolbar_item_named(mforms::ToolBar *toolbar, const std::string& name)
 {
   mforms::ToolBarItem *item = toolbar->find_item(name);
   if (item)
@@ -431,10 +431,16 @@ void mforms::gtk::ToolBarImpl::set_item_text(mforms::ToolBarItem *item, const st
       }
       break;
     }
+    case mforms::SearchFieldItem:
+    {
+      Gtk::Entry* e = cast<Gtk::Entry*>(item->get_data_ptr());
+      if (e)
+        e->set_text(label);
+      break;
+    }
     case mforms::SeparatorItem:
     case mforms::ExpanderItem:
     case mforms::ImageBoxItem:
-    case mforms::SearchFieldItem:
       break;
   }
 }
@@ -464,6 +470,13 @@ std::string mforms::gtk::ToolBarImpl::get_item_text(mforms::ToolBarItem *item)
       }
       break;
     }
+    case mforms::SearchFieldItem:
+    {
+      Gtk::Entry* e = cast<Gtk::Entry*>(item->get_data_ptr());
+      if (e)
+        text = e->get_text();
+      break;
+    }
     default:
     {
       Gtk::Widget* btn = cast<Gtk::Widget*>(item->get_data_ptr());
@@ -540,12 +553,16 @@ void mforms::gtk::ToolBarImpl::set_selector_items(ToolBarItem* item, const std::
     if (w)
     {
       w->set_data("ignore_signal", (void*)1);
-
+      #if ((GTKMM_MAJOR_VERSION == 2 && GTKMM_MINOR_VERSION >= 24) || GTKMM_MAJOR_VERSION > 2)
+      w->remove_all();
+      #else
+      w->clear_items();
+      #endif
       const int size = values.size();
       for (int i = 0; i < size; ++i)
         w->append_text(values[i]);
 
-      if (w->get_active_row_number() < 0)
+      if (w->get_active_row_number() < 0 && !values.empty())
         w->set_active_text(values[0]);
 
       w->set_data("ignore_signal", 0);
