@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,6 +20,11 @@
 //#include "config.h"
 #if defined(HAVE_GNOME_KEYRING) || defined(HAVE_OLD_GNOME_KEYRING)
 extern "C" {
+// gnome-keyring has been deprecated in favor of libsecret
+// More informations can be found here  https://mail.gnome.org/archives/commits-list/2013-October/msg08876.html
+// Below defines will turn off deprecations and allow build with never Gnome until we will not move to libsecret.
+  #define GNOME_KEYRING_DEPRECATED
+  #define GNOME_KEYRING_DEPRECATED_FOR(x)
   #include <gnome-keyring.h>
 };
   #include <string.h>
@@ -46,6 +51,42 @@ extern "C" {
 namespace mforms {
 namespace gtk {
 
+/**
+  * We need to use our implemenation of g_environ_unsetenv function because on OL6 that function is not avaiable.
+  * TODO: replace that function with g_environ_unsetenv when OL6 support will be dropped
+  */
+static gchar **wb_environ_unsetenv_internal(gchar **envp, const gchar *variable)
+{
+	g_return_val_if_fail(variable != NULL, NULL);
+	g_return_val_if_fail(strchr (variable, '=') == NULL, NULL);
+
+	if (envp == NULL)
+		return NULL;
+
+	gint len;
+	gchar **_envp, **envp_tmp;
+
+	len = strlen(variable);
+
+	/* Note that we remove *all* environment entries for
+	 * the variable name, not just the first.*/
+	_envp = envp_tmp = envp;
+	while (*_envp != NULL)
+	{
+		if (strncmp(*_envp, variable, len) != 0 || (*_envp)[len] != '=')
+		{
+			*envp_tmp = *_envp;
+			envp_tmp++;
+		} else
+			g_free(*_envp);
+
+		_envp++;
+	}
+	*envp_tmp = NULL;
+
+	return envp;
+}
+
 /**
   * Get the current active window for this application
   */
@@ -205,10 +246,14 @@ void UtilitiesImpl::open_url(const std::string &url)
     quoted_url,
     NULL 
   };
+  char **envp = g_get_environ();
+  envp = wb_environ_unsetenv_internal(envp, "LD_PRELOAD");
+
   GError *error = NULL;
-  gboolean result = g_spawn_async(NULL, (gchar**)argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error);
+  gboolean result = g_spawn_async(NULL, (gchar**)argv, envp, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error);
   
   free(quoted_url);
+  g_strfreev(envp);
   
   if (!result)
   {
@@ -377,6 +422,14 @@ bool UtilitiesImpl::find_password(const std::string &service, const std::string
                                                             "account", account.c_str(),
                                                             NULL);
 
+  if ( res == GNOME_KEYRING_RESULT_CANCELLED)
+  {
+    if (kpwd)
+      gnome_keyring_free_password(kpwd);
+    kpwd = 0;
+    throw grt::user_cancelled("User cancelled password lookup.");
+  }
+
   if ( res != GNOME_KEYRING_RESULT_OK && res != GNOME_KEYRING_RESULT_NO_MATCH )
   {
     if (kpwd)
@@ -419,6 +472,9 @@ void UtilitiesImpl::forget_password(const std::string &service, const std::strin
                                                             "account", account.c_str(),
                                                             NULL);
 
+  if ( res == GNOME_KEYRING_RESULT_CANCELLED)
+    throw grt::user_cancelled("User cancelled password lookup.");
+
   if ( res != GNOME_KEYRING_RESULT_OK && res != GNOME_KEYRING_RESULT_NO_MATCH)
     throw std::runtime_error(std::string("forget_password ") + gnome_keyring_result_to_message(res));
 }
@@ -494,6 +550,13 @@ static guint32 find_password_and_id(const std::string &service, const std::strin
                                                                   attrs,
                                                                   &found_items);
 
+  if (result == GNOME_KEYRING_RESULT_CANCELLED)
+  {
+    if (found_items)
+      gnome_keyring_found_list_free(found_items);
+    throw grt::user_cancelled("User cancelled password lookup.");
+  }
+
   if (result != GNOME_KEYRING_RESULT_OK)
   {
     if (found_items)
@@ -686,8 +749,8 @@ void TransparentMessage::show_message(const std::string& title, const std::strin
 #if GTK_VERSION_GE(2,12)
   window->set_opacity(0.85);
 #endif
-  window->process_updates(true);
   show_all();
+  window->process_updates(true);
 }
 
 void TransparentMessage::run()
@@ -788,13 +851,20 @@ bool UtilitiesImpl::move_to_trash(const std::string &path)
 //------------------------------------------------------------------------------
 void UtilitiesImpl::reveal_file(const std::string &path)
 {
+  std::string dirname = base::dirname(path);
   const gchar *argv[] = {
     "xdg-open",
-    base::dirname(path).c_str(),
+    dirname.c_str(),
     NULL 
   };
   GError *error = NULL;
-  if (!g_spawn_async(NULL, (gchar**)argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error))
+  char **envp = g_get_environ();
+  envp = wb_environ_unsetenv_internal(envp, "LD_PRELOAD");
+
+  gboolean result = g_spawn_async(NULL, (gchar**)argv, envp, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error);
+  g_strfreev(envp);
+
+  if (!result)
   {
     char *err = g_strdup_printf("Error opening folder with xdg-open: %s", error->message);
     g_error_free(error);
@@ -859,18 +929,19 @@ void UtilitiesImpl::stop_cancelable_wait_message()
   }
 }
 
+static std::map<std::string, Glib::RefPtr<Gdk::Pixbuf> > icon_cache;
    
 Glib::RefPtr<Gdk::Pixbuf> UtilitiesImpl::get_cached_icon(const std::string &icon)
 {
-  std::map<std::string, Glib::RefPtr<Gdk::Pixbuf> > cache;
 
-  if (cache.find(icon) != cache.end())
-    return cache[icon];
+
+  if (icon_cache.find(icon) != icon_cache.end())
+    return icon_cache[icon];
 
   if (icon == "folder")
   {
     Glib::RefPtr<Gdk::Pixbuf> pix = get_mainwindow()->render_icon(Gtk::Stock::DIRECTORY, Gtk::ICON_SIZE_MENU);
-    cache[icon] = pix;
+    icon_cache[icon] = pix;
     return pix;
   }
   else
@@ -878,8 +949,8 @@ Glib::RefPtr<Gdk::Pixbuf> UtilitiesImpl::get_cached_icon(const std::string &icon
     std::string path = mforms::App::get()->get_resource_path(icon);
     if (!path.empty() && g_file_test(path.c_str(), G_FILE_TEST_IS_REGULAR))
     {
-      cache[icon] = Gdk::Pixbuf::create_from_file(path);
-      return cache[icon];
+      icon_cache[icon] = Gdk::Pixbuf::create_from_file(path);
+      return icon_cache[icon];
     }
     else
       g_warning("Can't find icon %s", icon.c_str());
