@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -35,7 +35,7 @@ Gtk::Widget *ViewImpl::get_inner() const
 }
 
 ViewImpl::ViewImpl(::mforms::View *view)
-  : ObjectImpl(view),  _last_btn_down(NULL), _target(NULL), _drag_image(NULL)
+  : ObjectImpl(view), _back_image_alignment(mforms::NoAlign),  _last_btn_down(NULL), _target(NULL), _drag_image(NULL)
 {}
 
 
@@ -839,13 +839,9 @@ void ViewImpl::slot_drag_data_received(const Glib::RefPtr<Gdk::DragContext> &con
   }
   else
   {
-    //We need to remove scheme if it exists:
-    std::string file_schema = "file://";
+    //We need to convert uris to file names
     for(std::vector<std::string>::iterator it = files.begin(); it < files.end(); ++it)
-    {
-      if((*it).compare(0, file_schema.size(), file_schema) == 0)
-        (*it) = (*it).substr(file_schema.size());
-    }
+        (*it) = Glib::filename_from_uri((*it));
 
     drop_delegate->files_dropped((mforms::View*)owner, base::Point(x, y), allowedOperations, files);
   }
