@@ -16,8 +16,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-#ifndef _MFORMS_BOX_H_
-#define _MFORMS_BOX_H_
+
+#pragma once
 
 #include <mforms/container.h>
 #include <map>
@@ -31,12 +31,12 @@ namespace mforms {
 #ifndef SWIG
   struct BoxImplPtrs
   {
-    bool (__stdcall *create)(Box *self, bool horizontal);
-    void (__stdcall *add)(Box *self, View *child, bool expand, bool fill);
-    void (__stdcall *add_end)(Box *self, View *child, bool expand, bool fill);
-    void (__stdcall *remove)(Box *self, View *child);
-    void (__stdcall *set_homogeneous)(Box *self, bool);
-    void (__stdcall *set_spacing)(Box *self, int);
+    bool (*create)(Box *self, bool horizontal);
+    void (*add)(Box *self, View *child, bool expand, bool fill);
+    void (*add_end)(Box *self, View *child, bool expand, bool fill);
+    void (*remove)(Box *self, View *child);
+    void (*set_homogeneous)(Box *self, bool);
+    void (*set_spacing)(Box *self, int);
   };
 #endif
 #endif
@@ -93,5 +93,3 @@ namespace mforms {
   };
   
 };
-
-#endif /* _MFORMS_BOX_H_ */
