@@ -51,6 +51,8 @@ namespace mforms {
     LanguageHtml,   // includes embedded xml, javascript, php, vb, python
     LanguagePython,
     LanguageCpp,    // Lexer for C++, C, Java, and JavaScript (which includes JSON).
+    LanguageJS,
+    LanguageJson,
 
     LanguageMySQL = LanguageMySQL56, // Always the latest (released) language.
   };
@@ -206,7 +208,6 @@ public:
 
     /** Replaces the text in the editor. */
     void set_text(const char* text);
-
     void set_value(const std::string& text);
 
     /** Replaces the text in the editor but preserves top line, caret position and selection.
