@@ -17,8 +17,7 @@
  * 02110-1301  USA
  */
 
-#ifndef _MFORMS_DRAWBOX_H_
-#define _MFORMS_DRAWBOX_H_
+#pragma once
 
 #include <mforms/base.h>
 #include <mforms/view.h>
@@ -121,5 +120,3 @@ namespace mforms
     DrawBoxImplPtrs *_drawbox_impl;
   };
 };
-
-#endif
