@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,8 +16,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-#ifndef _MFILECHOOSERS_FILECHOOSER_H_
-#define _MFILECHOOSERS_FILECHOOSER_H_
+
+#pragma once
 
 #include "base/file_utilities.h"
 #include <mforms/base.h>
@@ -39,18 +39,18 @@ namespace mforms {
 #ifndef SWIG
   struct MFORMS_EXPORT FileChooserImplPtrs
   {
-    bool (__stdcall *create)(FileChooser *self, mforms::Form *owner, FileChooserType type, bool show_hidden);
-    void (__stdcall *set_title)(FileChooser *self, const std::string &title);
-    bool (__stdcall *run_modal)(FileChooser *self);
-    void (__stdcall *set_directory)(FileChooser *self, const std::string &path);
-    void (__stdcall *set_path)(FileChooser *self, const std::string &path);//TODO windows
-    std::string (__stdcall *get_directory)(FileChooser *self);
-    std::string (__stdcall *get_path)(FileChooser *self);
-    void (__stdcall *set_extensions)(FileChooser *self, const std::string &extensions, const std::string &default_extension);
+    bool (*create)(FileChooser *self, mforms::Form *owner, FileChooserType type, bool show_hidden);
+    void (*set_title)(FileChooser *self, const std::string &title);
+    bool (*run_modal)(FileChooser *self);
+    void (*set_directory)(FileChooser *self, const std::string &path);
+    void (*set_path)(FileChooser *self, const std::string &path);
+    std::string (*get_directory)(FileChooser *self);
+    std::string (*get_path)(FileChooser *self);
+    void (*set_extensions)(FileChooser *self, const std::string &extensions, const std::string &default_extension, bool allow_all_file_types);
 
-    void (__stdcall *add_selector_option)(FileChooser *self, const std::string &name, 
-        const std::string &label, const std::vector<std::pair<std::string, std::string> > &options);//TODO windows
-    std::string (__stdcall *get_selector_option_value)(FileChooser *self, const std::string &name); //TODO windows
+    void (*add_selector_option)(FileChooser *self, const std::string &name, 
+        const std::string &label, const std::vector<std::pair<std::string, std::string> > &options);
+    std::string (*get_selector_option_value)(FileChooser *self, const std::string &name);
   };
 #endif
 #endif
@@ -102,7 +102,7 @@ namespace mforms {
      The format is "Foo files (*.foo)|*.foo|SQL Scripts (*.sql)|*.sql"
      default_extension selects the default (ie "foo")
      */
-    void set_extensions(const std::string &extensions, const std::string &default_extension);
+    void set_extensions(const std::string &extensions, const std::string &default_extension, bool allow_all_file_types = true);
     
     /** Adds a selector type option to the file dialog, with the givan label and list of value caption/identifiers.
      If name is 'format', the option is treated as a file extension, where the identifier of the option is the file extension */
@@ -116,4 +116,3 @@ namespace mforms {
     static std::string last_directory;
   };
 };
-#endif
