@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,7 @@
  * 02110-1301  USA
  */
 
-#ifndef _MFORMS_MENU_H_
-#define _MFORMS_MENU_H_
+#pragma once
 
 #include <map>
 #include "boost/signals2.hpp"
@@ -31,10 +30,6 @@ namespace mforms {
   class Menu;
 }
 
-#ifdef _WIN32
-  #pragma make_public(mforms::Menu) // To allow using Menu in interface functions in assemblies.
-#endif
-
 namespace mforms {
  
 #ifndef DOXYGEN_SHOULD_SKIP_THIS 
@@ -92,5 +87,3 @@ namespace mforms {
 #endif
   };
 };
-
-#endif
