@@ -17,8 +17,7 @@
  * 02110-1301  USA
  */
 
-#ifndef _MFORMS_POPUP_H_
-#define _MFORMS_POPUP_H_
+#pragma once
 
 #include <mforms/base.h>
 #include <mforms/view.h>
@@ -44,13 +43,13 @@ namespace mforms
 #ifndef SWIG
   struct PopupImplPtrs
   {
-    bool (__stdcall *create)(Popup *, PopupStyle);
-    void (__stdcall *destroy)(Popup *);                      // TODO: Linux
-    void (__stdcall *set_needs_repaint)(Popup *);
-    void (__stdcall *set_size)(Popup *, int, int);
-    int (__stdcall *show)(Popup *, int, int);
-    base::Rect (__stdcall *get_content_rect)(Popup *);
-    void (__stdcall *set_modal_result)(Popup *, int result);
+    bool (*create)(Popup *, PopupStyle);
+    void (*destroy)(Popup *);
+    void (*set_needs_repaint)(Popup *);
+    void (*set_size)(Popup *, int, int);
+    int (*show)(Popup *, int, int);
+    base::Rect (*get_content_rect)(Popup *);
+    void (*set_modal_result)(Popup *, int result);
   };
 #endif
 #endif
@@ -75,13 +74,13 @@ namespace mforms
 
     virtual void repaint(cairo_t *cr, int x, int y, int w, int h) {}
 
-    virtual void mouse_down(int button, int x, int y) {}
-    virtual void mouse_up(int button, int x, int y) {}
-    virtual void mouse_click(int button, int x, int y) {}
-    virtual void mouse_double_click(int button, int x, int y) {}
-    virtual void mouse_enter() {}
-    virtual void mouse_leave() {}
-    virtual void mouse_move(int x, int y) {}
+    virtual bool mouse_down(mforms::MouseButton button, int x, int y) { return false; }
+    virtual bool mouse_up(mforms::MouseButton button, int x, int y) { return false; }
+    virtual bool mouse_click(mforms::MouseButton button, int x, int y) { return false; }
+    virtual bool mouse_double_click(mforms::MouseButton button, int x, int y) { return false; }
+    virtual bool mouse_enter() { return false; }
+    virtual bool mouse_leave() { return false; }
+    virtual bool mouse_move(mforms::MouseButton button, int x, int y) { return false; }
 #endif
 #endif
   protected:
@@ -89,5 +88,3 @@ namespace mforms
     boost::signals2::signal<void ()> _on_close; // Callback for non-modal popups.
   };
 };
-
-#endif // _MFORMS_POPUP_H_
