@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,7 @@
  * 02110-1301  USA
  */
 
-#ifndef _MFORMS_TABSWITCHER_H_
-#define _MFORMS_TABSWITCHER_H_
+#pragma once
 
 #include <vector>
 
@@ -31,84 +30,74 @@
 
 namespace mforms
 {
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
-#ifndef SWIG
-   struct TabItem {
-    std::string title;
-    std::string sub_title;
-    cairo_surface_t *icon;
-    cairo_surface_t *alt_icon;
-    int width;
-    int text_width;             // The width of the widest entry (either main caption or sub caption).
-    int text_height;            // The height of the entire text.
-    int text_y_advance;         // Vertical offset from the main caption to the sub caption.
-
-    TabItem() : icon(0), alt_icon(0), width(0) {}
-    ~TabItem() { if (icon) cairo_surface_destroy(icon); if (alt_icon) cairo_surface_destroy(alt_icon); }
+  enum TabSwitcherType {
+    VerticalIconSwitcher
   };
-#endif // !SWIG
-#endif
 
   enum TabElementPart {
-    TabBackground  = 0,
-    TabMainCaption = 1,
-    TabSubCaption  = 2,
-    TabLineColor   = 3
+    TabActiveBackground  = 0,
+    TabInactiveBackground,
+    TabActiveForeground,
+    TabInactiveForeground,
+    
+    TabMainCaption,
+    TabSubCaption,
+    TabLineColor,
+    TabLastElement
   };
   
+  class TabSwitcherPimpl;
+  
   class MFORMS_EXPORT TabSwitcher : public DrawBox
   {
   public:
-    TabSwitcher();
+    TabSwitcher(TabSwitcherType type);
     virtual ~TabSwitcher();
     
     void attach_to_tabview(TabView *tabView);
     
     int add_item(const std::string &title, const std::string &sub_title,
                   const std::string &icon_path, const std::string &alt_icon_path);
+    void remove_item(int index);
+
     void set_icon(int index, const std::string &icon_path, const std::string &alt_icon_path);
 
-    void set_color(TabElementPart parts, base::Color new_color);
-    
     void set_selected(int index);
-    int get_selected() { return _selected; }
+    int get_selected();
     void set_needs_relayout();
     
+    void set_collapsed(bool flag);
+    bool get_collapsed();
+    
 #ifndef SWIG
     boost::signals2::signal<void ()>* signal_changed() { return &_signal_changed; };
+    boost::signals2::signal<void ()>* signal_collapse_changed() { return &_signal_collapse_changed; };
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
     virtual int get_preferred_height();
     virtual void set_layout_dirty();
     
     virtual void repaint(cairo_t *cr, int x, int y, int w, int h);
-    virtual void mouse_down(int button, int x, int y);
-    virtual void mouse_up(int button, int x, int y);
-    virtual void mouse_click(int button, int x, int y);
+    virtual bool mouse_down(mforms::MouseButton button, int x, int y);
+    virtual bool mouse_up(mforms::MouseButton button, int x, int y);
+    virtual bool mouse_click(mforms::MouseButton button, int x, int y);
+    virtual bool mouse_enter();
+    virtual bool mouse_leave();
 #endif
 #endif
   private:
+    TabSwitcherPimpl *_pimpl;
     boost::signals2::signal<void ()> _signal_changed;
+    boost::signals2::signal<void ()> _signal_collapse_changed;
     TabView *_tabView;
-    base::Color _colors[4];
-    int _selected;
+    TimeoutHandle _timeout;
+
     int _last_clicked;
-
-    std::vector<TabItem*> _items;
     bool _needs_relayout;
+    bool _was_collapsed;
     
-    // Patterns for drawing the selected tab visual cues.
-    cairo_pattern_t* _background_pattern;
-    cairo_pattern_t* _line_pattern;
-
-    void prepare_patterns();
-    void destroy_patterns();
-    void layout(cairo_t *cr);
-    int index_from_point(int x, int y);
     void tab_changed();
+    
+    bool collapse();
   };
 };
-
-#endif
-
-
