@@ -21,8 +21,6 @@
 
 #include "base/log.h"
 #include "base/threading.h"
-#include <string.h>
-#include <stdlib.h>
 #include <errno.h>
 #include <cstdlib>
 #include <cstring>
