@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,7 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-#include "stdafx.h"
 
 #include "mforms/mforms.h"
 
@@ -72,9 +71,7 @@ void RadioButton::callback()
     _updating = true;
     _radio_activated(_group_id);
     _updating = false;
-    _signal_toggled();
   }
-
+  // We leave it here as there's additional if(!_updating) check inside the callback()
   Button::callback();
 }
-
