@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "mforms/mforms.h"
 
 using namespace mforms;
@@ -207,11 +205,13 @@ void HeaderBox::repaint(cairo_t *cr, int x, int y, int w, int h)
 
 //--------------------------------------------------------------------------------------------------
 
-void HeaderBox::mouse_down(int button, int x, int y)
+bool HeaderBox::mouse_down(mforms::MouseButton button, int x, int y)
 {
   // Check if the mouse position is on the icon and toggle the section box if that is the case.
   if (_owner->_expandable && x >= _icon_left && x <= _icon_right && y >= _icon_top && y <= _icon_bottom)
     _owner->toggle();
+
+  return true;
 }
 
 //----------------- SectionBox ---------------------------------------------------------------------
