@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,7 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-#include "tut_stdafx.h"
 
 #include "workbench/wb_context.h"
 
@@ -48,7 +47,6 @@
 #include "../stub_treenodeview.h"
 #include "../stub_codeeditor.h"
 #include "../stub_toolbar.h"
-#include "../stub_grid.h"
 #include "mforms/dockingpoint.h"
 
 class DockingPointDelegate : public mforms::DockingPointDelegate
@@ -60,45 +58,48 @@ class DockingPointDelegate : public mforms::DockingPointDelegate
   virtual bool select_view(mforms::AppView *view) { return false; }
   virtual void undock_view(mforms::AppView *view) {}
   virtual void set_view_title(mforms::AppView *view, const std::string &title) {}
-  virtual void get_size(int *w, int *h) {}
+  virtual std::pair<int, int> get_size() { return std::make_pair(0, 0); }
+
+  virtual mforms::AppView *selected_view() { return NULL; }
+  virtual int view_count() { return 0; }
+  virtual mforms::AppView *view_at_index(int index) { return NULL; }
 };
 
 DockingPointDelegate deleg;
 
-boost::function<void (std::string)> mforms::stub::UtilitiesImpl::open_url_slot;
+boost::function<void (std::string)> mforms::stub::UtilitiesWrapper::open_url_slot;
 
 void ::mforms::stub::init(wb::WBOptions *options)
 {
   App::instantiate(&deleg, false);
-  AppImpl::init(options);
-  ViewImpl::init();
-  BoxImpl::init();
-  FormImpl::init();
-  ButtonImpl::init();
-  RadioButtonImpl::init();
-  CheckBoxImpl::init();
-  TextEntryImpl::init();
-  TextBoxImpl::init();
-  LabelImpl::init();
-  ImageBoxImpl::init();
-  SelectorImpl::init();
-  ListBoxImpl::init();
-  PanelImpl::init();
-  TabViewImpl::init();
-  FileChooserImpl::init();
-  ProgressBarImpl::init();
-  TableImpl::init();
-  ScrollPanelImpl::init();
-  WizardImpl::init();
-  UtilitiesImpl::init();
-  DrawBoxImpl::init();
-  SplitterImpl::init();
-  MenuImpl::init();
-  MenuItemImpl::init();
-  CodeEditorImpl::init();
-  ToolBarImpl::init();
-  GridImpl::init();
-  TreeNodeViewImpl::init();
+  AppWrapper::init(options);
+  ViewWrapper::init();
+  BoxWrapper::init();
+  FormWrapper::init();
+  ButtonWrapper::init();
+  RadioButtonWrapper::init();
+  CheckBoxWrapper::init();
+  TextEntryWrapper::init();
+  TextBoxWrapper::init();
+  LabelWrapper::init();
+  ImageBoxWrapper::init();
+  SelectorWrapper::init();
+  ListBoxWrapper::init();
+  PanelWrapper::init();
+  TabViewWrapper::init();
+  FileChooserWrapper::init();
+  ProgressBarWrapper::init();
+  TableWrapper::init();
+  ScrollPanelWrapper::init();
+  WizardWrapper::init();
+  UtilitiesWrapper::init();
+  DrawBoxWrapper::init();
+  SplitterWrapper::init();
+  MenuWrapper::init();
+  MenuItemWrapper::init();
+  CodeEditorWrapper::init();
+  ToolBarWrapper::init();
+  TreeNodeViewWrapper::init();
 
   if (getenv("VERBOSE"))
     puts("done setting up mforms stubs...");
