@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,25 +24,25 @@
 namespace mforms {
 namespace stub {
 
-class CodeEditorImpl : public ViewImpl
+class CodeEditorWrapper : public ViewWrapper
 {
  protected:
-  CodeEditorImpl(::mforms::CodeEditor *self)
-    : ViewImpl(self)
+  CodeEditorWrapper(::mforms::CodeEditor *self)
+    : ViewWrapper(self)
   {
   }
 
-  static bool __stdcall create(CodeEditor* self)
+  static bool create(CodeEditor* self)
   {
     return true;
   }
 
-  static sptr_t __stdcall send_editor(CodeEditor* self, unsigned int message, uptr_t wParam, sptr_t lParam)
+  static sptr_t send_editor(CodeEditor* self, unsigned int message, uptr_t wParam, sptr_t lParam)
   {
     return 0;
   }
 
-  static void __stdcall set_status_text(CodeEditor* self, const std::string &text)
+  static void set_status_text(CodeEditor* self, const std::string &text)
   {
   }
 
@@ -51,9 +51,9 @@ public:
   {
     ::mforms::ControlFactory *f = ::mforms::ControlFactory::get_instance();
 
-    f->_code_editor_impl.create = &CodeEditorImpl::create;
-    f->_code_editor_impl.send_editor = &CodeEditorImpl::send_editor;
-    f->_code_editor_impl.set_status_text = &CodeEditorImpl::set_status_text;
+    f->_code_editor_impl.create = &CodeEditorWrapper::create;
+    f->_code_editor_impl.send_editor = &CodeEditorWrapper::send_editor;
+    f->_code_editor_impl.set_status_text = &CodeEditorWrapper::set_status_text;
   }
 };
 
