@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,11 +21,15 @@
 
 #include "stub_base.h"
 
+namespace wb {
+  struct WBOptions;
+};
+
 namespace mforms
 { 
 namespace stub 
 {
-  void init();
+  void init(wb::WBOptions *options);
   void check();
 };
 };
