@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,40 +24,40 @@
 namespace mforms {
 namespace stub {
 
-class PopupImpl : public ViewImpl
+class PopupImpl : public ViewWrapper
 {
  protected:
   PopupImpl(::mforms::WebBrowser *self)
-    : ViewImpl(self)
+    : ViewWrapper(self)
   {
   }
 
-  static bool __stdcall create(Popup *, PopupStyle)
+  static bool create(Popup *, PopupStyle)
   {
     return true;
   }
 
-  static void __stdcall set_needs_repaint(Popup *)
+  static void set_needs_repaint(Popup *)
   {
   }
 
-  static void __stdcall set_size(Popup *, int, int)
+  static void set_size(Popup *, int, int)
   {
   }
 
-  static int __stdcall show(Popup *, int, int)
+  static int show(Popup *, int, int)
   {
     return 0;
   }
 
-  static base::Rect __stdcall get_content_rect(Popup *)
+  static base::Rect get_content_rect(Popup *)
   {
     base::Rect ret_val;
 
     return ret_val;
   }
 
-  static void __stdcall set_modal_result(Popup *, int result)
+  static void set_modal_result(Popup *, int result)
   {
   }
 
