@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2013, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,3 +20,8 @@
 #pragma once
 
 #define HAVE_ROUND
+
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
+
+#include <glib.h>
