@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,7 +16,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-#include "stdafx.h"
+
+#include "mforms/mforms.h"
 
 using namespace mforms;
 
@@ -49,38 +50,31 @@ void Table::set_column_spacing(int c)
 
 void Table::add(View *subview, int row_left, int row_right, int col_top, int col_bottom, int flags)
 {
-  if (subview == this)
-    throw std::logic_error("mforms: Can't add a table inside itself");
-  
-  subview->retain();
-  subview->set_parent(this);
-  
   if (row_left > row_right)
     throw std::invalid_argument("table cell left must be <= right");
   if (col_top > col_bottom)
     throw std::invalid_argument("table cell top must be <= bottom");
   
+  cache_view(subview);
   _table_impl->add(this, subview, row_left, row_right, col_top, col_bottom, flags);
   subview->show();
-  add_subview(subview);
-  
-  relayout();
+#ifdef _WIN32 // XXX this shouldn't be needed here, the plat specific code is supposed to do this
+  //relayout();
+#endif
 }
 
-void Table::remove(View *subview)
+void Table::remove(View *sv)
 {
-  subview->set_parent(0);
-  remove_subview(subview);
-  (*_table_impl->remove)(this, subview);
-
-  subview->release();
-  
-  relayout();
+  _table_impl->remove(this, sv);
+  View::remove_from_cache(sv);
+#ifdef _WIN32 // XXX this shouldn't be needed here, the plat specific code is supposed to do this
+    relayout();
+#endif
 }
 
 
 void Table::set_homogeneous(bool value)
 {
-  (*_table_impl->set_homogeneous)(this, value);
+  _table_impl->set_homogeneous(this, value);
 }
 
