@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,9 +17,10 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "mforms/mforms.h"
+#include "base/log.h"
+
+DEFAULT_LOG_DOMAIN("mforms");
 
 using namespace mforms;
 
@@ -83,14 +84,28 @@ void ToolBar::add_item(ToolBarItem *item)
 }
 
 
+ToolBarItem *ToolBar::add_separator_item(const std::string &name)
+{
+  ToolBarItem *item = mforms::manage(new ToolBarItem(SeparatorItem));
+  item->set_name(name);
+  add_item(item);
+  return item;
+}
+
+
 void ToolBar::insert_item(int index, ToolBarItem *item)
 {
   assert(item->is_managed());
 
   if (index < 0 || index > (int) _items.size())
-    index = _items.size();
+    index = (int)_items.size();
   _impl->insert_item(this, index, item);
-  item->retain();
+
+  if (!item->_release_on_add) // Means: don't increase the ref count, the caller retained already, but won't release.
+    item->retain();
+  else
+    item->_release_on_add = false;
+
   _items.push_back(item);
 }
 
@@ -117,7 +132,7 @@ void ToolBar::remove_item(ToolBarItem *item)
 
 
 ToolBarItem::ToolBarItem(ToolBarItemType type, const bool expandable)
-: _type(type), _expandable(expandable)
+: _type(type), _updating(false), _expandable(expandable)
 {
   _impl = &mforms::ControlFactory::get_instance()->_tool_bar_impl;
   _impl->create_tool_item(this, type);
@@ -125,7 +140,9 @@ ToolBarItem::ToolBarItem(ToolBarItemType type, const bool expandable)
 
 void ToolBarItem::set_text(const std::string &text)
 {
+  _updating = true;
   _impl->set_item_text(this, text);
+  _updating = false;
 }
 
 std::string ToolBarItem::get_text()
@@ -163,7 +180,9 @@ bool ToolBarItem::get_enabled()
 
 void ToolBarItem::set_checked(bool flag)
 {
+  _updating = true;
   _impl->set_item_checked(this, flag);
+  _updating = false;
 }
 
 
@@ -180,12 +199,23 @@ void ToolBarItem::set_name(const std::string &name)
 
 void ToolBarItem::set_selector_items(const std::vector<std::string>& values)
 {
+  _updating = true;
   _impl->set_selector_items(this, values);
+  _updating = false;
 }
 
 void ToolBarItem::callback()
 {
-  _clicked_signal(this);
+  try
+  {
+    if (!_updating)
+      _clicked_signal(this);
+  }
+  catch (std::exception &exc)
+  {
+    log_error("Unhandled exception in toolbar callback for %s: %s\n", _name.c_str(), exc.what());
+    mforms::Utilities::show_error("Unhandled Exception", exc.what(), "OK");
+  }
 }
 
 void ToolBarItem::validate()
