@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -133,6 +133,19 @@ TreeNodeRef TreeNode::find_child_with_tag(const std::string &tag)
   return TreeNodeRef();
 }
 
+//--------------------------------------------------------------------------------------------------
+
+void TreeNode::toggle()
+{
+  if (can_expand())
+  {
+    if (!is_expanded())
+      expand();
+    else
+      collapse();
+  }
+}
+
 //----------------- TreeNodeView -------------------------------------------------------------------
 
 TreeNodeView::TreeNodeView(TreeOptions options)
@@ -267,18 +280,24 @@ void TreeNodeView::clear_selection()
 
 void TreeNodeView::select_node(TreeNodeRef node)
 {
-  _update_count++;
-  clear_selection();
-  _treeview_impl->set_selected(this, node, true);
-  _update_count--;
+  if (node.is_valid())
+  {
+    _update_count++;
+    clear_selection();
+    _treeview_impl->set_selected(this, node, true);
+    _update_count--;
+  }
 }
 
 
 void TreeNodeView::set_node_selected(TreeNodeRef node, bool flag)
 {
-  _update_count++;
-  _treeview_impl->set_selected(this, node, flag);
-  _update_count--;
+  if (node.is_valid())
+  {
+    _update_count++;
+    _treeview_impl->set_selected(this, node, flag);
+    _update_count--;
+  }
 }
 
 
