@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -29,6 +29,8 @@
 #include "mforms/mforms.h"
 #include "mforms/password_cache.h"
 
+#include "mdc_image.h"
+
 DEFAULT_LOG_DOMAIN(DOMAIN_MFORMS_BE);
 
 using namespace mforms;
@@ -50,11 +52,45 @@ void Utilities::beep()
 
 //--------------------------------------------------------------------------------------------------
 
+static void* _show_dialog(const DialogType type, const std::string &title, const std::string &text,
+    const std::string &ok, const std::string &cancel,
+    const std::string &other)
+{
+  int *ret = new int;
+  switch(type)
+  {
+  case DialogMessage:
+    *ret = ControlFactory::get_instance()->_utilities_impl.show_message(title, text, ok, cancel, other);
+    break;
+  case DialogWarning:
+    *ret = ControlFactory::get_instance()->_utilities_impl.show_warning(title, text, ok, cancel, other);
+    break;
+  case DialogError:
+    *ret = ControlFactory::get_instance()->_utilities_impl.show_error(title, text, ok, cancel, other);
+    break;
+  default:
+    *ret = mforms::ResultUnknown;
+  }
+
+  return (void*)ret;
+}
+
+static int void_to_int(void* val)
+{
+  int *ret = (int*)val;
+  int ret_val = *ret;
+  delete ret;
+  return ret_val;
+}
+
 int Utilities::show_message(const std::string &title, const std::string &text,
                             const std::string &ok, const std::string &cancel,
                             const std::string &other)
 {
-  return ControlFactory::get_instance()->_utilities_impl.show_message(title, text, ok, cancel, other);
+  if (Utilities::in_main_thread())
+    return void_to_int(_show_dialog(DialogMessage, title, text, ok, cancel, other));
+  else
+    return void_to_int(Utilities::perform_from_main_thread(boost::bind(&_show_dialog, DialogMessage, title, text, ok, cancel, other)));
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -63,7 +99,10 @@ int Utilities::show_error(const std::string &title, const std::string &text,
                           const std::string &ok, const std::string &cancel,
                           const std::string &other)
 {
-  return ControlFactory::get_instance()->_utilities_impl.show_error(title, text, ok, cancel, other);
+  if (Utilities::in_main_thread())
+    return void_to_int(_show_dialog(DialogError, title, text, ok, cancel, other));
+  else
+    return void_to_int(Utilities::perform_from_main_thread(boost::bind(&_show_dialog, DialogError, title, text, ok, cancel, other)));
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -72,7 +111,10 @@ int Utilities::show_warning(const std::string &title, const std::string &text,
                             const std::string &ok, const std::string &cancel,
                             const std::string &other)
 {
-  return ControlFactory::get_instance()->_utilities_impl.show_warning(title, text, ok, cancel, other);
+  if (Utilities::in_main_thread())
+    return void_to_int(_show_dialog(DialogWarning, title, text, ok, cancel, other));
+  else
+    return void_to_int(Utilities::perform_from_main_thread(boost::bind(&_show_dialog, DialogWarning, title, text, ok, cancel, other)));
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -114,7 +156,7 @@ void Utilities::set_message_answers_storage_path(const std::string &path)
       if (ptr)
       {
         *ptr= 0;
-        remembered_message_answers[line] = atoi(ptr+1);
+        remembered_message_answers[line] = base::atoi<int>(ptr+1, 0);
       }
     }
     fclose(f);
@@ -472,9 +514,17 @@ bool Utilities::find_cached_password(const std::string &service, const std::stri
 
 //--------------------------------------------------------------------------------------------------
 
+void Utilities::forget_cached_password(const std::string &service, const std::string &account)
+{
+  log_debug2("Forgetting cached password for '%s'@'%s'\n", account.c_str(), service.c_str());
+  PasswordCache::get()->remove_password(service, account);
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void Utilities::forget_password(const std::string &service, const std::string &account)
 {
-  PasswordCache::get()->remove_password(service, account);
+  Utilities::forget_cached_password(service, account);
   
   log_debug("Forgetting password for '%s'@'%s'\n", account.c_str(), service.c_str());
   ControlFactory::get_instance()->_utilities_impl.forget_password(service, account);
@@ -618,7 +668,7 @@ static bool _ask_for_password(const std::string &title, const std::string &servi
   if (Utilities::in_main_thread())
     return _ask_for_password_main(title, service, &username, prompt_storage, &ret_password, &ret_store) != NULL;
   else
-    return Utilities::perform_from_main_thread(boost::bind(&_ask_for_password_main, 
+    return Utilities::perform_from_main_thread(boost::bind(&_ask_for_password_main,
                                                title, service, &username, prompt_storage, &ret_password, &ret_store)) != NULL;
 }
 
@@ -736,10 +786,15 @@ cairo_surface_t* Utilities::load_icon(const std::string& name, bool allow_hidpi)
   if (name.empty())
     return NULL;
 
+#ifdef __APPLE__
+  allow_hidpi = true; // For OSX we always want hires images.
+#endif
+
   if (allow_hidpi && mforms::App::get()->backing_scale_factor() > 1.0)
   {
     std::string hidpi_name = base::strip_extension(name) + "@2x" + base::extension(name);
-    cairo_surface_t *tmp = load_icon(hidpi_name, false);
+    std::string icon_path = App::get()->get_resource_path(hidpi_name);
+    cairo_surface_t *tmp = mdc::surface_from_png_image(icon_path);
     if (tmp)
     {
       // Mark the surface as being a hi-res variant of a standard icon.
@@ -748,15 +803,8 @@ cairo_surface_t* Utilities::load_icon(const std::string& name, bool allow_hidpi)
     }
   }
 
-  std::string icon_path= App::get()->get_resource_path(name);
-  cairo_surface_t* result= cairo_image_surface_create_from_png(icon_path.c_str());
-  if (result && cairo_surface_status(result) != CAIRO_STATUS_SUCCESS)
-  {
-    cairo_surface_destroy(result);
-    result= NULL;
-  }
-
-  return result;
+  std::string icon_path = App::get()->get_resource_path(name);
+  return mdc::surface_from_png_image(icon_path);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -770,7 +818,7 @@ bool Utilities::is_hidpi_icon(cairo_surface_t *s)
 
 bool Utilities::icon_needs_reload(cairo_surface_t *s)
 {
-  float scale = s && mforms::Utilities::is_hidpi_icon(s) ? 2 : 1;
+  float scale = s && mforms::Utilities::is_hidpi_icon(s) ? 2.0f : 1.0f;
   return mforms::App::get()->backing_scale_factor() != scale;
 }
 
@@ -890,13 +938,11 @@ std::string Utilities::shorten_string(cairo_t* cr, const std::string& text, doub
   return "";
 }
 
+//--------------------------------------------------------------------------------------------------
 
 double Utilities::get_text_width(const std::string &text, const std::string &font)
 {
-  if (ControlFactory::get_instance()->_utilities_impl.get_text_width)
-    return ControlFactory::get_instance()->_utilities_impl.get_text_width(text, font);
-  else
-    return text.length() * 20; // temporary hardcoded until implemented in Windows and Linux
+  return ControlFactory::get_instance()->_utilities_impl.get_text_width(text, font);
 }
 
 //--------------------------------------------------------------------------------------------------
