@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,20 +17,19 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
-#include "mforms/mforms.h"
+#include "wf_base.h"
 #include "wf_view.h"
 #include "wf_box.h"
+#include "wf_label.h"
 
-#include "ConvUtils.h"
+using namespace System::Collections::Generic;
+using namespace System::Drawing;
+using namespace System::Windows::Forms;
 
 using namespace MySQL;
 using namespace MySQL::Forms;
 using namespace MySQL::Utilities;
 
-using namespace System::Drawing;
-
 ref class ControlEntry {
 public:
   Control^ control;
@@ -55,7 +54,7 @@ void maximize_children(ControlList% list, bool horizontal, Size containerSize, i
   {
     for each (ControlEntry^ entry in list)
     {
-      ViewImpl::remove_auto_resize(entry->control, AutoResizeMode::ResizeVertical);
+      ViewWrapper::remove_auto_resize(entry->control, AutoResizeMode::ResizeVertical);
       entry->bounds.Y = padding;
       entry->bounds.Height= containerSize.Height;
     };
@@ -64,14 +63,17 @@ void maximize_children(ControlList% list, bool horizontal, Size containerSize, i
   {
     for each (ControlEntry^ entry in list)
     {
-      ViewImpl::remove_auto_resize(entry->control, AutoResizeMode::ResizeHorizontal);
+      ViewWrapper::remove_auto_resize(entry->control, AutoResizeMode::ResizeHorizontal);
       entry->bounds.X = padding;
 
-      // Determine the preferred size based on the new width (for labels, as they can be in auto wrap mode).
-      if (entry->control->GetType()->IsSubclassOf(Windows::Forms::Label::typeid))
-        entry->bounds.Size= entry->control->GetPreferredSize(Size(containerSize.Width, 0));
+      // For labels determine the preferred height based on the new width (as they can be in auto wrap mode).
+      if (is<WrapControlLabel>(entry->control))
+      {
+        Size size = entry->control->GetPreferredSize(Size(containerSize.Width, 0));
+        entry->bounds.Size = Size(containerSize.Width, size.Height);
+      }
       else
-        entry->bounds.Width= containerSize.Width;
+        entry->bounds.Width = containerSize.Width;
     };
   }
 }
@@ -95,7 +97,7 @@ void apply_bounds(ControlList% list, bool horizontal)
     // The parameter horizontal tells us if the layout direction is horizontal or vertical.
     // Always resize orthogonal to the layout direction but resize in layout direction only if fill is set
     // or the computed size is smaller than the current size.
-    ViewImpl::remove_auto_resize(entry->control, AutoResizeMode::ResizeBoth);
+    ViewWrapper::remove_auto_resize(entry->control, AutoResizeMode::ResizeBoth);
     Drawing::Rectangle newBounds = entry->control->Bounds;
 
     if (horizontal)
@@ -103,7 +105,7 @@ void apply_bounds(ControlList% list, bool horizontal)
       // Workaround: usually, scaling a button vertically does not produce good results.
       // Buttons are better left at their normal height, so we switch that auto-scaling off.
       // In vertical layout the scaling is controllable by the fill flag.
-      if (entry->control->GetType() != Button::typeid)
+      if (!is<Button>(entry->control))
         newBounds.Height = entry->bounds.Height;
       if (entry->fills || entry->bounds.Width < newBounds.Width)
         newBounds.Width = entry->bounds.Width;
@@ -202,7 +204,7 @@ int get_largest_height(ControlList% list1, ControlList% list2)
  * @param fraction The amount of pixels the controls must be resized/moved.
  * @param mirrored Do the adjustment in a mirrored fashion (for right aligned controls).
  */
-void expand_horizontally(ControlList% list, Box^ box, int fraction, bool mirrored)
+void expand_horizontally(ControlList% list, LayoutBox ^box, int fraction, bool mirrored)
 {
   for (int i = 0; i < list.Count; i++)
   {
@@ -237,7 +239,7 @@ void expand_horizontally(ControlList% list, Box^ box, int fraction, bool mirrore
  * @param fraction The amount of pixels the controls must be resized/moved.
  * @param mirrored Do the adjustment in a mirrored fashion (for bottom aligned controls).
  */
-void expand_vertically(ControlList% list, Box^ box, int fraction, bool mirrored)
+void expand_vertically(ControlList% list, LayoutBox ^box, int fraction, bool mirrored)
 {
   for (int i= 0; i < list.Count; i++)
   {
@@ -275,7 +277,7 @@ void expand_vertically(ControlList% list, Box^ box, int fraction, bool mirrored)
  *                      (when doing a relayout) or not (when computing the preferred size).
  * @return The resulting size of the box.
  */
-System::Drawing::Size HorizontalGtkBoxLayout::ComputeLayout(Box^ box, System::Drawing::Size proposedSize,
+System::Drawing::Size HorizontalGtkBoxLayout::ComputeLayout(LayoutBox ^box, System::Drawing::Size proposedSize,
   bool resizeChildren)
 {
   ControlList left_aligned;
@@ -301,13 +303,16 @@ System::Drawing::Size HorizontalGtkBoxLayout::ComputeLayout(Box^ box, System::Dr
     entry->control = control;
     entry->fills = box->GetControlFills(control);
 
-    bool use_min_width = ViewImpl::use_min_width_for_layout(control);
-    bool use_min_height = ViewImpl::use_min_height_for_layout(control);
+    // Keep track of the tallest control, so we can adjust the container's height properly.
+    ViewWrapper::set_full_auto_resize(control);
+
+    bool use_min_width = ViewWrapper::use_min_width_for_layout(control);
+    bool use_min_height = ViewWrapper::use_min_height_for_layout(control);
     if (use_min_width && use_min_height)
       entry->bounds.Size = control->MinimumSize;
     else
     {
-      entry->bounds= System::Drawing::Rectangle(Point(0, 0), control->GetPreferredSize(Size(0, proposedSize.Height)));
+      entry->bounds = System::Drawing::Rectangle(Point(0, 0), control->GetPreferredSize(Size(0, proposedSize.Height)));
       if (use_min_width)
         entry->bounds.Width = control->MinimumSize.Width;
       if (use_min_height)
@@ -316,17 +321,28 @@ System::Drawing::Size HorizontalGtkBoxLayout::ComputeLayout(Box^ box, System::Dr
 
     // Weird behavior of combo boxes. They report a one pixel too small height (you cannot set them
     // to this smaller height, though). So this wrong value is messing up our computed overall height.
-    if (entry->control->GetType() == ComboBox::typeid)
+    if (is<ComboBox>(entry->control))
       entry->bounds.Height++;
     
     // Buttons have a very own understanding of their preferred size. Flat buttons always add 3 px padding
     // in addition to their normal Padding. System buttons ignore the image in size calculations.
     // We fix flat buttons for now as they can come with images only. Normal buttons usually have text.
-    if (entry->control->GetType() == Button::typeid)
+    if (is<Button>(entry->control))
     {
       Button ^button = (Button ^)entry->control;
       if (button->FlatStyle == FlatStyle::Flat)
         entry->bounds = Drawing::Rectangle::Inflate(entry->bounds, -3, -3);
+      else
+      {
+        // If a smaller size than the preferred size is set for a button use this instead.
+        if (button->MinimumSize.Height > 0 && button->MinimumSize.Height < control->PreferredSize.Height)
+          entry->bounds.Height = button->MinimumSize.Height;
+      }
+      if (entry->fills)
+      {
+        Button^ btn = dynamic_cast<Button^>(entry->control);
+        btn->AutoSizeMode = AutoSizeMode::GrowAndShrink;
+      }
     }
 
     // Sort control into the proper alignment list.
@@ -346,7 +362,7 @@ System::Drawing::Size HorizontalGtkBoxLayout::ComputeLayout(Box^ box, System::Dr
   }
 
   // Adjust height of the container if it is too small or auto resizing is enabled.
-  if (proposedSize.Height < max_height || ViewImpl::can_auto_resize_vertically(box))
+  if (proposedSize.Height < max_height || ViewWrapper::can_auto_resize_vertically(box))
   {
     proposedSize.Height = max_height;
     if (proposedSize.Height < box->MinimumSize.Height - box->Padding.Vertical)
@@ -414,7 +430,7 @@ System::Drawing::Size HorizontalGtkBoxLayout::ComputeLayout(Box^ box, System::Dr
     resulting_width += -offset - spacing - box->Padding.Right;
 
   // Adjust width of the container if it is too small or auto resizing is enabled.
-  if (proposedSize.Width < resulting_width || ViewImpl::can_auto_resize_horizontally(box))
+  if (proposedSize.Width < resulting_width || ViewWrapper::can_auto_resize_horizontally(box))
   {
     proposedSize.Width = resulting_width;
     if (proposedSize.Width < box->MinimumSize.Width - box->Padding.Horizontal)
@@ -455,26 +471,26 @@ System::Drawing::Size HorizontalGtkBoxLayout::ComputeLayout(Box^ box, System::Dr
 
 //--------------------------------------------------------------------------------------------------
 
-bool HorizontalGtkBoxLayout::Layout(Object^ container, LayoutEventArgs^ arguments)
+bool HorizontalGtkBoxLayout::Layout(Object ^container, LayoutEventArgs ^arguments)
 {
-  Box^ box = (Box^) container;
-  if (!ViewImpl::can_layout(box, arguments->AffectedProperty))
+  LayoutBox ^box = static_cast<LayoutBox^>(container);
+  if (!ViewWrapper::can_layout(box, arguments->AffectedProperty))
     return false;
 
-  ViewImpl::adjust_auto_resize_from_docking(box);
+  ViewWrapper::adjust_auto_resize_from_docking(box);
   System::Drawing::Size boxSize = ComputeLayout(box, box->Size, true);
 
   // Finally resize container if necessary.
   bool parentLayoutNeeded= !box->Size.Equals(boxSize);
   if (parentLayoutNeeded)
-    ViewImpl::resize_with_docking(box, boxSize);
+    ViewWrapper::resize_with_docking(box, boxSize);
 
   return parentLayoutNeeded;
 }
 
 //--------------------------------------------------------------------------------------------------
 
-System::Drawing::Size HorizontalGtkBoxLayout::GetPreferredSize(Box^ container,
+System::Drawing::Size HorizontalGtkBoxLayout::GetPreferredSize(LayoutBox ^container,
   System::Drawing::Size proposedSize)
 {
   proposedSize = ComputeLayout(container, proposedSize, false);
@@ -494,7 +510,7 @@ System::Drawing::Size HorizontalGtkBoxLayout::GetPreferredSize(Box^ container,
  *                       (when doing a relayout) or not (when computing the preferred size).
  * @return The resulting size of the box.
  */
-System::Drawing::Size VerticalGtkBoxLayout::ComputeLayout(Box^ box, System::Drawing::Size proposedSize,
+System::Drawing::Size VerticalGtkBoxLayout::ComputeLayout(LayoutBox ^box, System::Drawing::Size proposedSize,
   bool resizeChildren)
 {
   ControlList top_aligned;
@@ -526,10 +542,10 @@ System::Drawing::Size VerticalGtkBoxLayout::ComputeLayout(Box^ box, System::Draw
       top_aligned.Add(entry);
 
     // Keep track of the widest control, so we can adjust the container's width properly.
-    ViewImpl::set_full_auto_resize(control);
+    ViewWrapper::set_full_auto_resize(control);
 
-    bool use_min_width = ViewImpl::use_min_width_for_layout(control);
-    bool use_min_height = ViewImpl::use_min_height_for_layout(control);
+    bool use_min_width = ViewWrapper::use_min_width_for_layout(control);
+    bool use_min_height = ViewWrapper::use_min_height_for_layout(control);
     if (use_min_width && use_min_height)
       entry->bounds.Size = control->MinimumSize;
     else
@@ -543,9 +559,15 @@ System::Drawing::Size VerticalGtkBoxLayout::ComputeLayout(Box^ box, System::Draw
 
     // Weird behavior of combo boxes. They report a one pixel too small height (you cannot set them
     // to this smaller height, though). So this wrong value is messing up our computed overall height.
-    if (entry->control->GetType() == ComboBox::typeid)
+    if (is<ComboBox>(entry->control))
       entry->bounds.Height++;
 
+    if (is<Button>(entry->control) && entry->fills)
+    {
+      Button^ btn = dynamic_cast<Button^>(entry->control);
+      btn->AutoSizeMode = AutoSizeMode::GrowAndShrink;
+    }
+
     if (max_width < entry->bounds.Width)
       max_width= entry->bounds.Width;
 
@@ -556,7 +578,7 @@ System::Drawing::Size VerticalGtkBoxLayout::ComputeLayout(Box^ box, System::Draw
   }
 
   // Adjust height of the container if it is too small or auto resizing is enabled.
-  if (proposedSize.Width < max_width || ViewImpl::can_auto_resize_horizontally(box))
+  if (proposedSize.Width < max_width || ViewWrapper::can_auto_resize_horizontally(box))
   {
     proposedSize.Width = max_width;
     if (proposedSize.Width < box->MinimumSize.Width - box->Padding.Horizontal)
@@ -624,7 +646,7 @@ System::Drawing::Size VerticalGtkBoxLayout::ComputeLayout(Box^ box, System::Draw
     resulting_height += -offset - spacing - box->Padding.Bottom;
 
   // Adjust height of the container if it is too small or auto resizing is enabled.
-  if (proposedSize.Height < resulting_height || ViewImpl::can_auto_resize_vertically(box))
+  if (proposedSize.Height < resulting_height || ViewWrapper::can_auto_resize_vertically(box))
   {
     proposedSize.Height= resulting_height;
     if (proposedSize.Height < box->MinimumSize.Height - box->Padding.Vertical)
@@ -665,34 +687,34 @@ System::Drawing::Size VerticalGtkBoxLayout::ComputeLayout(Box^ box, System::Draw
 
 //--------------------------------------------------------------------------------------------------
 
-bool VerticalGtkBoxLayout::Layout(Object^ container, LayoutEventArgs^ arguments)
+bool VerticalGtkBoxLayout::Layout(Object ^container, LayoutEventArgs ^arguments)
 {
-  Box^ box = (Box^) container;
-  if (!ViewImpl::can_layout(box, arguments->AffectedProperty))
+  LayoutBox ^box = static_cast<LayoutBox^>(container);
+  if (!ViewWrapper::can_layout(box, arguments->AffectedProperty))
     return false;
 
-  ViewImpl::adjust_auto_resize_from_docking(box);
+  ViewWrapper::adjust_auto_resize_from_docking(box);
   System::Drawing::Size boxSize = ComputeLayout(box, box->Size, true);
 
   // Finally resize container if necessary.
   bool parentLayoutNeeded= !box->Size.Equals(boxSize);
   if (parentLayoutNeeded)
-    ViewImpl::resize_with_docking(box, boxSize);
+    ViewWrapper::resize_with_docking(box, boxSize);
 
   return parentLayoutNeeded;
 }
 
 //-------------------------------------------------------------------------------------------------
 
-System::Drawing::Size VerticalGtkBoxLayout::GetPreferredSize(Box^ container,
+System::Drawing::Size VerticalGtkBoxLayout::GetPreferredSize(LayoutBox ^container,
   System::Drawing::Size proposedSize)
 {
   return ComputeLayout(container, proposedSize, false);
 }
 
-//----------------- Box ---------------------------------------------------------------------------
+//----------------- LayoutBox ----------------------------------------------------------------------
 
-Box::Box()
+LayoutBox::LayoutBox()
 {
   SetStyle(ControlStyles::UserPaint, true);
   SetStyle(ControlStyles::AllPaintingInWmPaint, true);
@@ -708,150 +730,106 @@ Box::Box()
 
 //--------------------------------------------------------------------------------------------------
 
-System::Drawing::Size Box::GetPreferredSize(System::Drawing::Size proposedSize)
+System::Drawing::Size LayoutBox::GetPreferredSize(System::Drawing::Size proposedSize)
 {
   return layoutEngine->GetPreferredSize(this, proposedSize);
 }
 
 //--------------------------------------------------------------------------------------------------
 
-void Box::OnPaintBackground(PaintEventArgs^ args)
+void LayoutBox::OnPaintBackground(PaintEventArgs^ args)
 {
   Panel::OnPaintBackground(args);
 
-  ViewImpl ^view = ObjectImpl::get_wrapper<ViewImpl>(this);
-  view->draw_background(args);
+  BoxWrapper *wrapper = BoxWrapper::GetWrapper<BoxWrapper>(this);
+  wrapper->DrawBackground(args);
 }
 
-//----------------- BoxImpl -----------------------------------------------------------------------
+//----------------- BoxWrapper -----------------------------------------------------------------------
 
-bool BoxImpl::create(mforms::Box *self, bool horizontal)
+BoxWrapper::BoxWrapper(mforms::Box *box)
+  : ViewWrapper(box)
 {
-  BoxImpl ^box = gcnew BoxImpl(self);
+}
 
-  if (box != nullptr)
-  {
-    Box^ native_box = ViewImpl::create<Box>(self, box);
-    native_box->Horizontal = horizontal;
-    return true;
-  }
-  return false;
+//--------------------------------------------------------------------------------------------------
+
+bool BoxWrapper::create(mforms::Box *backend, bool horizontal)
+{
+  BoxWrapper *wrapper = new BoxWrapper(backend);
+  LayoutBox ^box = Create<LayoutBox>(backend, wrapper);
+  box->Horizontal = horizontal;
+
+  return true;
 }
 
 //-------------------------------------------------------------------------------------------------
 
-void BoxImpl::add(mforms::Box *self, mforms::View *child, bool expand, bool fill)
+void BoxWrapper::add(mforms::Box *backend, mforms::View *child, bool expand, bool fill)
 {
-  BoxImpl^ box= (BoxImpl^)ObjectImpl::FromUnmanaged(self);
-
-  if (box != nullptr)
-  {
-    box->add(((ViewImpl^)ObjectImpl::FromUnmanaged(child))->get_control<Control>(), expand, fill);
-    self->set_layout_dirty(true);
-  }
-}
-
-//-------------------------------------------------------------------------------------------------
-
-void BoxImpl::add_end(mforms::Box *self, mforms::View *child, bool expand, bool fill)
-{
-  BoxImpl^ box= (BoxImpl^)ObjectImpl::FromUnmanaged(self);
-
-  if (box != nullptr)
-  {
-    box->add_end(((ViewImpl^)ObjectImpl::FromUnmanaged(child))->get_control<Control>(), expand, fill);
-    self->set_layout_dirty(true);
-  }
-}
-
-//-------------------------------------------------------------------------------------------------
-
-void BoxImpl::remove(mforms::Box *self, mforms::View *child)
-{
-  BoxImpl^ box= (BoxImpl^)ObjectImpl::FromUnmanaged(self);
-
-  if (box != nullptr)
-  {
-    box->remove(((ViewImpl^)ObjectImpl::FromUnmanaged(child))->get_control<Control>());
-    self->set_layout_dirty(true);
-  }
-}
-
-//-------------------------------------------------------------------------------------------------
-
-void BoxImpl::set_spacing(mforms::Box *self, int space)
-{
-  BoxImpl^ box= (BoxImpl^)ObjectImpl::FromUnmanaged(self);
-
-  if (box != nullptr)
-  {
-    box->set_spacing(space);
-    self->set_layout_dirty(true);
-  }
-}
-
-//-------------------------------------------------------------------------------------------------
-
-void BoxImpl::set_homogeneous(mforms::Box *self, bool value)
-{
-  BoxImpl^ box= (BoxImpl^)ObjectImpl::FromUnmanaged(self);
-
-  if (box != nullptr)
-  {
-    box->set_homogeneous(value);
-    self->set_layout_dirty(true);
-  }
-}
-
-//-------------------------------------------------------------------------------------------------
-
-void BoxImpl::add(Windows::Forms::Control ^control, bool expand, bool fill)
-{
-  Box ^box= get_control<Box>();
-
-  if (box->Horizontal)
-    control->Dock= DockStyle::Left;
+  Control ^control = BoxWrapper::GetControl(child);
+  if (backend->is_horizontal())
+    control->Dock = DockStyle::Left;
   else 
-    control->Dock= DockStyle::Top;
+    control->Dock = DockStyle::Top;
 
+  LayoutBox ^box = BoxWrapper::GetManagedObject<LayoutBox>(backend);
   box->Add(control, expand, fill);
+  backend->set_layout_dirty(true);
 }
 
 //-------------------------------------------------------------------------------------------------
 
-void BoxImpl::add_end(Windows::Forms::Control ^control, bool expand, bool fill)
+void BoxWrapper::add_end(mforms::Box *backend, mforms::View *child, bool expand, bool fill)
 {
-  Box ^box= get_control<Box>();
-
-  if (box->Horizontal)
+  Control ^control = BoxWrapper::GetControl(child);
+  if (backend->is_horizontal())
     control->Dock= DockStyle::Right;
   else 
     control->Dock= DockStyle::Bottom;
 
+  LayoutBox ^box = BoxWrapper::GetManagedObject<LayoutBox>(backend);
   box->Add(control, expand, fill);
+  backend->set_layout_dirty(true);
 }
 
 //-------------------------------------------------------------------------------------------------
 
-void BoxImpl::remove(Windows::Forms::Control ^control)
+void BoxWrapper::remove(mforms::Box *backend, mforms::View *child)
 {
-  Box ^box= get_control<Box>();
-
-  box->Remove(control);
+  LayoutBox ^box = BoxWrapper::GetManagedObject<LayoutBox>(backend);
+  box->Remove(BoxWrapper::GetControl(child));
+  backend->set_layout_dirty(true);
 }
 
 //-------------------------------------------------------------------------------------------------
 
-void BoxImpl::set_homogeneous(bool homogeneous)
+void BoxWrapper::set_spacing(mforms::Box *backend, int space)
 {
-  get_control<Box>()->Homogeneous= homogeneous;
+  LayoutBox ^box = BoxWrapper::GetManagedObject<LayoutBox>(backend);
+  box->Spacing = space;
+  backend->set_layout_dirty(true);
 }
 
 //-------------------------------------------------------------------------------------------------
 
-void BoxImpl::set_spacing(int space)
+void BoxWrapper::set_homogeneous(mforms::Box *backend, bool value)
 {
-  get_control<Box>()->Spacing= space;
+  LayoutBox ^box = BoxWrapper::GetManagedObject<LayoutBox>(backend);
+  box->Homogeneous = value;
+  backend->set_layout_dirty(true);
 }
 
-//-------------------------------------------------------------------------------------------------
+//--------------------------------------------------------------------------------------------------
+
+void BoxWrapper::init()
+{
+  mforms::ControlFactory *f = mforms::ControlFactory::get_instance();
+
+  f->_box_impl.create = &BoxWrapper::create;
+  f->_box_impl.add = &BoxWrapper::add;
+  f->_box_impl.add_end = &BoxWrapper::add_end;
+  f->_box_impl.remove = &BoxWrapper::remove;
+  f->_box_impl.set_spacing = &BoxWrapper::set_spacing;
+  f->_box_impl.set_homogeneous = &BoxWrapper::set_homogeneous;
+}
