@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,22 +16,16 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-/**
- * Implementation of the label wrapper for the mforms library.
- */
 
-#include "stdafx.h"
-
-#include "mforms/mforms.h"
+#include "wf_base.h"
 #include "wf_view.h"
 #include "wf_label.h"
 
 #include "base/log.h"
-#include "ConvUtils.h"
+#include "base/drawing.h"
 
-using namespace System;
-using namespace Drawing;
-using namespace Windows::Forms;
+using namespace System::Drawing;
+using namespace System::Windows::Forms;
 
 using namespace MySQL::Forms;
 
@@ -74,6 +68,7 @@ WrapControlLabel::WrapControlLabel()
 {
   autoWrapping = false;
   lastSize = Drawing::Size(-1, -1);
+  ForeColor = Color::FromKnownColor(KnownColor::WindowText);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -142,138 +137,70 @@ Drawing::Size WrapControlLabel::GetPreferredSize(Drawing::Size proposedSize)
   return result;
 }
 
-//----------------- Static functions ---------------------------------------------------------------
+//----------------- LabelWrapper -------------------------------------------------------------------
 
-bool LabelImpl::create(mforms::Label *self)
-{
-  LabelImpl ^label= gcnew LabelImpl(self);
-
-  if (label != nullptr)
-  {
-    WrapControlLabel^ l= ViewImpl::create<WrapControlLabel>(self, label);
-    l->TextAlign= ContentAlignment::MiddleLeft;
-    l->BorderStyle= BorderStyle::None;
-    l->ForeColor = Color::Black;
-    return true;
-  }
-  return false;
-}
-
-//--------------------------------------------------------------------------------------------------
-
-void LabelImpl::set_style(mforms::Label *self, mforms::LabelStyle style)
-{
-  LabelImpl^ label= (LabelImpl^)ObjectImpl::FromUnmanaged(self);
-
-  if (label != nullptr)
-    label->set_style(style);
-}
-
-//--------------------------------------------------------------------------------------------------
-
-void LabelImpl::set_text(mforms::Label *self, const std::string &text)
-{
-  LabelImpl^ label= (LabelImpl^)ObjectImpl::FromUnmanaged(self);
-
-  if (label != nullptr)
-  {
-    if (!text.empty() && text[text.size() - 1] == '\n') // Add a space if there's a trailing linebreak. Otherwise it would be ignored.
-      label->set_text(text + " ");
-    else
-      label->set_text(text);
-    self->set_layout_dirty(true);
-  }
-}
-
-//--------------------------------------------------------------------------------------------------
-
-void LabelImpl::set_text_align(mforms::Label *self, mforms::Alignment align)
-{
-  LabelImpl^ label= (LabelImpl^)ObjectImpl::FromUnmanaged(self);
-
-  if (label != nullptr)
-  {
-    label->set_text_align(align);
-    self->set_layout_dirty(true);
-  }
-}
-
-//--------------------------------------------------------------------------------------------------
-
-void LabelImpl::set_color(mforms::Label *self, const std::string &color)
-{
-  LabelImpl^ label= (LabelImpl^)ObjectImpl::FromUnmanaged(self);
-
-  if (label != nullptr)
-    label->set_color(color);
-}
-
-//--------------------------------------------------------------------------------------------------
-
-void LabelImpl::set_wrap_text(mforms::Label *self, bool flag)
-{
-  LabelImpl^ label= (LabelImpl^)ObjectImpl::FromUnmanaged(self);
-
-  if (label != nullptr)
-  {
-    label->set_wrap_text(flag);
-    self->set_layout_dirty(true);
-  }
-}
-
-//----------------- Actual implementation ----------------------------------------------------------
-
-LabelImpl::LabelImpl(mforms::Label *self)
-  : ViewImpl(self)
+LabelWrapper::LabelWrapper(mforms::Label *backend)
+  : ViewWrapper(backend)
 {
 }
 
 //--------------------------------------------------------------------------------------------------
 
-void LabelImpl::set_style(mforms::LabelStyle style)
+bool LabelWrapper::create(mforms::Label *backend)
 {
-  WrapControlLabel^ label= get_control<WrapControlLabel>();
+  LabelWrapper *wrapper = new LabelWrapper(backend);
+  WrapControlLabel  ^label = Create<WrapControlLabel>(backend, wrapper);
+  label->TextAlign= ContentAlignment::MiddleLeft;
+  label->BorderStyle= BorderStyle::None;
 
+  return true;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void LabelWrapper::set_style(mforms::Label *backend, mforms::LabelStyle style)
+{
+  WrapControlLabel ^label = LabelWrapper::GetManagedObject<WrapControlLabel>(backend);
   try
   {
     switch (style)
     {
     case mforms::NormalStyle:
-      label->Font= gcnew Font(DEFAULT_FONT, DEFAULT_FONT_SIZE, FontStyle::Regular, GraphicsUnit::Pixel);
+      label->Font = gcnew Font(DEFAULT_FONT_FAMILY, DEFAULT_FONT_SIZE, FontStyle::Regular, GraphicsUnit::Pixel);
       break;
     case mforms::BoldStyle: 
-      label->Font= gcnew Font(DEFAULT_FONT, DEFAULT_FONT_SIZE, FontStyle::Bold, GraphicsUnit::Pixel);
+      label->Font = gcnew Font(DEFAULT_FONT_FAMILY, DEFAULT_FONT_SIZE, FontStyle::Bold, GraphicsUnit::Pixel);
       break;
     case mforms::BigStyle:
-      label->Font= gcnew Font(DEFAULT_FONT, DEFAULT_FONT_SIZE + 4, FontStyle::Regular, GraphicsUnit::Pixel);
+      label->Font = gcnew Font(DEFAULT_FONT_FAMILY, DEFAULT_FONT_SIZE + 4, FontStyle::Regular, GraphicsUnit::Pixel);
       break;
     case mforms::BigBoldStyle:
-      label->Font= gcnew Font(DEFAULT_FONT, DEFAULT_FONT_SIZE + 4, FontStyle::Bold, GraphicsUnit::Pixel);
+      label->Font = gcnew Font(DEFAULT_FONT_FAMILY, DEFAULT_FONT_SIZE + 4, FontStyle::Bold, GraphicsUnit::Pixel);
       break;
     case mforms::SmallBoldStyle:
-      label->Font= gcnew Font(DEFAULT_FONT, 8, FontStyle::Bold, GraphicsUnit::Pixel);
+      label->Font = gcnew Font(DEFAULT_FONT_FAMILY, 8, FontStyle::Bold, GraphicsUnit::Pixel);
       break;
     case mforms::SmallStyle:
-      label->Font= gcnew Font(DEFAULT_FONT, 8, FontStyle::Regular, GraphicsUnit::Point);
+      label->Font = gcnew Font(DEFAULT_FONT_FAMILY, 8, FontStyle::Regular, GraphicsUnit::Point);
       break;
     case mforms::VerySmallStyle:
-      label->Font= gcnew Font(DEFAULT_SMALL_FONT, 7.5, FontStyle::Regular, GraphicsUnit::Point);
+      label->Font = gcnew Font(DEFAULT_SMALL_FONT, 7.5, FontStyle::Regular, GraphicsUnit::Point);
       break;
     case mforms::InfoCaptionStyle:
-      label->Font= gcnew Font(DEFAULT_FONT, label->Font->Size, FontStyle::Regular, GraphicsUnit::Pixel);
+      label->Font = gcnew Font(DEFAULT_FONT_FAMILY, label->Font->Size, FontStyle::Regular, GraphicsUnit::Pixel);
       break;
     case mforms::BoldInfoCaptionStyle:
-      label->Font= gcnew Font(DEFAULT_FONT, label->Font->Size, FontStyle::Bold, GraphicsUnit::Pixel);
+      label->Font = gcnew Font(DEFAULT_FONT_FAMILY, label->Font->Size, FontStyle::Bold, GraphicsUnit::Pixel);
       break;
     case mforms::WizardHeadingStyle:
-      label->Font= gcnew Font(DEFAULT_FONT, 12.0, FontStyle::Bold, GraphicsUnit::Pixel);
-      label->ForeColor= ColorTranslator::FromHtml("#003392");
+      label->Font = gcnew Font(DEFAULT_FONT_FAMILY, 12.0, FontStyle::Bold, GraphicsUnit::Pixel);
+      label->ForeColor = ColorTranslator::FromHtml("#003392");
       break;
     case mforms::SmallHelpTextStyle:
-      label->Font= gcnew Font(DEFAULT_FONT, 11, FontStyle::Regular, GraphicsUnit::Pixel);
+      label->Font = gcnew Font(DEFAULT_FONT_FAMILY, 11, FontStyle::Regular, GraphicsUnit::Pixel);
       break;
     case mforms::VeryBigStyle:
-      label->Font= gcnew Font(DEFAULT_FONT, 13, FontStyle::Regular, GraphicsUnit::Point);
+      label->Font = gcnew Font(DEFAULT_FONT_FAMILY, 13, FontStyle::Regular, GraphicsUnit::Point);
       break;
     }
 
@@ -291,10 +218,16 @@ void LabelImpl::set_style(mforms::LabelStyle style)
 
 //--------------------------------------------------------------------------------------------------
 
-void LabelImpl::set_text(const std::string &text)
+void LabelWrapper::set_text(mforms::Label *backend, const std::string &text)
 {
-  String^ new_text = CppStringToNative(text);
-  WrapControlLabel^ label = get_control<WrapControlLabel>();
+  WrapControlLabel ^label = LabelWrapper::GetManagedObject<WrapControlLabel>(backend);
+
+  String^ new_text;
+  if (!text.empty() && text[text.size() - 1] == '\n') // Add a space if there's a trailing linebreak. Otherwise it would be ignored.
+    new_text = CppStringToNative(text + " ");
+  else
+    new_text = CppStringToNative(text);
+
   if (label->Text != new_text)
   {
     label->Text = new_text;
@@ -304,28 +237,48 @@ void LabelImpl::set_text(const std::string &text)
       newSize.Width = label->Size.Width;
     label->Size = newSize;
   };
+
+  backend->set_layout_dirty(true);
 }
 
 //--------------------------------------------------------------------------------------------------
 
-void LabelImpl::set_text_align(mforms::Alignment align)
+void LabelWrapper::set_text_align(mforms::Label *backend, mforms::Alignment align)
 {
-  get_control<WrapControlLabel>()->TextAlign = get_alignment(align);
+  WrapControlLabel ^label = LabelWrapper::GetManagedObject<WrapControlLabel>(backend);
+  label->TextAlign = get_alignment(align);
+  backend->set_layout_dirty(true);
 }
 
 //--------------------------------------------------------------------------------------------------
 
-void LabelImpl::set_color(const std::string &color)
+void LabelWrapper::set_color(mforms::Label *backend, const std::string &color)
 {
-  get_control<WrapControlLabel>()->ForeColor = ColorTranslator::FromHtml(CppStringToNative(color));
+  WrapControlLabel ^label = LabelWrapper::GetManagedObject<WrapControlLabel>(backend);
+  label->ForeColor = ColorTranslator::FromHtml(CppStringToNative(color));
 }
 
 //--------------------------------------------------------------------------------------------------
 
-void LabelImpl::set_wrap_text(bool flag)
+void LabelWrapper::set_wrap_text(mforms::Label *backend, bool flag)
 {
-  get_control<WrapControlLabel>()->AutoWrapping = flag;
+  WrapControlLabel ^label = LabelWrapper::GetManagedObject<WrapControlLabel>(backend);
+  label->AutoWrapping = flag;
+  backend->set_layout_dirty(true);
 }
 
 //--------------------------------------------------------------------------------------------------
 
+void LabelWrapper::init()
+{
+  mforms::ControlFactory *f = mforms::ControlFactory::get_instance();
+
+  f->_label_impl.create = &LabelWrapper::create;
+  f->_label_impl.set_text = &LabelWrapper::set_text;
+  f->_label_impl.set_style = &LabelWrapper::set_style;
+  f->_label_impl.set_text_align = &LabelWrapper::set_text_align;
+  f->_label_impl.set_color = &LabelWrapper::set_color;
+  f->_label_impl.set_wrap_text = &LabelWrapper::set_wrap_text;
+}
+
+//--------------------------------------------------------------------------------------------------
