@@ -17,18 +17,16 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
+#include "grtpp.h"
 
 #include "changefactory.h"
 #include "changeobjects.h"
 #include "grts/structs.h"
+
 #include "base/log.h"
 
 #ifdef DEBUG_DIFF
-DEFAULT_LOG_DOMAIN("diff")
-#endif
 
-#ifdef DEBUG_DIFF
 static std::string rep(const grt::ValueRef &value)
 {
   switch (value.type())
@@ -50,8 +48,6 @@ bool is_any(ValueRef& s);
 boost::shared_ptr<DiffChange> ChangeFactory::create_value_added_change(boost::shared_ptr<DiffChange> parent, const ValueRef &source, const ValueRef &target, bool dupvalue)
 {
 #ifdef DEBUG_DIFF
-  XXX log_* calls are not meant to be switched at compile time. Either make this always-on log_debug3 or
-  just use g_message or something.
   log_info("value_added (%s)\n", rep(target).c_str());
 #endif
   return boost::shared_ptr<DiffChange>(new ValueAddedChange(ValueAdded, target, dupvalue));
