@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include <memory>
 #include "grtlistdiff.h"
 #include "grtpp.h"
@@ -27,8 +25,6 @@
 #include "grtdiff.h"
 #include "base/log.h"
 
-DEFAULT_LOG_DOMAIN("diff")
-
 #include <memory>
 #include <algorithm>
 
@@ -209,8 +205,7 @@ boost::shared_ptr<MultiChange> GrtListDiff::diff(const BaseListRef &source, cons
   {
     internal::List::raw_const_iterator It_target = find_if (target.content().raw_begin(), target.content().raw_end(), std::bind2nd(OmfEqPred(comparer), source.get(*It)));
     prev_value = It_target == target.content().raw_begin()?ValueRef():*(It_target-1);
-    boost::shared_ptr<ListItemOrderChange> orderchange(
-                                                       new ListItemOrderChange(source.get(*It), *It_target, omf, prev_value, target.get_index(*It_target)));
+    boost::shared_ptr<ListItemOrderChange> orderchange(new ListItemOrderChange(source.get(*It), *It_target, omf, prev_value, target.get_index(*It_target)));
     //    if (!orderchange->subchanges()->empty())
     changes.push_back(orderchange);
   }
