@@ -131,6 +131,17 @@ namespace grt {
     user_cancelled(const std::string &exc) : std::runtime_error(exc) {}
   };  
 
+  class MYSQLGRT_PUBLIC server_denied : public std::runtime_error
+  {
+
+  public:
+    int errNo;
+
+    server_denied(const server_denied &other) : std::runtime_error(other), errNo(other.errNo) {}
+    server_denied(const std::string &exc, int err) : std::runtime_error(exc), errNo(err) {}
+    virtual ~server_denied() THROW() {}
+  };
+
   class MYSQLGRT_PUBLIC db_error : public std::runtime_error
   {
     int _error;
@@ -552,11 +563,11 @@ namespace grt {
   #define DEFINE_SSIZE_T_FUNCTIONS
 
   #ifdef __LP64__
-  #define DEFINE_INT_FUNCTIONS
+    #define DEFINE_INT_FUNCTIONS
   #endif
 
   #ifdef __APPLE__
-    // Probably also depending on 32bit vs 64bit, but for now we compile only 32bit on Mac.
+    // On OSX we only support the 64bit arch.
     #define DEFINE_UINT64_T_FUNCTIONS
     #define DEFINE_INT_FUNCTIONS
 
@@ -643,6 +654,18 @@ namespace grt {
     {
     }
 
+#ifdef DEFINE_UINT64_T_FUNCTIONS
+    Ref(int64_t value)
+      : ValueRef(internal::Integer::get(value))
+    {
+    }
+
+    Ref(uint64_t value)
+      : ValueRef(internal::Integer::get(value))
+    {
+    }
+#endif
+
     inline operator storage_type () const { return *content(); }
     inline storage_type operator *() const { return *content(); }
 
