@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,14 +16,11 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-
-#include "stdafx.h"
-
+#include "grtpp_shell_python.h"
 #include "base/string_utilities.h"
 #include "base/threading.h"
 #include "base/log.h"
 
-#include "grtpp_shell_python.h"
 #include "grtpp.h"
 #include "grtpp_util.h"
 #include "grtpp_shell.h"
@@ -31,10 +28,8 @@
 #include "grtpp_undo_manager.h"
 #include "grtpp_notifications.h"
 
+#include <cppconn/exception.h>
 #include <algorithm>
-
-#include "grtpp_shell_lua.h"
-
 #include <glib.h>
 
 #include "serializer.h"
@@ -144,6 +139,11 @@ type_error::type_error(const std::string &expected, const std::string &actual, T
 {
 }  
 
+db_error::db_error(const sql::SQLException &exc)
+: std::runtime_error(exc.what()), _error(exc.getErrorCode())
+{
+}
+
 //--------------------------------------------------------------------------------------------------
 
 StringRef StringRef::format(const char *format, ...)
@@ -180,8 +180,6 @@ GRT::GRT()
   _default_undo_manager= new UndoManager(this);
 
   add_module_loader(new CPPModuleLoader(this));
-
-  g_static_rec_mutex_init(&_message_mutex);
   
   // register metaclass for base class
   add_metaclass(MetaClass::create_base_class(this));
@@ -219,8 +217,6 @@ GRT::~GRT()
        iter != _metaclasses.end(); ++iter)
     delete iter->second;
   _metaclasses.clear();
-
-  g_static_rec_mutex_free(&_message_mutex);
 }
 
 void GRT::push_undo_manager(UndoManager *um)
@@ -378,7 +374,7 @@ int GRT::scan_metaclasses_in(const std::string &directory, std::multimap<std::st
   
   g_dir_close(dir);
   
-  return _metaclasses.size() - old_count;
+  return (int)(_metaclasses.size() - old_count);
 }
 
 
@@ -436,14 +432,14 @@ void GRT::end_loading_metaclasses(bool check_class_binding)
     if (iter->second->placeholder())
     {
       undefined= true;
-      g_warning("MetaClass '%s' is undefined but was referred in '%s'",
+      log_warning("MetaClass '%s' is undefined but was referred in '%s'\n",
                 iter->second->name().c_str(), iter->second->source().c_str());
     }
     if (!iter->second->validate())
       validate_error= true;
   }
   if (undefined)
-    throw std::runtime_error("One or more undefined metaclass were referred by other structs");
+    throw std::runtime_error("One or more undefined meta classes were referred by other structs");
   if (validate_error)
     throw std::runtime_error("Validation error in loaded metaclasses");
 
@@ -522,23 +518,34 @@ ObjectRef GRT::find_object_by_id(const std::string &id, const std::string &subpa
   return ObjectRef();
    */
   
+  std::map<std::string, ObjectRef >::const_iterator iter;
+  if ((iter= _objects_cache.find(id)) != _objects_cache.end())
+    return iter->second;
+
   ValueRef start= get(subpath);
+  ObjectRef result = ObjectRef();
 
   if (start.is_valid())
   {
     switch (start.type())
     {
     case ListType:
-      return find_child_object(BaseListRef::cast_from(start), id);
+      result = find_child_object(BaseListRef::cast_from(start), id);
+      break;
     case DictType:
-      return find_child_object(DictRef::cast_from(start), id);
+      result = find_child_object(DictRef::cast_from(start), id);
+      break;
     case ObjectType:
-      return find_child_object(ObjectRef::cast_from(start), id);
+      result = find_child_object(ObjectRef::cast_from(start), id);
+      break;
     default:
       throw std::invalid_argument("Value at "+subpath+" is not a container");
     }
   }
-  return ObjectRef();
+  if (result.is_valid())
+    _objects_cache[id] = result;
+
+  return result;
 }
 
 
@@ -554,17 +561,22 @@ void GRT::serialize(const ValueRef &value, const std::string &path,
   ser.save_to_xml(value, path, doctype, version, list_objects_as_links);
 }
 
-
-ValueRef GRT::unserialize(const std::string &path)
+boost::shared_ptr<grt::internal::Unserializer> GRT::get_unserializer()
 {
-  internal::Unserializer unser(this, _check_serialized_crc);
+  return boost::shared_ptr<grt::internal::Unserializer> (new internal::Unserializer(this, _check_serialized_crc));
+};
+
+ValueRef GRT::unserialize(const std::string &path, boost::shared_ptr<grt::internal::Unserializer> unserializer)
+{
+  if(!unserializer)
+    unserializer = boost::shared_ptr<grt::internal::Unserializer>(new internal::Unserializer(this, _check_serialized_crc));
 
   if (!g_file_test(path.c_str(), G_FILE_TEST_EXISTS))
     throw os_error(path);
 
   try
   {
-    return unser.load_from_xml(path);
+    return unserializer->load_from_xml(path);
   }
   catch (std::exception &exc)
   {
@@ -642,7 +654,6 @@ void GRT::add_module_loader(ModuleLoader *loader)
   loader->refresh();
 }
 
-
 bool GRT::load_module(const std::string &path, bool refresh)
 {
   for (std::list<ModuleLoader*>::iterator loader= _loaders.begin();
@@ -650,16 +661,11 @@ bool GRT::load_module(const std::string &path, bool refresh)
   {
     if ((*loader)->check_file_extension(path))
     {
-      if (_verbose)
-        g_message("Trying to load module '%s' (%s)", path.c_str(), (*loader)->get_loader_name().c_str());
+      log_debug2("Trying to load module '%s' (%s)\n", path.c_str(), (*loader)->get_loader_name().c_str());
       
-      Module *module= (*loader)->init_module(path);
-      if (!module)
-      {
-//        if (_verbose)
-          g_message("Failed loading module '%s' (%s)", path.c_str(), (*loader)->get_loader_name().c_str());
-      }
-      else
+      // Problems, if any, are logged in init_module.
+      Module *module = (*loader)->init_module(path);
+      if (module)
       {
         try 
         {
@@ -668,10 +674,10 @@ bool GRT::load_module(const std::string &path, bool refresh)
           else
             register_new_module(module);
         }
-        catch (...)
+        catch (std::exception &exc)
         {
-         // crashes in some systems, works in others
-         // delete module;
+          log_debug("Deleting module %s because of %s\n", module->name().c_str(), exc.what());
+          delete module;
           throw;
         }
         return true;
@@ -795,7 +801,7 @@ int GRT::scan_modules_in(const std::string &path, const std::list<std::string> &
 
 static bool compare_modules(Module *a, Module *b)
 {
-  return g_strcasecmp(a->name().c_str(), b->name().c_str()) < 0;
+  return g_ascii_strcasecmp(a->name().c_str(), b->name().c_str()) < 0;
 }
 
 void GRT::end_loading_modules()
@@ -910,6 +916,9 @@ void GRT::refresh_module(Module *module)
 
 void GRT::register_new_interface(Interface *iface)
 {
+  if (get_interface(iface->name()))
+      throw std::logic_error("Duplicate interface "+iface->name());
+
   _interfaces[iface->name()]= iface;
 } 
 
@@ -954,12 +963,8 @@ void *GRT::get_context_data(const std::string &key)
 
 bool GRT::init_shell(const std::string &shell_type)
 {
-  if (shell_type == LanguageLua)
-    _shell= new LuaShell(this);
-#ifdef ENABLE_PYTHON_MODULES
-  else if (shell_type == LanguagePython)
+  if (shell_type == LanguagePython)
     _shell= new PythonShell(this);
-#endif
   else
     throw std::runtime_error("Invalid shell type "+shell_type);
   
@@ -970,12 +975,9 @@ bool GRT::init_shell(const std::string &shell_type)
 
 std::string GRT::shell_type()
 {
-  if (dynamic_cast<LuaShell*>(_shell))
-    return LanguageLua;
-#ifdef ENABLE_PYTHON_MODULES
-  else if (dynamic_cast<PythonShell*>(_shell))
+  if (dynamic_cast<PythonShell*>(_shell))
     return LanguagePython;
-#endif
+
   return "";
 }
 
@@ -988,14 +990,14 @@ Shell *GRT::get_shell()
 
 void GRT::push_message_handler(const MessageSlot &slot)
 {
-  GStaticRecMutexLock lock(_message_mutex);  
+  base::RecMutexLock lock(_message_mutex);
   _message_slot_stack.push_back(slot);
 }
 
 
 void GRT::pop_message_handler()
 {
-  GStaticRecMutexLock lock(_message_mutex);
+  base::RecMutexLock lock(_message_mutex);
   if (_message_slot_stack.empty())
     log_error("pop_message_handler() called on empty handler stack");
   else
@@ -1005,16 +1007,16 @@ void GRT::pop_message_handler()
 
 bool GRT::handle_message(const Message &msg, void *sender)
 {
-  if (_message_slot_stack.empty())
-    log_error("Unhandled message: %s\n", msg.format().c_str());
-  else
+  // Don't log any message if there's no message slot is occupied. It just means
+  // we don't want anything logged.
+  if (!_message_slot_stack.empty())
   {
     int i = 0;
     MessageSlot slot;
     for (;;)
     {
       {
-        GStaticRecMutexLock lock(_message_mutex);
+        base::RecMutexLock lock(_message_mutex);
         if ((int)_message_slot_stack.size() - i - 1 >= 0)
         {
           slot = _message_slot_stack[_message_slot_stack.size() - i - 1];
@@ -1027,7 +1029,7 @@ bool GRT::handle_message(const Message &msg, void *sender)
         return true;
     }
   }
-  log_error("Unhandled message (%lu): %s\n", _message_slot_stack.size(), msg.format().c_str());
+  log_error("Unhandled message (%lu): %s\n", (unsigned long)_message_slot_stack.size(), msg.format().c_str());
   return false;
 }
 
@@ -1055,7 +1057,7 @@ bool GRT::query_status()
 
 void GRT::make_output_visible(void *sender)
 {
-  g_static_rec_mutex_lock(&_message_mutex);
+  base::RecMutexLock lock(_message_mutex);
   Message msg;
   msg.type= ControlMsg;
   msg.text= "show";
@@ -1063,13 +1065,12 @@ void GRT::make_output_visible(void *sender)
   msg.timestamp= time(NULL);
   msg.progress= 0.0;
   handle_message(msg, sender);
-  g_static_rec_mutex_unlock(&_message_mutex);
 }
 
 // XXX: these handlers should go and be replaced by pure log_* calls.
 void GRT::send_error(const std::string &message, const std::string &details, void *sender)
 {
-  g_static_rec_mutex_lock(&_message_mutex);
+  base::RecMutexLock lock(_message_mutex);
   Message msg;
   msg.type= ErrorMsg;
   msg.text= message;
@@ -1077,7 +1078,6 @@ void GRT::send_error(const std::string &message, const std::string &details, voi
   msg.timestamp= time(NULL);
   msg.progress= 0.0;
   handle_message(msg, sender);
-  g_static_rec_mutex_unlock(&_message_mutex);
   
   log_error("%s\t%s\n", message.c_str(), details.c_str());
 }
@@ -1085,7 +1085,7 @@ void GRT::send_error(const std::string &message, const std::string &details, voi
 
 void GRT::send_warning(const std::string &message, const std::string &details, void *sender)
 {
-  g_static_rec_mutex_lock(&_message_mutex);
+  base::RecMutexLock lock(_message_mutex);
   Message msg;
   msg.type= WarningMsg;
   msg.text= message;
@@ -1093,15 +1093,14 @@ void GRT::send_warning(const std::string &message, const std::string &details, v
   msg.timestamp= time(NULL);
   msg.progress= 0.0;
   handle_message(msg, sender);
-  g_static_rec_mutex_unlock(&_message_mutex);
   
-  log_warning("%s\t%s", message.c_str(), details.c_str());
+  log_warning("%s\t%s\n", message.c_str(), details.c_str());
 }
 
 
 void GRT::send_info(const std::string &message, const std::string &details, void *sender)
 {
-  g_static_rec_mutex_lock(&_message_mutex);
+  base::RecMutexLock lock(_message_mutex);
   Message msg;
   msg.type= InfoMsg;
   msg.text= message;
@@ -1109,7 +1108,6 @@ void GRT::send_info(const std::string &message, const std::string &details, void
   msg.timestamp= time(NULL);
   msg.progress= 0.0;
   handle_message(msg, sender);
-  g_static_rec_mutex_unlock(&_message_mutex);
   
   log_info("%s\t%s\n", message.c_str(), details.c_str());
 }
@@ -1147,7 +1145,7 @@ void GRT::end_progress_step()
 
 void GRT::send_progress(float percentage, const std::string &message, const std::string &details, void *sender)
 {
-  g_static_rec_mutex_lock(&_message_mutex);
+  base::RecMutexLock lock(_message_mutex);
   Message msg;
   msg.type= ProgressMsg;
   msg.text= message;
@@ -1157,14 +1155,12 @@ void GRT::send_progress(float percentage, const std::string &message, const std:
   // calculate the actual progress percentage
   if (!_progress_step_stack.empty())
   {
-    for (int i = _progress_step_stack.size()-1; i >= 0; --i)
-    {
-      percentage = _progress_step_stack[i].first + (_progress_step_stack[i].second - _progress_step_stack[i].first) * percentage;
-    } 
+    std::vector<std::pair<float,float> >::reverse_iterator rit;
+    for (rit = _progress_step_stack.rbegin(); rit != _progress_step_stack.rend(); ++rit)
+      percentage = (*rit).first + ((*rit).second - (*rit).first) * percentage;
   }
   msg.progress= percentage;
   handle_message(msg, sender);
-  g_static_rec_mutex_unlock(&_message_mutex);
 
   // Progress messages are often sent without a real message, just to trigger some actions.
   // Don't log empty lines.
@@ -1175,7 +1171,7 @@ void GRT::send_progress(float percentage, const std::string &message, const std:
 
 void GRT::send_verbose(const std::string &message, void *sender)
 {
-  g_static_rec_mutex_lock(&_message_mutex);
+  base::RecMutexLock lock(_message_mutex);
   Message msg;
   msg.type= VerboseMsg;
   msg.text= message;
@@ -1183,7 +1179,6 @@ void GRT::send_verbose(const std::string &message, void *sender)
   msg.timestamp= time(NULL);
   msg.progress= 0.0;
   handle_message(msg, sender);
-  g_static_rec_mutex_unlock(&_message_mutex);
 
   log_debug2("%s", message.c_str());
 }
@@ -1191,7 +1186,7 @@ void GRT::send_verbose(const std::string &message, void *sender)
 
 void GRT::send_output(const std::string &message, void *sender)
 {
-  g_static_rec_mutex_lock(&_message_mutex);
+  base::RecMutexLock lock(_message_mutex);
   Message msg;
   msg.type= OutputMsg;
   msg.text= message;
@@ -1199,9 +1194,9 @@ void GRT::send_output(const std::string &message, void *sender)
   msg.timestamp= time(NULL);
   msg.progress= 0.0;
   handle_message(msg, sender);
-  g_static_rec_mutex_unlock(&_message_mutex);
 
   // Log send_output only when verbose is on to avoid duplicate prints to stdout, when logged text also goes to stderr/out
+  // TODO: fix the actual cause for sending duplicates and remove _verbose (doesn't fit to our logging).
   if (_verbose)
     log_debug("%s", message.c_str());
 }
