@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,9 +17,6 @@
  * 02110-1301  USA
  */
 
-
-#include "stdafx.h"
-
 #include "grtpp_undo_manager.h"
 #include "base/string_utilities.h"
 
@@ -1086,7 +1083,7 @@ void UndoManager::add_undo(UndoAction *cmd)
     if (!flag)
     {
       if (debug_undo && !dynamic_cast<UndoGroup*>(cmd))
-        g_message("added undo action that's not a group to top");
+        log_debug2("added undo action that's not a group to top");
       _undo_stack.push_back(cmd);
       trim_undo_stack();
     }
