@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,9 +17,6 @@
  * 02110-1301  USA
  */
 
-
-#include "stdafx.h"
-
 #include "grtpp_module_python.h"
 
 #include "grtpp_util.h"
@@ -104,7 +101,7 @@ std::string grt::fmt_simple_type_spec(const SimpleTypeSpec &type)
 {
   switch (type.type)
   {
-    case IntegerType: return "int";
+    case IntegerType: return "ssize_t";
     case DoubleType: return "double";
     case StringType: return "string";
     case ListType: return "list";
@@ -119,7 +116,7 @@ std::string grt::fmt_type_spec(const TypeSpec &type)
 {
   switch (type.base.type)
   {
-    case IntegerType: return "int";
+    case IntegerType: return "ssize_t";
     case DoubleType: return "double";
     case StringType: return "string";
     case ListType:
@@ -287,8 +284,9 @@ bool grt::set_value_by_path(const ValueRef &value, const std::string &path, cons
     else if (container.type() == ListType)
     {
       BaseListRef list(BaseListRef::cast_from(container));
-      size_t i;
-      if (sscanf(last.c_str(), "%zi", &i) != 1 || i >= list.count())
+
+      int i; // Unlikely we will ever need an index beyond 32bit.
+      if (sscanf(last.c_str(), "%i", &i) != 1 || i >= (int)list.count())
         return false;
       
       list.gset(i, new_value);
@@ -827,7 +825,7 @@ void grt::merge_contents_by_name(ObjectListRef target, ObjectListRef source,
   std::map<std::string, int> known_names;
 
   for (size_t c= target.count(), i= 0; i < c; i++)
-    known_names[StringRef::cast_from(target[i].get_member("name"))]= i;
+    known_names[StringRef::cast_from(target[i].get_member("name"))]= (int)i;
 
   for (size_t c= source.count(), i= 0; i < c; i++)
   {
@@ -1078,7 +1076,6 @@ void grt::remove_list_items_matching(ObjectListRef list, const boost::function<b
 
 bool grt::init_python_support(grt::GRT *grt, const std::string &module_path)
 {
-#ifdef ENABLE_PYTHON_MODULES
   PythonModuleLoader *loader= new PythonModuleLoader(grt, module_path);
   if (!module_path.empty())
   {
@@ -1089,18 +1086,14 @@ bool grt::init_python_support(grt::GRT *grt, const std::string &module_path)
   }
   grt->add_module_loader(loader);
   return true;
-#endif
-  return false;
 }
 
 
 void grt::add_python_module_dir(grt::GRT *grt, const std::string &python_module_path)
 {
-#ifdef ENABLE_PYTHON_MODULES
   PythonModuleLoader *loader= dynamic_cast<PythonModuleLoader *>(grt->get_module_loader("python"));
   if (loader && !python_module_path.empty())
     loader->get_python_context()->add_module_path(python_module_path, true);
-#endif
 }
 
 static void dump_value(const grt::ValueRef &value, int level, bool skip_spacing=false);
