@@ -595,6 +595,15 @@ ValueRef Dict::operator [](const std::string &key) const
   return iter->second;
 }
 
+Dict::const_iterator Dict::begin() const
+{
+  return _content.begin();
+}
+
+Dict::const_iterator Dict::end() const
+{
+  return _content.end();
+}
 
 std::vector<std::string> Dict::keys() const
 {
@@ -1617,7 +1626,6 @@ const char* fixed_strins[] = {
 "sslCipher",
 "getSchemata",
 "useAnsiQuotes",
-"useLegacyAuth",
 "sslCert",
 "Path to Certificate file for SSL.",
 "mysqlcppconn",
