@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,10 +17,23 @@
  * 02110-1301  USA
  */
 
-#ifndef _GRTPP_VALUE_H_
-#define _GRTPP_VALUE_H_
+#pragma once
 
-#include "grtpp.h"
+#ifdef _WIN32
+  #ifdef _WIN64
+    typedef __int64 ssize_t;
+  #else
+    typedef int ssize_t;
+  #endif
+#endif
+
+#include <boost/signals2.hpp>
+#include "base/threading.h"
+
+#if defined(ENABLE_DEBUG) || defined(_DEBUG)
+//#define GRT_LEAK_DETECTOR_ENABLED
+//#define GRT_LEAK_DETECTOR_RECORD_CALL_STACK
+#endif
 
 namespace grt {
   class GRT;
@@ -105,6 +118,26 @@ namespace grt {
   };
 
   //------------------------------------------------------------------------------------------------
+#ifdef GRT_LEAK_DETECTOR_ENABLED
+    namespace LeakDetector
+    {
+        MYSQLGRT_PUBLIC void print_leaks();
+        MYSQLGRT_PUBLIC void update_objects_data();
+        MYSQLGRT_PUBLIC void enable();
+        MYSQLGRT_PUBLIC void disable();
+        MYSQLGRT_PUBLIC void reset();
+    };
+#else
+    namespace LeakDetector
+    {
+        inline void print_leaks(){};
+        inline void update_objects_data(){};
+        inline void enable(){};
+        inline void disable(){};
+        inline void reset(){};
+    };
+
+#endif
 
   namespace internal {
     
@@ -113,21 +146,29 @@ namespace grt {
     class MYSQLGRT_PUBLIC Value
     {
     public:
-      virtual Type get_type() const= 0;
+      virtual Type get_type() const = 0;
     
-      Value *retain() { /*uses mutex in some OSs g_atomic_int_inc(&_refcount);*/++_refcount; return this; }
+      Value *retain()
+      {
+        g_atomic_int_inc(&_refcount);
+        return this;
+      }
+
       void release() {
 #ifdef DEBUG
-        if (_refcount == 0) g_warning("GRT: releasing invalid object");
-#endif          
-        //if (g_atomic_int_dec_and_test(&_refcount)) // returns true if --refcount==0
-        if (--_refcount == 0)
+        if (_refcount == 0)
+          g_warning("GRT: releasing invalid object");
+#endif
+        if (g_atomic_int_dec_and_test(&_refcount))
           delete this;
       }
 
-      virtual std::string repr() const= 0;
+      virtual std::string repr() const = 0;
 
-      inline gint refcount() const { return _refcount; } //g_atomic_int_get(&_refcount); }
+      inline base::refcount_t refcount() const
+      {
+        return g_atomic_int_get(&_refcount);
+      }
 
       virtual void mark_global() const {}
       virtual void unmark_global() const {}
@@ -136,7 +177,7 @@ namespace grt {
       virtual bool less_than(const Value *) const= 0;
       
       // reset_references helps to free memory allocated
-      // by Value. The method is overriden in Object, List and Dict
+      // by Value. The method is overridden in Object, List and Dict.
       virtual void reset_references() {}
 
     protected:
@@ -146,15 +187,14 @@ namespace grt {
     private:
       Value(const Value&) {}
 
-      mutable gint _refcount;
+      volatile mutable base::refcount_t _refcount;
     };
     
-    
-    
+    // 32 bit or 64 bit integer type.
     class MYSQLGRT_PUBLIC Integer : public Value
     {
     public:
-      typedef long storage_type;
+      typedef ssize_t storage_type;
       
     public:    
       Integer(storage_type value);
@@ -358,11 +398,11 @@ namespace grt {
       
       ValueRef operator [](const std::string &key) const;
       
-      const_iterator begin() const { return _content.begin(); }
-      const_iterator end() const { return _content.end(); }
+      const_iterator begin() const;
+      const_iterator end() const;
       
-      iterator begin() { return _content.begin(); }
-      iterator end() { return _content.end(); }
+      //iterator begin() { return _content.begin(); }
+      //iterator end() { return _content.end(); }
       
       bool has_key(const std::string &key) const;
       ValueRef get(const std::string &key) const;
@@ -371,6 +411,8 @@ namespace grt {
       virtual void reset_entries();
       size_t count() const { return _content.size(); }
 
+      std::vector<std::string> keys() const;
+
       virtual bool equals(const Value*) const;
       virtual bool less_than(const Value *) const;
 
@@ -408,85 +450,6 @@ namespace grt {
     };
 
     //------------------------------------------------------------------------------------------------
-    //QQQ Do we really need this, looks like this enum is never used
-    enum ListChangeType
-    {
-      ListAdded,
-      ListRemoved,
-      ListReplaced
-    };
-    
-    
-    /** Object validity flag for weak-references
-     * 
-     * Allocated by objects to signal whether it's valid and held by weak references
-     * to check whether the object is still valid.
-     * @ingroup GRT
-     */
-    class ObjectValidFlag
-    {
-    public:
-      ObjectValidFlag(bool valid) : flagp(new Flag(valid))
-      {}
-      
-      ObjectValidFlag(const ObjectValidFlag &copy) : flagp(copy.flagp)
-      {
-        ref_added();
-      }
-      
-      inline ObjectValidFlag &operator= (const ObjectValidFlag &other)
-      {
-        ObjectValidFlag tmp(other); 
-        swap(tmp);
-        return *this;
-      }
-      
-      ~ObjectValidFlag()
-      {
-        ref_deleted();
-      }
-      
-      bool valid() const
-      {
-        return flagp->valid;
-      }
-      
-      void invalidate() 
-      {
-        flagp->valid= false;
-      }
-      
-      inline void swap(ObjectValidFlag &other)
-      {
-         if (flagp != other.flagp)
-         {
-           ref_deleted();
-           flagp= other.flagp;
-           ref_added();
-         }
-      }
-      
-    private:
-      struct Flag {
-        gint refcount;
-        bool valid;
-        
-        inline Flag(bool initial) : refcount(1), valid(initial) {}
-      };
-      
-      Flag *flagp;
-      
-      inline void ref_added()
-      {
-        g_atomic_int_inc(&flagp->refcount);
-      }
-      
-      inline void ref_deleted()
-      {
-        if (g_atomic_int_dec_and_test(&flagp->refcount))
-          delete flagp; // if no more refcounts, delete it
-      }
-    };
 
     /** Base GRT Object class.
      * 
@@ -538,6 +501,8 @@ namespace grt {
       
       void __set_id(const std::string &id);
 
+      virtual void mark_global() const;
+      virtual void unmark_global() const;
     protected:
       friend class OwnedList;
       friend class OwnedDict;
@@ -546,13 +511,9 @@ namespace grt {
 
       explicit Object(MetaClass *gclass);
       Object(GRT *grt, MetaClass *gclass); /* deprecated */
-      virtual ~Object()
-      {
-        _valid_flag.invalidate();
-      }
-
-      virtual void mark_global() const;
-      virtual void unmark_global() const;
+#ifdef GRT_LEAK_DETECTOR_ENABLED
+      virtual ~Object();
+#endif
 
       void owned_member_changed(const std::string &name, const grt::ValueRef &ovalue, const grt::ValueRef &nvalue);
       void member_changed(const std::string &name, const grt::ValueRef &ovalue, const grt::ValueRef &nvalue);
@@ -566,16 +527,16 @@ namespace grt {
       //use _metaclass->get_grt() GRT *_grt;
       MetaClass* _metaclass;
       std::string _id;
-      boost::signals2::signal<void (const std::string&, const ValueRef&)> _changed_signal;
+      boost::signals2::signal<void (const std::string&, const grt::ValueRef&)> _changed_signal;
       boost::signals2::signal<void (OwnedList*, bool, const grt::ValueRef&)> _list_changed_signal;
       boost::signals2::signal<void (OwnedDict*, bool, const std::string&)> _dict_changed_signal;
 
-      ObjectValidFlag _valid_flag;
+      //ObjectValidFlag _valid_flag;
       
       mutable short _is_global; // whether object is attached to the global GRT tree
       
-    public:
-      const ObjectValidFlag &weakref_valid_flag() const { return _valid_flag; }
+//    public:
+//      const ObjectValidFlag &weakref_valid_flag() const { return _valid_flag; }
     };
 
 
@@ -623,5 +584,3 @@ namespace grt {
 
   }; // internal
 }; //grt
-
-#endif // _GRTPP_VALUE_H_
