@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -39,6 +39,10 @@
 
 #include "base/log.h"
 
+#ifdef _WIN32
+#include "base/event_log.h"
+#endif
+
 DEFAULT_LOG_DOMAIN("python context")
 
 using namespace grt;
@@ -321,7 +325,6 @@ void PythonContext::handle_grt_notification(const std::string &name, ObjectRef s
   }
 }
 
-
 void PythonContext::handle_notification(const std::string &name, void *sender, base::NotificationInfo &info)
 {
   if (_grt_notification_observer)
@@ -703,6 +706,96 @@ static PyObject *grt_get_by_path(PyObject *self, PyObject *args)
   return ctx->from_grt(value);
 }
 
+void PythonContext::setEventlogCallback(PyObject *obj)
+{
+  _grtEventLogNotification = obj;
+}
+
+void PythonContext::printResult(std::map<std::string, std::string> &output)
+{
+  if (_grtEventLogNotification)
+  {
+    WillEnterPython lock;
+    PyObject *dict = PyDict_New();
+    std::map<std::string, std::string>::iterator end = output.end();
+    for (std::map<std::string, std::string>::iterator it = output.begin(); it != end; ++it)
+    {
+      PyObject *str = PyString_FromString(it->second.c_str());
+      PyDict_SetItemString(dict, it->first.c_str(), str);
+      Py_DECREF(str);
+    }
+    PyObject *args = Py_BuildValue("sO", "", dict);
+    PyObject *res;
+    if (!(res = PyObject_CallObject(_grtEventLogNotification, args)))
+    {
+      log_python_error("Error forwarding GRT notification to Python");
+    }
+    if (res)
+    {
+      Py_DECREF(res);
+    }
+    Py_DECREF(args);
+    Py_DECREF(dict);
+  }
+}
+
+#ifdef _WIN32
+static void printResultCallback(std::map<std::string, std::string> &output)
+{
+  PythonContext *ctx;
+
+  if (!(ctx = PythonContext::get_and_check()))
+    return;
+
+  ctx->printResult(output);
+}
+#endif
+
+static PyObject *getEventLogEntry(PyObject *self, PyObject *args)
+{
+  PythonContext *ctx;
+
+  if (!(ctx = PythonContext::get_and_check()))
+    return NULL;
+
+  char *query = NULL;
+  long seek = 0;
+  if (!PyArg_ParseTuple(args, "ls", &seek, &query))
+    return NULL;
+
+#ifdef _WIN32
+  EventLogReader reader(query, printResultCallback);
+  reader.SetPosition(seek);
+  reader.ReadEvents();
+#endif
+
+  Py_INCREF(Py_None);
+  return Py_None;
+}
+
+static PyObject *setEventlogCallback(PyObject *self, PyObject *args)
+{
+  PythonContext *ctx;
+  if (!(ctx = PythonContext::get_and_check()))
+    return NULL;
+
+  PyObject *object;
+
+  if (!PyArg_ParseTuple(args, "O", &object))
+    return NULL;
+
+  if (!PyCallable_Check(object))
+  {
+    PyErr_SetString(PyExc_ValueError, "notification observer argument must be a callable");
+    return NULL;
+  }
+
+  ctx->setEventlogCallback(object);
+
+  Py_INCREF(Py_None);
+  return Py_None;
+}
+
 
 static PyObject *grt_readline(PyObject *self, PyObject *args)
 {
@@ -1129,6 +1222,9 @@ static PyMethodDef GrtModuleMethods[] = {
 {"togrt", grt_wrap_pyobject, METH_VARARGS,
   "Wraps a Python object in a grt_PyObject GRT object, which you can then use to reference from GRT objects (such as GRT dicts and lists)"},
 
+{ "getEventLogEntry", getEventLogEntry, METH_VARARGS, "Read logentry from windows event log." },
+{ "setEventlogCallback", setEventlogCallback, METH_VARARGS, "Pushes a callback that print log entry form a EventViewer." },
+
 {NULL, NULL, 0, NULL}        /* Sentinel */
 };
 
@@ -1199,8 +1295,6 @@ void PythonContext::register_grt_module()
 }
 
 
-
-
 PyObject *PythonContext::get_grt_module()
 {
   return _grt_module;
