@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,12 +17,9 @@
  * 02110-1301  USA
  */
  
-#include "stdafx.h"
-
-#ifdef ENABLE_PYTHON_MODULES
-
-#include "base/string_utilities.h"
 #include "python_context.h"
+#include "base/string_utilities.h"
+
 #include "grtpp_util.h"
 
 #include "python_module.h"
@@ -127,6 +124,10 @@ static PyObject* function_call(PyGRTFunctionObject *self, PyObject *args, PyObje
     PythonContext::set_python_error(exc, strfmt("%s.%s()", self->module->name().c_str(), self->function->name.c_str()));
     return NULL;
   }
+  catch (grt::python_error)
+  {
+    return NULL;
+  }
   catch (std::exception &exc)
   {
     PythonContext::set_python_error(exc, strfmt("%s.%s()", self->module->name().c_str(), self->function->name.c_str()));
@@ -464,8 +465,3 @@ void grt::PythonContext::init_grt_module_type()
     _grt_function_class= PyDict_GetItemString(PyModule_GetDict(get_grt_module()), "Function");
   }    
 }
-
-
-
-#endif
-
