@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,13 +21,8 @@
 #ifndef __PYTHON_GRTMODULE_H__
 #define __PYTHON_GRTMODULE_H__
 
-#ifdef __APPLE__
-#include <Python/Python.h>
-#include <Python/object.h>
-#else
 #include <Python.h>
 #include <object.h>
-#endif
 
 #include "grtpp.h"
 
