@@ -1,31 +1,31 @@
-/* 
- * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
+/*
+ * Copyright (c) 2011, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; version 2 of the
  * License.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
 
-#include "tut_stdafx.h"
-
 #include "testgrt.h"
-#include "db_rev_eng_be.h"
+#include "backend/db_rev_eng_be.h"
 #include "module_db_mysql.h"
 
-#include "changelistobjects.h"
+#include "diff/changelistobjects.h"
+#include "grtdb/db_helpers.h"
+#include "wb_helpers.h"
 
-using namespace std;
+using namespace parser;
 using namespace tut;
 
 #define VERBOSE_TESTING 0
@@ -58,7 +58,7 @@ protected:
       std::string target_version = tester.wb->get_grt_manager()->get_app_option_string("DefaultTargetMySQLVersion");
       if (target_version.empty())
         target_version = "5.5";
-      tester.get_rdbms()->version(CatalogHelper::parse_version(tester.grt, target_version));
+      tester.get_rdbms()->version(parse_version(tester.grt, target_version));
 
       // init database connection
       connection= tester.create_connection_for_import();
@@ -66,12 +66,12 @@ protected:
       sql_parser= SqlFacade::instance_for_rdbms_name(tester.grt, "Mysql");
       ensure("failed to get sqlparser module", (NULL != sql_parser));
   }
-  
+
   TEST_DATA_DESTRUCTOR(grtdiff_alter_test)
   {
     std::auto_ptr<sql::Statement> stmt(connection->createStatement());
     std::string sql_string = "DROP DATABASE IF EXISTS grtdiff_alter_test;";
-    
+
     execute_script(stmt.get(), sql_string, tester.wb->get_grt_manager());
   }
 
@@ -91,7 +91,7 @@ TEST_FUNCTION(3)
   tester.wb->close_document_finish();
 }
 
-  static struct 
+  static struct
   {
     const char *description;
     const char *object_name;
@@ -119,19 +119,19 @@ TEST_FUNCTION(3)
     //  "alter CHARACTER SET",
     //  "grtdiff_alter_test",
     //  "DROP DATABASE IF EXISTS grtdiff_alter_test",
-    //  "CREATE DATABASE grtdiff_alter_test DEFAULT CHARACTER SET 'utf8'", 
+    //  "CREATE DATABASE grtdiff_alter_test DEFAULT CHARACTER SET 'utf8'",
     //  "CREATE DATABASE grtdiff_alter_test DEFAULT CHARACTER SET 'latin1'"
     //}, {
     //  "alter COLLATE",
     //  "grtdiff_alter_test",
     //  "DROP DATABASE IF EXISTS grtdiff_alter_test",
-    //  "CREATE DATABASE grtdiff_alter_test DEFAULT COLLATE 'utf8_general_ci'", 
+    //  "CREATE DATABASE grtdiff_alter_test DEFAULT COLLATE 'utf8_general_ci'",
     //  "CREATE DATABASE grtdiff_alter_test DEFAULT COLLATE 'utf8_general_cs'"
     //}, {
     //  "alter CHARACTER SET and COLLATE",
     //  "grtdiff_alter_test",
     //  "DROP DATABASE IF EXISTS grtdiff_alter_test",
-    //  "CREATE DATABASE grtdiff_alter_test DEFAULT CHARACTER SET 'utf8' COLLATE 'utf8_general_ci'", 
+    //  "CREATE DATABASE grtdiff_alter_test DEFAULT CHARACTER SET 'utf8' COLLATE 'utf8_general_ci'",
     //  "CREATE DATABASE grtdiff_alter_test DEFAULT CHARACTER SET 'latin1' COLLATE 'utf8_general_cs'"
     //},
 #endif
@@ -150,7 +150,7 @@ TEST_FUNCTION(3)
       "C C C+",
       "grtdiff_alter_test.t1",
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1",
-      "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) DEFAULT NULL, `id2` int(11) DEFAULT NULL) ENGINE=InnoDB DEFAULT CHARSET=latin1", 
+      "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) DEFAULT NULL, `id2` int(11) DEFAULT NULL) ENGINE=InnoDB DEFAULT CHARSET=latin1",
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) DEFAULT NULL, `id2` int(11) DEFAULT NULL, t TEXT) ENGINE=InnoDB DEFAULT CHARSET=latin1"
     }, {
       "C C C+ C+",
@@ -256,7 +256,7 @@ TEST_FUNCTION(3)
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1",
       "CREATE TABLE grtdiff_alter_test.t1 (t TEXT, t2 TEXT, `id` int(11) DEFAULT NULL, t3 TEXT, t4 TEXT, `id2` int(11) DEFAULT NULL, t5 TEXT, t6 TEXT) ENGINE=InnoDB DEFAULT CHARSET=latin1",
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) DEFAULT NULL, `id2` int(11) DEFAULT NULL) ENGINE=InnoDB DEFAULT CHARSET=latin1"
-    }, 
+    },
     // ALTER TABLE tests (ADD/DROP COLUMN  mix)
 #if 0 // crash due to server bug #31145
     {
@@ -345,7 +345,7 @@ TEST_FUNCTION(3)
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1",
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) DEFAULT NULL, `id2` int(11) DEFAULT NULL, t2 TEXT, `id3` int(11) DEFAULT NULL) ENGINE=InnoDB DEFAULT CHARSET=latin1",
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) DEFAULT NULL, t TEXT, `id2` int(11) DEFAULT NULL, `id3` int(11) DEFAULT NULL) ENGINE=InnoDB DEFAULT CHARSET=latin1"
-    }, 
+    },
 #if 0 // crash due to server bug #31145
     {
       "C C C- C+",
@@ -361,7 +361,7 @@ TEST_FUNCTION(3)
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1",
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) DEFAULT NULL, `id2` int(11) DEFAULT NULL, t TEXT, t2 TEXT) ENGINE=InnoDB DEFAULT CHARSET=latin1",
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) DEFAULT NULL, `id2` int(11) DEFAULT NULL, t3 TEXT) ENGINE=InnoDB DEFAULT CHARSET=latin1"
-    }, 
+    },
     // ALTER TABLE tests (CHANGE COLUMN position change, content change)
     {
       "C> C C",
@@ -385,7 +385,7 @@ TEST_FUNCTION(3)
       "C*", // a test for presicion/scale
       "grtdiff_alter_test.t1",
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1",
-      "CREATE TABLE grtdiff_alter_test.t1 (`id` DECIMAL) ENGINE=InnoDB DEFAULT CHARSET=latin1", 
+      "CREATE TABLE grtdiff_alter_test.t1 (`id` DECIMAL) ENGINE=InnoDB DEFAULT CHARSET=latin1",
       "CREATE TABLE grtdiff_alter_test.t1 (`id` DECIMAL(8,2)) ENGINE=InnoDB DEFAULT CHARSET=latin1"
     }, {
       "C C*> C",
@@ -471,7 +471,7 @@ TEST_FUNCTION(3)
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1",
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) DEFAULT NULL, `id2` int(11) DEFAULT NULL, t TEXT) ENGINE=InnoDB DEFAULT CHARSET=latin1",
       "CREATE TABLE grtdiff_alter_test.t1 (t TEXT, `id` int(11) DEFAULT NULL) ENGINE=InnoDB DEFAULT CHARSET=latin1"
-    }, 
+    },
     // TODO: test USING HASH and other indexKind values
     {
       "I(1)+",
@@ -577,21 +577,21 @@ TEST_FUNCTION(3)
       "F+",
       "grtdiff_alter_test.t1",
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1; DROP TABLE IF EXISTS grtdiff_alter_test.ref_t1;",
-      
+
       "CREATE TABLE grtdiff_alter_test.ref_t1 (`id` int(11) NOT NULL DEFAULT '0', PRIMARY KEY (`id`)) ENGINE=InnoDB DEFAULT CHARSET=latin1;"
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0', PRIMARY KEY (`id`)) ENGINE=InnoDB DEFAULT CHARSET=latin1;",
-      
+
       "CREATE TABLE grtdiff_alter_test.ref_t1 (`id` int(11) NOT NULL DEFAULT '0', PRIMARY KEY (`id`)) ENGINE=InnoDB DEFAULT CHARSET=latin1;"
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0', PRIMARY KEY (`id`), CONSTRAINT `c1` FOREIGN KEY (`id`) REFERENCES `grtdiff_alter_test`.`ref_t1` (`id`) ) ENGINE=InnoDB DEFAULT CHARSET=latin1"
     }, {
       "F+ F+",
       "grtdiff_alter_test.t1",
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1; DROP TABLE IF EXISTS grtdiff_alter_test.ref_t1; DROP TABLE IF EXISTS grtdiff_alter_test.ref_t2;",
-      
+
       "CREATE TABLE grtdiff_alter_test.ref_t1 (`id` int(11) NOT NULL DEFAULT '0', PRIMARY KEY (`id`)) ENGINE=InnoDB DEFAULT CHARSET=latin1;"
       "CREATE TABLE grtdiff_alter_test.ref_t2 (`id` int(11) NOT NULL DEFAULT '0', PRIMARY KEY (`id`)) ENGINE=InnoDB DEFAULT CHARSET=latin1;"
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0', PRIMARY KEY (`id`)) ENGINE=InnoDB DEFAULT CHARSET=latin1;",
-      
+
       "CREATE TABLE grtdiff_alter_test.ref_t1 (`id` int(11) NOT NULL DEFAULT '0', PRIMARY KEY (`id`)) ENGINE=InnoDB DEFAULT CHARSET=latin1;"
       "CREATE TABLE grtdiff_alter_test.ref_t2 (`id` int(11) NOT NULL DEFAULT '0', PRIMARY KEY (`id`)) ENGINE=InnoDB DEFAULT CHARSET=latin1;"
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0', PRIMARY KEY (`id`), CONSTRAINT `c1` FOREIGN KEY (`id`) REFERENCES `grtdiff_alter_test`.`ref_t1` (`id`), CONSTRAINT `c2` FOREIGN KEY (`id`) REFERENCES `grtdiff_alter_test`.`ref_t2` (`id`) ) ENGINE=InnoDB DEFAULT CHARSET=latin1"
@@ -599,12 +599,12 @@ TEST_FUNCTION(3)
       "F-",
       "grtdiff_alter_test.t1",
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1; DROP TABLE IF EXISTS grtdiff_alter_test.ref_t1;",
-            
+
       "CREATE TABLE grtdiff_alter_test.ref_t1 (`id` int(11) NOT NULL DEFAULT '0', PRIMARY KEY (`id`)) ENGINE=InnoDB DEFAULT CHARSET=latin1;"
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0', PRIMARY KEY (`id`), CONSTRAINT `c1` FOREIGN KEY (`id`) REFERENCES `grtdiff_alter_test`.`ref_t1` (`id`) ) ENGINE=InnoDB DEFAULT CHARSET=latin1",
 
       "CREATE TABLE grtdiff_alter_test.ref_t1 (`id` int(11) NOT NULL DEFAULT '0', PRIMARY KEY (`id`)) ENGINE=InnoDB DEFAULT CHARSET=latin1;"
-      "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0', PRIMARY KEY (`id`)) ENGINE=InnoDB DEFAULT CHARSET=latin1;" 
+      "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0', PRIMARY KEY (`id`)) ENGINE=InnoDB DEFAULT CHARSET=latin1;"
     }, {
       "F- F-",
       "grtdiff_alter_test.t1",
@@ -708,7 +708,7 @@ TEST_FUNCTION(3)
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1;",
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0') ENGINE=InnoDB DEFAULT CHARSET=latin1",
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0') ENGINE=InnoDB DEFAULT CHARSET=latin1 DELAY_KEY_WRITE = 1"
-    }, { 
+    }, {
 #if 0  // TODO: need a fix from rev-eng
 
       "Change UNION, INSERT_METHOD attribute",
@@ -877,10 +877,10 @@ TEST_FUNCTION(3)
       "Add range partitioning ",
       "grtdiff_alter_test.t1",
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1;",
-      
+
       "CREATE TABLE grtdiff_alter_test.t1 (id int(11) NOT NULL DEFAULT '0', hired DATE NOT NULL)"
       " ENGINE=InnoDB DEFAULT CHARSET=latin1",
-      
+
       "CREATE TABLE grtdiff_alter_test.t1 (id int(11) NOT NULL DEFAULT '0', hired DATE NOT NULL)"
       " ENGINE=InnoDB DEFAULT CHARSET=latin1"
       " PARTITION BY RANGE(YEAR(hired)) ("
@@ -893,7 +893,7 @@ TEST_FUNCTION(3)
       "Remove range partitioning ",
       "grtdiff_alter_test.t1",
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1;",
-      
+
       "CREATE TABLE grtdiff_alter_test.t1 (id int(11) NOT NULL DEFAULT '0', hired DATE NOT NULL)"
       " ENGINE=InnoDB DEFAULT CHARSET=latin1"
       " PARTITION BY RANGE(YEAR(hired)) ("
@@ -908,7 +908,7 @@ TEST_FUNCTION(3)
       "Range partitioning: change expression, partition LESS THAN values ",
       "grtdiff_alter_test.t1",
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1;",
-      
+
       "CREATE TABLE grtdiff_alter_test.t1 (id int(11) NOT NULL DEFAULT '0', hired DATE NOT NULL)"
       " ENGINE=InnoDB DEFAULT CHARSET=latin1"
       " PARTITION BY RANGE(YEAR(hired)) ("
@@ -928,7 +928,7 @@ TEST_FUNCTION(3)
       "Range partitioning: add partition",
       "grtdiff_alter_test.t1",
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1;",
-      
+
       "CREATE TABLE grtdiff_alter_test.t1 (id int(11) NOT NULL DEFAULT '0', hired DATE NOT NULL)"
       " ENGINE=InnoDB DEFAULT CHARSET=latin1"
       " PARTITION BY RANGE(YEAR(hired)) ("
@@ -945,7 +945,7 @@ TEST_FUNCTION(3)
       "Range partitioning: drop 2 partitions",
       "grtdiff_alter_test.t1",
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1;",
-      
+
       "CREATE TABLE grtdiff_alter_test.t1 (id int(11) NOT NULL DEFAULT '0', hired DATE NOT NULL)"
       " ENGINE=InnoDB DEFAULT CHARSET=latin1"
       " PARTITION BY RANGE(YEAR(hired)) ("
@@ -963,7 +963,7 @@ TEST_FUNCTION(3)
       "Range partitioning: reorganize partitions",
       "grtdiff_alter_test.t1",
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1;",
-      
+
       "CREATE TABLE grtdiff_alter_test.t1 (id int(11) NOT NULL DEFAULT '0', hired DATE NOT NULL)"
       " ENGINE=InnoDB DEFAULT CHARSET=latin1"
       " PARTITION BY RANGE(YEAR(hired)) ("
@@ -983,7 +983,7 @@ TEST_FUNCTION(3)
       "List partitioning: reorganize partitions",
       "grtdiff_alter_test.t1",
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1;",
-      
+
       "CREATE TABLE grtdiff_alter_test.t1 (id int(11) NOT NULL DEFAULT '0', hired DATE NOT NULL)"
       " ENGINE=InnoDB DEFAULT CHARSET=latin1"
       " PARTITION BY LIST(id) ("
@@ -1003,7 +1003,7 @@ TEST_FUNCTION(3)
       "List partitioning: add, remove, change partitions",
       "grtdiff_alter_test.t1",
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1;",
-      
+
       "CREATE TABLE grtdiff_alter_test.t1 (id int(11) NOT NULL DEFAULT '0', hired DATE NOT NULL)"
       " ENGINE=InnoDB DEFAULT CHARSET=latin1"
       " PARTITION BY LIST(id) ("
@@ -1058,13 +1058,13 @@ TEST_FUNCTION(3)
       "DROP PROCEDURE IF EXISTS grtdiff_alter_test.p1;",
       "CREATE DATABASE IF NOT EXISTS grtdiff_alter_test;",
       "DELIMITER //\nCREATE DEFINER=`root`@`localhost` PROCEDURE `grtdiff_alter_test`.`p1`()\nBEGIN SELECT 1; SELECT 2; END//"
-    }, {  
+    }, {
       "Drop procedure",
       "grtdiff_alter_test.p1",
       "DROP PROCEDURE IF EXISTS grtdiff_alter_test.p1;",
       "DELIMITER //\nCREATE DEFINER=`root`@`localhost` PROCEDURE `grtdiff_alter_test`.`p1`()\nBEGIN SELECT 1; SELECT 2; END//",
       "CREATE DATABASE IF NOT EXISTS grtdiff_alter_test;"
-    }, {  
+    }, {
       "Change procedure",
       "grtdiff_alter_test.p1",
       "DROP PROCEDURE IF EXISTS grtdiff_alter_test.p1;",
@@ -1076,7 +1076,7 @@ TEST_FUNCTION(3)
     {
       "Create trigger",
       "grtdiff_alter_test.tr1",
-      
+
       "DROP TRIGGER IF EXISTS grtdiff_alter_test.tr1;DROP TABLE IF EXISTS grtdiff_alter_test.t1;",
 
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0') ENGINE=MyISAM DEFAULT CHARSET=latin1;\n",
@@ -1087,7 +1087,7 @@ TEST_FUNCTION(3)
     }, {
       "Drop trigger",
       "grtdiff_alter_test.tr1",
-      
+
       "DROP TRIGGER IF EXISTS grtdiff_alter_test.tr1;DROP TABLE IF EXISTS grtdiff_alter_test.t1;",
 
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0') ENGINE=MyISAM DEFAULT CHARSET=latin1;\n"
@@ -1098,7 +1098,7 @@ TEST_FUNCTION(3)
     }, {
       "Change trigger",
       "grtdiff_alter_test.tr1",
-      
+
       "DROP TRIGGER IF EXISTS grtdiff_alter_test.tr1;DROP TABLE IF EXISTS grtdiff_alter_test.t1;",
 
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0') ENGINE=MyISAM DEFAULT CHARSET=latin1;\n"
@@ -1108,7 +1108,7 @@ TEST_FUNCTION(3)
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0') ENGINE=MyISAM DEFAULT CHARSET=latin1;\n"
       "USE grtdiff_alter_test;\n"
       "DELIMITER //\nCREATE\nDEFINER=`root`@`localhost`\nTRIGGER `grtdiff_alter_test`.`tr1`\nBEFORE INSERT ON `grtdiff_alter_test`.`t1`\nFOR EACH ROW\nBEGIN DELETE FROM t2; END//"
-    }, 
+    },
 #endif  // TRIGGER_TESTS
 #ifdef CREATE_TESTS
     {
@@ -1186,36 +1186,39 @@ TEST_FUNCTION(3)
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0') ENGINE=MyISAM DEFAULT CHARSET=latin1 COLLATE=latin1_german1_ci",
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0') ENGINE=MyISAM DEFAULT CHARSET=latin1"
     },
-    {
-      NULL, NULL, NULL
-    }
+    {NULL, NULL, NULL, NULL, NULL}
   };
 
 
-TEST_FUNCTION(6)
+TEST_FUNCTION(10)
 {
   boost::shared_ptr<DiffChange> alter_change;
   boost::shared_ptr<DiffChange> empty_change;
 
   // column insertion
-  
+
   ensure("connection is NULL", connection.get() != NULL);
 
   {
     std::auto_ptr<sql::Statement> stmt(connection->createStatement());
     execute_script(stmt.get(), "DROP DATABASE IF EXISTS grtdiff_alter_test;"
-                               "DROP DATABASE IF EXISTS grtdiff_alter_test2",tester.wb->get_grt_manager());
+                               "DROP DATABASE IF EXISTS grtdiff_alter_test2", tester.wb->get_grt_manager());
     execute_script(stmt.get(), "CREATE DATABASE IF NOT EXISTS grtdiff_alter_test /*!40100 DEFAULT CHARACTER SET latin1 */",tester.wb->get_grt_manager());
   }
 
-  for(int i= 0; data[i].description != NULL; i++)
-  {
-#if VERBOSE_TESTING
-    std::cout << "iteration " << i << std::endl;
-#endif
+  MySQLParserServices::Ref services = MySQLParserServices::get(tester.grt);
+  ParserContext::Ref context = services->createParserContext(tester.get_rdbms()->characterSets(),
+    tester.get_rdbms()->version(), false);
 
-    db_mysql_CatalogRef org_cat= create_empty_catalog_for_import(tester.grt);
-    db_mysql_CatalogRef mod_cat= create_empty_catalog_for_import(tester.grt);
+  grt::DictRef options(tester.grt);
+  for (int i = 0; data[i].description != NULL; i++)
+  {
+    std::cout << ".";
+    if ((i + 1) % 30 == 0)
+      std::cout << std::endl;
+
+    db_mysql_CatalogRef org_cat = create_empty_catalog_for_import(tester.grt);
+    db_mysql_CatalogRef mod_cat = create_empty_catalog_for_import(tester.grt);
 
     {
       std::string org_script;
@@ -1223,7 +1226,8 @@ TEST_FUNCTION(6)
         .append("CREATE DATABASE IF NOT EXISTS grtdiff_alter_test /*!40100 DEFAULT CHARACTER SET latin1 */;\n")
         .append(data[i].org)
         .append(";");
-      sql_parser->parseSqlScriptString(org_cat, org_script);
+      services->parseSQLIntoCatalog(context, org_cat, org_script, options);
+
     }
     {
       std::string mod_script;
@@ -1231,7 +1235,7 @@ TEST_FUNCTION(6)
         .append("CREATE DATABASE IF NOT EXISTS grtdiff_alter_test /*!40100 DEFAULT CHARACTER SET latin1 */;\n")
         .append(data[i].mod)
         .append(";");
-      sql_parser->parseSqlScriptString(mod_cat, mod_script);
+      services->parseSQLIntoCatalog(context, mod_cat, mod_script, options);
     }
 
     if(strcmp(data[i].description, "C CR C") == 0)
@@ -1253,14 +1257,14 @@ TEST_FUNCTION(6)
     normalizer.init_omf(&omf);
     grt::ValueRef default_engine = tester.wb->get_grt_manager()->get_app_option("db.mysql.Table:tableEngine");
     std::string default_engine_name;
-    if(grt::StringRef::can_wrap(default_engine))
-        default_engine_name = grt::StringRef::cast_from(default_engine);
+    if (grt::StringRef::can_wrap(default_engine))
+      default_engine_name = grt::StringRef::cast_from(default_engine);
 
     bec::CatalogHelper::apply_defaults(mod_cat, default_engine_name);
     bec::CatalogHelper::apply_defaults(org_cat, default_engine_name);
 
-    alter_change= diff_make(org_cat, mod_cat, &omf);
-    ensure("Empty alter:",alter_change);
+    alter_change = diff_make(org_cat, mod_cat, &omf);
+    ensure("Empty alter:", (bool)alter_change);
 
 #if VERBOSE_TESTING
     alter_change->dump_log(0);
@@ -1276,13 +1280,13 @@ TEST_FUNCTION(6)
     options.set("CaseSensitive", grt::IntegerRef(omf.case_sensitive));
 
     diffsql_module->generateSQL(mod_cat, options, alter_change);
-    diffsql_module->makeSQLSyncScript(options, alter_map, alter_object_list);
+    diffsql_module->makeSQLSyncScript(mod_cat, options, alter_map, alter_object_list);
     std::string export_sql_script= options.get_string("OutputScript");
 
     // 2. apply it to server
     std::auto_ptr<sql::Statement> stmt(connection->createStatement());
 
-    try 
+    try
     {
       execute_script(stmt.get(), data[i].cleanup,tester.wb->get_grt_manager());
       execute_script(stmt.get(), data[i].org,tester.wb->get_grt_manager());
@@ -1325,15 +1329,13 @@ TEST_FUNCTION(6)
 
     normalizer.init_omf(&omf);
     empty_change= diff_make(cat, mod_cat, &omf);
-    tester.wb->close_document();
-    tester.wb->close_document_finish();
 
     if (empty_change)
     {
       alter_map.clear();
       alter_object_list.clear();
       diffsql_module->generateSQL(mod_cat, options, empty_change);
-      diffsql_module->makeSQLSyncScript(options, alter_map, alter_object_list);
+      diffsql_module->makeSQLSyncScript(mod_cat, options, alter_map, alter_object_list);
 
       // we can't check for the changeset to make sure there are not changes, because some changes from the diff
       // don't cause a script to be generated (like foreign keys being reordered)
@@ -1348,18 +1350,18 @@ TEST_FUNCTION(6)
           fail(data[i].description);
       }
     }
+    tester.wb->close_document();
+    tester.wb->close_document_finish();
   }
+  std::cout << std::endl;
 }
 
-TEST_FUNCTION(7)
+TEST_FUNCTION(20)
 {
   boost::shared_ptr<DiffChange> alter_change;
   boost::shared_ptr<DiffChange> empty_change;
 
   // column insertion
-
-
-  
   ensure("connection is NULL", connection.get() != NULL);
 
   {
@@ -1369,109 +1371,116 @@ TEST_FUNCTION(7)
     execute_script(stmt.get(), "CREATE DATABASE IF NOT EXISTS grtdiff_alter_test /*!40100 DEFAULT CHARACTER SET latin1 */",tester.wb->get_grt_manager());
   }
 
-  for(int i= 2; data[i].description != NULL; i++)
-	  for (int j = 0; j <= 1; ++j)
+  MySQLParserServices::Ref services = MySQLParserServices::get(tester.grt);
+  ParserContext::Ref context = services->createParserContext(tester.get_rdbms()->characterSets(),
+    tester.get_rdbms()->version(), false);
+
+  grt::DictRef options(tester.grt);
+  for (int i = 0; data[i].description != NULL; i++)
   {
-#if VERBOSE_TESTING
-    std::cout << "iteration " << i << std::endl;
-#endif
-
-    db_mysql_CatalogRef org_cat= create_empty_catalog_for_import(tester.grt);
-    db_mysql_CatalogRef mod_cat= create_empty_catalog_for_import(tester.grt);
+    std::cout << ".";
+    if ((i + 1) % 30 == 0)
+      std::cout << std::endl;
 
+    for (int j = 0; j <= 1; ++j)
     {
-      std::string org_script;
-      org_script
-        .append("CREATE DATABASE IF NOT EXISTS grtdiff_alter_test /*!40100 DEFAULT CHARACTER SET latin1 */;\n")
-        .append(data[i].org)
-        .append(";");
-      sql_parser->parseSqlScriptString(org_cat, org_script);
-    }
-    {
-      std::string mod_script;
-      mod_script
-        .append("CREATE DATABASE IF NOT EXISTS grtdiff_alter_test /*!40100 DEFAULT CHARACTER SET latin1 */;\n")
-        .append(data[i].mod)
-        .append(";");
-      sql_parser->parseSqlScriptString(mod_cat, mod_script);
-    }
+      db_mysql_CatalogRef org_cat = create_empty_catalog_for_import(tester.grt);
+      db_mysql_CatalogRef mod_cat = create_empty_catalog_for_import(tester.grt);
 
-    if(strcmp(data[i].description, "C CR C") == 0)
-      mod_cat->schemata().get(0)->tables().get(0)->columns().get(1)->oldName("t");
-    else if(strcmp(data[i].description, "C CR> C") == 0)
-      mod_cat->schemata().get(0)->tables().get(0)->columns().get(2)->oldName("t");
-    else if(strcmp(data[i].description, "C CR< C") == 0)
-      mod_cat->schemata().get(0)->tables().get(0)->columns().get(0)->oldName("t");
-    else if(strcmp(data[i].description, "C CR*> C") == 0)
-      mod_cat->schemata().get(0)->tables().get(0)->columns().get(2)->oldName("t");
-    else if(strcmp(data[i].description, "C CR*< C") == 0)
-      mod_cat->schemata().get(0)->tables().get(0)->columns().get(0)->oldName("t");
-    else if(strcmp(data[i].description, "Rename table") == 0)
-      mod_cat->schemata().get(0)->tables().get(0)->oldName("t1");
-    else if(strcmp(data[i].description, "Rename view") == 0)
-      mod_cat->schemata().get(0)->views().get(0)->oldName("v1");
+      {
+        std::string org_script;
+        org_script
+          .append("CREATE DATABASE IF NOT EXISTS grtdiff_alter_test /*!40100 DEFAULT CHARACTER SET latin1 */;\n")
+          .append(data[i].org)
+          .append(";");
+        services->parseSQLIntoCatalog(context, org_cat, org_script, options);
+      }
+      {
+        std::string mod_script;
+        mod_script
+          .append("CREATE DATABASE IF NOT EXISTS grtdiff_alter_test /*!40100 DEFAULT CHARACTER SET latin1 */;\n")
+          .append(data[i].mod)
+          .append(";");
+        services->parseSQLIntoCatalog(context, mod_cat, mod_script, options);
+      }
 
-    grt::NormalizedComparer normalizer(tester.grt);
-    normalizer.init_omf(&omf);
-        grt::ValueRef default_engine = tester.wb->get_grt_manager()->get_app_option("db.mysql.Table:tableEngine");
-    std::string default_engine_name;
-    if(grt::StringRef::can_wrap(default_engine))
+      if (strcmp(data[i].description, "C CR C") == 0)
+        mod_cat->schemata().get(0)->tables().get(0)->columns().get(1)->oldName("t");
+      else if (strcmp(data[i].description, "C CR> C") == 0)
+        mod_cat->schemata().get(0)->tables().get(0)->columns().get(2)->oldName("t");
+      else if (strcmp(data[i].description, "C CR< C") == 0)
+        mod_cat->schemata().get(0)->tables().get(0)->columns().get(0)->oldName("t");
+      else if (strcmp(data[i].description, "C CR*> C") == 0)
+        mod_cat->schemata().get(0)->tables().get(0)->columns().get(2)->oldName("t");
+      else if (strcmp(data[i].description, "C CR*< C") == 0)
+        mod_cat->schemata().get(0)->tables().get(0)->columns().get(0)->oldName("t");
+      else if (strcmp(data[i].description, "Rename table") == 0)
+        mod_cat->schemata().get(0)->tables().get(0)->oldName("t1");
+      else if (strcmp(data[i].description, "Rename view") == 0)
+        mod_cat->schemata().get(0)->views().get(0)->oldName("v1");
+
+      grt::NormalizedComparer normalizer(tester.grt);
+      normalizer.init_omf(&omf);
+      grt::ValueRef default_engine = tester.wb->get_grt_manager()->get_app_option("db.mysql.Table:tableEngine");
+      std::string default_engine_name;
+      if (grt::StringRef::can_wrap(default_engine))
         default_engine_name = grt::StringRef::cast_from(default_engine);
 
-    bec::CatalogHelper::apply_defaults(mod_cat, default_engine_name);
-    bec::CatalogHelper::apply_defaults(org_cat, default_engine_name);
+      bec::CatalogHelper::apply_defaults(mod_cat, default_engine_name);
+      bec::CatalogHelper::apply_defaults(org_cat, default_engine_name);
 
 
-    alter_change= diff_make(org_cat, mod_cat, &omf);
-    ensure("Empty alter:",alter_change);
+      alter_change = diff_make(org_cat, mod_cat, &omf);
+      ensure("Empty alter:", (bool)alter_change);
 
 #if VERBOSE_TESTING
-    alter_change->dump_log(0);
+      alter_change->dump_log(0);
 #endif
+      
+      const char TemplateFile[] = "data/reporting/Basic_Text.tpl/basic_text_report.txt.tpl";
 
-    const char TemplateFile[] = "data/reporting/Basic_Text.tpl/basic_text_report.txt.tpl";
+      // 1. generate alter
+      grt::StringListRef alter_map(tester.grt);
+      grt::DictRef options(tester.grt);
+      options.set("UseFilteredLists", grt::IntegerRef(0));
+      options.set("OutputContainer", alter_map);
+      options.set("TemplateFile", grt::StringRef(TemplateFile));
+      options.set("UseShortNames", grt::IntegerRef(j));
+      options.set("SeparateForeignKeys", grt::IntegerRef(0));
 
-    // 1. generate alter
-    grt::StringListRef alter_map(tester.grt);
-    grt::DictRef options(tester.grt);
-    options.set("UseFilteredLists", grt::IntegerRef(0));
-    options.set("OutputContainer", alter_map);
-    options.set("TemplateFile", grt::StringRef(TemplateFile));
-    options.set("UseShortNames", grt::IntegerRef(j));
-    options.set("SeparateForeignKeys", grt::IntegerRef(0));
- 
-    grt::StringRef report = diffsql_module->generateReport(org_cat, options, alter_change);
+      std::string report = diffsql_module->generateReport(org_cat, options, alter_change);
+      std::string reportFile = base::strfmt("data/reporting/Basic_Text.tpl/reports/testres%s%d.txt",
+        (j != 0) ? "_longname" : "_shortname", i);
 
-    char buf1[1000];
-    
-    g_snprintf(buf1, sizeof(buf1), "data/reporting/Basic_Text.tpl/reports/testres%s%d.txt",j?"_longname":"_shortname", i);
+      std::ifstream rep;
+      rep.open(reportFile.c_str());
+      std::string expected((std::istreambuf_iterator<char>(rep)), std::istreambuf_iterator<char>());
 
-    std::ifstream rep;
-    rep.open(buf1);
-    std::string str((std::istreambuf_iterator<char>(rep)), std::istreambuf_iterator<char>());
+      if (report != expected)
+      {
+        std::cout << std::endl << std::endl << "Reports differ, expected:" << std::endl;
+        //alter_change->dump_log(0);
+        std::cout << expected << std::endl << std::endl;
+        std::cout << "==================================================" << std::endl << std::endl << "actual: " << std::endl;
+        std::cout << report << std::endl << std::endl;
+        fail(reportFile);
+      }
 
-    if (report != str)
-    {
-        alter_change->dump_log(0);
-        int pp = strcmp(report.c_str(),str.c_str());
-        std::cout<<report.c_str()<<std::endl;
-        std::cout<<"=======================================================================================================================\n";
-        std::cout<<str<<std::endl;
-        fail(buf1);
+      // Test Data generation
+      /*
+        sprintf(buf1, "testres%s%d.txt",j?"_longname":"_shortname", i);
+        std::ofstream out;
+        out.open(buf1);
+        out<<(*report).c_str();
+        out.close();
+      */
     }
-// Test Data generation
-/*
-	sprintf(buf1, "testres%s%d.txt",j?"_longname":"_shortname", i);
-	std::ofstream out;
-	out.open(buf1);
-	out<<(*report).c_str();
-	out.close();
-*/
   }
+  std::cout << std::endl;
 }
 
 
-static struct 
+static struct
 {
   const char *description;
   const char *object_name;
@@ -1522,18 +1531,15 @@ static struct
       "CREATE TABLE grtdiff_alter_test.t1 (`a` enum('a','b','c'))",
       "CREATE TABLE grtdiff_alter_test.t1 (`a` ENUM('a', 'b',    'c'  ))"
     },
-    {NULL, NULL}
+    {NULL, NULL, NULL, NULL, NULL}
 };
 
-
-
-
-TEST_FUNCTION(5)
+TEST_FUNCTION(30)
 {
   boost::shared_ptr<DiffChange> empty_change;
 
   // column insertion
-  
+
   ensure("connection is NULL", connection.get() != NULL);
 
   {
@@ -1543,11 +1549,16 @@ TEST_FUNCTION(5)
     execute_script(stmt.get(), "CREATE DATABASE IF NOT EXISTS grtdiff_alter_test /*!40100 DEFAULT CHARACTER SET latin1 */",tester.wb->get_grt_manager());
   }
 
-  for(int i= 0; neg_data[i].description != NULL; i++)
+  MySQLParserServices::Ref services = MySQLParserServices::get(tester.grt);
+  ParserContext::Ref context = services->createParserContext(tester.get_rdbms()->characterSets(),
+    tester.get_rdbms()->version(), false);
+
+  grt::DictRef options(tester.grt);
+  for (int i = 0; neg_data[i].description != NULL; i++)
   {
-#if VERBOSE_TESTING
-    std::cout << "iteration " << i << std::endl;
-#endif
+    std::cout << ".";
+    if ((i + 1) % 30 == 0)
+      std::cout << std::endl;
 
     db_mysql_CatalogRef org_cat= create_empty_catalog_for_import(tester.grt);
     db_mysql_CatalogRef mod_cat= create_empty_catalog_for_import(tester.grt);
@@ -1558,7 +1569,7 @@ TEST_FUNCTION(5)
         .append("CREATE DATABASE IF NOT EXISTS grtdiff_alter_test /*!40100 DEFAULT CHARACTER SET latin1 */;\n")
         .append(neg_data[i].org)
         .append(";");
-      sql_parser->parseSqlScriptString(org_cat, org_script);
+      services->parseSQLIntoCatalog(context, org_cat, org_script, options);
     }
     {
       std::string mod_script;
@@ -1566,7 +1577,7 @@ TEST_FUNCTION(5)
         .append("CREATE DATABASE IF NOT EXISTS grtdiff_alter_test /*!40100 DEFAULT CHARACTER SET latin1 */;\n")
         .append(neg_data[i].mod)
         .append(";");
-      sql_parser->parseSqlScriptString(mod_cat, mod_script);
+      services->parseSQLIntoCatalog(context, mod_cat, mod_script, options);
     }
 
     if(strcmp(neg_data[i].description, "C CR C") == 0)
@@ -1610,7 +1621,7 @@ TEST_FUNCTION(5)
       alter_map.clear();
       alter_object_list.clear();
       diffsql_module->generateSQL(mod_cat, options, empty_change);
-      diffsql_module->makeSQLSyncScript(options, alter_map, alter_object_list);
+      diffsql_module->makeSQLSyncScript(mod_cat, options, alter_map, alter_object_list);
 
       // we can't check for the changeset to make sure there are not changes, because some changes from the diff
       // don't cause a script to be generated (like foreign keys being reordered)
@@ -1625,8 +1636,8 @@ TEST_FUNCTION(5)
       }
     }
   }
+  std::cout << std::endl;
 }
 
 
 END_TESTS
-
