@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "tut_stdafx.h"
-
 #include <grtpp_util.h>
 
 #include "testgrt.h"
@@ -64,7 +62,7 @@ TEST_FUNCTION(2)
   grt::internal::Value* value= iv.valueptr();
   iv.clear();
   ensure("Completely release value", iv.valueptr() == NULL);
-  ensure_equals("Reference count", value->refcount(), 3U);
+  ensure_equals("Reference count", value->refcount(), 3);
 
   value->release();
   value->release();
@@ -85,13 +83,13 @@ TEST_FUNCTION(2)
 TEST_FUNCTION(4)
 {
   IntegerRef iv(1234);
-  int i;
+  ssize_t i;
 
   ensure("IntegerRef(NULL)", !IntegerRef((grt::internal::Integer*)NULL).is_valid());
   ensure("IntegerRef(0)", IntegerRef(0).is_valid());
   ensure("IntegerRef()", !IntegerRef().is_valid());
 
-  i= iv;
+  i = iv;
   ensure_equals("IntegerRef to int", i, 1234);
 
   ensure_equals("IntegerRef refcount", iv.refcount(), 1);
