@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,8 +22,10 @@
 #include <string>
 #include <stack>
 #include <set>
+#include <map>
 
 #include <antlr3.h>
+#include <glib.h>
 
 #include "MySQLLexer.h"  // The generated lexer.
 #include "MySQLParser.h" // The generated parser.
@@ -58,10 +60,6 @@ std::string get_token_name(pANTLR3_UINT8 *tokenNames, ANTLR3_UINT32 token)
     return "opening curly brace";
   case CLOSE_CURLY_SYMBOL:
     return "closing curly brace";
-  case OPEN_BRACKET_SYMBOL:
-    return "opening bracket";
-  case CLOSE_BRACKET_SYMBOL:
-    return "closing bracket";
   case NULL2_SYMBOL:
     return "null escape sequence";
   case PARAM_MARKER:
@@ -80,44 +78,185 @@ std::string get_token_name(pANTLR3_UINT8 *tokenNames, ANTLR3_UINT32 token)
 
 //--------------------------------------------------------------------------------------------------
 
-bool handle_lexer_error(pANTLR3_BASE_RECOGNIZER recognizer, pANTLR3_EXCEPTION exception,
+std::string formatVersion(long version)
+{
+  long major = version / 10000, minor = (version / 100) % 100, release = version % 100;
+  return base::strfmt("%ld.%ld.%ld", major, minor, release);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Parses a server version predicate to get out the version + a word for the error message
+ * that describes the version relationship.
+ */
+std::string handleServerVersion(const std::vector<std::string> parts, bool withPrefix)
+{
+  bool includesEquality = parts[1].size() == 2;
+  std::string version = formatVersion(atoi(parts[2].c_str()));
+  switch (parts[1][0])
+  {
+    case '<': // A max version.
+      if (!includesEquality)
+        return withPrefix ? "server versions before " + version : "before " + version;
+      return withPrefix ? "server versions up to " + version : "up to " + version;
+    case '=': // An exact version.
+      return "the server version " + version;
+    case '>': // A min version.
+      if (!includesEquality)
+        withPrefix ? "server versions after " + version : "after " + version;
+      return withPrefix ? "server versions starting with " + version : "starting with " + version;
+
+    default:
+      return "specific versions"; // Unexpected operator found. Return a generic message part.
+  }
+}
+
+//------------------------------------------------------------------------------------------------
+
+/**
+ * Parses the given predicate and constructs an error message that tells why there is that error.
+ */
+std::string createErrorFromPredicate(std::string predicate, long version)
+{
+  // Parsable predicates have one of these forms:
+  // - "SERVER_VERSION >= 50100"
+  // - "(SERVER_VERSION >= 50105) && (SERVER_VERSION < 50500)"
+  // - "SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES)"
+  // - "!SQL_MODE_ACTIVE(SQL_MODE_ANSI_QUOTES)"
+  //
+  // We don't do full expression parsing here. Only what is given above.
+  predicate = base::trim(predicate);
+  std::vector<std::string> parts = base::split(predicate, "&&");
+  std::string message = "\nThis syntax is only allowed for %s. The current version is " + formatVersion(version);
+  switch (parts.size())
+  {
+    case 2:
+    {
+      // Min and max values for server versions.
+      std::string messagePart = "unknown conditions";
+      std::string expression = base::trim(parts[0]);
+      if (base::starts_with(expression, "(") && base::ends_with(expression, ")"))
+        expression = expression.substr(1, expression.size() - 2);
+      std::vector<std::string> expressionParts = base::split(expression, " ");
+      if ((expressionParts[0] == "SERVER_VERSION") && (expressionParts.size() == 3))
+        messagePart = handleServerVersion(expressionParts, true);
+
+      expression = base::trim(parts[1]);
+      if (base::starts_with(expression, "(") && base::ends_with(expression, ")"))
+        expression = expression.substr(1, expression.size() - 2);
+      expressionParts = base::split(expression, " ");
+      if ((expressionParts[0] == "SERVER_VERSION") && (expressionParts.size() == 3))
+        messagePart += " and " + handleServerVersion(expressionParts, false);
+
+      return base::strfmt(message.c_str(), messagePart.c_str());
+    }
+
+    case 1:
+    {
+      // A single expression.
+      std::string messagePart = "unknown conditions";
+      std::vector<std::string> expressionParts = base::split(predicate, " ");
+      if (expressionParts.size() == 1)
+      {
+        message = "\nThis syntax is only allowed if ";
+        if (base::starts_with(predicate, "SQL_MODE_ACTIVE("))
+          message += predicate.substr(16, predicate.size() - 17) + " is active.";
+        else if (base::starts_with(predicate, "!SQL_MODE_ACTIVE("))
+          message += predicate.substr(17, predicate.size() - 18) + " is not active.";
+      }
+      else
+      {
+        if ((expressionParts[0] == "SERVER_VERSION") && (expressionParts.size() == 3))
+          messagePart = handleServerVersion(expressionParts, true);
+      }
+
+      return base::strfmt(message.c_str(), messagePart.c_str());
+    }
+    default:
+      return "";
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+bool handleLexerError(pANTLR3_BASE_RECOGNIZER recognizer, pANTLR3_EXCEPTION exception,
   ANTLR3_MARKER &start, ANTLR3_MARKER &length, std::string &message)
 {
+  std::ostringstream error;
   pANTLR3_LEXER lexer = (pANTLR3_LEXER)(recognizer->super);
   start = recognizer->state->tokenStartCharIndex;
 
-  // For debugging use the native error message that contains the grammar line.
-  // std::string native = (pANTLR3_UINT8)(exception->message);
-  length = ANTLR3_UINT32_CAST(((pANTLR3_UINT8)(lexer->input->data) + (lexer->input->size(lexer->input))) - (pANTLR3_UINT8)(exception->index));
+  length = exception->index - start;
 
-  if (length <= 0)
+  std::string tokenText((char*)start, length);
+  switch (exception->type)
   {
-    message = "unexpected end of input (unfinished string or quoted identifier)";
-    length = ANTLR3_UINT32_CAST(((pANTLR3_UINT8)(lexer->input->data) + (lexer->input->size(lexer->input))) - (pANTLR3_UINT8)(lexer->rec->state->tokenStartCharIndex));
-  }
-  else
-  {
-    switch (exception->type)
-    {
     case ANTLR3_RECOGNITION_EXCEPTION:
-      // Invalid character. Can currently never appear because any input from the Unicode BMP
-      // is acceptable input for our parser. We cannot parse input beyond the BMP.
-      message += "'";
-      if (isprint(exception->c))
-        message += (char)exception->c;
-      else
-        message += (ANTLR3_UINT8)(exception->c);
-
-      message += "' is not valid input";
+    {
+      switch (tokenText[0])
+      {
+        case '/':
+          error << "unfinished multiline comment";
+          break;
+        case 'x':
+        case 'X':
+          error << "unfinished hex string literal";
+          break;
+        case 'b':
+        case 'B':
+          error << "unfinished binary string literal";
+          break;
+        default:
+          error << "unexpected input";
+          break;
+      }
       break;
     }
+
+    case ANTLR3_NO_VIABLE_ALT_EXCEPTION:
+    {
+      switch (recognizer->state->type) {
+        case DOUBLE_QUOTE:
+          error << "unfinished double quote string";
+          break;
+
+        case SINGLE_QUOTE:
+          error << "unfinished single quote string";
+          break;
+
+        case BACK_TICK:
+          error << "unfinished back tick quote string";
+          break;
+
+        default:
+          error << "unexpected input";
+          break;
+      }
+      break;
+    }
+
+    case ANTLR3_FAILED_PREDICATE_EXCEPTION:
+    {
+      // One of the semantic predicates failed. Since most of those are our version check predicates
+      // we can use that to give the user a hint about this.
+      std::string predicate = (const char*)exception->message;
+      RecognitionContext *context = (RecognitionContext*)recognizer->state->userp;
+      error << "'" << tokenText << "' is not a valid keyword" << createErrorFromPredicate(predicate, context->version);
+      break;
+    }
+
+    default:
+      return false;
   }
+
+  message = error.str();
   return true;
 }
 
 //--------------------------------------------------------------------------------------------------
 
-bool handle_parser_error(pANTLR3_BASE_RECOGNIZER recognizer, pANTLR3_EXCEPTION exception,
+bool handleParserError(pANTLR3_BASE_RECOGNIZER recognizer, pANTLR3_EXCEPTION exception,
   pANTLR3_UINT8 *tokenNames, ANTLR3_MARKER &start, ANTLR3_MARKER &length, std::string &message)
 {
   std::ostringstream error;
@@ -135,11 +274,10 @@ bool handle_parser_error(pANTLR3_BASE_RECOGNIZER recognizer, pANTLR3_EXCEPTION e
   if (eoi)
   {
     // We are at the end of the input. Seek back one token to have a meaningful error indicator.
-    // If we cannot get a previous token then the stream is messed up enough to not
-    // give us a useful error *here*. In that case we very likely have already a lexer error.
-    error_token = parser->tstream->_LT(parser->tstream, -1);
-    if (error_token == NULL)
-      return false;
+    // If we cannot get a previous token then issue a generic eoi error.
+    pANTLR3_COMMON_TOKEN previous_token = parser->tstream->_LT(parser->tstream, -1);
+    if (previous_token != NULL)
+      error_token = previous_token;
   }
   else
     token_name = get_token_name(tokenNames, error_token->type);
@@ -149,7 +287,10 @@ bool handle_parser_error(pANTLR3_BASE_RECOGNIZER recognizer, pANTLR3_EXCEPTION e
   {
   case ANTLR3_RECOGNITION_EXCEPTION:
     // Unpredicted input.
-    error << token_text << " (" << token_name << ") is not valid input at this position";
+    if (error_token->type == INVALID_INPUT) // Our catch all rule for any char not allowed in MySQL.
+      error << token_text << " is not allowed at all";
+    else
+      error << token_text << " (" << token_name << ") is not valid input at this position";
     break;
 
   case ANTLR3_MISMATCHED_TOKEN_EXCEPTION:
@@ -164,7 +305,7 @@ bool handle_parser_error(pANTLR3_BASE_RECOGNIZER recognizer, pANTLR3_EXCEPTION e
   case ANTLR3_NO_VIABLE_ALT_EXCEPTION:
     // No alternative to choose from here.
     if (eoi)
-      error << "unexpected end of statement";
+      error << "unexpected end of input";
     else
       error << "unexpected " << token_text << " (" << token_name << ")";
 
@@ -206,13 +347,14 @@ bool handle_parser_error(pANTLR3_BASE_RECOGNIZER recognizer, pANTLR3_EXCEPTION e
     break;
 
   case ANTLR3_FAILED_PREDICATE_EXCEPTION:
-    // Appears when a gated semantic predicate is used in the grammar, but not for predicting an alternative,
-    // e.g. ... some_rule {condition}? => some_rule.
-    // If the condition does not match we get a failed predicate error. So it's more like a grammar error
-    // unless this is by intention (which is never the case in the MySQL grammar where predicates are only used
-    // to guide the parser).
-    error << "failed predicate";
-    break;
+    {
+      // One of the semantic predicates failed. Since most of those are our version check predicates
+      // we can use that to give the user a hint about this.
+      std::string predicate = (const char*)exception->message;
+      RecognitionContext *context = (RecognitionContext*)recognizer->state->userp;
+      error << token_text << " (" << token_name << ") is not valid input here." << createErrorFromPredicate(predicate, context->version);
+      break;
+    }
 
   case ANTLR3_MISMATCHED_TREE_NODE_EXCEPTION:
     // This is very likely a tree parser error and hence not relevant here (no info in ANTLR docs).
@@ -230,7 +372,7 @@ bool handle_parser_error(pANTLR3_BASE_RECOGNIZER recognizer, pANTLR3_EXCEPTION e
     // this when the token that follows this unwanted token would normally be part of the
     // syntactically correct stream.
     if	(exception->expecting == ANTLR3_TOKEN_EOF)
-      error << "extraneous input found - expected end of statement";
+      error << "extraneous input found - expected end of input";
     else
       error << "extraneous input found - expected '" << get_token_name(tokenNames, exception->expecting) << "'";
     break;
@@ -247,7 +389,7 @@ bool handle_parser_error(pANTLR3_BASE_RECOGNIZER recognizer, pANTLR3_EXCEPTION e
       {
         if (exception->expecting == ANTLR3_TOKEN_EOF)
           // Will probably not occur since ANTLR3_UNWANTED_TOKEN_EXCEPTION will kick in instead.
-          error << "expected end of statement";
+          error << "expected end of input";
         else
           error << "missing '" << get_token_name(tokenNames, exception->expecting) << "'";
       }
@@ -289,7 +431,7 @@ extern "C" {
   /**
    * Error report function which is set in the parser (see MySQL.g where this is done).
    */
-  void on_parse_error(pANTLR3_BASE_RECOGNIZER recognizer, pANTLR3_UINT8 *tokenNames)
+  void onMySQLParseError(pANTLR3_BASE_RECOGNIZER recognizer, pANTLR3_UINT8 *tokenNames)
   {
     pANTLR3_EXCEPTION exception = recognizer->state->exception;
 
@@ -304,26 +446,42 @@ extern "C" {
       switch (recognizer->type)
       {
       case ANTLR3_TYPE_LEXER:
-        if (!handle_lexer_error(recognizer, exception, start, length, message))
+        if (!handleLexerError(recognizer, exception, start, length, message))
           return;
         break;
 
       case ANTLR3_TYPE_PARSER:
-        if (!handle_parser_error(recognizer, exception, tokenNames, start, length, message))
+        if (!handleParserError(recognizer, exception, tokenNames, start, length, message))
           return;
         break;
       }
 
-      pANTLR3_COMMON_TOKEN error_token = (pANTLR3_COMMON_TOKEN)(exception->token);
       MySQLRecognitionBase *our_recognizer = (MySQLRecognitionBase*)((RecognitionContext*)recognizer->state->userp)->payload;
-      our_recognizer->add_error("Syntax error: " + message,
-        (error_token == NULL) ? 0 : error_token->type, start,
-        exception->line, exception->charPositionInLine, length);
+      our_recognizer->add_error("Syntax error: " + message, recognizer->state->type, start, exception->line,
+        exception->charPositionInLine, length);
     }
   }
 
 } // extern "C"
 
+//--------------------------------------------------------------------------------------------------
+
+class MySQLRecognizer::Private
+{
+public:
+  const char *_text;
+  size_t _text_length;
+  int _input_encoding;
+  RecognitionContext _context;
+
+  pANTLR3_INPUT_STREAM _input;
+  pMySQLLexer _lexer;
+  pANTLR3_COMMON_TOKEN_STREAM _tokens;
+  pMySQLParser _parser;
+  pANTLR3_BASE_TREE _ast;
+};
+
+
 //----------------- MySQLTreeWalker ----------------------------------------------------------------
 
 struct compare_token_index
@@ -780,12 +938,41 @@ bool MySQLRecognizerTreeWalker::skip_token_sequence(unsigned int start_token, ..
 //--------------------------------------------------------------------------------------------------
 
 /**
- * Advance to the nth next token if the current one is that given by @token.
+ * Advances to the nth next token if the current one is that given by @token.
+ * Returns true if we skipped actually.
  */
-void MySQLRecognizerTreeWalker::skip_if(unsigned int token, size_t count)
+bool MySQLRecognizerTreeWalker::skip_if(unsigned int token, size_t count)
 {
   if (token_type() == token)
+  {
     next(count);
+    return true;
+  }
+  return false;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+* Advances to the next node after the current subtree. If the current node is not a subtree then
+* we simply skip to the next node after that.
+* Properly handles subtrees at the end of a child list where there is no next sibling.
+*/
+void MySQLRecognizerTreeWalker::skip_subtree()
+{
+  if (is_subtree())
+  {
+    if (next_sibling())
+      return;
+
+    do {
+      up();
+    } while (!next_sibling()); // Terminates reliably on the top level as there is always the EOF node.
+
+    return;
+  }
+
+  next();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -882,6 +1069,16 @@ void MySQLRecognizerTreeWalker::remove_tos()
 
 //--------------------------------------------------------------------------------------------------
 
+/**
+* Returns true if the current token is of the given type.
+*/
+bool MySQLRecognizerTreeWalker::is(unsigned int type)
+{
+  return _tree->getType(_tree) == type;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Returns true if the current token is empty, false otherwise.
  */
@@ -969,9 +1166,9 @@ bool MySQLRecognizerTreeWalker::is_operator()
  * parsed query (if it is a lexer symbol) or the textual expression of the constant name for abstract
  * tokens.
  */
-std::string MySQLRecognizerTreeWalker::token_text()
+std::string MySQLRecognizerTreeWalker::token_text(bool keepQuotes)
 {
-  return _recognizer->token_text(_tree);
+  return _recognizer->token_text(_tree, keepQuotes);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1023,7 +1220,7 @@ ANTLR3_MARKER MySQLRecognizerTreeWalker::token_index()
 size_t MySQLRecognizerTreeWalker::token_offset()
 {
   pANTLR3_COMMON_TOKEN token = _tree->getToken(_tree);
-  return (size_t)(token->start - (ANTLR3_MARKER)_recognizer->text());
+  return (size_t)(token->start - (ANTLR3_MARKER)_recognizer->lineStart());
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1147,21 +1344,6 @@ MySQLQueryType MySQLRecognizerTreeWalker::get_main_query_type()
 
 //----------------- MySQLRecognizer ----------------------------------------------------------------
 
-class MySQLRecognizer::Private
-{
-public:
-  const char *_text;
-  size_t _text_length;
-  int _input_encoding;
-  RecognitionContext _context;
-
-  pANTLR3_INPUT_STREAM _input;
-  pMySQLLexer _lexer;
-  pANTLR3_COMMON_TOKEN_STREAM _tokens;
-  pMySQLParser _parser;
-  pANTLR3_BASE_TREE _ast;
-};
-
 MySQLRecognizer::MySQLRecognizer(long server_version, const std::string &sql_mode, const std::set<std::string> &charsets)
   : MySQLRecognitionBase(charsets)
 {
@@ -1204,7 +1386,7 @@ MySQLRecognizer::~MySQLRecognizer()
  * @param parse_unit used to restrict parsing to a particular query type. 
  *                   Note: only a few types are supported, everything else is just parsed as a query.
  */
-void MySQLRecognizer::parse(const char *text, size_t length, bool is_utf8, MySQLQueryType parse_unit)
+void MySQLRecognizer::parse(const char *text, size_t length, bool is_utf8, MySQLParseUnit parse_unit)
 {
   // If the text is not using utf-8 (which it should) then we interpret as 8bit encoding
   // (everything requiring only one byte per char as Latin1, ASCII and similar).
@@ -1235,6 +1417,8 @@ void MySQLRecognizer::parse(const char *text, size_t length, bool is_utf8, MySQL
   }
   else
   {
+    // TODO: looks like there's still a problem with reusing a parser in certain circumstances.
+    //       mysql_parser_test consumes endless memory with at least one query when reusing the recognizer.
     d->_input->reuse(d->_input, (pANTLR3_UINT8)d->_text, (ANTLR3_UINT32)d->_text_length, (pANTLR3_UINT8)"");
     d->_tokens->reset(d->_tokens);
     d->_lexer->reset(d->_lexer);
@@ -1251,19 +1435,42 @@ void MySQLRecognizer::parse(const char *text, size_t length, bool is_utf8, MySQL
 
   switch (parse_unit)
   {
-  case QtCreateTrigger:
+  case PuCreateTable:
+    d->_ast = d->_parser->create_table(d->_parser).tree;
+    break;
+  case PuCreateTrigger:
     d->_ast = d->_parser->create_trigger(d->_parser).tree;
     break;
-  case QtCreateView:
+  case PuCreateView:
     d->_ast = d->_parser->create_view(d->_parser).tree;
     break;
-  case QtCreateRoutine:
+  case PuCreateRoutine:
     d->_ast = d->_parser->create_routine(d->_parser).tree;
     break;
-  case QtCreateEvent:
-    d->_ast = d->_parser->create_trigger(d->_parser).tree;
+  case PuCreateEvent:
+    d->_ast = d->_parser->create_event(d->_parser).tree;
+    break;
+  case PuCreateIndex:
+    d->_ast = d->_parser->create_index(d->_parser).tree;
+    break;
+  case PuGrant:
+    d->_ast = d->_parser->parse_grant(d->_parser).tree;
+    break;
+  case PuDataType:
+    d->_ast = d->_parser->data_type_definition(d->_parser).tree;
+    break;
+  case PuCreateLogfileGroup:
+    d->_ast = d->_parser->create_logfile_group(d->_parser).tree;
+    break;
+  case PuCreateServer:
+    d->_ast = d->_parser->create_server(d->_parser).tree;
+    break;
+  case PuCreateTablespace:
+    d->_ast = d->_parser->create_tablespace(d->_parser).tree;
+    break;
   default:
     d->_ast = d->_parser->query(d->_parser).tree;
+    break;
   }
 }
 
@@ -1300,9 +1507,9 @@ std::string MySQLRecognizer::dump_tree(pANTLR3_BASE_TREE tree, const std::string
       token_name = state->tokenNames[token_type];
 
 #ifdef  ANTLR3_USE_64BIT
-    result = base::strfmt("%s(line: %i, offset: %i, length: %li, index: %li, %s[%i])    %s\n",
-                          indentation.c_str(), line, char_pos, token->stop - token->start + 1, token->index, token_name,
-                          token_type, utf8);
+    result = base::strfmt("%s(line: %i, offset: %i, length: %" PRId64 ", index: %" PRId64 ", %s[%i])    %s\n",
+                               indentation.c_str(), line, char_pos, token->stop - token->start + 1, token->index, token_name,
+                               token_type, utf8);
 #else
     result = base::strfmt("%s(line: %i, offset: %i, length: %i, index: %i, %s[%i])    %s\n",
                           indentation.c_str(), line, char_pos, token->stop - token->start + 1, token->index, token_name,
@@ -1326,7 +1533,14 @@ std::string MySQLRecognizer::dump_tree(pANTLR3_BASE_TREE tree, const std::string
 
 //--------------------------------------------------------------------------------------------------
 
-const char* MySQLRecognizer::text()
+std::string MySQLRecognizer::text()
+{
+  return std::string(d->_text, d->_text_length);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+const char* MySQLRecognizer::lineStart()
 {
   return d->_text;
 }
@@ -1346,7 +1560,7 @@ MySQLRecognizerTreeWalker MySQLRecognizer::tree_walker()
 void MySQLRecognizer::set_sql_mode(const std::string &new_mode)
 {
   MySQLRecognitionBase::set_sql_mode(new_mode);
-  d->_context.sql_mode = sql_mode(); // Parsed SQL mode.
+  d->_context.sqlMode = sql_mode(); // Parsed SQL mode.
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1365,68 +1579,6 @@ long MySQLRecognizer::server_version()
 
 //--------------------------------------------------------------------------------------------------
 
-/**
- * Returns the text of the given node. The result depends on the type of the node. If it represents
- * a quoted text entity then two consecutive quote chars are replaced by a single one and if
- * escape sequence parsing is not switched off (as per sql mode) then such sequences are handled too.
- */
-std::string MySQLRecognizer::token_text(pANTLR3_BASE_TREE node)
-{
-  pANTLR3_STRING text = node->getText(node);
-  if (text == NULL)
-    return "";
-
-  std::string chars;
-  pANTLR3_COMMON_TOKEN token = node->getToken(node);
-  ANTLR3_UINT32 type = (token != NULL) ? token->type : ANTLR3_TOKEN_INVALID;
-
-  if (type == STRING_TOKEN)
-  {
-    // STRING is the grouping subtree for multiple consecutive string literals, which
-    // must be concatenated.
-    for (ANTLR3_UINT32 index = 0; index < node->getChildCount(node); index++)
-    {
-      pANTLR3_BASE_TREE child = (pANTLR3_BASE_TREE)node->getChild(node, index);
-      chars += token_text(child);
-    }
-
-    return chars;
-  }
-
-  chars = (const char*)text->chars;
-  std::string quote_char;
-  switch (type)
-  {
-    case BACK_TICK_QUOTED_ID:
-      quote_char = "`";
-      break;
-    case SINGLE_QUOTED_TEXT:
-      quote_char = "'";
-      break;
-    case DOUBLE_QUOTED_TEXT:
-      quote_char = "\"";
-      break;
-    default:
-      return chars;
-  }
-
-  std::string double_quotes = quote_char + quote_char;
-
-  if ((d->_context.sql_mode & SQL_MODE_NO_BACKSLASH_ESCAPES) == 0)
-    chars = base::unescape_sql_string(chars, quote_char[0]);
-  else
-    if (token->user1 > 0)
-    {
-      // The field user1 is set by the parser to the number of quote char pairs it found.
-      // So we can use it to shortcut our handling here.
-      base::replace(chars, double_quotes, quote_char);
-    }
-
-  return chars.substr(1, chars.size() - 2);
-}
-
-//--------------------------------------------------------------------------------------------------
-
 /**
  * Determines the type of the query the recognizer is set to (and has parsed).
  * We don't check the validity of a query here, but only scan as much info as needed to
@@ -1435,16 +1587,15 @@ std::string MySQLRecognizer::token_text(pANTLR3_BASE_TREE node)
 
 MySQLQueryType MySQLRecognizer::query_type()
 {
-  return query_type(d->_ast);
+  return query_type(d->_ast); // XXX: obsolete with the new MySQLQueryIdentifier.
 }
 
 //--------------------------------------------------------------------------------------------------
 
+// XXX: obsolete with the new MySQLQueryIdentifier.
 MySQLQueryType MySQLRecognizer::query_type(pANTLR3_BASE_TREE node)
 {
   MySQLRecognizerTreeWalker walker(this, node);
-  if (node->getToken(node) == NULL) // If we are at the root node advance to the first real node.
-    walker.next();
 
   switch (walker.token_type())
   {
@@ -1520,11 +1671,17 @@ MySQLQueryType MySQLRecognizer::query_type(pANTLR3_BASE_TREE node)
     case ONLINE_SYMBOL:
     case OFFLINE_SYMBOL:
     case INDEX_SYMBOL:
+    case UNIQUE_SYMBOL:
+    case FULLTEXT_SYMBOL:
+    case SPATIAL_SYMBOL:
       return QtCreateIndex;
 
     case DATABASE_SYMBOL:
       return QtCreateDatabase;
 
+    case TRIGGER_SYMBOL:
+      return QtCreateTrigger;
+
     case DEFINER_SYMBOL: // Can be event, view, procedure, function, UDF, trigger.
       {
         if (!walker.next_sibling())
@@ -1537,20 +1694,17 @@ MySQLQueryType MySQLRecognizer::query_type(pANTLR3_BASE_TREE node)
 
         case VIEW_SYMBOL:
         case SQL_SYMBOL:
-          return QtAlterView;
+          return QtCreateView;
 
         case PROCEDURE_SYMBOL:
           return QtCreateProcedure;
 
         case FUNCTION_SYMBOL:
           {
-            if (!walker.next() || !walker.is_identifier())
-              return QtAmbiguous;
-
             if (!walker.next())
               return QtAmbiguous;
 
-            if (walker.token_type() == RETURNS_SYMBOL)
+            if (walker.is(UDF_NAME_TOKEN))
               return QtCreateUdf;
             return QtCreateFunction;
           }
@@ -1563,10 +1717,23 @@ MySQLQueryType MySQLRecognizer::query_type(pANTLR3_BASE_TREE node)
         }
       }
 
+    case VIEW_SYMBOL:
     case OR_SYMBOL:        // CREATE OR REPLACE ... VIEW
     case ALGORITHM_SYMBOL: // CREATE ALGORITHM ... VIEW
       return QtCreateView;
 
+    case EVENT_SYMBOL:
+      return QtCreateEvent;
+
+    case FUNCTION_SYMBOL:
+      return QtCreateFunction;
+
+    case AGGREGATE_SYMBOL:
+      return QtCreateUdf;
+
+    case PROCEDURE_SYMBOL:
+      return QtCreateProcedure;
+
     case LOGFILE_SYMBOL:
       return QtCreateLogFileGroup;
 
@@ -2128,6 +2295,22 @@ MySQLQueryType MySQLRecognizer::query_type(pANTLR3_BASE_TREE node)
 
 //--------------------------------------------------------------------------------------------------
 
+/**
+ *	Returns the token start of this token or it's first real child/grandchild etc.
+ */
+static ANTLR3_MARKER getRealTokenStart(pANTLR3_BASE_TREE node)
+{
+  if (node->getChildCount(node) == 0)
+  {
+    pANTLR3_COMMON_TOKEN token = node->getToken(node);
+    return token->start;
+  }
+
+  return getRealTokenStart((pANTLR3_BASE_TREE)node->getChild(node, 0));
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * If the given node is a subtree this function collects the original text for all tokens in this tree,
  * including all tokens on the hidden channel (whitespaces).
@@ -2139,11 +2322,22 @@ std::string MySQLRecognizer::text_for_tree(pANTLR3_BASE_TREE node)
 
   std::string result;
 
-  pANTLR3_BASE_TREE child = (pANTLR3_BASE_TREE)node->getChild(node, 0);
+  // Find the start of the first real node (no virtual nodes).
+  pANTLR3_BASE_TREE child = node;
+  do
+  {
+    child = (pANTLR3_BASE_TREE)child->getChild(child, 0);
+  } while (node->getChildCount(child) > 0);
   pANTLR3_COMMON_TOKEN token = child->getToken(child);
   ANTLR3_MARKER start = token->start;
 
-  child = (pANTLR3_BASE_TREE)node->getChild(node, node->getChildCount(node) - 1);
+  // Similar for the last node.
+  child = node;
+  do
+  {
+    child = (pANTLR3_BASE_TREE)child->getChild(child, child->getChildCount(child) - 1);
+  } while (node->getChildCount(child) > 0);
+
   token = child->getToken(child);
   ANTLR3_MARKER stop = token->stop;
   return std::string((char*)start, stop - start + 1);
