@@ -18,6 +18,30 @@
 import re
 from grt import log_error
 
+def get_exe_path(cmd):
+    import os
+    import mforms
+
+    filepath = mforms.App.get().get_executable_path(cmd).encode("utf8")
+    if len(filepath) != 0:
+        return filepath;
+    
+    filepath, filename = os.path.split(cmd)
+    
+    def is_executable(filepath):
+        return os.path.isfile(filepath) and os.access(filepath, os.X_OK)
+    
+    if filepath:
+        if is_executable(filepath):
+            return cmd
+    else:
+        for path in os.environ["PATH"].split(os.pathsep):
+            path = path.strip('"')
+            exe = os.path.join(path, cmd)
+            if is_executable(exe):
+                return exe
+    return None
+
 def human_size(num):
     for x in ['bytes','KiB','MiB','GiB']:
         if num < 1024.0:
@@ -120,14 +144,12 @@ class Version:
         self.minorNumber = minor
         self.releaseNumber = release
 
-
     def __str__(self):
         if self.releaseNumber >= 0:
             return "%i.%i.%i" % (self.majorNumber, self.minorNumber, self.releaseNumber)
         else:
             return "%i.%i" % (self.majorNumber, self.minorNumber)
 
-
     @classmethod
     def fromgrt(cls, v):
         return Version(v.majorNumber, v.minorNumber, v.releaseNumber)
@@ -148,22 +170,48 @@ class Version:
             return Version(v[0], v[1], v[2])
         else:
             raise ValueError("Invalid version string %s" % s)
-
-
-    def __cmp__(self, v):
-        if not isinstance(v, Version):
+    
+    def compare(self, other):
+        other_version = None
+        if isinstance(other, Version):
+            other_version = other
+        elif isinstance(other, basestring):
+            other_version = Version.fromstr(other)
+        else:
             raise TypeError("Unexpected type")
-        
-        return cmp(self.majorNumber * 10000 + self.minorNumber * 100 + max(0, self.releaseNumber),
-                   v.majorNumber * 10000 + v.minorNumber * 100 + max(0, v.releaseNumber))
 
+        this_version_number = self.majorNumber * 10000 + self.minorNumber * 100 + max(0, self.releaseNumber)
+        other_version_number = other_version.majorNumber * 10000 + other_version.minorNumber * 100 + max(0, other_version.releaseNumber)
+        
+        if this_version_number < other_version_number:
+            return -1
+        elif this_version_number > other_version_number:
+            return 1
+        return 0
+    
+    def __lt__(self, other):
+        return self.compare(other) < 0
+
+    def __eq__(self, other):
+        return self.compare(other) == 0
+
+    def __ne__(self, other):
+        return self.compare(other) != 0
+
+    def __gt__(self, other):
+        return self.compare(other) > 0
+
+    def __ge__(self, other):
+        return self.compare(other) >= 0
+
+    def __le__(self, other):
+        return self.compare(other) <= 0
 
     def is_supported_mysql_version(self):
         if self.majorNumber == 5 and self.minorNumber in (1, 5, 6, 7):
             return True
         return False
 
-
     def is_supported_mysql_version_at_least(self, major, minor = None, release=-1):
         assert type(major) == int or isinstance(major, Version)
         if isinstance(major, Version):
