@@ -1,4 +1,4 @@
-# Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2012, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -30,8 +30,8 @@ import os
 import mforms
 
 import paramiko
-from workbench.log import log_warning, log_error, log_debug, log_debug2
-from wb_common import SSHFingerprintNewError
+from workbench.log import log_warning, log_error, log_debug, log_debug2, log_debug3, log_info
+from wb_common import SSHFingerprintNewError, format_bad_host_exception
 
 SSH_PORT = 22
 REMOTE_PORT = 3306
@@ -236,6 +236,23 @@ class Tunnel(threading.Thread):
         with self.lock:
             return self._server == server and self._username == username and self._target == target
 
+
+    def _get_ssh_config_path(self):
+        paths = []
+        if platform.system().lower() == "windows":
+            paths.append("%s\ssh\config" % mforms.App.get().get_user_data_folder())
+            paths.append("%s\ssh\ssh_config" % mforms.App.get().get_user_data_folder())
+        else:
+            paths.append("~/.ssh/config")
+            paths.append("~/.ssh/ssh_config")
+            
+        for path in paths:
+            if os.path.isfile(os.path.expanduser(path)):
+                return os.path.expanduser(path)
+        else:
+            log_debug3("ssh config file not found")
+            return None
+
     def _connect_ssh(self):
         """Create the SSH client and set up the connection.
         
@@ -247,27 +264,40 @@ class Tunnel(threading.Thread):
                                                this is a subclass of paramiko.AuthenticationException
         """
         try:
-            self._client.get_host_keys().clear()
-            ssh_known_hosts_file = '~/.ssh/known_hosts'
             
-            if platform.system().lower() == "windows":
-                ssh_known_hosts_file = '%s\ssh\known_hosts' % mforms.App.get().get_user_data_folder()
+            config = paramiko.config.SSHConfig()
+            config_file_path = self._get_ssh_config_path()
+            if config_file_path:
+                with open(config_file_path) as f:
+                    config.parse(f)
+                
+            opts = config.lookup(self._server[0])
+            ssh_known_hosts_file = None
+            if "userknownhostsfile" in opts:
+                ssh_known_hosts_file = opts["userknownhostsfile"]
+            else:
+                self._client.get_host_keys().clear()
+                ssh_known_hosts_file = '~/.ssh/known_hosts'
+                
+                if platform.system().lower() == "windows":
+                    ssh_known_hosts_file = '%s\ssh\known_hosts' % mforms.App.get().get_user_data_folder()
 
             try:
                 self._client.load_host_keys(os.path.expanduser(ssh_known_hosts_file))
             except IOError, e:
                 log_warning("IOError, probably caused by file %s not found, the message was: %s\n" % (ssh_known_hosts_file, e))
 
-            self._client.set_missing_host_key_policy(StoreIfConfirmedPolicy())
+            if "stricthostkeychecking" in opts and opts["stricthostkeychecking"].lower() == "no":
+                self._client.set_missing_host_key_policy(WarningPolicy())
+            else:
+                self._client.set_missing_host_key_policy(StoreIfConfirmedPolicy())
+                
             has_key = bool(self._keyfile)
             self._client.connect(self._server[0], self._server[1], username=self._username,
                                  key_filename=self._keyfile, password=self._password,
                                  look_for_keys=has_key, allow_agent=has_key)
         except paramiko.BadHostKeyException, exc:
-            if platform.system().lower() == "windows":
-                self.notify_exception_error('ERROR', "%s\nDelete entries for the host from the %s file" % (str(exc), '%s\ssh\known_hosts' % mforms.App.get().get_user_data_folder()))
-            else:
-                self.notify_exception_error('ERROR', "%s\nDelete entries for the host from the ~/.ssh/known_hosts file" % str(exc))
+            self.notify_exception_error('ERROR',format_bad_host_exception(exc, '%s\ssh\known_hosts' % mforms.App.get().get_user_data_folder() if platform.system().lower() == "windows" else "~/.ssh/known_hosts file"))
             return False
         except paramiko.BadAuthenticationType, exc:
             self.notify_exception_error('ERROR', "Bad authentication type, the server is not accepting this type of authentication.\nAllowed ones are:\n %s" % exc.allowed_types, sys.exc_info());
@@ -282,7 +312,7 @@ class Tunnel(threading.Thread):
             self.notify_exception_error('ERROR', "Error connecting SSH channel.\nPlease refer to logs for details: %s" % str(exc), sys.exc_info())
             return False
         except SSHFingerprintNewError, exc:
-            self.notify_exception_error('KEY_ERROR', { 'msg': "The authenticity of host '%(0)s (%(0)s)' can't be established.\nECDSA key fingerprint is %(1)s\nAre you sure you want to continue connecting?"  % {'0': "%s:%s" % (self._server[0], self._server[1]), '1': exc.fingerprint}, 'obj': exc})
+            self.notify_exception_error('KEY_ERROR', { 'msg': "The authenticity of host '%(0)s (%(0)s)' can't be established.\n%(1)s key fingerprint is %(2)s\nAre you sure you want to continue connecting?"  % {'0': "%s:%s" % (self._server[0], self._server[1]), '1': exc.key.get_name(), '2': exc.fingerprint}, 'obj': exc})
             return False
         except IOError, exc:
             #Io should be report to the user, so maybe he will be able to fix this issue
@@ -388,7 +418,7 @@ class TunnelManager:
 
         if found:
             with tunnel.lock:
-                print 'Reusing tunnel at port %d' % tunnel.local_port
+                log_debug('Reusing tunnel at port %d' % tunnel.local_port)
                 return tunnel.local_port
         else:
             tunnel = Tunnel(Queue.Queue(), server, username, target, password, keyfile)
@@ -438,7 +468,8 @@ class TunnelManager:
                     elif msg_type == 'IO_ERROR':
                         error = msg
                         break # Exit returning the error message
-
+                    else:
+                        time.sleep(0.3)
                     _msg = msg
                     if type(msg) is tuple:
                         msg = '\n' + ''.join(traceback.format_exception(*msg))
@@ -447,8 +478,8 @@ class TunnelManager:
                     if msg_type == 'ERROR':
                         error = _msg
                         break  # Exit returning the error message
-
-                if not tunnel.is_connecting() or not tunnel.isAlive():
+                
+                if (not tunnel.is_connecting() or not tunnel.isAlive()) and tunnel.q.empty():  
                     break
                 time.sleep(0.3)
         log_debug("returning from wait_connection(%s): %s\n" % (port, error))
@@ -466,6 +497,21 @@ class TunnelManager:
             return None
 
 
+    def set_keepalive(self, port, keepalive):
+        if keepalive == 0:
+            log_info("SSH KeepAlive setting skipped.\n")
+            return
+        tunnel = self.tunnel_by_port.get(port)
+        if not tunnel:
+            log_error("Looking up invalid port %s\n" % port)
+            return
+        transport = tunnel._client.get_transport()
+        if transport is None:
+            log_error("SSHTransport not ready yet %d\n" % port)
+            return
+
+        transport.set_keepalive(keepalive)
+
     def close(self, port):
         pass
         # tunnels auto-close when inactive
