@@ -1,4 +1,4 @@
-# Copyright (c) 2012, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -16,7 +16,7 @@
 # 02110-1301  USA
 
 from wb import DefineModule
-from workbench.utils import find_object_with_name, find_object_with_old_name
+from workbench.utils import find_object_with_old_name
 
 import grt
 
@@ -27,13 +27,17 @@ MYSQL_MAX_INDEX_KEY_LENGTH_NDBCLUSTER = 0
 ModuleInfo = DefineModule(name= "DbGenericMigration", author= "Oracle Corp.", version="1.0")
 
 truncated_identifier_serial = 0
+key_names = {}
 class GenericMigration(object):
     ## Note: do not add member variables in this class or subclasses
 
     def findMatchingTargetObject(self, state, sourceObject):
         """Finds the matching target object for a given source object, by searching in the migrationLog"""
         for log in state.migrationLog:
-            if log.logObject == sourceObject:
+            if type(log.logObject) == type(sourceObject)\
+               and type(log.logObject.owner) == type(sourceObject.owner)\
+               and log.logObject.owner.name == sourceObject.owner.name\
+               and log.logObject.name == sourceObject.name:
                 return log.refObject
         return None
         
@@ -61,7 +65,7 @@ class GenericMigration(object):
             return False
         return True
 
-    def migrateIdentifier(self, name, log):
+    def migrateIdentifier(self, name, log, dots_allowed=False):
         mysql_name = name
         # truncate too long identifiers
         if len(mysql_name) > 64:
@@ -128,6 +132,18 @@ class GenericMigration(object):
 
         return True
 
+    def getColumnDataType(self, column):
+        source_type = column.simpleType
+        if not source_type and column.userType:
+            source_type = column.userType.actualType
+        return source_type.name.upper() if source_type else None 
+
+    def migrateUpdateForChanges(self, state, target_catalog):
+        """
+        Create datatype cast expression for target column based on source datatype.
+        """
+        return target_catalog
+
     def migrateColumnDefaultValue(self, state, default_value, source_column, target_column):
         return default_value
         
@@ -178,7 +194,8 @@ class GenericMigration(object):
 
             # migrate foreign keys last, as they need the referenced objects which can be from different schemas to be ready
             for sourceSchema in source_catalog.schemata:
-                targetSchema = self.findMatchingTargetObject(state, sourceSchema)
+                global key_names
+                key_names[sourceSchema.name] = set()
                 for sourceTable in sourceSchema.tables:
                     if not self.shouldMigrate(state, 'tables', sourceTable):
                         continue
@@ -340,7 +357,7 @@ class GenericMigration(object):
         target_column = grt.classes.db_mysql_Column()
         target_column.owner = targetTable
         log = state.addMigrationLogEntry(0, source_column, target_column, "")
-        target_column.name = self.migrateIdentifier(source_column.name, log)
+        target_column.name = self.migrateIdentifier(source_column.name, log, dots_allowed=True)
         target_column.oldName = source_column.name
 
         for flag in source_column.flags:
@@ -386,7 +403,7 @@ class GenericMigration(object):
         target_index = grt.classes.db_mysql_Index()
         log = state.addMigrationLogEntry(0, source_index, target_index, "")
         target_index.owner = targetTable
-        target_index.name = self.migrateIdentifier(source_index.name, log)
+        target_index.name = self.migrateIdentifier(source_index.name, log, dots_allowed=True)
         target_index.oldName = source_index.name
             
         target_index.isPrimary = source_index.isPrimary
@@ -437,6 +454,7 @@ class GenericMigration(object):
 
     def migrateTableForeignKeyToMySQL(self, state, source_fk, targetTable):
         sourceTable = source_fk.owner
+        sourceSchema = sourceTable.owner
         if len(sourceTable.columns) == 0 or len(targetTable.columns) == 0:
             state.addMigrationLogEntry(2, source_fk, None,
                     'The migration of table %s foreign keys was attempted but either the source or the target table has no columns attribute' % sourceTable.name)
@@ -447,7 +465,21 @@ class GenericMigration(object):
         target_fk = grt.classes.db_mysql_ForeignKey()
         target_fk.owner = targetTable
         log = state.addMigrationLogEntry(0, source_fk, target_fk, "")
-        target_fk.name = self.migrateIdentifier(source_fk.name, log)
+
+        global key_names
+        fk_name = source_fk.name
+        if fk_name in key_names[sourceSchema.name]:  # Found duplicated fk name in this schema
+            idx = 1
+            while True:
+                fk_name = source_fk.name + '_%d' % idx
+                if not fk_name in key_names[sourceSchema.name]:
+                    break
+                idx += 1
+            state.addMigrationLogEntry(1, source_fk, target_fk,
+                'The foreign key constraint name "%s" is duplicated. Changed to "%s"' % (source_fk.name, fk_name))
+
+        target_fk.name = self.migrateIdentifier(fk_name, log, dots_allowed=True)
+        key_names[sourceSchema.name].add(fk_name)
         target_fk.oldName = source_fk.name
 
         target_fk.deleteRule = source_fk.deleteRule.replace('_', ' ')
@@ -485,7 +517,7 @@ class GenericMigration(object):
             try:
                 target_fk_col = self.findMatchingTargetObject(state, source_fk_column)
                 if not target_fk_col:
-                    raise ValueError, 'The column "%s" was not found in table "%s"' % (self.migrateIdentifier(source_fk_column.name, None), targetTable.name)
+                    raise ValueError, 'The column "%s" was not found in table "%s"' % (self.migrateIdentifier(source_fk_column.name, None, dots_allowed=True), targetTable.name)
                 target_fk.columns.append(target_fk_col)
             except Exception, err:
                 state.addMigrationLogEntry(2, source_fk, target_fk,
@@ -501,7 +533,7 @@ class GenericMigration(object):
             try:
                 target_fk_col = self.findMatchingTargetObject(state, source_fk_column)
                 if not target_fk_col:
-                    raise ValueError, 'The column "%s" was not found in table "%s"' % (self.migrateIdentifier(source_fk_column.name, None), target_fk.referencedTable.name)
+                    raise ValueError, 'The column "%s" was not found in table "%s"' % (self.migrateIdentifier(source_fk_column.name, None, dots_allowed=True), target_fk.referencedTable.name)
                 target_fk.referencedColumns.append(target_fk_col)
             except Exception, err:
                 state.addMigrationLogEntry(2, source_fk, target_fk, 
@@ -518,9 +550,7 @@ class GenericMigration(object):
 
 
     def migrateTriggerToMySQL(self, state, source_trigger, target_table):
-        copy_members = ["name", "oldName", "definer", "event", "condition", "order", "orientation", "timing", 
-             "referenceNewTable", "referenceOldTable", "referenceNewRow", "referenceOldRow", "sequenceNumber", "enabled",
-             "sqlDefinition"]
+        copy_members = ["name", "oldName", "definer", "event", "timing", "enabled", "sqlDefinition"]
         target_trigger = grt.classes.db_mysql_Trigger()
         state.addMigrationLogEntry(0, source_trigger, target_trigger, "")
         target_trigger.owner = target_table
@@ -546,8 +576,7 @@ class GenericMigration(object):
 
 
     def migrateRoutineToMySQL(self, state, source_routine, target_schema):
-        copy_members = ["name", "oldName", "routineType", "sequenceNumber",
-             "sqlDefinition"]
+        copy_members = ["name", "oldName", "routineType", "sqlDefinition"]
         target_routine = grt.classes.db_mysql_Routine()
         target_routine.owner = target_schema
         state.addMigrationLogEntry(0, source_routine, target_routine, "")
@@ -610,6 +639,11 @@ def migrateRoutineToMySQL(state, source_routine, target_schema):
     return instance.migrateRoutineToMySQL(state, source_routine, target_schema)
 
 
+@ModuleInfo.export(grt.classes.db_Catalog, grt.classes.db_migration_Migration, grt.classes.db_Catalog)
+def migrateUpdateForChanges(state, target_catalog):
+    return instance.migrateUpdateForChanges(state, target_catalog)
+
+
 @ModuleInfo.export((grt.LIST, grt.classes.db_migration_MigrationParameter), grt.classes.db_migration_Migration)
 def getMigrationOptions(state):
     list = grt.List(grt.OBJECT, grt.classes.db_migration_MigrationParameter.__grtclassname__)
