@@ -1067,253 +1067,6 @@ bool MysqlSqlFacadeImpl::parseDropStatement(const std::string &sql, std::string
   return ret_val;
 }
 
-grt::DictRef MysqlSqlFacadeImpl::parseStatement(const std::string &sql_statement, int server_version, int utf, const std::string &sql_mode)
-{
-  grt::DictRef ret_val = grt::DictRef();
-  std::set<std::string> charsets(_charsets.begin(), _charsets.end());
-
-  // 3 character sets were added in version 5.5.3. Remove them from the list if the current version
-  // is lower than that.
-  if (server_version < 50503)
-  {
-    charsets.erase("utf8mb4");
-    charsets.erase("utf16");
-    charsets.erase("utf32");
-  }
-  
-  MySQLRecognizer recognizer(server_version, sql_mode, charsets);
-  recognizer.parse(sql_statement.c_str(), sql_statement.length(), true, PuGrant);
-
-  if (!recognizer.has_errors())
-    ret_val = parseGrantStatement(recognizer);
-
-  return ret_val;
-}
-
-
-grt::StringRef MysqlSqlFacadeImpl::concatenateTokens(MySQLRecognizerTreeWalker &walker, std::unordered_set<int> &stop_symbols, const std::string &separator)
-{
-  std::string data;
-  bool is_id = walker.is_identifier() && !walker.is_keyword();
-
-  // Identifiers are back quoted
-  if (is_id)
-    data += "`";
-
-  // Starts adding the current token text
-  data += walker.token_text();
-  // Identifiers are back quoted
-  if (is_id)
-    data += "`";
-
-  walker.next();
-
-
-  // Now adds additional token text until the stopper is found
-  while (stop_symbols.find(walker.token_type()) == stop_symbols.end())
-  {
-    data += separator;
-
-	is_id = walker.is_identifier() && !walker.is_keyword();
-
-    // Identifiers are back quoted
-    if (is_id)
-      data += "`";
-
-    data += walker.token_text();
-
-    // Identifiers are back quoted
-    if (is_id)
-      data += "`";
-
-    walker.next();
-  }
-
-  return grt::StringRef(data);
-}
-
-
-grt::StringListRef MysqlSqlFacadeImpl::createList(MySQLRecognizerTreeWalker &walker, int separator_symbol, std::unordered_set<int> &stop_symbols)
-{
-  grt::StringListRef list = grt::StringListRef(get_grt());
-
-  std::unordered_set<int> item_separators(stop_symbols.begin(), stop_symbols.end());
-  item_separators.insert(separator_symbol);
-
-  // Dummy move to simplify logic inside the iteration below
-  walker.previous();
-
-  do 
-  {
-    walker.next();
-    list.insert(concatenateTokens(walker, item_separators));
-  }while (stop_symbols.find(walker.token_type()) == stop_symbols.end());
-
-  return list;
-}
-
-/*
- * parseUserDefinition: Provides a dctionary with the definition of a user
- * assumption : the walker is at the starting point of a user definition
- * returned dictionary:
- *                     user : the username or CURRENT_USER
- *           (optional)host : the host name if available
- *           (optional)id_method : the authentication method if available
- *           (optional)id_password: the authentication string if available
- */
-grt::DictRef MysqlSqlFacadeImpl::parseUserDefinition(MySQLRecognizerTreeWalker &walker)
-{
-  // Gets the user
-  std::string user_str = walker.token_text();
-  grt::DictRef user_dict = grt::DictRef(get_grt());
-
-  user_dict.set("user", grt::StringRef(user_str));
-
-  if (walker.token_type() == CURRENT_USER_SYMBOL)
-  {
-    // Skips the current user and parenthesis
-    walker.next();
-    walker.next();
-    walker.next();
-  }
-  else
-  {
-    // The user token(s) has been read, skips to the next token
-    if (!walker.next_sibling())
-    {
-      walker.up();
-      walker.next_sibling();
-    }
-
-    if (walker.token_type() == AT_SIGN_SYMBOL)
-    {
-      walker.next();
-      user_dict.set("host", grt::StringRef(walker.token_text()));
-
-      // The host token(s) have been read, sekips to the next token
-      if (!walker.next_sibling())
-      {
-        walker.up();
-        walker.next_sibling();
-      }
-    }
-  }
-
-  if (walker.token_type() == IDENTIFIED_SYMBOL)
-  {
-    walker.next();
-    if (walker.token_type() == BY_SYMBOL)
-    {
-      walker.next();
-
-      if (walker.token_type() == PASSWORD_SYMBOL)
-        walker.next();
-
-      user_dict.set("id_method", grt::StringRef("PASSWORD"));
-      user_dict.set("id_string", grt::StringRef(walker.token_text()));
-
-      if (!walker.next_sibling())
-      {
-        walker.up();
-        walker.next_sibling();
-      }
-    }
-    else if (walker.token_type() == WITH_SYMBOL)
-    {
-      walker.next();
-      user_dict.set("id_method", grt::StringRef(walker.token_text()));
-      walker.next();
-
-      if (walker.token_type() == AS_SYMBOL)
-      {
-        walker.next();
-        user_dict.set("id_string", grt::StringRef(walker.token_text()));
-        walker.next();
-      }
-    }
-  }
-
-  return user_dict;
-}
-
-grt::DictRef MysqlSqlFacadeImpl::parseGrantStatement(MySQLRecognizer &recognizer)
-{
-  grt::DictRef data = grt::DictRef(get_grt());
-  std::unordered_set<int>terminators;
-  terminators.insert(ANTLR3_TOKEN_EOF);
-  terminators.insert(PRIVILEGE_TARGET_TOKEN);
-
-  MySQLRecognizerTreeWalker walker = recognizer.tree_walker();
-
-  // Skips the GRANT token
-  walker.next();
-
-  // Gets the privilege list
-  data.set("privileges", createList(walker, COMMA_SYMBOL, terminators));
-
-  // Skips the PRIVILEGE_TARGET token
-  walker.next();
-
-  // Skips the ON token
-  walker.next();
-
-  // Doesn't hurt to let the others so we are ok to go
-  terminators.insert(TO_SYMBOL);
-
-  // Sets the privilege target as a single text string
-  data.set("target", concatenateTokens(walker, terminators, ""));
-
-  //Skips the TO symbol
-  walker.next();
-
-  // Now Parses the user definitions
-  terminators.clear();
-  terminators.insert(ANTLR3_TOKEN_EOF);
-  terminators.insert(WITH_SYMBOL);
-  terminators.insert(REQUIRE_SYMBOL);
-
-
-  grt::DictRef users = grt::DictRef(get_grt());
-  data.set("users", users);
-
-  while (terminators.find(walker.token_type()) == terminators.end())
-  {
-    grt::DictRef user_definition = parseUserDefinition(walker);
-    
-    std::string user_str = grt::StringRef::cast_from(user_definition.get("user"));
-
-    users.set(user_str, user_definition);
-
-    if (walker.token_type() == COMMA_SYMBOL)
-      walker.next();
-  }
-
-  data.set("users", users);
-
-  // Skips the REQUIRE info from the GRANT statement
-  bool options = walker.token_type() == WITH_SYMBOL;
-  if (!options)
-      options = walker.advance_to_type(WITH_SYMBOL, false);
-
-  if (options)
-  {
-    walker.next();
-    if (walker.token_type() == GRANT_SYMBOL)
-    {
-      data.set("option", grt::StringRef("GRANT"));
-    }
-    else
-    {
-      data.set("option", grt::StringRef(walker.token_text()));
-      walker.next();
-      data.set("option_value", grt::StringRef(walker.token_text()));
-    }
-  }
-
-	return data;
-}
-//--------------------------------------------------------------------------------------------------
-
 /**
  * Signals any ongoing process to stop. This must be called from a different thread than from where
  * the processing was started to make it work.
