@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,10 +17,9 @@
  * 02110-1301  USA
  */
 
-
-#include "stdafx.h"
-
 #include <glib.h>
+#include <cctype>
+
 #include "grtdb/charset_utils.h"
 #include "base/string_utilities.h"
 #include "base/threading.h"
@@ -28,6 +27,7 @@
 #include "mysql_sql_parser_fe.h"
 #include <sstream>
 #include <boost/scoped_array.hpp>
+#include <boost/shared_ptr.hpp>
 #include "myx_statement_parser.h"
 
 #define MYSQL_LEX 1
@@ -200,9 +200,9 @@ void remove_versioning_comments(const std::string &sql, std::string &effective_s
       *ignore_statement= (0 == strncmp(ptr, " CREATE TABLE", 13));
 
     {
-      register bool quoted= false;
-      register bool escaped= false;
-      register bool commented= false;
+      bool quoted= false;
+      bool escaped= false;
+      bool commented= false;
       int nested_comments_count= 1;
       char quot_sym= 0;
 
@@ -264,7 +264,7 @@ end: ;
 
     if (effective_sql.empty())
     {
-      *first_versioning_comment_pos= start_start - begin;
+      *first_versioning_comment_pos= (int)(start_start - begin);
       effective_sql= sql;
     }
 
@@ -334,7 +334,7 @@ max_err_count(-1)
 }
 
 
-GStaticMutex _parser_fe_critical_section= G_STATIC_MUTEX_INIT;
+boost::shared_ptr<base::Mutex> _parser_fe_critical_section(new base::Mutex);
 
 
 void Mysql_sql_parser_fe::reset()
@@ -365,7 +365,7 @@ void Mysql_sql_parser_fe::parse_sql_mode(const std::string &sql_mode_string)
 
 int Mysql_sql_parser_fe::parse_sql_script(const char* sql, fe_process_sql_statement_callback cb, void *user_data)
 {
-  base::GStaticMutexLock parser_fe_critical_section(_parser_fe_critical_section);
+  base::MutexLock parser_fe_critical_section(*_parser_fe_critical_section);
   reset();
   Context context= {this, cb, user_data, 0, ignore_dml, is_ast_generation_enabled, max_insert_statement_size, processing_create_statements, processing_alter_statements, processing_drop_statements, sql_mode};
   myx_process_sql_statements(sql, Lex_helper::charset(), &process_sql_statement_cb, &context, MYX_SPM_NORMAL_MODE);
@@ -375,7 +375,7 @@ int Mysql_sql_parser_fe::parse_sql_script(const char* sql, fe_process_sql_statem
 
 int Mysql_sql_parser_fe::parse_sql_script_file(const std::string &filename, fe_process_sql_statement_callback cb, void *user_data)
 {
-  base::GStaticMutexLock parser_fe_critical_section(_parser_fe_critical_section);
+  base::MutexLock parser_fe_critical_section(*_parser_fe_critical_section);
   reset();
   Context context= {this, cb, user_data, 0, ignore_dml, is_ast_generation_enabled, max_insert_statement_size, processing_create_statements, processing_alter_statements, processing_drop_statements, sql_mode};
   myx_process_sql_statements_from_file(filename.c_str(), Lex_helper::charset(), &process_sql_statement_cb, &context, MYX_SPM_NORMAL_MODE/*MYX_SPM_DELIMS_REQUIRED*/);
@@ -386,7 +386,7 @@ int Mysql_sql_parser_fe::parse_sql_script_file(const std::string &filename, fe_p
 int Mysql_sql_parser_fe::escape_string(const std::string &in_text, std::string &out_text)
 {
   boost::scoped_array<char> out(new char[in_text.size()*2+1]);
-  int res= escape_string(out.get(), 0, in_text.c_str(), in_text.size());
+  int res= escape_string(out.get(), 0, in_text.c_str(), (unsigned long)in_text.size());
   out_text= out.get();
   return res;
 }
@@ -632,7 +632,7 @@ void Mysql_sql_parser_fe::determine_token_position(const SqlAstNode *item, const
 
 std::string Mysql_sql_parser_fe::get_first_sql_token(const std::string &sql, const std::string &versioning_comment_subst_token)
 {
-  base::GStaticMutexLock parser_fe_critical_section(_parser_fe_critical_section);
+  base::MutexLock parser_fe_critical_section(*_parser_fe_critical_section);
   reset();
   static Mysql_sql_parser_fe::SqlMode sql_mode;
 
