@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,8 @@
  * 02110-1301  USA
  */
 
-
-#include "stdafx.h"
+#include <glib.h>
+#include <boost/signals2.hpp>
 
 #include "mysql_sql_schema_rename.h"
 #include "grtsqlparser/module_utils.h"
