@@ -1,4 +1,4 @@
-# Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2012, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -57,10 +57,15 @@ def connect(connection, password):
             grt.send_info("Connection restablished")
     except NotConnectedError:
         con = MySQLConnection(connection, password = password)
-        grt.send_info("Connecting to %s..." % connection.hostIdentifier)
-        con.connect()
-        grt.send_info("Connected")
+        host_identifier = connection.hostIdentifier
+        grt.send_info("Connecting to %s..." % host_identifier)
+        con.connect()        
         _connections[connection.__id__] = con
+        version = "Unknown version"
+        result = execute_query(connection, "SHOW VARIABLES LIKE 'version'")
+        if result and result.nextRow():
+            version = result.stringByIndex(2)
+        grt.send_info("Connected to %s, %s" % (host_identifier, version))
     return 1
 
 
@@ -176,6 +181,31 @@ def getFunctionNames(connection, catalog_name, schema_name):
     return names
 
 
+@ModuleInfo.export(grt.STRING, grt.classes.db_mgmt_Connection)
+def getOS(connection):
+    conn = get_connection(connection)
+    if conn:
+        result = conn.executeQuery("SELECT @@version_compile_os")
+        if result and result.nextRow():
+            compile_os = result.stringByIndex(1).lower()
+            if 'linux' in compile_os:
+                return 'linux'
+            elif 'win' in compile_os:
+                return 'windows'
+            elif 'osx' in compile_os:
+                return 'darwin'
+
+    return None
+
+
+@ModuleInfo.export(grt.STRING, grt.classes.db_mgmt_Connection)
+def getServerMode(connection):
+    result = execute_query(connection, "SELECT @@SESSION.sql_mode")
+    if result and result.nextRow():
+        return result.stringByIndex(1)
+    return None
+
+
 #########  Reverse Engineering functions #########
 
 
@@ -257,6 +287,8 @@ def reverseEngineer(connection, catalog_name, schemata_list, context):
         schema.owner = catalog
         schema.name = schema_name
         catalog.schemata.append(schema)
+        context = grt.modules.MySQLParserServices.createParserContext(catalog.characterSets, getServerVersion(connection), getServerMode(connection), 1)
+        options = {}
 
         if get_tables or get_views:
             grt.send_info("Reverse engineering tables from %s" % schema_name)
@@ -270,7 +302,7 @@ def reverseEngineer(connection, catalog_name, schemata_list, context):
                     sql = result.stringByIndex(2)
                     grt.push_message_handler(filter_warnings)
                     grt.begin_progress_step(0.1 + 0.9 * (i / total), 0.1 + 0.9 * ((i+0.5) / total))
-                    grt.modules.MysqlSqlFacade.parseSqlScriptString(catalog, wrap_sql(sql, schema_name))
+                    grt.modules.MySQLParserServices.parseSQLIntoCatalogSql(context, catalog, wrap_sql(sql, schema_name), options)
                     grt.end_progress_step()
                     grt.pop_message_handler()
                     i += 0.5
@@ -288,7 +320,7 @@ def reverseEngineer(connection, catalog_name, schemata_list, context):
                 if result and result.nextRow():
                     sql = result.stringByName("SQL Original Statement")
                     grt.begin_progress_step(0.1 + 0.9 * (i / total), 0.1 + 0.9 * ((i+0.5) / total))
-                    grt.modules.MysqlSqlFacade.parseSqlScriptString(catalog, wrap_sql(wrap_routine_sql(sql), schema_name))
+                    grt.modules.MySQLParserServices.parseSQLIntoCatalogSql(context, catalog, wrap_sql(wrap_routine_sql(sql), schema_name), options)
                     grt.end_progress_step()
                     i += 0.5
                 else:
@@ -306,7 +338,7 @@ def reverseEngineer(connection, catalog_name, schemata_list, context):
                 if result and result.nextRow():
                     sql = result.stringByName("Create Procedure")
                     grt.begin_progress_step(0.1 + 0.9 * (i / total), 0.1 + 0.9 * ((i+0.5) / total))
-                    grt.modules.MysqlSqlFacade.parseSqlScriptString(catalog, wrap_sql(wrap_routine_sql(sql), schema_name))
+                    grt.modules.MySQLParserServices.parseSQLIntoCatalogSql(context, catalog, wrap_sql(wrap_routine_sql(sql), schema_name), options)
                     grt.end_progress_step()
                     i += 0.5
                 else:
@@ -322,7 +354,7 @@ def reverseEngineer(connection, catalog_name, schemata_list, context):
                 if result and result.nextRow():
                     sql = result.stringByName("Create Function")
                     grt.begin_progress_step(0.1 + 0.9 * (i / total), 0.1 + 0.9 * ((i+0.5) / total))
-                    grt.modules.MysqlSqlFacade.parseSqlScriptString(catalog, wrap_sql(wrap_routine_sql(sql), schema_name))
+                    grt.modules.MySQLParserServices.parseSQLIntoCatalogSql(context, catalog, wrap_sql(wrap_routine_sql(sql), schema_name), options)
                     grt.end_progress_step()
                     i += 0.5
                 else:
@@ -358,7 +390,9 @@ def reverseEngineerTable(connection, schema_name, table_name):
     result = execute_query(connection, "SHOW CREATE TABLE `%s`.`%s`" % (escape_sql_identifier(schema_name), escape_sql_identifier(table_name)))
     if result and result.nextRow():
         sql = result.stringByIndex(2)
-        grt.modules.MysqlSqlFacade.parseSqlScriptString(catalog, ("USE `%s`;\n" % escape_sql_identifier(schema_name))+sql)
+        context = grt.modules.MySQLParserServices.createParserContext(catalog.characterSets, getServerVersion(connection), getServerMode(connection), 1)
+        options = {}
+        grt.modules.MySQLParserServices.parseSQLIntoCatalogSql(context, catalog, ("USE `%s`;\n" % escape_sql_identifier(schema_name)) + sql, options)
     else:
         raise Exception("Could not fetch table information for %s" % table_name)
 
@@ -384,7 +418,9 @@ def reverseEngineerTableToCatalog(connection, catalog, schema_name, table_name):
     result = execute_query(connection, "SHOW CREATE TABLE `%s`.`%s`" % (escape_sql_identifier(schema_name), escape_sql_identifier(table_name)))
     if result and result.nextRow():
         sql = result.stringByIndex(2)
-        grt.modules.MysqlSqlFacade.parseSqlScriptString(catalog, ("USE `%s`;\n" % escape_sql_identifier(schema_name))+sql)
+        context = grt.modules.MySQLParserServices.createParserContext(catalog.characterSets, getServerVersion(connection), getServerMode(connection), 1)
+        options = {}
+        grt.modules.MySQLParserServices.parseSQLIntoCatalogSql(context, catalog, ("USE `%s`;\n" % escape_sql_identifier(schema_name))+sql, options)
     else:
         raise Exception("Could not fetch table information for %s" % table_name)
 
