@@ -1,5 +1,23 @@
-#ifndef _DB_MYSQL_PARAMS_DEFINED_H_
-#define _DB_MYSQL_PARAMS_DEFINED_H_
+/*
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
+
+#pragma once
 
 #include "db_mysql_public_interface.h"
 
@@ -70,12 +88,7 @@ grt::ListRef<db_mysql_StorageEngine> MYSQLMODULEDBMYSQL_PUBLIC_FUNC
 get_known_engines(grt::GRT*);
 
 
-bool MYSQLMODULEDBMYSQL_PUBLIC_FUNC
-is_word_reserved(const char* str, grt::GRT*);
-
 bool MYSQLMODULEDBMYSQL_PUBLIC_FUNC
 check_valid_characters(const char* str);
 
 }
-
-#endif
