@@ -1,4 +1,4 @@
-# Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2012, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -38,7 +38,7 @@ class PostgresqlReverseEngineering(Sql92ReverseEngineering):
         r = super(PostgresqlReverseEngineering, cls).connect(connection, password)
         if r:
             ver = cls.execute_query(connection, "select version()").fetchone()[0]
-            grt.log_info("PostgreSQL RE", "Connected to %s, %s" % (connection.name, ver))
+            grt.log_info("PostgreSQL RE", "Connected to %s, %s\n" % (connection.name, ver))
             ver_parts = [int(n) for n in ver.split()[1].rstrip(",").split(".")] + 4*[0]
             version = grt.classes.GrtVersion()
             version.majorNumber, version.minorNumber, version.releaseNumber, version.buildNumber = ver_parts[:4]
@@ -91,7 +91,6 @@ FROM %s.%s"""
     @classmethod
     def reverseEngineerTableIndices(cls, connection, table):
         schema = table.owner
-        catalog = schema.owner
         
         if len(table.columns) == 0:
             grt.send_error('%s: reverseEngineerTableIndices', 
@@ -181,13 +180,13 @@ WHERE n.nspname = '%s' AND c.relname = '%s'"""
 
         catalog.userDatatypes.remove_all()
         
-        query_composite = """SELECT t.typname, at.attname, pg_catalog.format_type(at.atttypid, at.atttypmod)
-                  FROM pg_type t
-                  JOIN pg_class on (reltype = t.oid)
-                  JOIN pg_attribute at on (at.attrelid = pg_class.oid)
-                  JOIN pg_type a on (at.atttypid = a.oid)
-                  JOIN pg_namespace n on n.oid = t.typnamespace
-                  WHERE n.nspname NOT IN ('information_schema', 'pg_catalog') AND pg_class.relkind = 'c' """
+        #query_composite = """SELECT t.typname, at.attname, pg_catalog.format_type(at.atttypid, at.atttypmod)
+        #          FROM pg_type t
+        #          JOIN pg_class on (reltype = t.oid)
+        #          JOIN pg_attribute at on (at.attrelid = pg_class.oid)
+        #          JOIN pg_type a on (at.atttypid = a.oid)
+        #          JOIN pg_namespace n on n.oid = t.typnamespace
+        #          WHERE n.nspname NOT IN ('information_schema', 'pg_catalog') AND pg_class.relkind = 'c' """
         # TODO
             
         query_domains = """SELECT t.typname, pg_catalog.format_type(t.typbasetype, t.typtypmod)
@@ -196,6 +195,8 @@ WHERE n.nspname = '%s' AND c.relname = '%s'"""
             WHERE n.nspname NOT IN ('information_schema', 'pg_catalog') AND t.typtype = 'd' """
         domain_types = cls.execute_query(connection, query_domains)
         for type_name, type_def in domain_types:
+            if not type_def:
+                continue
             datatype = grt.classes.db_UserDatatype()
             datatype.name = type_name
             datatype.sqlDefinition = type_def
@@ -211,8 +212,8 @@ WHERE n.nspname = '%s' AND c.relname = '%s'"""
             datatype.owner = catalog
             catalog.userDatatypes.append(datatype)
 
-        query_ranges = """
-            """
+        #query_ranges = """
+        #    """
         # TODO
 
         query_enums = """SELECT t.typname, e.enumlabel
@@ -256,6 +257,20 @@ WHERE n.nspname = '%s' AND c.relname = '%s'"""
 
 
 
+    @classmethod
+    def getOS(cls, connection):
+        ver = cls.execute_query(connection, "select version()").fetchone()[0].lower()
+        if 'linux' in ver:
+            return 'linux'
+        elif 'visual c++' in ver:
+            return 'windows'
+        elif 'darwin' in ver or 'apple' in ver:
+            return 'darwin'
+
+        return None
+
+
+
 ###########################################################################################
 
 @ModuleInfo.export(grt.classes.db_mgmt_Rdbms)
@@ -361,3 +376,7 @@ def reverseEngineerFunctions(connection, schema):
 @ModuleInfo.export(grt.INT, grt.classes.db_mgmt_Connection, grt.classes.db_Schema)
 def reverseEngineerTriggers(connection, schema):
     return PostgresqlReverseEngineering.reverseEngineerTriggers(connection, schema)
+
+@ModuleInfo.export(grt.STRING, grt.classes.db_mgmt_Connection)
+def getOS(connection):
+    return PostgresqlReverseEngineering.getOS(connection)
