@@ -1,4 +1,4 @@
-# Copyright (c) 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2013, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -112,7 +112,7 @@ class SQLAnywhereReverseEngineering(GenericReverseEngineering):
             cls._connections[connection.__id__] = {'connection': con}
         if con:
             ver = cls.execute_query(connection, "SELECT @@version").fetchone()[0]
-            grt.log_info("SQLAnywhere RE", "Connected to %s, %s" % (connection.name, ver))
+            grt.log_info("SQLAnywhere RE", "Connected to %s, %s\n" % (connection.name, ver))
             ver_parts = server_version_str2tuple(ver) + (0, 0, 0, 0)
             version = grt.classes.GrtVersion()
             version.majorNumber, version.minorNumber, version.releaseNumber, version.buildNumber = ver_parts[:4]
@@ -188,6 +188,25 @@ WHERE st.table_name='%s' AND su.user_name='%s'""" % (table.name, table.owner.nam
 
         query = """SELECT UPPER(st.type_name), UPPER(base_type_str), UPPER(sd.domain_name )
         FROM SYSUSERTYPE st LEFT JOIN SYSDOMAIN sd ON st.domain_id=sd.domain_id"""
+
+        if cls.serverVersion().majorNumber < 12:
+            query = """SELECT
+            UPPER(st.type_name),
+            CASE
+                WHEN st.type_name = 'money' THEN UPPER(sd.domain_name+'(19,4)')
+                WHEN st.type_name = 'smallmoney' THEN UPPER(sd.domain_name+'(10,4)')
+                WHEN st.type_name = 'sysname' THEN UPPER(sd.domain_name+'(30)')
+                WHEN st.type_name = 'uniqueidentifierstr' THEN 'CHAR(36)'
+                WHEN st.type_name = 'uniqueidentifier' THEN 'BINARY(16)'
+            ELSE
+                UPPER(sd.domain_name)
+            END AS base_type_str,
+            UPPER(sd.domain_name )
+            FROM
+                SYSUSERTYPE st
+                    LEFT JOIN
+                SYSDOMAIN sd ON st.domain_id=sd.domain_id"""
+
         simple_datatypes = set()
         for datatype in cls._rdbms.simpleDatatypes:
             simple_datatypes.update([datatype.name] + list(datatype.synonyms))
@@ -260,7 +279,6 @@ ORDER BY sc.column_id""" % (table.name, table.owner.name)
         """Reverse engineers the primary key(s) for the given table."""
 
         schema = table.owner
-        catalog = schema.owner
 
 
         if len(table.columns) == 0:  # Table must have columns reverse engineered before we can rev eng its primary key(s)
@@ -380,7 +398,18 @@ ORDER BY sic.sequence""" % (other_table_id, other_index_id)
             table.foreignKeys.append(foreign_key)
         return 0
         
+    @classmethod
+    @release_cursors
+    def getOS(cls, connection):
+        _os = cls.execute_query(connection, "SELECT PROPERTY ('Platform')").fetchone()[0].lower()
+        if 'unix' in _os:
+            return 'linux'
+        elif 'windows' in _os:
+            return 'windows'
+        else:
+            return 'darwin'
 
+        return None
 ###########################################################################################
 
 @ModuleInfo.export(grt.classes.db_mgmt_Rdbms)
@@ -486,3 +515,7 @@ def reverseEngineerFunctions(connection, schema):
 @ModuleInfo.export(grt.INT, grt.classes.db_mgmt_Connection, grt.classes.db_Schema)
 def reverseEngineerTriggers(connection, schema):
     return SQLAnywhereReverseEngineering.reverseEngineerTriggers(connection, schema)
+
+@ModuleInfo.export(grt.LIST, grt.classes.db_mgmt_Connection)
+def getOS(connection):
+    return SQLAnywhereReverseEngineering.getOS(connection)
\ No newline at end of file
