@@ -1,4 +1,4 @@
-# Copyright (c) 2012, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2012, 2014 Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -179,6 +179,7 @@ def connect(connection, password):
         password:    a string with the password to use for the connection.
     '''
     con = None
+    host_identifier = connection.hostIdentifier
     try:
         con = get_connection(connection)
         try:
@@ -188,7 +189,7 @@ def connect(connection, password):
             grt.send_info("Connection to %s apparently lost, reconnecting..." % connection.hostIdentifier)
             raise NotConnectedError("Connection error")
     except NotConnectedError, exc:
-        grt.send_info("Connecting to %s..." % connection.hostIdentifier)
+        grt.send_info("Connecting to %s..." % host_identifier)
         import pyodbc
         try:
             con = db_driver.connect(connection, password)
@@ -216,10 +217,11 @@ def connect(connection, password):
         if not con:
             grt.send_error('Connection failed', str(exc))
             raise
-        grt.send_info("Connected")
         
         _connections[connection.__id__] = {"connection" : con }
         _connections[connection.__id__]["version"] = getServerVersion(connection)
+        version  = execute_query(connection, "SELECT @@version").fetchone()[0]
+        grt.send_info("Connected to %s, %s", (host_identifier, version))
     return 1
 
 
@@ -528,7 +530,7 @@ WHERE accessrule != NULL"""
         if simple_type:
             datatype.actualType = simple_type
         else:
-            grt.send_warning('Sybase reverseEngineerUserDatatypes', 'Could not found base type "%s" for user defined type "%s"' % (base_type, user_type_name) )
+            grt.send_warning('Sybase reverseEngineerUserDatatypes', 'Could not found base type "%s" for user defined type "%s"' % (base_type, name) )
         catalog.userDatatypes.append(datatype)
     return 0
 
@@ -952,3 +954,9 @@ def getTDSProtocolVersionChoices():
     result.append("4.1|Sybase Adaptive Server Enterprise 15 (4.1)")
 
     return result 
+
+
+@ModuleInfo.export(grt.INT, grt.classes.db_mgmt_Connection)
+def resetProgressFlags(connection):
+    _connections[connection.__id__]['_rev_eng_progress_flags'] = []
+    return 0
\ No newline at end of file
