@@ -27,15 +27,6 @@ def sortText(text):
   return "\n".join(lines)
 
 
-@ModuleInfo.exportFilter("wb.text.comment", "Un/Comment Selection")
-def commentText(text):
-  lines = text.split("\n")
-  if lines[0].startswith("-- "):
-    return "\n".join((line[3:] if line.startswith("-- ") else line) for line in lines)
-  else:
-    return "\n".join("-- "+line for line in lines)
-
-
 @ModuleInfo.exportFilter("wb.text.quoteString", "Quote and Escape String")
 def quoteString(text):
   escaped= ""
