@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,27 +17,18 @@
  * 02110-1301  USA
  */
 
-#ifndef _MYSQL_ROUTINE_EDITOR_H_
-#define _MYSQL_ROUTINE_EDITOR_H_
+#pragma once
 
 #include "grtdb/editor_routine.h"
-#include "grt/tree_model.h"
-#include "grts/structs.db.h"
-#include "grts/structs.db.mysql.h"
-#include "grts/structs.workbench.physical.h"
 
 #include "mysql_support_backend_public_interface.h"
 
 class MYSQLWBMYSQLSUPPORTBACKEND_PUBLIC_FUNC MySQLRoutineEditorBE : public bec::RoutineEditorBE
 {
 public:
-  MySQLRoutineEditorBE(bec::GRTManager *grtm, const db_mysql_RoutineRef &routine, const db_mgmt_RdbmsRef &rdbms); 
-  
-  std::string get_sql_definition_header();
-  
+  MySQLRoutineEditorBE(bec::GRTManager *grtm, const db_mysql_RoutineRef &routine); 
+
   void load_routine_sql();
   virtual void commit_changes();
-
+  virtual bool can_close();
 };
-
-#endif /* _MYSQL_ROUTINE_EDITOR_H_ */
