@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,7 @@
  * 02110-1301  USA
  */
 
-#ifndef _MYSQL_TABLE_EDITOR_H_
-#define _MYSQL_TABLE_EDITOR_H_
+#pragma once
 
 #include "grtdb/editor_table.h"
 #include "grts/structs.workbench.physical.h"
@@ -38,12 +37,15 @@ class MYSQLWBMYSQLSUPPORTBACKEND_PUBLIC_FUNC MySQLTableColumnsListBE : public ::
 public:
   enum MySQLColumnListColumns
   {
-    IsAutoIncrement= bec::TableColumnsListBE::LastColumn,
+    IsAutoIncrement = bec::TableColumnsListBE::LastColumn,
     IsAutoIncrementable,
+    IsGenerated,
+    GeneratedStorageType,
+    GeneratedExpression
   };
 
-  virtual bool set_field(const ::bec::NodeId &node, int column, const std::string &value);
-  virtual bool set_field(const ::bec::NodeId &node, int column, int value);
+  virtual bool set_field(const ::bec::NodeId &node, ColumnId column, const std::string &value);
+  virtual bool set_field(const ::bec::NodeId &node, ColumnId column, ssize_t value);
 
   MySQLTableColumnsListBE(MySQLTableEditorBE *owner);
   
@@ -51,7 +53,8 @@ public:
   virtual bec::MenuItemList get_popup_items_for_nodes(const std::vector<bec::NodeId> &nodes);
 protected:
   // for internal use only
-  virtual bool get_field_grt(const ::bec::NodeId &node, int column, ::grt::ValueRef &value);
+  virtual bool get_field_grt(const ::bec::NodeId &node, ColumnId column, ::grt::ValueRef &value);
+
 };
 
 
@@ -67,10 +70,10 @@ public:
 
   MySQLTableIndexListBE(MySQLTableEditorBE *owner);
 
-  virtual bool set_field(const ::bec::NodeId &node, int column, const std::string &value);
+  virtual bool set_field(const ::bec::NodeId &node, ColumnId column, const std::string &value);
 
 protected:
-  virtual bool get_field_grt(const ::bec::NodeId &node, int column, grt::ValueRef &value);
+  virtual bool get_field_grt(const ::bec::NodeId &node, ColumnId column, grt::ValueRef &value);
 };
 
 
@@ -95,16 +98,17 @@ public:
 
   virtual void refresh() {};
 
-  virtual int count_children(const ::bec::NodeId &parent);
-  virtual ::bec::NodeId get_child(const ::bec::NodeId &parent, int index);
+  virtual size_t count_children(const ::bec::NodeId &parent);
+  virtual ::bec::NodeId get_child(const ::bec::NodeId &parent, size_t index);
 
-  virtual bool set_field(const ::bec::NodeId &node, int column, const std::string &value);
+  virtual bool set_field(const ::bec::NodeId &node, ColumnId column, const std::string &value);
 
 protected:
-  virtual bool get_field_grt(const ::bec::NodeId &node, int column, grt::ValueRef &value);
-  virtual grt::Type get_field_type(const ::bec::NodeId &node, int column);
+  virtual bool get_field_grt(const ::bec::NodeId &node, ColumnId column, grt::ValueRef &value);
+  virtual grt::Type get_field_type(const ::bec::NodeId &node, ColumnId column);
 
   db_mysql_PartitionDefinitionRef get_definition(const ::bec::NodeId &node);
+
 };
 
 
@@ -114,16 +118,15 @@ class MYSQLWBMYSQLSUPPORTBACKEND_PUBLIC_FUNC MySQLTableEditorBE : public ::bec::
 {
   friend class MySQLTriggerPanel;
 public:
-  MySQLTableEditorBE(::bec::GRTManager *grtm, db_mysql_TableRef table, const db_mgmt_RdbmsRef &rdbms);
+  MySQLTableEditorBE(::bec::GRTManager *grtm, db_mysql_TableRef table);
   virtual ~MySQLTableEditorBE();
 
+  virtual void refresh_live_object();
+  virtual void commit_changes();
+
   virtual MySQLTableColumnsListBE *get_columns() { return &_columns; }
   virtual MySQLTableIndexListBE *get_indexes() { return &_indexes; }
 
-  virtual db_TableRef get_table() { return table(); }
-
-  db_mysql_TableRef table() { return _table; }
-
   virtual std::vector<std::string> get_index_types();
   virtual std::vector<std::string> get_index_storage_types();
   virtual std::vector<std::string> get_fk_action_options();
@@ -136,15 +139,13 @@ public:
   
   virtual bool check_column_referenceable_by_fk(const db_ColumnRef &column1, const db_ColumnRef &column2);
 
-  // interactive functions
-  std::string get_all_triggers_sql() const; // TODO: only used internally and in tests, should be protected.
   void load_trigger_sql();
 
   // triggers
   mforms::View *get_trigger_panel();
+  void add_trigger(const std::string &timing, const std::string &event);
 
   virtual bool can_close();
-  virtual void commit_changes();
 
   // partitioning
   bool set_partition_type(const std::string &type);
@@ -169,8 +170,8 @@ public:
 
   MySQLTablePartitionTreeBE *get_partitions() { return &_partitions; }
 
-  // whether partitions and subpartitions will be defined by the user or not 
-  // if false, only count is needed otherwise the partitionslist must be defined
+  // whether partitions and sub partitions will be defined by the user or not 
+  // if false, only count is needed otherwise the partitions list must be defined
   void set_explicit_partitions(bool flag);
   bool get_explicit_partitions();
   void set_explicit_subpartitions(bool flag);
@@ -179,7 +180,6 @@ public:
   virtual db_TableRef create_stub_table(const std::string &schema, const std::string &table);
   
 protected:
-  db_mysql_TableRef _table;
   MySQLTableColumnsListBE _columns;
   MySQLTablePartitionTreeBE _partitions;
   MySQLTableIndexListBE _indexes;
@@ -188,5 +188,3 @@ protected:
   
   void reset_partition_definitions(int parts, int subparts);
 };
-
-#endif /* _MYSQL_TABLE_EDITOR_H_ */
