@@ -21,6 +21,7 @@
 #include "gtk/mforms_gtk.h"
 #include <sstream>
 #include "text_list_columns_model.h"
+#include "workbench/wb_context.h"
 
 //==============================================================================
 //
@@ -63,7 +64,7 @@ class DbMySQLRoutineGroupEditor : public PluginEditorBase
 //------------------------------------------------------------------------------
 DbMySQLRoutineGroupEditor::DbMySQLRoutineGroupEditor(grt::Module *m, bec::GRTManager *grtm, const grt::BaseListRef &args)
     : PluginEditorBase(m, grtm, args, "modules/data/editor_rg.glade")
-    , _be(new MySQLRoutineGroupEditorBE(grtm, db_mysql_RoutineGroupRef::cast_from(args[0]), get_rdbms_for_db_object(args[0])))
+    , _be(new MySQLRoutineGroupEditorBE(grtm, db_mysql_RoutineGroupRef::cast_from(args[0])))
     , _routines_model(model_from_string_list(std::vector<std::string>(), &_routines_columns))
 {
   xml()->get_widget("mysql_rg_editor_notebook", _editor_notebook);
@@ -98,8 +99,9 @@ DbMySQLRoutineGroupEditor::DbMySQLRoutineGroupEditor(grt::Module *m, bec::GRTMan
   _rg_list->signal_row_activated().connect(sigc::mem_fun(this, &DbMySQLRoutineGroupEditor::activate_row));
 
   // Setup DnD
-  std::vector<Gtk::TargetEntry> targets;  
-  targets.push_back(Gtk::TargetEntry("x-mysql-wb/db.DatabaseObject", Gtk::TARGET_SAME_APP));
+  std::vector<Gtk::TargetEntry> targets;
+
+  targets.push_back(Gtk::TargetEntry(WB_DBOBJECT_DRAG_TYPE, Gtk::TARGET_SAME_APP));
   _rg_list->drag_dest_set(targets, Gtk::DEST_DEFAULT_ALL,Gdk::ACTION_COPY);
   _rg_list->signal_drag_data_received().connect(sigc::mem_fun(this, &DbMySQLRoutineGroupEditor::on_routine_drop));
   _rg_list->signal_event().connect(sigc::mem_fun(*this, &DbMySQLRoutineGroupEditor::process_event));
@@ -126,7 +128,7 @@ bool DbMySQLRoutineGroupEditor::switch_edited_object(bec::GRTManager *grtm, cons
   xml()->get_widget("rg_code_holder", code_win);
   delete _be;
  
-  _be = new MySQLRoutineGroupEditorBE(grtm, db_mysql_RoutineGroupRef::cast_from(args[0]), get_rdbms_for_db_object(args[0]));
+  _be = new MySQLRoutineGroupEditorBE(grtm, db_mysql_RoutineGroupRef::cast_from(args[0]));
   embed_code_editor(_be->get_sql_editor()->get_container(), code_win);
   _be->load_routines_sql();
   _be->set_refresh_ui_slot(sigc::mem_fun(this, &DbMySQLRoutineGroupEditor::refresh_form_data));
@@ -215,7 +217,7 @@ void DbMySQLRoutineGroupEditor::on_routine_drop(const Glib::RefPtr<Gdk::DragCont
 {
   bool dnd_status = false;
   
-  if ( selection_data.get_target() == "x-mysql-wb/db.DatabaseObject" )
+  if ( selection_data.get_target() == WB_DBOBJECT_DRAG_TYPE)
   {
     std::list<db_DatabaseObjectRef> objects;
 
@@ -230,10 +232,7 @@ void DbMySQLRoutineGroupEditor::on_routine_drop(const Glib::RefPtr<Gdk::DragCont
       {
         db_mysql_RoutineRef routine = db_mysql_RoutineRef::cast_from(*obj);
         if ( routine.is_valid() )
-        {
           _be->append_routine_with_id(routine.id());
-          std::string routine_name = _be->get_routine_name(routine.id());
-        }
       }
     }
 
