@@ -27,6 +27,7 @@ DbMySQLTableEditorColumnPage::DbMySQLTableEditorColumnPage(DbMySQLTableEditor *o
                              , _tv_holder(0)
                              , _edit_conn(0)
                              , _ce(0)
+                             , _auto_edit_pending(false)
                              , _editing(false)
 {
   _xml->get_widget("table_columns_holder", _tv_holder);
@@ -47,6 +48,19 @@ DbMySQLTableEditorColumnPage::DbMySQLTableEditorColumnPage(DbMySQLTableEditor *o
   collations.insert(collations.begin(), "*Table Default*");
   fill_combo_from_string_list(_collation_combo, collations);
   _collation_combo->signal_changed().connect(sigc::mem_fun(this, &DbMySQLTableEditorColumnPage::set_collation));
+
+  Gtk::Box *box;
+  _xml->get_widget("gc_storage_type_box", box);
+  if (box)
+    box->set_sensitive(false);
+
+  _xml->get_widget("gc_stored_radio", _radioStored);
+  _xml->get_widget("gc_virtual_radio", _radioVirtual);
+  if (_radioStored == 0 || _radioVirtual == 0)
+    throw std::logic_error("Glade file is missing gc_stored_radio or gc_virtual_radio");
+
+  _radioStored->signal_toggled().connect(sigc::mem_fun(this, &DbMySQLTableEditorColumnPage::set_gc_storage_type));
+  _radioVirtual->signal_toggled().connect(sigc::mem_fun(this, &DbMySQLTableEditorColumnPage::set_gc_storage_type));
 }
 
 //------------------------------------------------------------------------------
@@ -107,7 +121,7 @@ void DbMySQLTableEditorColumnPage::refill_columns_tv()
   Glib::RefPtr<ListModelWrapper> model = ListModelWrapper::create(_be->get_columns(), _tv, "DbMySQLTableEditorColumnPage");
 
   model->model().append_string_column(MySQLTableColumnsListBE::Name, "Column Name", EDITABLE, WITH_ICON);
-  model->model().append_combo_column(MySQLTableColumnsListBE::Type, "Datatype", model->model().create_model(get_types_for_table(_be->table())), EDITABLE);
+  model->model().append_combo_column(MySQLTableColumnsListBE::Type, "Datatype", model->model().create_model(get_types_for_table(_be->get_table())), EDITABLE);
   model->model().append_check_column(MySQLTableColumnsListBE::IsPK, "PK", EDITABLE);
   model->model().append_check_column(MySQLTableColumnsListBE::IsNotNull, "NN", EDITABLE);
   model->model().append_check_column(MySQLTableColumnsListBE::IsUnique,  "UQ", EDITABLE);
@@ -115,7 +129,8 @@ void DbMySQLTableEditorColumnPage::refill_columns_tv()
   model->model().append_check_column(MySQLTableColumnsListBE::IsUnsigned, "UN", EDITABLE);
   model->model().append_check_column(MySQLTableColumnsListBE::IsZerofill, "ZF", EDITABLE);
   model->model().append_check_column(MySQLTableColumnsListBE::IsAutoIncrement, "AI", EDITABLE);
-  model->model().append_string_column(MySQLTableColumnsListBE::Default, "Default", EDITABLE);
+  model->model().append_check_column(MySQLTableColumnsListBE::IsGenerated, "G", EDITABLE);
+  model->model().append_string_column(MySQLTableColumnsListBE::Default, "Default / Expression", EDITABLE);
   
   _model = model;
   new_tv.release();
@@ -306,6 +321,7 @@ void DbMySQLTableEditorColumnPage::type_cell_editing_started(GtkCellRenderer* cr
 //--------------------------------------------------------------------------------
 void DbMySQLTableEditorColumnPage::cell_editing_done(GtkCellEditable* ce, gpointer udata)
 {
+
   DbMySQLTableEditorColumnPage* columns_page = reinterpret_cast<DbMySQLTableEditorColumnPage*>(udata);
   columns_page->_editing = false;
 
@@ -341,6 +357,30 @@ void DbMySQLTableEditorColumnPage::cell_editing_done(GtkCellEditable* ce, gpoint
   }
   else
     columns_page->_tv->set_cursor(path);
+
+  //If it's Gtk::Entry, we try to find out if maybe user cancelled edit operation,
+  //if so we revert the placeholder row to the initial state.
+  if (GTK_IS_ENTRY(ce))
+  {
+    GtkEntry *entry = GTK_ENTRY(ce);
+    if (entry)
+    {
+      gboolean user_canceled = false;
+
+      g_object_get(entry, "editing-canceled", &user_canceled, NULL);
+      if (user_canceled)
+      {
+          std::string name;
+          bec::NodeId node(path.to_string());
+          columns_page->_be->get_columns()->reset_placeholder();
+          columns_page->_be->get_columns()->get_field(node, MySQLTableColumnsListBE::Name, name);
+          gtk_entry_set_completion(entry, NULL);
+          gtk_entry_set_text(entry, name.c_str());
+
+      }
+    }
+  }
+
 }
 
 //------------------------------------------------------------------------------
@@ -363,12 +403,12 @@ grt::StringListRef DbMySQLTableEditorColumnPage::get_types_for_table(const db_Ta
 //------------------------------------------------------------------------------
 void DbMySQLTableEditorColumnPage::check_resize(Gtk::Allocation& r)
 {
-  //   0       1        2  3  4   5   6   7   8       9
-  // name    type      PK  NN UQ BIN  UN  ZF  AI      Default
+  //   0       1        2  3  4   5   6   7   8   9    10
+  // name    type      PK  NN UQ BIN  UN  ZF  AI  G    Default
   const int step = r.get_width() / 10;
   _tv->get_column(0)->set_min_width(4 * step);
   _tv->get_column(1)->set_min_width(2 * step);
-  _tv->get_column(9)->set_min_width(2 * step);
+  _tv->get_column(10)->set_min_width(2 * step);
 }
 
 //------------------------------------------------------------------------------
@@ -455,6 +495,7 @@ void DbMySQLTableEditorColumnPage::update_column_details(const ::bec::NodeId& no
   }
   
   update_collation();
+  update_gc_storage_type();
 }
 
 //------------------------------------------------------------------------------
@@ -518,15 +559,65 @@ void DbMySQLTableEditorColumnPage::update_collation()
     collation_combo->set_sensitive(false);
   }
 }
+//--------------------------------------------------------------------------------
+void DbMySQLTableEditorColumnPage::update_gc_storage_type()
+{
+  const bec::NodeId node = get_selected();
+  if (node.is_valid())
+  {
+    std::string has_charset;
+    MySQLTableColumnsListBE* columns = _be->get_columns();
+    ssize_t isGenerated;
+    columns->get_field(node, MySQLTableColumnsListBE::IsGenerated, isGenerated);
+    Gtk::Box *box;
+    _xml->get_widget("gc_storage_type_box", box);
+    if (box)
+      box->set_sensitive(isGenerated != 0);
+
+    if (isGenerated != 0)
+    {
+      std::string storageType;
+      columns->get_field(node, MySQLTableColumnsListBE::GeneratedStorageType, storageType);
+
+      if (base::toupper(storageType) != "STORED")
+        _radioVirtual->activate();
+      else
+        _radioStored->activate();
+    }
+  }
+}
+
+//--------------------------------------------------------------------------------
+
+void DbMySQLTableEditorColumnPage::set_gc_storage_type()
+{
+  const bec::NodeId node = get_selected();
+  if (node.is_valid())
+  {
+    MySQLTableColumnsListBE* columns = _be->get_columns();
+    if (_radioVirtual->get_active())
+      columns->set_field(node, MySQLTableColumnsListBE::GeneratedStorageType, "VIRTUAL");
+    else
+      columns->set_field(node, MySQLTableColumnsListBE::GeneratedStorageType, "STORED");
+  }
+}
 
 //--------------------------------------------------------------------------------
 bool DbMySQLTableEditorColumnPage::do_on_visible(GdkEventVisibility*)
 {
-  MySQLTableColumnsListBE *columns = _be->get_columns();
-  if ( columns->count() == 1 )
+  if (!_auto_edit_pending && _be->get_columns()->count() == 1)
   {
-    ::bec::NodeId node(columns->get_node(0));
-    _tv->set_cursor(node2path(node), *(_tv->get_column(0)), true);
+    Glib::signal_idle().connect(sigc::bind_return(sigc::mem_fun(this, &DbMySQLTableEditorColumnPage::start_auto_edit), false));
+    _auto_edit_pending = true;
   }
   return false;
 }
+
+
+void DbMySQLTableEditorColumnPage::start_auto_edit()
+{
+  MySQLTableColumnsListBE *columns = _be->get_columns();
+  ::bec::NodeId node(columns->get_node(0));
+  _tv->set_cursor(node2path(node), *(_tv->get_column(0)), true);
+}
+
