@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "grtdb/db_object_helpers.h"
 
 #include "grts/structs.h"
@@ -54,6 +52,8 @@ DbMySQLSQLExport::DbMySQLSQLExport(bec::GRTManager *grtm, db_mysql_CatalogRef ca
   _users_are_selected= true;
   _catalog= catalog;
   _case_sensitive = true;
+  _gen_doc_props = false;
+  _gen_attached_scripts = false;
 
   if(!_catalog.is_valid())
     _catalog= get_model_catalog();  // call own version
@@ -87,9 +87,9 @@ void DbMySQLSQLExport::set_option(const std::string& name, bool value)
   else if(name.compare("GenerateCreateIndex") == 0)
     _gen_create_index= value;
   else if(name.compare("NoViewPlaceholders") == 0)
-    _no_users_just_privileges=value;
-  else if(name.compare("NoUsersJustPrivileges") == 0)
     _no_view_placeholders=value;
+  else if(name.compare("NoUsersJustPrivileges") == 0)
+    _no_users_just_privileges=value;
   else if(name.compare("GenerateInserts") == 0)
     _gen_inserts=value;
   else if(name.compare("NoFKForInserts") == 0)
@@ -114,12 +114,30 @@ void DbMySQLSQLExport::set_option(const std::string& name, bool value)
     _skip_foreign_keys= value;
   else if(name.compare("SkipFKIndexes") == 0)
     _skip_fk_indexes= value;
+  else if(name.compare("GenerateDocumentProperties") == 0)
+    _gen_doc_props= value;
+  else if(name.compare("GenerateAttachedScripts") == 0)
+    _gen_attached_scripts= value;
 }
 
 void DbMySQLSQLExport::set_option(const std::string& name, const std::string& value)
 {
   if(name.compare("OutputFileName") == 0)
     _output_filename= value;
+  else if(name.compare("OutputScriptHeader") == 0)
+    _output_header= value;
+}
+
+void DbMySQLSQLExport::set_db_options_for_version(const GrtVersionRef &version)
+{
+  SQLGeneratorInterfaceImpl *diffsql_module= dynamic_cast<SQLGeneratorInterfaceImpl*>(version.get_grt()->get_module("DbMySQL"));
+  if (diffsql_module != NULL)
+    _db_options = diffsql_module->getTraitsForServerVersion((int)version->majorNumber(), (int)version->minorNumber(), (int)version->releaseNumber());
+}
+
+void DbMySQLSQLExport::set_db_options(grt::DictRef &db_options)
+{
+    _db_options = db_options;
 }
 
 grt::StringListRef convert_string_vector_to_grt_list(grt::GRT *grt, const std::vector<std::string>& v)
@@ -174,7 +192,7 @@ grt::DictRef DbMySQLSQLExport::get_options_as_dict(grt::GRT *grt)
   options.set("GenerateWarnings", grt::IntegerRef(_gen_warnings ? 1 : 0));
   options.set("GenerateCreateIndex", grt::IntegerRef(_gen_create_index ? 1 : 0));
   options.set("NoUsersJustPrivileges", grt::IntegerRef(_no_users_just_privileges ? 1 : 0));
-  options.set("NoViewPlaceholders", grt::IntegerRef(_no_users_just_privileges ? 1 : 0));
+  options.set("NoViewPlaceholders", grt::IntegerRef(_no_view_placeholders ? 1 : 0));
   options.set("GenerateInserts", grt::IntegerRef(_gen_inserts ? 1 : 0));
   options.set("NoFKForInserts", grt::IntegerRef(_no_FK_for_inserts ? 1 : 0));
   options.set("TriggersAfterInserts", grt::IntegerRef(_triggers_after_inserts ? 1 : 0));
@@ -182,6 +200,10 @@ grt::DictRef DbMySQLSQLExport::get_options_as_dict(grt::GRT *grt)
   options.set("GenerateUse", grt::IntegerRef(_generate_use ? 1 : 0));
   options.set("SkipForeignKeys", grt::IntegerRef(_skip_foreign_keys ? 1 : 0));
   options.set("SkipFKIndexes", grt::IntegerRef(_skip_fk_indexes ? 1 : 0));
+  options.set("GenerateDocumentProperties", grt::IntegerRef(_gen_doc_props ? 1 : 0));
+  options.set("GenerateAttachedScripts", grt::IntegerRef(_gen_attached_scripts ? 1 : 0));
+
+  options.set("OutputScriptHeader", grt::StringRef(_output_header));
 
   std::vector<std::string> schemata_names;
   db_mysql_CatalogRef cat(get_model_catalog());
@@ -220,10 +242,11 @@ grt::DictRef DbMySQLSQLExport::get_options_as_dict(grt::GRT *grt)
   return options;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DbMySQLSQLExport::start_export(bool wait_finish)
 {
-  bec::GRTTask *task= new bec::GRTTask("SQL export", 
+  bec::GRTTask::Ref task = bec::GRTTask::create_task("SQL export", 
     _manager->get_dispatcher(), 
     boost::bind(&DbMySQLSQLExport::export_task, this, _1, grt::StringRef()));
 
@@ -235,6 +258,8 @@ void DbMySQLSQLExport::start_export(bool wait_finish)
     _manager->get_dispatcher()->add_task(task);
 }
 
+//--------------------------------------------------------------------------------------------------
+
 void DbMySQLSQLExport::export_finished(grt::ValueRef res)
 {
   CatalogMap cmap;
@@ -265,6 +290,9 @@ ValueRef DbMySQLSQLExport::export_task(grt::GRT* grt, grt::StringRef)
 
     if(_db_options.is_valid())
     {
+      if (_db_options.count() == 0)
+        log_error("internal error: Supplied dboptions is empty!?\n");
+        
         _db_options.set("CaseSensitive", grt::IntegerRef(1));
         options.set("DBSettings",_db_options);
     }else
@@ -284,15 +312,18 @@ ValueRef DbMySQLSQLExport::export_task(grt::GRT* grt, grt::StringRef)
     grt::StringListRef strlist= grt::StringListRef::cast_from(options.get("ViewFilterList"));
 
     // generateSQLForDifferences() will set DiffCaseSensitiveness to the used value
-    _case_sensitive = options.get_int("DiffCaseSensitiveness", _case_sensitive);
+    _case_sensitive = options.get_int("DiffCaseSensitiveness", _case_sensitive) != 0;
     options.set("CaseSensitive", grt::IntegerRef(_case_sensitive));
+    if (_db_options.is_valid())
+      _db_options.set("CaseSensitive", grt::IntegerRef(_case_sensitive));
 
     if (diffsql_module->makeSQLExportScript(_catalog, options, create_map, drop_map))
     {
       return grt::StringRef("\nSQL Script Export Error: SQL Script Export Module Returned Error");
     }
     
-    _export_sql_script= options.get_string("OutputScript");
+    _export_sql_script= options.get_string("OutputScriptHeader") + options.get_string("OutputScript");
+
     if (!_output_filename.empty())
     {
       g_file_set_contents(_output_filename.c_str(), 
