@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include <ostream>
 #include <sstream>
 #include <memory>
@@ -128,6 +126,46 @@ std::string Db_plugin::db_objects_struct_name_by_type(Db_object_type db_object_t
   return obj.get_metaclass()->get_member_type(attr_name).content.object_class;
 }
 
+/**
+ * Check if on the server we're connecting we can expirence some case sensitivity problems,
+ * return -1 if it's impossible to check
+ * return 0 if everything is ok
+ * return 1 if there can be some problems
+ */
+int Db_plugin::check_case_sensitivity_problems()
+{
+
+  sql::ConnectionWrapper dbc_conn = _db_conn->get_dbc_connection();
+  boost::scoped_ptr<sql::Statement> statement(dbc_conn->createStatement());
+
+  std::string compile_os;
+  int lower_case_table_names = -1;
+  {
+    boost::scoped_ptr<sql::ResultSet> rs(statement->executeQuery("SELECT @@version_compile_os"));
+    if (rs->next())
+    {
+      compile_os = rs->getString(1);
+    }
+  }
+  {
+    boost::scoped_ptr<sql::ResultSet> rs(statement->executeQuery("SELECT @@lower_case_table_names"));
+    if (rs->next())
+    {
+      lower_case_table_names = rs->getInt(1);
+    }
+  }
+  if (compile_os.empty() || lower_case_table_names == -1)
+    return -1;
+
+  if (lower_case_table_names == 0 && (base::starts_with(compile_os, "Win") || base::starts_with(compile_os, "osx")))
+    return 1;
+
+  if (lower_case_table_names == 2 && base::starts_with(compile_os, "Win"))
+    return 1;
+
+  return 0;
+}
+
 void Db_plugin::load_schemata(std::vector<std::string> &schemata)
 {
   _schemata.clear();
@@ -512,7 +550,12 @@ grt::StringRef Db_plugin::apply_script_to_db(grt::GRT *grt)
 int Db_plugin::process_sql_script_error(long long err_no, const std::string &err_msg, const std::string &statement)
 {
   std::ostringstream oss;
-  oss << _("Error ") << err_no << ": " << err_msg << std::endl << statement << std::endl;
+  std::string stmt = base::trim(statement, "\n");
+
+  base::replace(stmt, "\n", "\n        ");
+  stmt = "        " + stmt;
+
+  oss << _("Error ") << err_no << ": " << err_msg << std::endl << _("SQL Code:") << std::endl << stmt << std::endl;
   _grtm->get_grt()->send_error(oss.str());
   return 0;
 }
