@@ -1,4 +1,21 @@
-#include "tut_stdafx.h"
+/*
+* Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
+*
+* This program is free software; you can redistribute it and/or
+* modify it under the terms of the GNU General Public License as
+* published by the Free Software Foundation; version 2 of the
+* License.
+*
+* This program is distributed in the hope that it will be useful,
+* but WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+* GNU General Public License for more details.
+*
+* You should have received a copy of the GNU General Public License
+* along with this program; if not, write to the Free Software
+* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+* 02110-1301  USA
+*/
 
 #include "testgrt.h"
 #include "grt_test_utility.h"
@@ -14,21 +31,20 @@
 #include "grtdb/db_object_helpers.h"
 
 #include "cppdbc.h"
-#include "db_rev_eng_be.h"
+#include "backend/db_rev_eng_be.h"
 
 #include "grtsqlparser/sql_facade.h"
 #include "db_mysql_diffsqlgen.h"
 
-#include "diffchange.h"
-#include "grtdiff.h"
-#include "changeobjects.h"
-#include "changelistobjects.h"
+#include "diff/diffchange.h"
+#include "diff/grtdiff.h"
+#include "diff/changeobjects.h"
+#include "diff/changelistobjects.h"
 #include "grtdb/diff_dbobjectmatch.h"
 
 #include "myx_statement_parser.h"
-//#include "test_import_helpers.h"
-#include "db_mysql_sql_script_sync.h"
-#include "db_mysql_sql_export.h"
+#include "backend/db_mysql_sql_script_sync.h"
+#include "backend/db_mysql_sql_export.h"
 #include "module_db_mysql.h"
 #include "wb_helpers.h"
 
@@ -122,13 +138,6 @@ END_TEST_DATA_CLASS
 
 TEST_MODULE(model_diff_apply, "db.mysql plugin test");
 
-static int process_sql_statement_callback(const MyxStatementParser *splitter, const char *sql, void *user_data)
-{
-  sql::Statement *stmt= static_cast<sql::Statement *>(user_data);
-  stmt->execute(sql);
-  return 1;
-}
-
 db_mysql_CatalogRef tut::Test_object_base<model_diff_apply>::create_catalog_from_script(
   const std::string& sql)
 {
@@ -751,6 +760,8 @@ TEST_FUNCTION(42)
     ensure("FK not removed", objects.t2->foreignKeys().count() == 0);
 }
 
+
+
 /*
 As there is no t1 definition in sql a stub will be created when creating t2
 FK referring it, and as iSStub cols aren't diffed original t1 will be left 
@@ -791,9 +802,9 @@ TEST_FUNCTION(45)
 
     all_objects_mwb objects = get_model_objects();
 
-    ensure_equals("New column added", objects.t1->columns().count(), 4);
+    ensure_equals("New column added", objects.t1->columns().count(), 4U);
     ensure_equals("New column name", *objects.t1->columns()[3]->name(), "newcol");
-    ensure_equals("New index added", objects.t1->indices().count(), 2);
+    ensure_equals("New index added", objects.t1->indices().count(), 2U);
     ensure_equals("New index name", *objects.t1->indices()[0]->name(), "newindex");
     
     // BUG #14588524 - MYSQL WORKBENCH SEGFAULTS WHEN UPDATING MODEL FROM A DATABASE
@@ -802,6 +813,33 @@ TEST_FUNCTION(45)
       objects.t1->columns()[3]->id());
 }
 
+TEST_FUNCTION(46)
+{
+
+  static const char * create_table2 = "CREATE  TABLE IF NOT EXISTS `mydb`.`table2` (\n"
+  "  `table1_idtable1` INT NULL ,\n"
+  "  INDEX `fktable1_idx` (`table1_idtable1` ASC) ,\n"
+  "  CONSTRAINT `fktable1`\n"
+  "    FOREIGN KEY (`table1_idtable1` )\n"
+  "    REFERENCES `mydb`.`table1` (`idtable1` )\n"
+  "    ON DELETE RESTRICT\n"
+  "    ON UPDATE RESTRICT);\n";
+
+  all_objects_sql sql;
+  sql.table2_sql = create_table2;
+  tester.wb->open_document("data/workbench/all_objects.mwb");
+  all_objects_mwb initial_objects = get_model_objects();
+  apply_sql_to_model(sql.get_sql());
+
+  all_objects_mwb objects = get_model_objects();
+  all_objects_mwb_validator validator;
+  validator.validate_FK.disable();
+  validator.validate(objects);
+
+  ensure("FK ON UPDATE wasn't changed", objects.t2->foreignKeys().get(0)->updateRule() == "RESTRICT");
+  ensure("FK ON DELETE wasn't changed", objects.t2->foreignKeys().get(0)->deleteRule() == "RESTRICT");
+}
+
 
 //View
 TEST_FUNCTION(51)
