@@ -1,5 +1,5 @@
 /* 
-* Copyright (c) 2012, 2013 Oracle and/or its affiliates. All rights reserved.
+* Copyright (c) 2012, 2014 Oracle and/or its affiliates. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
@@ -17,7 +17,6 @@
 * 02110-1301  USA
 */
 
-#include "stdafx.h"
 #include "DbSearchPanel.h"
 #include <sstream>
 #include <boost/assign/list_of.hpp>
@@ -108,8 +107,8 @@ private:
     int _matched_rows;
     std::string _cast_to;
     int _search_data_type;
-    GMutex* _search_result_mutex;
-    GMutex* _pause_mutex;
+    base::Mutex _search_result_mutex;
+    base::Mutex _pause_mutex;
 
 protected:
     typedef boost::function<void (const std::string&, const std::string&, const std::list<std::string>&, const std::list<std::string>&, const std::string&, const bool match_PK)> select_func_t;
@@ -132,16 +131,11 @@ public:
         _working(false), _stop(false), _starting(false), _paused(false), _invert(invert), 
         _searched_tables(0), _matched_rows(0),
         _cast_to(cast_to), _search_data_type(search_data_type)
-    {
-        _pause_mutex = g_mutex_new();
-        _search_result_mutex = g_mutex_new();
-    }
+    {}
 
     ~DBSearch()
     {
         stop();
-        g_mutex_free(_pause_mutex);
-        g_mutex_free(_search_result_mutex);
     };
 
     std::string get_keyword() { return _search_keyword;}
@@ -152,22 +146,22 @@ public:
     {
         _paused = !_paused;
         if(_paused)
-            g_mutex_lock(_pause_mutex);
+          _pause_mutex.lock();
         else
-            g_mutex_unlock(_pause_mutex);
+          _pause_mutex.unlock();
     }
     void wait_if_paused()
     {
         if (is_paused())
         {
-            base::GMutexLock lock(_pause_mutex);//Wait for unlock
+            base::MutexLock lock(_pause_mutex);//Wait for unlock
         };
     };
     bool is_paused() const {return _paused;}
     float get_progress() const {return _progress;}
     std::string get_state() const {return _state;}
     const std::vector<SearchResultEntry>& search_results() const {return _search_result;}
-    GMutex* get_search_result_mutex() {return _search_result_mutex;};
+    base::Mutex &get_search_result_mutex() {return _search_result_mutex;};
     int searched_table_count() { return _searched_tables; }
     int matched_rows() { return _matched_rows; }
     bool is_working() const {return _working;}
@@ -192,8 +186,8 @@ void DBSearch::stop()
 
 std::string DBSearch::build_where(const std::string& col, const std::string& data) const
 {
-  static const std::vector<std::string> select_modes = boost::assign::list_of(std::string("="))("LIKE")("REGEXP");
-  static const std::vector<std::string> inverted_select_modes = boost::assign::list_of(std::string("<>"))("NOT LIKE")("NOT REGEXP");
+  static const std::vector<std::string> select_modes = boost::assign::list_of(std::string("LIKE"))("=")("LIKE")("REGEXP");
+  static const std::vector<std::string> inverted_select_modes = boost::assign::list_of(std::string("LIKE"))("<>")("NOT LIKE")("NOT REGEXP");
 
   std::string where_condition;
   if (_cast_to.empty())
@@ -208,7 +202,10 @@ std::string DBSearch::build_where(const std::string& col, const std::string& dat
 
   where_condition.append(" ");
   where_condition.append(_invert ? inverted_select_modes[_search_mode].c_str() : select_modes[_search_mode].c_str());
-  where_condition.append(std::string(base::sqlstring(" ? ", 0) << data));
+  if (_search_mode == Contains)
+    where_condition.append(std::string(base::sqlstring(" ? ", 0) << "%" + data + "%"));
+  else
+    where_condition.append(std::string(base::sqlstring(" ? ", 0) << data));
   return where_condition;
 }
 
@@ -280,7 +277,7 @@ void DBSearch::count_data(const std::string& schema_name, const std::string& tab
     boost::scoped_ptr<sql::Statement> stmt(_db_conn->createStatement());
     boost::scoped_ptr<sql::ResultSet> rs(stmt->executeQuery(query));
     if (_limit_counter > 0)
-        _limit_counter -= rs->rowsCount();
+      _limit_counter -= (int)rs->rowsCount();
     SearchResultEntry result;
     result.schema = schema_name;
     result.table = table_name;
@@ -288,14 +285,13 @@ void DBSearch::count_data(const std::string& schema_name, const std::string& tab
     result.query = query;
     while(rs->next())
     {
-        size_t col_idx = 1;
         std::vector<std::pair<std::string, std::string> > data;
         data.reserve(select_columns.size());
         data.push_back(std::pair<std::string, std::string>("COUNT", rs->getString(1)));
         _matched_rows += rs->getInt(1);
         result.data.push_back(data);
     }
-    base::GMutexLock lock(_search_result_mutex);
+    base::MutexLock lock(_search_result_mutex);
     _search_result.push_back(result);
 };
 
@@ -307,7 +303,7 @@ void DBSearch::select_data(const std::string& schema_name, const std::string& ta
     boost::scoped_ptr<sql::Statement> stmt(_db_conn->createStatement());
     boost::scoped_ptr<sql::ResultSet> rs(stmt->executeQuery(query));
     if (_limit_counter > 0)
-        _limit_counter -= rs->rowsCount();
+      _limit_counter -= (int)rs->rowsCount();
     SearchResultEntry result;
     result.schema = schema_name;
     result.table = table_name;
@@ -319,14 +315,14 @@ void DBSearch::select_data(const std::string& schema_name, const std::string& ta
         std::vector<std::pair<std::string, std::string> > data;
         data.reserve(select_columns.size());
         for (std::list<std::string>::const_iterator It = select_columns.begin(); It != select_columns.end(); ++It)
-           data.push_back(std::pair<std::string, std::string>(*It, rs->getString(col_idx++)));
+           data.push_back(std::pair<std::string, std::string>(*It, rs->getString((int)col_idx++)));
         if (!data.empty())
             result.data.push_back(data);
     }
-    _matched_rows += result.data.size();
+    _matched_rows += (int)result.data.size();
     if (!result.data.empty())
     {
-        base::GMutexLock lock(_search_result_mutex);
+        base::MutexLock lock(_search_result_mutex);
         _search_result.push_back(result);
     }
 };
@@ -530,7 +526,7 @@ void DBSearch::run(select_func_t select_func)
             std::string limit_clause("");
             if (_limit_counter > 0)
             {
-                int limit = std::min(_limit_counter, _limt_per_table);
+                size_t limit = std::min(_limit_counter, _limt_per_table);
                 std::stringstream sout;
                 sout <<"LIMIT "<< limit;
                 limit_clause = sout.str();
@@ -682,7 +678,7 @@ void DBSearchPanel::activate_menu_item(const std::string &action)
 
     for (size_t c = selection.front()->count(), i = 0; i < c; i++)
     {
-      mforms::TreeNodeRef child = selection.front()->get_child(i);
+      mforms::TreeNodeRef child = selection.front()->get_child((int)i);
       if (!pks.empty())
         pks.append(",");
       pks.append(child->get_string(2));
@@ -706,7 +702,7 @@ void DBSearchPanel::activate_menu_item(const std::string &action)
     std::string pks;
     for (size_t c = selection.front()->count(), i = 0; i < c; i++)
     {
-      mforms::TreeNodeRef child = selection.front()->get_child(i);
+      mforms::TreeNodeRef child = selection.front()->get_child((int)i);
       if (!pks.empty())
         pks.append(",");
       pks.append(child->get_string(2));
@@ -859,7 +855,7 @@ bool DBSearchPanel::update()
     bool is_working = false;
     if (_searcher)
     {
-        base::GMutexLock search_lock(_searcher->get_search_result_mutex());
+        base::MutexLock search_lock(_searcher->get_search_result_mutex());
         is_working = _searcher->is_working(); 
         if (_searcher->is_paused())
         {
