@@ -96,6 +96,15 @@ class MigrationTarget(object):
     def module_db(self):
         return grt.modules.DbMySQL
 
+    def get_os(self):
+        if self.checkConnection():
+            os_name = self.module_fe().getOS(self.connection)
+            self.disconnect()
+            return os_name
+
+        return None
+
+
 
 class MigrationSource(object):
     def __init__(self, state, connection):
@@ -242,16 +251,20 @@ class MigrationSource(object):
             if only_these_catalogs:
                 inexistent_catalogs = set(only_these_catalogs).difference(catalog_names)
                 if inexistent_catalogs:
-                    grt.send_warning('The following catalogs where not found: ' + ', '.join(list(inexistent_catalogs)))
+                    grt.send_warning('The following catalogs were not found: ' + ', '.join(list(inexistent_catalogs)))
                 catalog_names = list(set(only_these_catalogs).difference(inexistent_catalogs)) or self.getCatalogNames()
             self._catalog_names = catalog_names
             grt.send_progress(0.1, "Fetching schema names...")
             accumulated_progress = 0.1
             step_progress_share = 1.0 / (len(catalog_names) + 1e-10)
             for catalog in catalog_names:
+                if not catalog:
+                    continue
                 grt.send_progress(accumulated_progress, 'Fetching schema names from %s...' % catalog)
                 schema_names = self.getSchemaNames(catalog)
                 for schema in schema_names:
+                    if not schema:
+                        continue
                     self.state.sourceSchemataNames.append("%s.%s" % (self._db_module.quoteIdentifier(catalog), self._db_module.quoteIdentifier(schema)))
                 accumulated_progress += 0.9 * step_progress_share
         elif self.rdbms.doesSupportCatalogs == 0:  # The rdbms doesn't support catalogs
@@ -363,6 +376,15 @@ class MigrationSource(object):
         if hasattr(self._rev_eng_module, "cleanup"):
             self._rev_eng_module.cleanup()
 
+    def get_os(self):
+        if hasattr(self.module_re(), 'getOS'):
+            return self.module_re().getOS(self.connection)
+        else:
+            return None
+
+    def get_source_instance(self):
+        return self.module_re().getSourceInstance(self.connection)
+
 
 
 class MigrationPlan(object):
@@ -393,11 +415,11 @@ class MigrationPlan(object):
         elif sys.platform == "darwin":
             self.wbcopytables_path = mforms.App.get().get_executable_path("wbcopytables")
         else:
-            self.wbcopytables_path = mforms.App.get().get_executable_path("wbcopytables")
+            self.wbcopytables_path = mforms.App.get().get_executable_path("wbcopytables-bin")
             if not os.path.exists(self.wbcopytables_path):
-                self.wbcopytables_path = os.path.join(os.path.dirname(grt.root.wb.registry.appExecutablePath), "wbcopytables")
+                self.wbcopytables_path = os.path.join(os.path.dirname(grt.root.wb.registry.appExecutablePath), "wbcopytables-bin")
             if not os.path.exists(self.wbcopytables_path):
-                self.wbcopytables_path = "wbcopytables"
+                self.wbcopytables_path = "wbcopytables-bin"
         if type(self.wbcopytables_path) == unicode:
             self.wbcopytables_path = self.wbcopytables_path.encode("UTF-8")
 
