@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -79,6 +79,7 @@ static const char *mysql_field_type_to_name(enum enum_field_types type)
     case MYSQL_TYPE_VAR_STRING: return "MYSQL_TYPE_VAR_STRING";
     case MYSQL_TYPE_STRING: return "MYSQL_TYPE_STRING";
     case MYSQL_TYPE_GEOMETRY: return "MYSQL_TYPE_GEOMETRY";
+    case MYSQL_TYPE_JSON: return "MYSQL_TYPE_JSON";
     default:
       return "UNKNOWN";
   }
@@ -240,6 +241,7 @@ RowBuffer::RowBuffer(boost::shared_ptr<std::vector<ColumnInfo> > columns,
       case MYSQL_TYPE_STRING:
       case MYSQL_TYPE_VAR_STRING:
       case MYSQL_TYPE_BIT:
+      case MYSQL_TYPE_JSON:
         if (!col->is_long_data)
           bind.buffer_length = (unsigned)col->source_length+1;
 
@@ -600,10 +602,6 @@ ODBCCopyDataSource::ODBCCopyDataSource(SQLHENV env,
   _utf8_blob_buffer = NULL;
   _force_utf8_input = force_utf8_input;
 
-  _iconv = iconv_open("UTF-8", "UCS-2LE");
-  if (_iconv == (iconv_t)-1)
-    throw std::runtime_error("Could not create UCS-2 to UTF-8 conversion context");
-
   SQLAllocHandle(SQL_HANDLE_DBC, env, &_dbc);
 
   // 5s timeout
@@ -633,7 +631,6 @@ ODBCCopyDataSource::~ODBCCopyDataSource()
     free(_blob_buffer);
   if (_utf8_blob_buffer)
     free(_utf8_blob_buffer);
-  iconv_close(_iconv);
 
   if (_stmt_ok)
     SQLFreeHandle(SQL_HANDLE_ENV, _stmt);
@@ -643,8 +640,6 @@ ODBCCopyDataSource::~ODBCCopyDataSource()
 
 void ODBCCopyDataSource::ucs2_to_utf8(char *inbuf, size_t inbuf_len, char *&utf8buf, size_t &utf8buf_len)
 {
-  size_t converted;
-  char *outbuf = (char*)_utf8_blob_buffer;
   size_t outbuf_len = _max_blob_chunk_size;
 
   // outside Windows, SQLWCHAR is wchar_t, which is not always 2 bytes
@@ -662,18 +657,16 @@ void ODBCCopyDataSource::ucs2_to_utf8(char *inbuf, size_t inbuf_len, char *&utf8
   //log_debug3("Convert string with %i chars to buffer size %i\n", inbuf_len, outbuf_len);
 
   // convert data from UCS-2 to utf-8
-#ifdef _WIN32
-  converted = iconv(_iconv,
-                    (const char**)&inbuf, &inbuf_len,
-                    (char**)&outbuf, &outbuf_len);
-#else
-  converted = iconv(_iconv,
-                    (char**)&inbuf, &inbuf_len,
-                    (char**)&outbuf, &outbuf_len);
-#endif
-  if (converted == (size_t)-1)
+  std::string s_outbuf = base::wstring_to_string((wchar_t*)inbuf);
+  outbuf_len = s_outbuf.size();
+  if (outbuf_len > _max_blob_chunk_size - 1)
+	  throw std::logic_error("Output buffer size is greater than max blob chunk size.");
+
+  if (s_outbuf.empty())
     throw std::logic_error(base::strfmt("Error during charset conversion of wstring: %s", strerror(errno)));
 
+  std::strcpy(_utf8_blob_buffer, s_outbuf.c_str());
+
   if (inbuf_len > 0)
     log_warning("%li characters could not be converted to UTF-8 during copy\n",
                 (long)inbuf_len);//, (*_columns)[column-1].source_name.c_str());
@@ -685,18 +678,21 @@ void ODBCCopyDataSource::ucs2_to_utf8(char *inbuf, size_t inbuf_len, char *&utf8
 
 SQLRETURN ODBCCopyDataSource::get_wchar_buffer_data(RowBuffer &rowbuffer, int column)
 {
-  unsigned long *out_length;
-  SQLRETURN ret;
-  SQLLEN len_or_indicator;
-  char* out_buffer;
-  size_t out_buffer_len;
-  SQLWCHAR tmpbuf[64*1024];
+  unsigned long *out_length = NULL;
+  SQLLEN len_or_indicator = 0;
+  char* out_buffer = NULL;
+  size_t out_buffer_len = 0;
+  wchar_t tmpbuf[64 * 1024] = { 0 };
 
-  ret = SQLGetData(_stmt, column, _column_types[column-1], tmpbuf, sizeof(tmpbuf), &len_or_indicator);
+  if(typeid(wchar_t*) != typeid(SQLWCHAR*))
+    log_warning("Forcing wchar_t but SQLWCHAR is of different type which shouldn't happen. Potential problems during migration may occur.\n");
+
+  SQLRETURN ret = SQLGetData(_stmt, column, _column_types[column - 1], tmpbuf, sizeof(tmpbuf), &len_or_indicator);
   // check if the data fits
   //if (len_or_indicator > out_buffer_len)
   //  ;
   rowbuffer.prepare_add_string(out_buffer, out_buffer_len, out_length);
+  memset(out_buffer, 0, out_buffer_len);
   if (SQL_SUCCEEDED(ret))
   {
     if (len_or_indicator == SQL_NO_TOTAL)
@@ -704,43 +700,24 @@ SQLRETURN ODBCCopyDataSource::get_wchar_buffer_data(RowBuffer &rowbuffer, int co
 
     if (len_or_indicator != SQL_NULL_DATA)
     {
-      char *inbuf = (char*)tmpbuf;
-      size_t inbuf_len = len_or_indicator;
-      char *outbuf = (char*)out_buffer;
       size_t outbuf_len = out_buffer_len;
-      size_t converted;
-
-      if (sizeof(SQLWCHAR) > sizeof(unsigned short))
-      {
-        SQLWCHAR *in = (SQLWCHAR*)inbuf;
-        unsigned short *out = (unsigned short*)inbuf;
-        for (size_t i = 0; i < inbuf_len/sizeof(SQLWCHAR); i++)
-          out[i] = in[i];
-        inbuf_len = (inbuf_len/sizeof(SQLWCHAR)) * sizeof(unsigned short);
-      }
-      else if (sizeof(SQLWCHAR) < sizeof(unsigned short))
-        throw std::logic_error("Unexpected architecture. sizeof(SQLWCHAR) < sizeof(unsigned short)!");
 
       //log_debug3("Convert string with %i chars to buffer size %i\n", inbuf_len, outbuf_len);
 
       // convert data from UCS-2 to utf-8
-#ifdef _WIN32
-      converted = iconv(_iconv,
-                        (const char**)&inbuf, &inbuf_len,
-                        (char**)&outbuf, &outbuf_len);
-#else
-      converted = iconv(_iconv,
-                        (char**)&inbuf, &inbuf_len,
-                        (char**)&outbuf, &outbuf_len);
-#endif
-      if (converted == (size_t)-1)
+      std::string s_outbuf = base::wstring_to_string(tmpbuf);
+      //log_debug3("get_wchar_buffer_data wstring_to_string i<%S> o<%s>\n", (wchar_t*)tmpbuf, s_outbuf.c_str());
+
+      outbuf_len = s_outbuf.size();
+      if (outbuf_len > _max_blob_chunk_size - 1)
+      	  throw std::logic_error("Output buffer size is greater than max blob chunk size.");
+
+      if (s_outbuf.empty())
         throw std::logic_error(base::strfmt("Error during charset conversion of wstring: %s", strerror(errno)));
 
-      if (inbuf_len > 0)
-        log_warning("%li characters could not be converted to UTF-8 from column %s during copy\n",
-                    (long)inbuf_len, (*_columns)[column-1].source_name.c_str());
+      std::strcpy(out_buffer, s_outbuf.c_str());
 
-      *out_length = (unsigned long)(out_buffer_len - outbuf_len);
+      *out_length = outbuf_len;
     }
     rowbuffer.finish_field(len_or_indicator == SQL_NULL_DATA);
   }
@@ -759,7 +736,12 @@ SQLRETURN ODBCCopyDataSource::get_date_time_data(RowBuffer &rowbuffer, int colum
   ret = SQLGetData(_stmt, column, SQL_C_CHAR, &out_date, sizeof(out_date), &len_or_indicator);
   if (SQL_SUCCEEDED(ret))
   {
-    if (out_date != NULL)
+    //When driver cannot determine the number of bytes of long data
+    //still available to return in an output buffer it return SQL_NO_TOTAL
+    if (len_or_indicator == SQL_NO_TOTAL)
+      throw std::runtime_error(base::strfmt("Got SQL_NO_TOTAL for string size during copy of column %i", column));
+
+    if (len_or_indicator != SQL_NULL_DATA)
       BaseConverter::convert_date_time(out_date, (MYSQL_TIME*)out_buffer, type);
     else
       ((MYSQL_TIME*)out_buffer)->time_type = MYSQL_TIMESTAMP_NONE;
@@ -798,15 +780,19 @@ SQLRETURN ODBCCopyDataSource::get_char_buffer_data(RowBuffer &rowbuffer, int col
 
 SQLRETURN ODBCCopyDataSource::get_geometry_buffer_data(RowBuffer &rowbuffer, int column)
 {
-  unsigned long *out_length;
-  SQLRETURN ret;
-  SQLLEN len_or_indicator;
-  char* out_buffer;
-  size_t out_buffer_len;
-  SQLWCHAR tmpbuf[64*1024];
-  ret = SQLGetData(_stmt, column, SQL_C_WCHAR, tmpbuf, sizeof(tmpbuf), &len_or_indicator);
+  unsigned long *out_length = NULL;
+  SQLLEN len_or_indicator = 0;
+  char* out_buffer = NULL;
+  size_t out_buffer_len = 0;
+  wchar_t tmpbuf[64 * 1024] = { 0 };
+
+  if(typeid(wchar_t*) != typeid(SQLWCHAR*))
+    log_warning("Forcing wchar_t but SQLWCHAR is of different type which shouldn't happen. Potential problems during migration may occur.\n");
+
+  SQLRETURN ret = SQLGetData(_stmt, column, SQL_C_WCHAR, tmpbuf, sizeof(tmpbuf), &len_or_indicator);
 
   rowbuffer.prepare_add_geometry(out_buffer, out_buffer_len, out_length);
+  memset(out_buffer, 0, out_buffer_len);
   if (SQL_SUCCEEDED(ret))
   {
     if (len_or_indicator == SQL_NO_TOTAL)
@@ -814,41 +800,20 @@ SQLRETURN ODBCCopyDataSource::get_geometry_buffer_data(RowBuffer &rowbuffer, int
 
     if (len_or_indicator != SQL_NULL_DATA)
     {
-      char *inbuf = (char*)tmpbuf;
-      size_t inbuf_len = len_or_indicator;
-      char *outbuf = out_buffer;
       size_t outbuf_len = out_buffer_len;
-      size_t converted;
-
-      if (sizeof(SQLWCHAR) > sizeof(unsigned short))
-      {
-        SQLWCHAR *in = (SQLWCHAR*)inbuf;
-        unsigned short *out = (unsigned short*)inbuf;
-        for (size_t i = 0; i < inbuf_len/sizeof(SQLWCHAR); i++)
-          out[i] = in[i];
-        inbuf_len = (inbuf_len/sizeof(SQLWCHAR)) * sizeof(unsigned short);
-      }
-      else if (sizeof(SQLWCHAR) < sizeof(unsigned short))
-        throw std::logic_error("Unexpected architecture. sizeof(SQLWCHAR) < sizeof(unsigned short)!");
 
       // convert data from UCS-2 to utf-8
-#ifdef _WIN32
-      converted = iconv(_iconv,
-                        (const char**)&inbuf, &inbuf_len,
-                        (char**)&outbuf, &outbuf_len);
-#else
-      converted = iconv(_iconv,
-                        (char**)&inbuf, &inbuf_len,
-                        (char**)&outbuf, &outbuf_len);
-#endif
-      if (converted == (size_t)-1)
+      std::string s_outbuf = base::wstring_to_string(tmpbuf);
+      outbuf_len = s_outbuf.size();
+      if (outbuf_len > _max_blob_chunk_size - 1)
+      	  throw std::logic_error("Output buffer size is greater than max blob chunk size.");
+
+      if (s_outbuf.empty())
         throw std::logic_error(base::strfmt("Error during charset conversion of wstring: %s", strerror(errno)));
 
-      if (inbuf_len > 0)
-        log_warning("%lu characters could not be converted to UTF-8 from column %s during copy\n",
-                    (unsigned long)inbuf_len, (*_columns)[column-1].source_name.c_str());
+      std::strcpy(out_buffer, s_outbuf.c_str());
 
-      *out_length = (unsigned long)(out_buffer_len - outbuf_len);
+      *out_length = outbuf_len;
     }
     rowbuffer.finish_field(len_or_indicator == SQL_NULL_DATA);
   }
@@ -865,14 +830,15 @@ size_t ODBCCopyDataSource::count_rows(const std::string &schema, const std::stri
     throw ConnectionError("SQLAllocHandle", ret, SQL_HANDLE_DBC, _dbc);
 
   std::string q;
+  std::string countStr = _source_rdbms_type == "Mssql" ? "count_big" : "count";
 
   switch (spec.type)
   {
     case CopyAll:
       if (spec.resume && last_pkeys.size())
-        q = base::strfmt("SELECT count(*) FROM %s.%s WHERE %s", schema.c_str(), table.c_str(), get_where_condition(pk_columns, last_pkeys).c_str());
+        q = base::strfmt("SELECT %s(*) FROM %s.%s WHERE %s", countStr.c_str(), schema.c_str(), table.c_str(), get_where_condition(pk_columns, last_pkeys).c_str());
       else
-        q = base::strfmt("SELECT count(*) FROM %s.%s", schema.c_str(), table.c_str());
+        q = base::strfmt("SELECT %s(*) FROM %s.%s", countStr.c_str(), schema.c_str(), table.c_str());
       break;
     case CopyRange:
     {
@@ -883,22 +849,22 @@ size_t ODBCCopyDataSource::count_rows(const std::string &schema, const std::stri
         end_expr = base::strfmt("%s <= %lli", spec.range_key.c_str(), spec.range_end);
       start_expr = base::strfmt("%s >= %lli", spec.range_key.c_str(), spec.range_start);
       if (!end_expr.empty())
-        q = base::strfmt("SELECT count(*) FROM %s.%s WHERE %s AND %s", schema.c_str(), table.c_str(), start_expr.c_str(), end_expr.c_str());
+        q = base::strfmt("SELECT %s(*) FROM %s.%s WHERE %s AND %s", countStr.c_str(), schema.c_str(), table.c_str(), start_expr.c_str(), end_expr.c_str());
       else
-        q = base::strfmt("SELECT count(*) FROM %s.%s WHERE %s", schema.c_str(), table.c_str(), start_expr.c_str());
+        q = base::strfmt("SELECT %s(*) FROM %s.%s WHERE %s", countStr.c_str(), schema.c_str(), table.c_str(), start_expr.c_str());
       break;
     }
     case CopyCount:
     {
       if (spec.resume && last_pkeys.size())
-        q = base::strfmt("SELECT count(*) FROM %s.%s WHERE %s", schema.c_str(), table.c_str(), get_where_condition(pk_columns, last_pkeys).c_str());
+        q = base::strfmt("SELECT %s(*) FROM %s.%s WHERE %s", countStr.c_str(), schema.c_str(), table.c_str(), get_where_condition(pk_columns, last_pkeys).c_str());
       else
-        q = base::strfmt("SELECT count(*) FROM %s.%s", schema.c_str(), table.c_str());
+        q = base::strfmt("SELECT %s(*) FROM %s.%s", countStr.c_str(), schema.c_str(), table.c_str());
       break;
     }
     case CopyWhere:
     {
-      q = base::strfmt("SELECT count(*) FROM %s.%s WHERE %s", schema.c_str(), table.c_str(), spec.where_expression.c_str());
+      q = base::strfmt("SELECT %s(*) FROM %s.%s WHERE %s", countStr.c_str(), schema.c_str(), table.c_str(), spec.where_expression.c_str());
       break;
     }
   }
@@ -1307,7 +1273,7 @@ bool ODBCCopyDataSource::fetch_row(RowBuffer &rowbuffer)
 
 MySQLCopyDataSource::MySQLCopyDataSource(const std::string &hostname, int port,
                     const std::string &username, const std::string &password,
-                    const std::string &socket)
+                    const std::string &socket, bool use_cleartext_plugin)
   : _select_stmt(NULL), _has_long_data(false)
 {
   std::string host = hostname;
@@ -1340,6 +1306,17 @@ MySQLCopyDataSource::MySQLCopyDataSource(const std::string &hostname, int port,
            socket.c_str(), username.c_str());
   }
 
+
+#if defined(MYSQL_VERSION_MAJOR) && defined(MYSQL_VERSION_MINOR) && defined(MYSQL_VERSION_PATCH)
+#if MYSQL_CHECK_VERSION(5,5,27)
+  my_bool use_cleartext = use_cleartext_plugin;
+  mysql_options(&_mysql, MYSQL_ENABLE_CLEARTEXT_PLUGIN, &use_cleartext);
+#else
+  if (use_cleartext_plugin)
+    log_warning("Trying to use the ClearText plugin, but it's not supported by libmysqlclient\n");
+#endif
+#endif
+
   if (!mysql_real_connect(&_mysql, host.c_str(), username.c_str(), password.c_str(), NULL, port, socket.c_str(),
                           CLIENT_COMPRESS))
   {
@@ -1491,6 +1468,7 @@ boost::shared_ptr<std::vector<ColumnInfo> > MySQLCopyDataSource::begin_select_ta
           if(info.is_long_data)
             _has_long_data = true;
 
+          log_debug2("%i - %s: %s\n", i + 1, info.source_name.c_str(), info.source_type.c_str());
           columns->push_back(info);
         }
 
@@ -1501,7 +1479,12 @@ boost::shared_ptr<std::vector<ColumnInfo> > MySQLCopyDataSource::begin_select_ta
         throw ConnectionError("mysql_stmt_result_metadata", &_mysql);
     }
     else
+    {
+      if(mysql_stmt_close(stmt))
+        throw ConnectionError("mysql_stmt_close", &_mysql);
+
       throw ConnectionError("mysql_stmt_prepare", &_mysql);
+    }
   }
   else
     throw ConnectionError("mysql_stmt_init", &_mysql);
@@ -1525,7 +1508,7 @@ bool MySQLCopyDataSource::fetch_row(RowBuffer &rowbuffer)
   bool ret_val = true;
 
   if (mysql_stmt_bind_result(_select_stmt, &(rowbuffer[0])) != 0)
-    throw ConnectionError("mysql_stmt_bind_result", &_mysql);
+    throw ConnectionError(base::strfmt("mysql_stmt_bind_result: %s", mysql_stmt_error(_select_stmt)), &_mysql);
 
   int errcode = mysql_stmt_fetch(_select_stmt);
 
@@ -1542,7 +1525,8 @@ bool MySQLCopyDataSource::fetch_row(RowBuffer &rowbuffer)
             rowbuffer[index].buffer_type == MYSQL_TYPE_LONG_BLOB ||
             rowbuffer[index].buffer_type == MYSQL_TYPE_BLOB ||
             rowbuffer[index].buffer_type == MYSQL_TYPE_STRING ||
-            rowbuffer[index].buffer_type == MYSQL_TYPE_GEOMETRY)
+            rowbuffer[index].buffer_type == MYSQL_TYPE_GEOMETRY ||
+            rowbuffer[index].buffer_type == MYSQL_TYPE_JSON)
           {
             if (rowbuffer[index].buffer_length)
               free(rowbuffer[index].buffer);
@@ -1747,16 +1731,23 @@ void MySQLCopyDataTarget::get_server_version()
 
   std::vector<std::string> parsed_version = base::split(version, ".");
 
-  _major_version = atoi(parsed_version[0].c_str());
+  _major_version = base::atoi<int>(parsed_version[0], 0);
   if (parsed_version.size() > 1)
-    _minor_version = atoi(parsed_version[1].c_str());
+    _minor_version = base::atoi<int>(parsed_version[1], 0);
 
   if (parsed_version.size() > 2)
-    _build_version = atoi(parsed_version[2].c_str());
+    _build_version = base::atoi<int>(parsed_version[2], 0);
 
   log_debug("Detected server version=%s\n", version.c_str());
 }
 
+bool MySQLCopyDataTarget::is_mysql_version_at_least(const int _major, const int _minor, const int _build)
+{
+  return _major_version > _major
+           || (_major_version == _major && _minor_version > _minor)
+           || (_major_version == _major && _minor_version == _minor && _build_version >= _build);
+}
+
 std::string MySQLCopyDataTarget::ps_query()
 {
   std::string q("INSERT INTO ");
@@ -1864,7 +1855,10 @@ enum enum_field_types MySQLCopyDataTarget::field_type_to_ps_param_type(enum enum
       ftype = MYSQL_TYPE_STRING;
       break;
     case MYSQL_TYPE_GEOMETRY:
-      ftype = MYSQL_TYPE_GEOMETRY;
+      if (_source_rdbms_type == "Mysql")
+        ftype = MYSQL_TYPE_BLOB;
+      else
+        ftype = MYSQL_TYPE_GEOMETRY;
       break;
     default:
       break;
@@ -1874,11 +1868,12 @@ enum enum_field_types MySQLCopyDataTarget::field_type_to_ps_param_type(enum enum
 
 MySQLCopyDataTarget::MySQLCopyDataTarget(const std::string &hostname, int port,
                     const std::string &username, const std::string &password,
-                    const std::string &socket, const std::string &app_name,
-                    const std::string &incoming_charset)
+                    const std::string &socket, bool use_cleartext_plugin, const std::string &app_name,
+                    const std::string &incoming_charset, const std::string &source_rdbms_type)
 : _insert_stmt(NULL), _max_allowed_packet(1000000), _max_long_data_size(1000000),// 1M default
-  _row_buffer(NULL), _major_version(0), _minor_version(0), _build_version(0), _use_bulk_inserts(true),
-  _bulk_insert_batch(0)
+_row_buffer(NULL), _major_version(0), _minor_version(0), _build_version(0),
+_use_bulk_inserts(true), _bulk_insert_buffer(this), _bulk_insert_record(this),
+  _bulk_insert_batch(0), _source_rdbms_type(source_rdbms_type)
 {
   std::string host = hostname;
   _truncate = false;
@@ -1890,7 +1885,8 @@ MySQLCopyDataTarget::MySQLCopyDataTarget(const std::string &hostname, int port,
   mysql_init(&_mysql);
 #if defined(MYSQL_VERSION_MAJOR) && defined(MYSQL_VERSION_MINOR) && defined(MYSQL_VERSION_PATCH)
 #if MYSQL_CHECK_VERSION(5,6,6)
-  mysql_options4(&_mysql, MYSQL_OPT_CONNECT_ATTR_ADD, "program_name", app_name.c_str());
+  if (is_mysql_version_at_least(5,6,6))
+    mysql_options4(&_mysql, MYSQL_OPT_CONNECT_ATTR_ADD, "program_name", app_name.c_str());
 #endif
 #endif
 
@@ -1925,6 +1921,16 @@ MySQLCopyDataTarget::MySQLCopyDataTarget(const std::string &hostname, int port,
            socket.c_str(), username.c_str());
   }
 
+#if defined(MYSQL_VERSION_MAJOR) && defined(MYSQL_VERSION_MINOR) && defined(MYSQL_VERSION_PATCH)
+#if MYSQL_CHECK_VERSION(5,5,27)
+  my_bool use_cleartext = use_cleartext_plugin;
+  mysql_options(&_mysql, MYSQL_ENABLE_CLEARTEXT_PLUGIN, &use_cleartext);
+#else
+  if (use_cleartext_plugin)
+    log_warning("Trying to use the ClearText plugin, but it's not supported by libmysqlclient\n");
+#endif
+#endif
+
 
   if (!mysql_real_connect(&_mysql, hostname.c_str(), username.c_str(), password.c_str(), NULL, port, socket.c_str(),
                           CLIENT_COMPRESS))
@@ -1982,7 +1988,7 @@ void MySQLCopyDataTarget::set_target_table(const std::string &schema, const std:
             (*columns)[i].is_unsigned = (fields[i].flags & UNSIGNED_FLAG) != 0;
             if (_get_field_lengths_from_target)
                 (*columns)[i].source_length = fields[i].length;
-            log_debug2("%i - %s: %s\n", i+1, (*columns)[i].target_name.c_str(),
+            log_debug2("%i - %s: %s\n", i + 1, (*columns)[i].target_name.c_str(),
                        mysql_field_type_to_name((*columns)[i].target_type));
           }
         }
@@ -2325,6 +2331,7 @@ bool MySQLCopyDataTarget::append_bulk_column(size_t col_index)
     case MYSQL_TYPE_STRING:
     case MYSQL_TYPE_ENUM:
     case MYSQL_TYPE_SET:
+    case MYSQL_TYPE_JSON:
       _bulk_insert_record.append("'", 1);
       ret_val = _bulk_insert_record.append_escaped((char*)(*_row_buffer)[col_index].buffer, *(*_row_buffer)[col_index].length);
       _bulk_insert_record.append("'", 1);
@@ -2626,7 +2633,7 @@ void MySQLCopyDataTarget::restore_triggers(std::set<std::string> &schemas)
         log_debug("Restoring trigger %s\n", trigger_name.at(trigger_index).c_str());
 
         std::string trigger_def(trigger_sql.at(trigger_index));
-        if (mysql_query(&_mysql, trigger_def.data()) != 0)
+        if (mysql_query(&_mysql, trigger_def.c_str()) != 0)
         {
           // It is ok having an error for duplicated triggers
           if (mysql_errno(&_mysql) != 1235)
@@ -2701,7 +2708,7 @@ void CopyDataTask::copy_table(const TableParam &task)
 {
   boost::shared_ptr<std::vector<ColumnInfo> > columns;
 
-  long long i = 0, total;
+  long long i = 0, total = 0;
   int inserted_records;
 
   time_t start = time(NULL);
@@ -2828,6 +2835,16 @@ bool MySQLCopyDataTarget::InsertBuffer::append_escaped(const char *data, size_t
   // This function is used to create a legal SQL string that you can use in an SQL statement
   // This is needed because the escaping depends on the character set in use by the server
   length += mysql_real_escape_string(_mysql, buffer + length, data, (unsigned long)dlength);
+//  #if defined(MYSQL_VERSION_MAJOR) && defined(MYSQL_VERSION_MINOR) && defined(MYSQL_VERSION_PATCH)
+//  #if MYSQL_CHECK_VERSION(5, 7, 6)
+//    if (_target->is_mysql_version_at_least(5, 7, 6))
+//      length += mysql_real_escape_string_quote(_mysql, buffer + length, data, (unsigned long)dlength, '`');
+//    else
+//      length += mysql_real_escape_string(_mysql, buffer + length, data, (unsigned long)dlength);
+//  #else
+//    length += mysql_real_escape_string(_mysql, buffer + length, data, (unsigned long)dlength);
+//  #endif
+//  #endif
 
   return true;
 }
