@@ -1,4 +1,4 @@
-# Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2012, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -73,7 +73,7 @@ class MainView(WizardPage):
         box.set_spacing(8)
 
         self._keep_schema = mforms.newCheckBox()
-        self._keep_schema.set_text("Keep schemata if they already exist. Objects that already exist will not be recreated or updated.")
+        self._keep_schema.set_text("Keep schemas if they already exist. Objects that already exist will not be recreated or updated.")
         box.add(self._keep_schema, False, True)
 
         self._create_db.set_active(True)
@@ -132,11 +132,12 @@ class MainView(WizardPage):
 
 class CreationProgressView(WizardProgressPage):
     def __init__(self, main):
-        WizardProgressPage.__init__(self, main, "Create Schemata", description="""The SQL scripts generated for the migrated schema objects will now be executed
+        WizardProgressPage.__init__(self, main, "Create Schemas", description="""The SQL scripts generated for the migrated schema objects will now be executed
 in the target database. You can monitor execution in the logs and if there are errors 
 you may correct them in the next step. Table data will be migrated at a later step.""")
 
-        main.add_wizard_page(self, "ObjectMigration", "Create Schemata")
+        self._autostart = True
+        main.add_wizard_page(self, "ObjectMigration", "Create Schemas")
 
 
     def create_ui(self):
@@ -144,7 +145,7 @@ you may correct them in the next step. Table data will be migrated at a later st
         self._script_task = self.add_task(self._create_script_task, "Create Script File")
         self._db_task1 = self.add_task(self._connect_task, "Connect to Target Database")
         self._db_task2 = self.add_task(self._validate_existing_schemata, "Perform Checks in Target")
-        self._db_task3 = self.add_threaded_task(self._create_task, "Create Schemata and Objects")
+        self._db_task3 = self.add_threaded_task(self._create_task, "Create Schemas and Objects")
 
     def page_activated(self, advancing):
         WizardProgressPage.page_activated(self, advancing)
@@ -364,7 +365,7 @@ the schema creation or return to the Manual Editing page to correct them there a
         if object and hasattr(object, "temp_sql"):
             self._code.set_text(object.temp_sql)
             self._code.set_enabled(not object.commentedOut)
-            self._comment_check.set_active(object.commentedOut)
+            self._comment_check.set_active(object.commentedOut != 0)
             self._advbox.set_enabled(True)
             self._apply_btn.set_enabled(False)
             self._revert_btn.set_enabled(False)
@@ -395,7 +396,7 @@ the schema creation or return to the Manual Editing page to correct them there a
         object = self._selected_object()
         if object:
             self._code.set_text(object.temp_sql)
-            self._comment_check.set_active(object.commentedOut)
+            self._comment_check.set_active(object.commentedOut != 0)
         self._apply_btn.set_enabled(False)
         self._revert_btn.set_enabled(False)
   
