@@ -76,7 +76,8 @@ layout = [ ( 'General',
           'lc-messages-dir',
           'skip-character-set-client-handshake']),
       ( 'Backup',
-        [ 'backup_history_log',
+        [ 'backup_elevation',
+          'backup_history_log',
           'backup_history_log_file',
           'backup_progress_log',
           'backup_progress_log_file',
@@ -475,14 +476,6 @@ layout = [ ( 'General',
           'secure-file-priv',
           'skip-grant-tables',
           'skip-show-database']),
-      ( 'Validate password',
-        [ 'backup_elevation',
-          'validate_password_dictionary_file',
-          'validate_password_length',
-          'validate_password_mixed_case_count',
-          'validate_password_number_count',
-          'validate_password_policy',
-          'validate_password_special_char_count']),
       ( 'Authentication',
         [ 'default_authentication_plugin',
           'default_password_lifetime',
