@@ -1,5 +1,13 @@
-variable_groups=[ ('Aborted_clients', ['Networking/Stats']),
+variable_groups = [ ('Aborted_clients', ['Networking/Stats']),
   ('Aborted_connects', ['Networking/Stats']),
+  ('Audit_log_current_size', []),
+  ('Audit_log_event_max_drop_size', []),
+  ('Audit_log_events', []),
+  ('Audit_log_events_filtered', []),
+  ('Audit_log_events_lost', []),
+  ('Audit_log_events_written', []),
+  ('Audit_log_total_size', []),
+  ('Audit_log_write_waits', []),
   ('Binlog_cache_disk_use', ['Binlog']),
   ('Binlog_cache_use', ['Binlog']),
   ('Binlog_stmt_cache_disk_use', ['Binlog']),
@@ -7,7 +15,6 @@ variable_groups=[ ('Aborted_clients', ['Networking/Stats']),
   ('Bytes_received', ['Networking/Stats']),
   ('Bytes_sent', ['Networking/Stats']),
   ('Com_admin_commands', ['Commands/Admin']),
-  ('Com_assign_to_keycache', ['Commands/General']),
   ('Com_alter_db', ['Commands/DDL']),
   ('Com_alter_db_upgrade', ['Commands/DDL']),
   ('Com_alter_event', ['Commands/DDL']),
@@ -18,11 +25,15 @@ variable_groups=[ ('Aborted_clients', ['Networking/Stats']),
   ('Com_alter_tablespace', ['Commands/DDL']),
   ('Com_alter_user', ['Commands/Admin']),
   ('Com_analyze', ['Commands/General']),
+  ('Com_assign_to_keycache', ['Commands/General']),
+  ('Com_backup', []),
+  ('Com_backup_table', []),
   ('Com_begin', ['Commands/Transaction']),
   ('Com_binlog', ['Binlog', 'Commands/General']),
   ('Com_call_procedure', ['Commands/DML']),
   ('Com_change_db', ['Commands/General']),
   ('Com_change_master', ['Commands/Admin']),
+  ('Com_change_repl_filter', []),
   ('Com_check', ['Commands/Admin']),
   ('Com_checksum', ['Commands/General']),
   ('Com_commit', ['Commands/General']),
@@ -53,6 +64,7 @@ variable_groups=[ ('Aborted_clients', ['Networking/Stats']),
   ('Com_drop_view', ['Commands/DDL']),
   ('Com_empty_query', ['Commands/General']),
   ('Com_execute_sql', ['Commands/General']),
+  ('Com_explain_other', []),
   ('Com_flush', ['Commands/General']),
   ('Com_get_diagnostics', ['Commands/Admin']),
   ('Com_grant', ['Commands/Admin']),
@@ -65,12 +77,15 @@ variable_groups=[ ('Aborted_clients', ['Networking/Stats']),
   ('Com_install_plugin', ['Commands/Admin']),
   ('Com_kill', ['Commands/Admin']),
   ('Com_load', ['Commands/Admin']),
+  ('Com_load_master_data', []),
+  ('Com_load_master_table', []),
   ('Com_lock_tables', ['Commands/General']),
   ('Com_optimize', ['Commands/Admin']),
   ('Com_preload_keys', ['Commands/Admin']),
   ('Com_prepare_sql', ['Commands/DML']),
   ('Com_purge', ['Commands/Admin']),
   ('Com_purge_before_date', ['Commands/Admin']),
+  ('Com_purge_bup_log', []),
   ('Com_release_savepoint', ['Commands/Admin']),
   ('Com_rename_table', ['Commands/DDL']),
   ('Com_rename_user', ['Commands/Admin']),
@@ -79,6 +94,8 @@ variable_groups=[ ('Aborted_clients', ['Networking/Stats']),
   ('Com_replace_select', ['Commands/DML']),
   ('Com_reset', ['Commands/Admin']),
   ('Com_resignal', ['Commands/Admin']),
+  ('Com_restore', []),
+  ('Com_restore_table', []),
   ('Com_revoke', ['Commands/Admin']),
   ('Com_revoke_all', ['Commands/Admin']),
   ('Com_rollback', ['Commands/Transaction']),
@@ -86,12 +103,12 @@ variable_groups=[ ('Aborted_clients', ['Networking/Stats']),
   ('Com_savepoint', ['Commands/Transaction']),
   ('Com_select', ['Commands/General']),
   ('Com_set_option', ['Commands/General']),
-  ('Com_signal', ['Commands/Admin']),
   ('Com_show_authors', ['Commands/Show']),
   ('Com_show_binlog_events', ['Commands/Show']),
   ('Com_show_binlogs', ['Commands/Show']),
   ('Com_show_charsets', ['Commands/Show']),
   ('Com_show_collations', ['Commands/Show']),
+  ('Com_show_column_types', []),
   ('Com_show_contributors', ['Commands/Show']),
   ('Com_show_create_db', ['Commands/Show']),
   ('Com_show_create_event', ['Commands/Show']),
@@ -99,18 +116,23 @@ variable_groups=[ ('Aborted_clients', ['Networking/Stats']),
   ('Com_show_create_proc', ['Commands/Show']),
   ('Com_show_create_table', ['Commands/Show']),
   ('Com_show_create_trigger', ['Commands/Show']),
+  ('Com_show_create_user', []),
   ('Com_show_databases', ['Commands/Show']),
   ('Com_show_engine_logs', ['Commands/Show']),
   ('Com_show_engine_mutex', ['Commands/Show']),
   ('Com_show_engine_status', ['Commands/Show']),
-  ('Com_show_events', ['Commands/Show']),
   ('Com_show_errors', ['Commands/Show']),
+  ('Com_show_events', ['Commands/Show']),
   ('Com_show_fields', ['Commands/Show']),
   ('Com_show_function_code', ['Commands/Show']),
   ('Com_show_function_status', ['Commands/Show']),
   ('Com_show_grants', ['Commands/Show']),
+  ('Com_show_innodb_status', []),
   ('Com_show_keys', ['Commands/Show']),
+  ('Com_show_logs', []),
   ('Com_show_master_status', ['Commands/Show']),
+  ('Com_show_ndb_status', []),
+  ('Com_show_new_master', []),
   ('Com_show_open_tables', ['Commands/Show']),
   ('Com_show_plugins', ['Commands/Show']),
   ('Com_show_privileges', ['Commands/Show']),
@@ -122,6 +144,7 @@ variable_groups=[ ('Aborted_clients', ['Networking/Stats']),
   ('Com_show_relaylog_events', ['Commands/Show']),
   ('Com_show_slave_hosts', ['Commands/Show']),
   ('Com_show_slave_status', ['Commands/Show']),
+  ('Com_show_slave_status_nonblocking', []),
   ('Com_show_status', ['Commands/Show']),
   ('Com_show_storage_engines', ['Commands/Show']),
   ('Com_show_table_status', ['Commands/Show']),
@@ -129,6 +152,7 @@ variable_groups=[ ('Aborted_clients', ['Networking/Stats']),
   ('Com_show_triggers', ['Commands/Show']),
   ('Com_show_variables', ['Commands/Show']),
   ('Com_show_warnings', ['Commands/Show']),
+  ('Com_signal', ['Commands/Admin']),
   ('Com_slave_start', ['Commands/Replication']),
   ('Com_slave_stop', ['Commands/Replication']),
   ('Com_stmt_close', ['Commands/Prepared Statement']),
@@ -153,7 +177,7 @@ variable_groups=[ ('Aborted_clients', ['Networking/Stats']),
   ('Connection_errors_accept', ['Networking/Errors']),
   ('Connection_errors_internal', ['Networking/Errors']),
   ('Connection_errors_max_connections', ['Networking/Errors']),
-  ('Connection_errors_peer_address', ['Networking/Errors']),
+  ('Connection_errors_peer_addr', []),
   ('Connection_errors_select', ['Networking/Errors']),
   ('Connection_errors_tcpwrap', ['Networking/Errors']),
   ('Connections', ['Networking/Stats']),
@@ -163,6 +187,9 @@ variable_groups=[ ('Aborted_clients', ['Networking/Stats']),
   ('Delayed_errors', ['General']),
   ('Delayed_insert_threads', ['General']),
   ('Delayed_writes', ['General']),
+  ('Firewall_access_denied', []),
+  ('Firewall_access_granted', []),
+  ('Firewall_cached_entries', []),
   ('Flush_commands', ['General']),
   ('Handler_commit', ['Handler']),
   ('Handler_delete', ['Handler']),
@@ -182,19 +209,25 @@ variable_groups=[ ('Aborted_clients', ['Networking/Stats']),
   ('Handler_savepoint_rollback', ['Handler']),
   ('Handler_update', ['Handler']),
   ('Handler_write', ['Handler']),
+  ('Innodb_available_undo_logs', ['InnoDB/Stats']),
+  ('Innodb_buffer_pool_bytes_data', []),
+  ('Innodb_buffer_pool_bytes_dirty', []),
   ('Innodb_buffer_pool_dump_status', ['InnoDB/Buffer pool']),
   ('Innodb_buffer_pool_load_status', ['InnoDB/Buffer pool']),
   ('Innodb_buffer_pool_pages_data', ['InnoDB/Buffer pool']),
   ('Innodb_buffer_pool_pages_dirty', ['InnoDB/Buffer pool']),
   ('Innodb_buffer_pool_pages_flushed', ['InnoDB/Buffer pool']),
   ('Innodb_buffer_pool_pages_free', ['InnoDB/Buffer pool']),
+  ('Innodb_buffer_pool_pages_latched', []),
   ('Innodb_buffer_pool_pages_misc', ['InnoDB/Buffer pool']),
   ('Innodb_buffer_pool_pages_total', ['InnoDB/Buffer pool']),
-  ('Innodb_buffer_pool_read_ahead_rnd', ['InnoDB/Buffer pool']),
   ('Innodb_buffer_pool_read_ahead', ['InnoDB/Buffer pool']),
   ('Innodb_buffer_pool_read_ahead_evicted', ['InnoDB/Buffer pool']),
+  ('Innodb_buffer_pool_read_ahead_rnd', ['InnoDB/Buffer pool']),
+  ('Innodb_buffer_pool_read_ahead_seq', []),
   ('Innodb_buffer_pool_read_requests', ['InnoDB/Buffer pool']),
   ('Innodb_buffer_pool_reads', ['InnoDB/Buffer pool']),
+  ('Innodb_buffer_pool_resize_status', []),
   ('Innodb_buffer_pool_wait_free', ['InnoDB/Buffer pool']),
   ('Innodb_buffer_pool_write_requests', ['InnoDB/Buffer pool']),
   ('Innodb_data_fsyncs', ['InnoDB/Data']),
@@ -208,9 +241,12 @@ variable_groups=[ ('Aborted_clients', ['Networking/Stats']),
   ('Innodb_dblwr_pages_written', ['InnoDB/General']),
   ('Innodb_dblwr_writes', ['InnoDB/General']),
   ('Innodb_have_atomic_builtins', ['InnoDB/General']),
+  ('Innodb_have_sync_atomic', []),
+  ('Innodb_heap_enabled', []),
   ('Innodb_log_waits', ['InnoDB/Stats']),
   ('Innodb_log_write_requests', ['InnoDB/Stats']),
   ('Innodb_log_writes', ['InnoDB/Stats']),
+  ('Innodb_num_open_files', ['InnoDB/Stats']),
   ('Innodb_os_log_fsyncs', ['InnoDB/Stats']),
   ('Innodb_os_log_pending_fsyncs', ['InnoDB/Stats']),
   ('Innodb_os_log_pending_writes', ['InnoDB/Stats']),
@@ -228,9 +264,8 @@ variable_groups=[ ('Aborted_clients', ['Networking/Stats']),
   ('Innodb_rows_inserted', ['InnoDB/Stats']),
   ('Innodb_rows_read', ['InnoDB/Stats']),
   ('Innodb_rows_updated', ['InnoDB/Stats']),
-  ('Innodb_num_open_files', ['InnoDB/Stats']),
   ('Innodb_truncated_status_writes', ['InnoDB/Stats']),
-  ('Innodb_available_undo_logs', ['InnoDB/Stats']),
+  ('Innodb_wake_ups', []),
   ('Key_blocks_not_flushed', ['Keycache']),
   ('Key_blocks_unused', ['Keycache']),
   ('Key_blocks_used', ['Keycache']),
@@ -240,8 +275,109 @@ variable_groups=[ ('Aborted_clients', ['Networking/Stats']),
   ('Key_writes', ['Keycache']),
   ('Last_query_cost', ['Performance']),
   ('Last_query_partial_plans', ['Performance']),
+  ('Locked_connects', []),
+  ('Max_statement_time_exceeded', []),
+  ('Max_statement_time_set', []),
+  ('Max_statement_time_set_failed', []),
   ('Max_used_connections', ['Networking/Stats']),
+  ('Max_used_connections_time', []),
+  ('mecab_charset', []),
+  ('ndb-nodeid', []),
+  ('Ndb_api_bytes_received_count', []),
+  ('Ndb_api_bytes_received_count_session', []),
+  ('Ndb_api_bytes_received_count_slave', []),
+  ('Ndb_api_bytes_sent_count', []),
+  ('Ndb_api_bytes_sent_count_session', []),
+  ('Ndb_api_bytes_sent_count_slave', []),
+  ('Ndb_api_event_bytes_count', []),
+  ('Ndb_api_event_bytes_count_injector', []),
+  ('Ndb_api_event_data_count', []),
+  ('Ndb_api_event_data_count_injector', []),
+  ('Ndb_api_event_nondata_count', []),
+  ('Ndb_api_event_nondata_count_injector', []),
+  ('Ndb_api_pk_op_count', []),
+  ('Ndb_api_pk_op_count_session', []),
+  ('Ndb_api_pk_op_count_slave', []),
+  ('Ndb_api_pruned_scan_count', []),
+  ('Ndb_api_pruned_scan_count_session', []),
+  ('Ndb_api_pruned_scan_count_slave', []),
+  ('Ndb_api_range_scan_count', []),
+  ('Ndb_api_range_scan_count_session', []),
+  ('Ndb_api_range_scan_count_slave', []),
+  ('Ndb_api_read_row_count', []),
+  ('Ndb_api_read_row_count_session', []),
+  ('Ndb_api_read_row_count_slave', []),
+  ('Ndb_api_scan_batch_count', []),
+  ('Ndb_api_scan_batch_count_session', []),
+  ('Ndb_api_scan_batch_count_slave', []),
+  ('Ndb_api_table_scan_count', []),
+  ('Ndb_api_table_scan_count_session', []),
+  ('Ndb_api_table_scan_count_slave', []),
+  ('Ndb_api_trans_abort_count', []),
+  ('Ndb_api_trans_abort_count_session', []),
+  ('Ndb_api_trans_abort_count_slave', []),
+  ('Ndb_api_trans_close_count', []),
+  ('Ndb_api_trans_close_count_session', []),
+  ('Ndb_api_trans_close_count_slave', []),
+  ('Ndb_api_trans_commit_count', []),
+  ('Ndb_api_trans_commit_count_session', []),
+  ('Ndb_api_trans_commit_count_slave', []),
+  ('Ndb_api_trans_local_read_row_count', []),
+  ('Ndb_api_trans_local_read_row_count_session', []),
+  ('Ndb_api_trans_local_read_row_count_slave', []),
+  ('Ndb_api_trans_start_count', []),
+  ('Ndb_api_trans_start_count_session', []),
+  ('Ndb_api_trans_start_count_slave', []),
+  ('Ndb_api_uk_op_count', []),
+  ('Ndb_api_uk_op_count_session', []),
+  ('Ndb_api_uk_op_count_slave', []),
+  ('Ndb_api_wait_exec_complete_count', []),
+  ('Ndb_api_wait_exec_complete_count_session', []),
+  ('Ndb_api_wait_exec_complete_count_slave', []),
+  ('Ndb_api_wait_meta_request_count', []),
+  ('Ndb_api_wait_meta_request_count_session', []),
+  ('Ndb_api_wait_meta_request_count_slave', []),
+  ('Ndb_api_wait_nanos_count', []),
+  ('Ndb_api_wait_nanos_count_session', []),
+  ('Ndb_api_wait_nanos_count_slave', []),
+  ('Ndb_api_wait_scan_result_count', []),
+  ('Ndb_api_wait_scan_result_count_session', []),
+  ('Ndb_api_wait_scan_result_count_slave', []),
+  ('Ndb_cluster_node_id', []),
+  ('Ndb_config_from_host', []),
+  ('Ndb_config_from_port', []),
+  ('Ndb_conflict_fn_epoch', []),
+  ('Ndb_conflict_fn_epoch2', []),
+  ('Ndb_conflict_fn_epoch2_trans', []),
+  ('Ndb_conflict_fn_epoch_trans', []),
+  ('Ndb_conflict_fn_max', []),
+  ('Ndb_conflict_fn_max_del_win', []),
+  ('Ndb_conflict_fn_old', []),
+  ('Ndb_conflict_last_stable_epoch', []),
+  ('Ndb_conflict_reflected_op_discard_count', []),
+  ('Ndb_conflict_reflected_op_prepare_count', []),
+  ('Ndb_conflict_refresh_op_count', []),
+  ('Ndb_conflict_trans_conflict_commit_count', []),
+  ('Ndb_conflict_trans_detect_iter_count', []),
+  ('Ndb_conflict_trans_reject_count', []),
+  ('Ndb_conflict_trans_row_conflict_count', []),
+  ('Ndb_conflict_trans_row_reject_count', []),
+  ('Ndb_epoch_delete_delete_count', []),
+  ('Ndb_execute_count', []),
+  ('Ndb_last_commit_epoch_server', []),
+  ('Ndb_last_commit_epoch_session', []),
+  ('Ndb_number_of_data_nodes', []),
+  ('Ndb_pruned_scan_count', []),
+  ('Ndb_pushed_queries_defined', []),
+  ('Ndb_pushed_queries_dropped', []),
+  ('Ndb_pushed_queries_executed', []),
+  ('Ndb_pushed_reads', []),
+  ('ndb_recv_thread_activation_threshold', []),
+  ('Ndb_scan_count', []),
   ('Not_flushed_delayed_rows', ['General']),
+  ('Ongoing_anonymous_gtid_violating_transaction_count', []),
+  ('Ongoing_anonymous_transaction_count', []),
+  ('Ongoing_automatic_gtid_violating_transaction_count', []),
   ('Open_files', ['General']),
   ('Open_streams', ['General']),
   ('Open_table_definitions', ['General']),
@@ -257,9 +393,15 @@ variable_groups=[ ('Aborted_clients', ['Networking/Stats']),
   ('Performance_schema_file_handles_lost', ['Performance schema']),
   ('Performance_schema_file_instances_lost', ['Performance schema']),
   ('Performance_schema_hosts_lost', ['Performance schema']),
+  ('Performance_schema_index_stat_lost', []),
   ('Performance_schema_locker_lost', ['Performance schema']),
+  ('Performance_schema_memory_classes_lost', []),
+  ('Performance_schema_metadata_lock_lost', []),
   ('Performance_schema_mutex_classes_lost', ['Performance schema']),
   ('Performance_schema_mutex_instances_lost', ['Performance schema']),
+  ('Performance_schema_nested_statement_lost', []),
+  ('Performance_schema_prepared_statements_lost', []),
+  ('Performance_schema_program_lost', []),
   ('Performance_schema_rwlock_classes_lost', ['Performance schema']),
   ('Performance_schema_rwlock_instances_lost', ['Performance schema']),
   ('Performance_schema_session_connect_attrs_lost', ['Performance schema']),
@@ -269,6 +411,7 @@ variable_groups=[ ('Aborted_clients', ['Networking/Stats']),
   ('Performance_schema_statement_classes_lost', ['Performance schema']),
   ('Performance_schema_table_handles_lost', ['Performance schema']),
   ('Performance_schema_table_instances_lost', ['Performance schema']),
+  ('Performance_schema_table_lock_stat_lost', []),
   ('Performance_schema_thread_classes_lost', ['Performance schema']),
   ('Performance_schema_thread_instances_lost', ['Performance schema']),
   ('Performance_schema_users_lost', ['Performance schema']),
@@ -283,6 +426,23 @@ variable_groups=[ ('Aborted_clients', ['Networking/Stats']),
   ('Qcache_total_blocks', ['Query cache']),
   ('Queries', ['General']),
   ('Questions', ['General']),
+  ('Rpl_semi_sync_master_clients', []),
+  ('Rpl_semi_sync_master_net_avg_wait_time', []),
+  ('Rpl_semi_sync_master_net_wait_time', []),
+  ('Rpl_semi_sync_master_net_waits', []),
+  ('Rpl_semi_sync_master_no_times', []),
+  ('Rpl_semi_sync_master_no_tx', []),
+  ('Rpl_semi_sync_master_status', []),
+  ('Rpl_semi_sync_master_timefunc_failures', []),
+  ('Rpl_semi_sync_master_tx_avg_wait_time', []),
+  ('Rpl_semi_sync_master_tx_wait_time', []),
+  ('Rpl_semi_sync_master_tx_waits', []),
+  ('Rpl_semi_sync_master_wait_pos_backtraverse', []),
+  ('Rpl_semi_sync_master_wait_sessions', []),
+  ('Rpl_semi_sync_master_yes_tx', []),
+  ('Rpl_semi_sync_slave_status', []),
+  ('Rpl_status', []),
+  ('Rsa_public_key', []),
   ('Select_full_join', ['General']),
   ('Select_full_range_join', ['General']),
   ('Select_range', ['General']),
