@@ -852,6 +852,9 @@ class FileOpsNope(object):
 
     def get_owner(self, path): # base operation to build file_exists and remote file selector
         return []
+      
+    def join_paths(self, path, *paths):
+        pass
 _file_ops_classes.append(FileOpsNope)
 
 class FileOpsLinuxBase(object):
@@ -1204,6 +1207,11 @@ class FileOpsLinuxBase(object):
                     
         else:
             raise PermissionDeniedError("Cannot write to target folder: %s" % target_dir)
+
+    def join_paths(self, path, *paths):
+        result = posixpath.join(path, *paths)
+        log_error("Remote join paths [FileOpsLinuxBase]: %s\n" % result)
+        return result
         
         
             
@@ -1312,6 +1320,11 @@ class FileOpsLocalUnix(FileOpsLinuxBase):
                     
         return file_list
 
+    def join_paths(self, path, *paths):
+        result = posixpath.join(path, *paths)
+        log_error("Remote join paths [FileOpsLocalUnix]: %s\n" % result)
+        return result
+
 _file_ops_classes.append(FileOpsLocalUnix)
 
 
@@ -1524,6 +1537,10 @@ class FileOpsLocalWindows(object): # Used for remote as well, if not using sftp
 
         return file_list
 
+    def join_paths(self, path, *paths):
+        result = ntpath.join(path, *paths)
+        log_error("Remote join paths [FileOpsLocalWindows]: %s\n" % result)
+        return result
 
 
 _file_ops_classes.append(FileOpsLocalWindows)
@@ -1642,6 +1659,10 @@ class FileOpsRemoteUnix(FileOpsLinuxBase):
 
         return tmpfilename
         
+    def join_paths(self, path, *paths):
+        result = posixpath.join(path, *paths)
+        log_error("Remote join paths [FileOpsRemoteUnix]: %s\n" % result)
+        return result
 
 _file_ops_classes.append(FileOpsRemoteUnix)
 
@@ -1690,7 +1711,7 @@ class FileOpsRemoteWindows(object):
             total = float(tokens[2].replace(",",""))
             index = 0
             while index < len(measures) and total > 1024:
-                total = total / 1024
+                total = float(total / 1024)
                 index = index + 1
 
 
@@ -1856,6 +1877,11 @@ class FileOpsRemoteWindows(object):
         for d in dirs:
             ret.append((d[0] + "/", d[1]) if include_size else d + "/")
         return ret + list(files)
+
+    def join_paths(self, path, *paths):
+        result = os.path.join(path, *paths)
+        log_error("Remote join paths [FileOpsRemoteWindows]: %s\n" % result)
+        return result
         
 _file_ops_classes.append(FileOpsRemoteWindows)
 
@@ -2000,6 +2026,10 @@ class ServerManagementHelper(object):
         return self.shell.list2cmdline(args)
 
 
+    def join_paths(self, path, *paths):
+        result = self.file.join_paths(path, *paths)
+        log_error("Remote join paths: %s\n" % result)
+        return result
 #===============================================================================
 
 class LocalInputFile(object):
