@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -19,15 +19,12 @@ import StringIO
 import socket
 import threading
 import time
-import os
 
 from mforms import App
-from workbench.db_utils import QueryError, MySQLError
 from wb_server_management import wbaOS
 from wb_common import Users
 
-from grt import log_info, log_error, log_warning, log_debug, log_debug2, log_debug3
-_this_file = os.path.basename(__file__)
+from workbench.log import log_error, log_debug, log_debug2, log_debug3
 
 
 #===============================================================================
@@ -83,7 +80,7 @@ class DBWidgetHandler(object):
             if result is not None and self.widget is not None:
                 try:
                     result = float(result)
-                except (ValueError, TypeError), e:
+                except (ValueError, TypeError):
                     print("Error! Calculation returned returned wrong value. Expected int or float but got", result)
                     result = 0.0
                 self.widget.set_value(result)
@@ -92,49 +89,13 @@ class DBWidgetHandler(object):
                     self.ctrl_be.uitask(self.label.set_text, lbl)
 
 #===============================================================================
-class DBStatsDataSource(DataSource):
-    stopped = 0
-    started = 1
-    skip_poll = 2
-    polling   = 3
-
-    def __init__(self, sql, mon_be):
+class DBStatusDataSource(DataSource):
+    def __init__(self, mon_be):
         DataSource.__init__(self, "sql_source", mon_be, None)
-        self.sql = sql
         self.mon_be = mon_be
         self.sources = {}
         self.rev_sources = {}
-        self.dbconn = None
-        self.ctrl_be.add_me_for_event("server_started", self)
-        self.ctrl_be.add_me_for_event("server_stopped", self)
-        self.ctrl_be.add_me_for_event("shutdown", self)
-        self.poll_control = self.stopped
-        log_debug2('%s:%s.__init__()' % (_this_file, self.__class__.__name__), 'DBStatsDataSource created. poll_control = %i\n' %  self.poll_control)
-
-    def server_started_event(self):
-        self.poll_control = self.started
-        if self.dbconn:
-            self.dbconn.disconnect()
-        self.dbconn = None
-        self.dbconn = self.ctrl_be.get_new_sql_connection()
-        self.poll_control = self.polling
-        log_debug2('%s:%s.server_started_event()' % (_this_file, self.__class__.__name__), 'DBStatsDataSource server started. poll_control = %i\n' %  self.poll_control)
-
-    def server_stopped_event(self):
-        self.poll_control = self.skip_poll
-        if self.dbconn:
-            self.dbconn.disconnect()
-        self.dbconn = None
-        self.poll_control = self.stopped
-        log_debug2('%s:%s.server_stopped_event()' % (_this_file, self.__class__.__name__), 'DBStatsDataSource server stopped. poll_control = %i\n' %  self.poll_control)
-
-    def shutdown_event(self):
-        self.poll_control = self.skip_poll
-        if self.dbconn:
-            self.dbconn.disconnect()
-        self.dbconn = None
-        self.poll_control = self.stopped
-        log_debug2('%s:%s.shutdown_event()' % (_this_file, self.__class__.__name__), 'DBStatsDataSource shutdown. poll_control = %i\n' %  self.poll_control)
+        log_debug2('DBStatusDataSource created.\n')
 
     def add_source(self, name, definition, widget):
         # Source is a dict which contains names of status variables, formula to calc
@@ -165,28 +126,16 @@ class DBStatsDataSource(DataSource):
             widget[0].set_value_range(definition['min'], definition['max'])
 
     def poll(self):
-        log_debug3('%s:%s.poll()' % (_this_file, self.__class__.__name__), 'DBStatsDataSource poll. poll_control = %i\n' %  self.poll_control)
-        if self.poll_control == self.polling and self.dbconn:
-            result = None
-            try:
-                result = self.dbconn.executeQuery(self.sql) #ctrl_be from DataSource
-            except QueryError, e:
-                result = None
-                if e.is_connection_error():
-                    self.dbconn.disconnect()
-                    if e.is_error_recoverable():
-                        self.dbconn = self.ctrl_be.get_new_sql_connection()
-            
-            if result is not None:
-                while result.nextRow():
-                    name = result.stringByName("Variable_name")
-                    if name in self.rev_sources:
-                        rev_src = self.rev_sources[name]
-                        value = float(result.stringByName("Value"))
-                        # rev_src contains list of sources and index of current variable in the sources
-                        for (src, i) in rev_src:
-                            src.set_var(i, value)
-                            res = src.calculate()
+        #log_debug3('%s:%s.poll()' % (_this_file, self.__class__.__name__), 'DBStatusDataSource poll.\n')
+        if self.ctrl_be.status_variables:
+            # update monitor stuff
+            for name in self.rev_sources:
+                rev_src = self.rev_sources[name]
+                value = float(self.ctrl_be.status_variables[name])
+                # rev_src contains list of sources and index of current variable in the sources
+                for (src, i) in rev_src:
+                    src.set_var(i, value)
+                    src.calculate()
 
 #===============================================================================
 class ShellDataSource(DataSource):
@@ -195,18 +144,20 @@ class ShellDataSource(DataSource):
     def __init__(self, name, detected_os_name, mon_be, cpu_widget):
         DataSource.__init__(self, name, mon_be, cpu_widget)
         self.os_name = detected_os_name
+        self._cpu_stat_return = None
 
     def poll(self):
         output = StringIO.StringIO()
         if self.ctrl_be.server_helper.execute_command("/usr/bin/uptime", output_handler=output.write) == 0:
-            data = output.getvalue().strip(" \r\t\n,:.")
+            data = output.getvalue().strip(" \r\t\n,:.").split("\n")[-1]
+            self._cpu_stat_return = None
             load_value = data.split()[-3]
             # in some systems, the format is x.xx x.xx x.xx and in others, it's x.xx, x.xx, x.xx
             load_value = load_value.rstrip(",")
             try:
                 result = float(load_value.replace(',','.'))
             except (ValueError, TypeError):
-                log_error(_this_file, "Shell source %s returned wrong value. Expected int or float but got %s\n" % (self.name, load_value))
+                log_error("Shell source %s returned wrong value. Expected int or float but got '%s'\n" % (self.name, load_value))
                 result = 0
 
             if self.widget is not None:
@@ -214,7 +165,10 @@ class ShellDataSource(DataSource):
                 if self.label_cb is not None:
                     self.ctrl_be.uitask(self.label.set_text, self.label_cb(result))
         else:
-            log_debug(_this_file, "CPU stat command returned error: %s\n" % output.getvalue())
+            value = output.getvalue()
+            if value != self._cpu_stat_return:
+                self._cpu_stat_return = value
+                log_debug("CPU stat command returned error: %s\n" % value)
 
 
 #===============================================================================
@@ -247,8 +201,7 @@ class WinRemoteStats(object):
         if code == 0 and dirpath is not None and dirpath != "%USERPROFILE%":
             script_path = App.get().get_resource_path("mysql_system_status_rmt.vbs")
             filename = "\"" + dirpath + "\\mysql_system_status_rmt.vbs\""
-            log_debug('%s:%s.__init__()' % (_this_file, self.__class__.__name__),
-                      'Script local path is "%s". Will be uploaded to "%s"\n' % (script_path, filename) )
+            log_debug('Script local path is "%s". Will be uploaded to "%s"\n' % (script_path, filename) )
             if script_path is not None and script_path != "":
                 #print "Uploading file to ", filename
                 try:
@@ -276,7 +229,7 @@ class WinRemoteStats(object):
                     self.script = "cscript //NoLogo " + filename + " /DoStdIn"
                     #run ssh in a thread
 
-                    log_debug2('%s:%s.__init__()' % (_this_file, self.__class__.__name__), 'About to run "%s"\n' % self.script)
+                    log_debug2('About to run "%s"\n' % self.script)
 
                     self.chan = None
                     self.out = ""
@@ -359,8 +312,7 @@ class WinRemoteStats(object):
                     ssh_session.close()
                     raise Exception("Can't read from remote Windows script")
 
-        log_debug('%s:%s.reader()' % (_this_file, self.__class__.__name__),
-                  'Leaving monitor thread which polls remote windows\n')
+        log_debug('Leaving monitor thread which polls remote windows\n')
 
     #-----------------------------------------------------------------------------
     def get_cpu(self):
@@ -436,7 +388,7 @@ class WBAdminMonitorBE(object):
                     stats = WinRemoteStats(ctrl_be, server_profile, self.running, cpu_widget)
                     self.sources.append(stats)
 
-        sql_sources = DBStatsDataSource('show global status', self)
+        sql_sources = DBStatusDataSource(self)
         for name, query in sql.iteritems():
             widget = None
             if name in widgets:
@@ -465,7 +417,11 @@ class WBAdminMonitorBE(object):
         return self.ctrl_be
 
     def server_started_event(self):
-        log_debug3('%s:%s.server_started_event()' % (_this_file, self.__class__.__name__), 'Enter\n')
+        log_debug3('Enter\n')
+
+        if self.poll_thread and self.running[0]:
+            # no need to restart the poll thread
+            return
 
         # This is needed to ensure an existing polling thread is finished
         # Before creating the new one
@@ -477,13 +433,13 @@ class WBAdminMonitorBE(object):
         self.running[0] = True
         self.poll_thread = threading.Thread(target = self.poll_sources)
         self.poll_thread.start()
-        log_debug3('%s:%s.server_started_event()' % (_this_file, self.__class__.__name__), 'Leave\n')
+        log_debug3('Leave\n')
 
     def server_stopped_event(self):
-        log_debug3('%s:%s.server_stopped_event()' % (_this_file, self.__class__.__name__), 'Enter\n')
+        log_debug3('Enter\n')
         self.running[0] = False
         self.poll_thread = None
-        log_debug3('%s:%s.server_stopped_event()' % (_this_file, self.__class__.__name__), 'Leave\n')
+        log_debug3('Leave\n')
 
     def stop(self):
         self.running[0] = False
@@ -497,4 +453,4 @@ class WBAdminMonitorBE(object):
             for cmd in self.sources:
                 cmd.poll()
             time.sleep(self.interval)
-        log_debug('%s:%s.poll_sources()' % (_this_file, self.__class__.__name__), 'Exiting monitor thread...\n')
+        log_debug('Exiting monitor thread...\n')
