@@ -1,4 +1,4 @@
-# Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -37,7 +37,7 @@ from workbench.utils import Version
 from workbench.notifications import NotificationCenter
 
 
-from workbench.log import log_info, log_warning, log_error, log_debug
+from workbench.log import log_info, log_warning, log_error, log_debug, log_debug2
 
 
 # How the Administrator Module Works
@@ -369,6 +369,9 @@ class AdministratorContext:
                 if first:
                     flags |= mforms.TaskSectionToggleModeButton
                     first = False
+                if grt.root.wb.options.options['DbSqlEditor:SidebarModeCombined'] == 1:
+                    flags |= mforms.TaskSectionToggleModeButtonPreSelected
+
                 self.sidebar.add_section(sname, stitle, flags)
                 for ident, ititle, icon_path in sitems:
                     self.sidebar.add_section_entry(sname, ident, ititle, icon_path, mforms.TaskEntryAlwaysActiveLink)
@@ -393,13 +396,14 @@ class AdministratorContext:
 #-------------------------------------------------------------------------------
 
 def attachToSQLEditor(name, sender, args):
-    # this is called when a new SQL Editor tab is created
-    # attach our WBA related things to it
-    context = AdministratorContext(sender)
-    sender.customData["adminContext"] = grt.togrt(context)
-    if not sender.isConnected:
-        ignore = mforms.Utilities.add_timeout(0.1, lambda:context.open_into_section("admin_server_status", True))
-        del ignore
+    if sender.connection:
+        # this is called when a new SQL Editor tab is created
+        # attach our WBA related things to it
+        context = AdministratorContext(sender)
+        sender.customData["adminContext"] = grt.togrt(context)
+        if not sender.isConnected:
+            ignore = mforms.Utilities.add_timeout(0.1, lambda:context.open_into_section("admin_server_status", True))
+            del ignore
 
 #-------------------------------------------------------------------------------
 def handleReconnect(name, sender, args):
@@ -673,11 +677,9 @@ test_ssh_connection_is_windows = None
 @ModuleInfo.export(grt.STRING, grt.STRING, grt.classes.db_mgmt_Connection, grt.classes.db_mgmt_ServerInstance)
 def testInstanceSettingByName(what, connection, server_instance):
     global test_ssh_connection
-
     log_debug("Test %s in %s\n" % (what, connection.name))
 
     profile = ServerProfile(connection, server_instance)
-
     if what == "connect_to_host":
         if test_ssh_connection:
             test_ssh_connection = None
@@ -685,12 +687,14 @@ def testInstanceSettingByName(what, connection, server_instance):
         log_info("Instance test: Connecting to %s\n" % profile.ssh_hostname)
 
         try:
-            test_ssh_connection = wb_admin_control.WbAdminControl(profile, None, connect_sql=False)
+            test_ssh_connection = wb_admin_control.WbAdminControl(profile, None, connect_sql=False, test_only = True)
             test_ssh_connection.init()
+                
             grt.send_info("connected.")
         except Exception, exc:
+            log_error("Exception: %s" % exc.message)
             import traceback
-            traceback.print_exc()
+            log_debug2("Backtrace was: " % traceback.format_stack())
             return "ERROR "+str(exc)
         except:
             print "Unknown error"
@@ -699,8 +703,9 @@ def testInstanceSettingByName(what, connection, server_instance):
         try:
             test_ssh_connection.acquire_admin_access()
         except Exception, exc:
+            log_error("Exception: %s" % exc.message)
             import traceback
-            traceback.print_exc()
+            log_debug2("Backtrace was: " % traceback.format_stack())
             return "ERROR "+str(exc)
 
         os_info = test_ssh_connection.detect_operating_system_version()
@@ -874,7 +879,7 @@ def resetPasswordCache(editor):
         if context.ctrl_be and context.ctrl_be.password_handler:
             context.ctrl_be.password_handler.pwd_store = {}
 
-        for service_type in ["ssh", "sshkey", "file", "service.startstop", "remoteshell"]:
+        for service_type in ["local", "ssh", "sshkey", "file", "service.startstop", "remoteshell"]:
             details = handler.get_password_parameters(service_type)
             if details and details != "UAC":
                 title, service, account = details
