@@ -30,7 +30,7 @@ from mforms import newBox, newLabel, newButton, newTextEntry, newTreeNodeView, n
 from wb_admin_utils import not_running_warning_label, make_panel_header
 from wb_admin_security_be import AdminSecurity, PrivilegeInfo, PrivilegeReverseDict, SecurityAdminRoles, WBSecurityValidationError
 from wb_common import PermissionDeniedError
-from workbench.log import log_error
+from workbench.log import log_error, log_debug3
 
 import grt
 from workbench import db_utils
@@ -694,14 +694,26 @@ class FirewallCommands:
         result.nextRow()
         return result.stringByIndex(1)
 
+    def reload_rules(self, userhost):
+        self.execute_multiresult_command("CALL mysql.sp_reload_firewall_rules('%s')" % userhost)
+
     def delete_user_rule(self, userhost, rule):
         result, cnt = self.execute_command("DELETE FROM mysql.firewall_whitelist WHERE USERHOST='%s' AND RULE='%s'" % (userhost, db_utils.escape_sql_string(rule)))
-        return cnt > 0
+        deleted_something = cnt > 0
+        if deleted_something:
+            self.reload_rules(userhost)
+        return deleted_something
 
-    def add_user_rule(self, userhost, rule):
-        firewall_rule = self.normalize_query(rule)
+    def add_user_rule(self, userhost, rule, normalized = False):
+        if not normalized:
+            firewall_rule = self.normalize_query(rule)
+        else:
+            firewall_rule = rule
+            
         if firewall_rule:
-            self.execute_command("INSERT INTO mysql.firewall_whitelist (USERHOST, RULE) VALUES ('%s', '%s')" % (userhost, db_utils.escape_sql_string(firewall_rule)))
+            result = self.execute_command("INSERT INTO mysql.firewall_whitelist (USERHOST, RULE) VALUES ('%s', '%s')" % (userhost, db_utils.escape_sql_string(firewall_rule)))
+            if result:
+                self.reload_rules(userhost)
             return True
         log_error("Adding a firewall user rule failed to normalize the query. Probably, the inserted query does not translate to a firewall rule.\n")
         return False
@@ -739,9 +751,24 @@ class FirewallCommands:
         
         result = multi_result[len(multi_result) - 1]
         
-        if result.nextRow():
-            return result.stringByIndex(1) == "OK"
+        try:
+            if result.nextRow():
+                if result.stringByIndex(1).startswith("ERROR"):
+                    log_error("Firewall: Failed to set user mode (user=%s, mode=%s): %s\n" % (userhost, mode, result.stringByIndex(1)))
+                    return False
+            log_debug3("Firewall: Returning 'True' due to lack of 'ERROR' records\n")
+            return True
+        except SystemError, err:  # sp_set_firewall_mode return resultset only on error, but nextRow will throw exception if result is missing resultset.
+            if self.owner.ctrl_be.target_version and self.owner.ctrl_be.target_version.is_supported_mysql_version_at_least(5, 6, 27):
+                log_debug3("Firewall: Returning 'True' due to server ('%s') not responding with the 'OK' record any more\n" % self.owner.ctrl_be.target_version)
+                return True
+            import traceback
+            log_error("Exception while setting firewall user mode: Expecting 'OK' record for this version ('%s')\n" % str(self.owner.ctrl_be.target_version))
+        except Exception, exc:
+            import traceback
+            log_error("Exception while setting firewall user mode: %s\n" % (traceback.format_exc()))
           
+        log_error("Firewall: Failed to set user mode (user=%s, mode=%s): Unknow error\n" % (userhost, mode))
         return False
 
     def reset_user(self, userhost):
@@ -804,7 +831,7 @@ class FirewallAddRuleDialog(mforms.Form):
     def __init__(self, owner):
         mforms.Form.__init__(self, None, mforms.FormResizable | mforms.FormMinimizable)
 
-        self.set_title("Add new rule")
+        self.set_title("Add new rule or SQL statement")
         self.set_size(400, 300)
         self.center()
 
@@ -818,6 +845,12 @@ class FirewallAddRuleDialog(mforms.Form):
         self.query_box.set_size(400, 200)
         self.content.add(self.query_box, True, True)
         
+        self.normalized = False
+        self.normalized_checkbox = mforms.newCheckBox(False)
+        self.normalized_checkbox.set_text("This is a normalized rule")
+        self.normalized_checkbox.add_clicked_callback(self.normalized_clicked)
+        self.content.add(self.normalized_checkbox, True, True)
+        
         button_box = mforms.newBox(True)
         self.content.add(button_box, False, True)
 
@@ -833,7 +866,10 @@ class FirewallAddRuleDialog(mforms.Form):
         self.cancel_button.set_text("Cancel")
         self.cancel_button.add_clicked_callback(self.cancel_button_pressed)
         button_box.add_end(self.cancel_button, False, True)
-        
+    
+    def normalized_clicked(self):
+        self.normalized = self.normalized_checkbox.get_active()
+    
     def ok_button_pressed(self):
         self.end_modal(True)
 
@@ -865,6 +901,11 @@ class FirewallUserInterface(FirewallUserInterfaceBase):
         self.note = mforms.newLabel("")
         self.build_note()
         firewall_rules_main_box.add(self.note, False, True)
+
+        info_box = mforms.newBox(True)
+        info_label = mforms.newLabel("Manage the rules for the current user. Changing the mode to RECORDING will start collecting the SQL commands used by your application.\nWhen all the neccessary rules were collected, you should set the mode to PROTECTING. You can then fine-tune the set of rules by adding or deleting them.")
+        info_box.add(info_label, True, True)
+        self.add(info_box, False, False)
         
         state_box = mforms.newBox(True)
         
@@ -872,6 +913,7 @@ class FirewallUserInterface(FirewallUserInterfaceBase):
         self.state.add_item("OFF")
         self.state.add_item("PROTECTING")
         self.state.add_item("RECORDING")
+        self.state.add_item("DETECTING")
         self.state.set_size(120, -1)
         self.state.add_changed_callback(self.change_state)
         state_box.add(mforms.newLabel("Mode:"), False, True)
@@ -966,11 +1008,11 @@ class FirewallUserInterface(FirewallUserInterfaceBase):
         self.white_list.clear()
         self.cache_list.clear()
 
-        self.available_rules_label.set_text("Active rules (%s):" % str(self.commands.get_rule_count(self.current_userhost)))
+        self.available_rules_label.set_text("Active rules (%s) - These are the rules used in PROTECTED mode for this user" % str(self.commands.get_rule_count(self.current_userhost)))
         for rule in self.commands.get_user_rules(self.current_userhost):
             self.white_list.add_item(rule)
 
-        self.cached_rules_label.set_text("Rules being recorded (%s):" % str(self.commands.get_cached_rule_count(self.current_userhost)))
+        self.cached_rules_label.set_text("Rules being recorded (%s) - These are the rules gathered while in RECORDING mode" % str(self.commands.get_cached_rule_count(self.current_userhost)))
         for rule in self.commands.get_cached_user_rules(self.current_userhost):
             self.cache_list.add_item(rule)
             
@@ -995,7 +1037,7 @@ class FirewallUserInterface(FirewallUserInterfaceBase):
         dialog = FirewallAddRuleDialog(self)
         if dialog.run():
             rule = dialog.query_box.get_string_value()
-            result = self.commands.add_user_rule(self.current_userhost, rule)
+            result = self.commands.add_user_rule(self.current_userhost, rule, dialog.normalized)
             if not result:
                 Utilities.show_error("Add user rule", "Add a new rule for this user failed to be inserted. Please check the log for more information.", "OK", "", "")
             self.owner.refresh()
@@ -1004,9 +1046,11 @@ class FirewallUserInterface(FirewallUserInterfaceBase):
     def add_from_file_button_click(self):
         dialog = mforms.FileChooser(mforms.OpenFile)
         dialog.set_title("Load firewall rules")
-        dialog.set_extensions("Firewall Rules (*.fwr)|*.fwr", ".fwr")
+        dialog.set_extensions("Firewall Rules (*.fwr)|*.fwr|SQL Statements (*.sql)|*.sql", ".fwr")
         if dialog.run_modal():
-            with open(dialog.get_path()) as f:
+            path = dialog.get_path()
+            is_rules_file = True if path.endswith(".fwr") else False
+            with open(path) as f:
                 content = [x.strip('\n') for x in f.readlines()]
 
             if not self.commands.set_user_mode(self.current_userhost, "OFF"):
@@ -1015,7 +1059,7 @@ class FirewallUserInterface(FirewallUserInterfaceBase):
             added_rules = []
             
             for rule in content:
-                if not self.commands.add_user_rule(self.current_userhost, rule):
+                if not self.commands.add_user_rule(self.current_userhost, rule, is_rules_file):
                     break
                 added_rules.append(rule)
         
@@ -1050,7 +1094,8 @@ class FirewallUserInterface(FirewallUserInterfaceBase):
                 self.owner.refresh()
         
     def save(self):
-        self.commands.set_user_mode(self.current_userhost, self.state.get_string_value())
+        if not self.commands.set_user_mode(self.current_userhost, self.state.get_string_value()):
+            Utilities.show_error("Setting user mode", "There was a problem setting the user mode. Please check the log for more details.", "OK", "", "")
 
 #############################
 
@@ -1162,7 +1207,10 @@ class SecurityAccount(mforms.Box):
         self.username = newTextEntry()
         self.username.set_size(180, -1)
         self.username.add_changed_callback(self.set_dirty)
-        self.username.set_max_length(16) # max username length for mysql
+        if self.owner.ctrl_be.target_version and self.owner.ctrl_be.target_version.is_supported_mysql_version_at_least(5, 7, 8):
+            self.username.set_max_length(32) # max username length for mysql since 5.7.8
+        else:
+            self.username.set_max_length(16) # max username length for mysql before 5.7.8
         self.password = newTextEntry(mforms.PasswordEntry)
         self.password.set_size(180, -1)
         self.password_advice = 'Consider using a password with 8 or more characters with\nmixed case letters, numbers and punctuation marks.'
