@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,8 +22,13 @@
 #pragma warning(disable: 4996) // 'std::_Uninitialized_copy0': Function call with parameters that may be unsafe
                                // A warning caused by the usage of boost in this context.
 
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
+
 #include <set>
 
+#include <glib.h>
+
 #include <boost/signals2.hpp>
 #include <boost/bind.hpp>
 
