@@ -29,7 +29,7 @@ from mforms import FileChooser
 import datetime
 import operator
 import json
-
+from workbench.utils import Version
 
 from workbench.log import log_debug3, log_debug2, log_error
 
@@ -89,6 +89,7 @@ class base_module:
         self._extension = None
         self._allow_remote = False
         self._editor = editor
+        self._targetVersion = Version.fromgrt(editor.serverVersion)
         self._local = True
         self._mapping = []
         self._new_table = False
@@ -103,12 +104,15 @@ class base_module:
         self._encoding = 'utf-8' #default encoding
         self._force_drop_table = False
         self._truncate_table = False
-        self._type_map = {'text':'is_string', 'int': 'is_number', 'double':'is_float'}
+        self._type_map = {'text':'is_string', 'bigint': 'is_bignumber', 'geometry': 'is_geometry', 'int': 'is_number', 'double':'is_float', 'json': 'is_json'}
         self.is_running = False
         self.progress_info = None
         self.item_count = 0
 
     def guess_type(self, vals):
+        def is_json(v):
+            return True if type(v) in [dict, list] else False
+        
         def is_float(v):
             v = str(v)
             try:
@@ -129,9 +133,16 @@ class base_module:
         
         cur_type = None
         for v in vals:
+            if is_json(v):
+                return "json" # If JSON then we can return immediately. 
             if is_int(v):
                 if not cur_type:
-                    cur_type = "int"
+                    #We know that it's int, but let's see if it's maybe big int
+                    val = int(v)
+                    if val > 2147483647 or val < -2147483647:
+                        cur_type = "bigint"
+                    else:
+                        cur_type = "int"
                 continue
     
             if is_float(v):
@@ -209,17 +220,20 @@ class base_module:
         self._columns = []
         for c in result.columns:
             self._columns.append({'name': c.name, 'type': c.columnType, 
-                   'is_string': True if c.columnType == "string" else False, 
-                   'is_number': True if c.columnType == "int" else False, 
-                   'is_date_or_time': any(x in c.columnType for x in ['time', 'datetime', 'date']), 
-                   'is_bin': any(x in c.columnType for x in ['geo', 'blob']),
-                   'is_float': True if c.columnType == "real" else False,
+                   'is_string': any(x in c.columnType for x in ['char', 'text', 'set', 'enum']),
+                   'is_json': any(x in c.columnType for x in ['json']), 
+                   'is_number': any(x in c.columnType for x in ['int', 'integer']),
+                   'is_bignumber':  any(x in c.columnType for x in ['bigint']),
+                   'is_geometry':  any(x in c.columnType for x in ['geometry','geometrycollection', 'linestring', 'multilinestring', 'multipoint', 'multipolygon', 'point' , 'polygon']),
+                   'is_date_or_time': any(x in c.columnType for x in ['timestamp', 'time', 'datetime', 'date']), 
+                   'is_bin': any(x in c.columnType for x in ['geo', 'blob', 'binary']),
+                   'is_float': any(x in c.columnType for x in ['decimal', 'float', 'double', 'real']),
                    'value': None})
     
     def prepare_new_table(self):
         try:
             
-            self._editor.executeManagementCommand(""" CREATE TABLE %s (%s)""" % (self._table_w_prefix, ", ".join(["`%s` %s" % (col['name'], col['type']) for col in self._mapping])), 1)
+            self._editor.executeManagementCommand(""" CREATE TABLE %s (%s)""" % (self._table_w_prefix, ", ".join(["`%s` %s" % (col['name'], col["type"]) for col in self._mapping])), 1)
             self.update_progress(0.0, "Prepared new table")
             # wee need to setup dest_col for each row, as the mapping is empty if we're creating new table
             for col in self._mapping:
@@ -297,7 +311,7 @@ class csv_module(base_module):
         base_module.__init__(self, editor, is_import)
         self.name = "csv"
         self.title = self.name
-        self.options = {'filedseparator': {'description':'Field Separator', 'type':'select', 'opts':{'TAB':'\t',';':';', ':':':'}, 'value':';', 'entry': None}, 
+        self.options = {'filedseparator': {'description':'Field Separator', 'type':'select', 'opts':{'TAB':'\t',';':';', ':':':', ',':','}, 'value':';', 'entry': None},
                 'lineseparator': {'description':'Line Separator', 'type':'select','opts':{"CR":'\r', "CR LF":'\r\n', "LF":'\n'}, 'value':'\n', 'entry': None}, 
                 'encolsestring': {'description':'Enclose Strings in', 'type':'text', 'value':'"', 'entry': None}};
         
@@ -311,7 +325,7 @@ class csv_module(base_module):
                 limit = "LIMIT %d" % int(self._limit)
                 if self._offset:
                     limit = "LIMIT %d,%d" % (int(self._offset), int(self._limit))
-            return """SELECT %s FROM %s %s""" % (",".join([value['name'] for value in self._columns]), self._table_w_prefix, limit)
+            return """SELECT %s FROM %s %s""" % (",".join(["`%s`" % value['name'] for value in self._columns]), self._table_w_prefix, limit)
         else:
             limit = ""
             if self._limit:
@@ -325,7 +339,7 @@ class csv_module(base_module):
             return """SELECT %s FROM %s INTO OUTFILE '%s' 
                         FIELDS TERMINATED BY '%s' 
                         ENCLOSED BY '%s' 
-                        LINES TERMINATED BY %s %s""" % (",".join([value['name'] for value in self._columns]), self._table_w_prefix, fpath,
+                        LINES TERMINATED BY %s %s""" % (",".join(["`%s`" % value['name'] for value in self._columns]), self._table_w_prefix, fpath,
                                                        self.options['filedseparator']['value'], self.options['encolsestring']['value'], repr(self.options['lineseparator']['value']), limit)
 
     def start_export(self):
@@ -368,10 +382,12 @@ class csv_module(base_module):
                         self.update_progress(round(self._current_row / self._max_rows, 2), "Data export")
                         row = []
                         for col in self._columns:
-                            if col['is_number']:
+                            if col['is_number'] or col['is_bignumber']:
                                 row.append(rset.intFieldValueByName(col['name']))
                             elif col['is_float']:
                                 row.append(rset.floatFieldValueByName(col['name']))
+                            elif col['is_geometry']:
+                                row.append(rset.geoStringFieldValueByName(col['name']))
                             else:
                                 row.append(rset.stringFieldValueByName(col['name']))
                         output.writerow(row)
@@ -404,6 +420,8 @@ class csv_module(base_module):
             col_order = dict([(i['dest_col'], i['col_no']) for i in self._mapping if i['active']])
             col_type = dict([(i['dest_col'], i['type']) for i in self._mapping if i['active']])
 
+            is_server_5_7 = self._targetVersion.is_supported_mysql_version_at_least(Version.fromstr("5.7.5"))
+            
             self._editor.executeManagementCommand(query, 1)
             try:
                 is_header = self.has_header
@@ -423,7 +441,7 @@ class csv_module(base_module):
                         is_header = False
                         continue
 
-                    self.item_count = self.item_count + 1
+                    
                     self.update_progress(round(self._current_row / self._max_rows, 2), "Data import")
 
                     for i, col in enumerate(col_order):
@@ -432,15 +450,27 @@ class csv_module(base_module):
                             result = False
                             break
                         val = row[col_order[col]]
-                        if col_type[col] == 'double':
-                            val = row[col_order[col]].replace(self._decimal_separator, ',')
-                        elif col_type[col] == 'datetime':
-                            val = datetime.datetime.strptime(row[col_order[col]], self._date_format).strftime("%Y-%m-%d %H:%M:%S")
+                        col_name = col_order[col]
 
-                        self._editor.executeManagementCommand("""SET @a%d = "%s" """ % (i, val), 0)
+                        if col_type[col] == "geometry":
+                            if is_server_5_7:
+                                val = """ST_GeomFromText("%s")""" % row[col_name]
+                            else:
+                                val = """GeomFromText("%s")""" % row[col_name]
+                                
+                            self._editor.executeManagementCommand("""SET @a%d = %s """ % (i, val), 0)
+                        else:
+                            if col_type[col] == 'double':
+                                val = row[col_name].replace(self._decimal_separator, '.')
+                            elif col_type[col] == 'datetime':
+                                val = datetime.datetime.strptime(row[col_name], self._date_format).strftime("%Y-%m-%d %H:%M:%S")
+                            if hasattr(val, "replace"):
+                                val = val.replace("\\", "\\\\").replace('"', '\\"')
+                            self._editor.executeManagementCommand("""SET @a%d = "%s" """ % (i, val), 0)
                     else:
                         try:
                             self._editor.executeManagementCommand("EXECUTE stmt USING %s" % ", ".join(['@a%d' % i for i, col in enumerate(col_order)]), 0)
+                            self.item_count = self.item_count + 1
                         except Exception, e:
                             log_error("Row import failed with error: %s" % e)
                 self.update_progress(1.0, "Import finished")
@@ -463,10 +493,14 @@ class csv_module(base_module):
                 self.dialect = csv.Sniffer().sniff(csvsample)
                 self.has_header = csv.Sniffer().has_header(csvsample)
                 csvfile.seek(0)
+                self.options['filedseparator']['value'] = self.dialect.delimiter 
+                self.options['lineseparator']['value'] = self.dialect.lineterminator 
+                self.options['encolsestring']['value'] = self.dialect.quotechar 
             else:
                 self.dialect.delimiter = self.options['filedseparator']['value']
                 self.dialect.lineterminator = self.options['lineseparator']['value']
                 self.dialect.quotechar = self.options['encolsestring']['value']
+                csvfile.seek(0)
                 
             try:
                 reader = UniReader(csvfile, self.dialect, encoding=self._encoding)
@@ -479,12 +513,18 @@ class csv_module(base_module):
                 
                 if row:
                     for col_value in row:
-                        self._columns.append({'name': col_value, 'type': 'text', 'is_string': True, 'is_number': False, 'is_date_or_time': False, 'is_bin': False, 'is_float':False, 'value': []})
+                        self._columns.append({'name': col_value, 'type': 'text', 'is_string': True, 'is_geometry': False, 'is_bignumber': False, 'is_number': False, 'is_date_or_time': False, 'is_bin': False, 'is_float':False, 'is_json':False,'value': []})
 
                     for i, row in enumerate(reader): #we will read only first few rows
                         if i < 5:
                             for j, col_value in enumerate(row):
-                                self._columns[j]['value'].append(col_value)
+                                try:
+                                    json_value = json.loads(col_value)
+                                    self._columns[j]['is_string'] = False
+                                    self._columns[j]['is_json'] = True
+                                    self._columns[j]['value'].append(json_value)
+                                except Exception as e:
+                                    self._columns[j]['value'].append(col_value)
                         else:
                             break
                     for col in self._columns:
@@ -503,7 +543,6 @@ class csv_module(base_module):
                                         col[attrib] = True
                                     else:
                                         col[attrib] = False
-
             except (UnicodeError, UnicodeDecodeError), e:
                 import traceback
                 log_error("Error analyzing file, probably encoding issue: %s\n Traceback is: %s" % (e, traceback.format_exc()))
@@ -564,12 +603,17 @@ class json_module(base_module):
                     self._current_row = rset.currentRow + 1
                     row = []
                     for col in self._columns:
-                        if col['is_string'] or col['is_bin']:
-                            row.append("\"%s\":%s" % (col['name'], json.dumps(rset.stringFieldValueByName(col['name']))))
-                        elif col['is_number']:
+                        if col['is_number'] or col['is_bignumber']:
                             row.append("\"%s\":%s" % (col['name'], json.dumps(rset.intFieldValueByName(col['name']))))
                         elif col['is_float']:
                             row.append("\"%s\":%s" % (col['name'], json.dumps(rset.floatFieldValueByName(col['name']))))
+                        elif col['is_geometry']:
+                            row.append("\"%s\":%s" % (col['name'], rset.geoJsonFieldValueByName(col['name'])))
+                        else:
+                            if col['type'] == "json":
+                                row.append("\"%s\":%s" % (col['name'], rset.stringFieldValueByName(col['name'])))
+                            else:
+                                row.append("\"%s\":%s" % (col['name'], json.dumps(rset.stringFieldValueByName(col['name']))))
                     ok = rset.nextRow()
                     jsonfile.write("{%s}%s" % (', '.join(row), ",\n " if ok else ""))
                     jsonfile.flush()
@@ -596,6 +640,7 @@ class json_module(base_module):
             query = """PREPARE stmt FROM 'INSERT INTO %s (%s) VALUES(%s)'""" % (self._table_w_prefix, ",".join(dest_col_order), ",".join(["?" for i in dest_col_order]))
             col_order = dict([(i['dest_col'], i['name']) for i in self._mapping if i['active']])
             col_type = dict([(i['name'], i['type']) for i in self._mapping if i['active']])
+            
             self._editor.executeManagementCommand(query, 1)
             try:
                 self._max_rows = len(data)
@@ -604,28 +649,48 @@ class json_module(base_module):
                         log_debug2("Worker thread was stopped by user")
                         self._editor.executeManagementCommand("DEALLOCATE PREPARE stmt", 1)
                         return False
-
                     self._current_row = self._current_row + 1
-                    self.item_count = self.item_count + 1
                     for i, col in enumerate(col_order):
                         if col_order[col] not in row:
                             log_error("Can't find col: %s in row: %s" % (col_order[col], row))
                             result = False
                             break
+
                         val = row[col_order[col]]
-                        if col_type[col] == 'double':
-                            val = row[col_order[col]].replace(self._decimal_separator, ',')
-                        elif col_type[col] == 'datetime':
-                            val = datetime.datetime.strptime(row[col_order[col]], self._date_format).strftime("%Y-%m-%d %H:%M:%S")
+                        col_name = col_order[col]
+
+                        if col_type[col] == "geometry":
+                            val = """ ST_GeomFromGeoJSON('%s')""" % json.dumps(val).replace("\\", "\\\\").replace("'", "\\'")
+                                 
+                            self._editor.executeManagementCommand("""SET @a%d = %s """ % (i, val), 0)
+                        else:
+                            if col_type[col_name] != "json" and hasattr(val, "replace"):
+                                val = val.replace("\\", "\\\\").replace('"', '\\"')
+                                
+                            if col_type[col_name] == 'double':
+                                val = val(str).replace(self._decimal_separator, '.')
+                            elif col_type[col_name] == 'datetime':
+                                val = datetime.datetime.strptime(val, self._date_format).strftime("%Y-%m-%d %H:%M:%S")
+                            elif col_type[col_name] == "json":
+                                val = json.dumps(val).replace("\\", "\\\\").replace("'", "\\'")
+    
+                            if col_type[col_name] == "int":
+                                self._editor.executeManagementCommand("""SET @a%d = %d """ % (i, int(val)), 0)
+                            elif col_type[col_name] == "json":
+                                self._editor.executeManagementCommand("""SET @a%d = '%s' """ % (i, val), 0)
+                            else:
+                                self._editor.executeManagementCommand("""SET @a%d = "%s" """ % (i, val), 0)
                             
-                        self._editor.executeManagementCommand("""SET @a%d = "%s" """ % (i, val), 0)
                     else:
                         try:
                             self._editor.executeManagementCommand("EXECUTE stmt USING %s" % ", ".join(['@a%d' % i for i, col in enumerate(col_order)]), 0)
+                            self.item_count = self.item_count + 1
                         except Exception, e:
                             log_error("Row import failed with error: %s" % e)
                         
             except Exception, e:
+                import traceback
+                log_debug3("Import failed traceback: %s" % traceback.format_exc())
                 log_error("Import failed: %s" % e)
             self._editor.executeManagementCommand("DEALLOCATE PREPARE stmt", 1)
             
@@ -640,7 +705,7 @@ class json_module(base_module):
             datachunk = []
             rowcount = 0 
             while True:
-                if f.tell() >= 4096:
+                if f.tell() >= 20480:
                     log_error("Json file contains data that's in unknown structure: %s" % (self._filepath))
                     return False
                 c = f.read(1)
@@ -677,10 +742,13 @@ class json_module(base_module):
         
         self._columns = []
         for elem in data[0]:
-            self._columns.append({'name': elem, 'type': 'text', 'is_string': True, 'is_number': False, 'is_date_or_time': False, 'is_bin': False, 'is_float':False, 'value': []})
+            self._columns.append({'name': elem, 'type': 'text', 'is_string': True, 'is_geometry': False, 'is_bignumber': False, 'is_number': False, 'is_date_or_time': False, 'is_bin': False, 'is_float':False, 'is_json':False, 'value': []})
 
         for row in data:
             for i, elem in enumerate(row):
+                if type(row[elem]) in [dict, list]:
+                    self._columns[i]['is_string'] = False
+                    self._columns[i]['is_json'] = True
                 self._columns[i]['value'].append(row[elem])
                 
         for col in self._columns:
@@ -695,7 +763,6 @@ class json_module(base_module):
                             col[attrib] = True
                         else:
                             col[attrib] = False
-        
         self._last_analyze = True
         return True
         
@@ -704,4 +771,4 @@ def create_module(type, editor, is_import):
     if type == "csv":
         return csv_module(editor, is_import);
     if type == "json":
-        return json_module(editor, is_import);
\ No newline at end of file
+        return json_module(editor, is_import);
